{==============================================================================
  PROGRAM: CARDSHOE.PAS
  DESCRIPTION: Delphi component to manage 1 to 8 decks of playing cards
  AUTHOR: Ralph W. Iden (riden@ais.net)

  Released into the public domain, enjoy.
 ==============================================================================}

unit Cardshoe;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

{$R CARDSHOE.RES}

type
  CardElement = record CardID:byte; RndValue:integer; end;

  TCardShoe = class(TComponent)
  public
    fOnEmptyDeck : TNotifyEvent;
  protected
    {}
  private
    fNumOfCards : integer;
    fNumOfDecks : integer;
    fTopOfDeck  : integer;
    fCards : Array[1..416] of CardElement; {Handles up to 8 decks}
  public
    constructor Create(AOwner:TComponent);
    function GetTopCard:integer;
    function GetTopCardAsStr:string;
    procedure SetNumberOfDecks(Decks:integer);
    procedure Shuffle;
  published
   {Properties}
    property NumOfDecks:Integer read fNumOfDecks write SetNumberOfDecks default 1;
   {Events}
    property OnEmptyDeck:TNotifyEvent read fOnEmptyDeck write fOnEmptyDeck;
  end;

procedure Register;

implementation

{The following values may conflict with your applications. I defined them
 here so I could link in CARDSHOE.RES into the unit. If there is a conflict,
 change the following values and use Resource Workshop to renumber the
 string identifiers in CARDSHOE.RES to match your new values}

{String resource identifiers}
const StrBase  : word = 60000;
const FaceBase : word = 60002;
const SuitBase : word = 60015;

procedure Register;
begin
  RegisterComponents('Samples', [TCardShoe]);
end;

constructor TCardShoe.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  fNumOfDecks:=1;
  fNumOfCards:=52;
  Shuffle;
end;

function TCardShoe.GetTopCard;
begin
  {If there are more cards to draw, return the next card else return -1}
  if fTopOfDeck <= fNumOfCards then
  begin
    Result:=fCards[fTopOfDeck].CardID;
    inc(fTopOfDeck);
  end else
  begin
    if assigned(fOnEmptyDeck) then fOnEmptyDeck(Self);
    Result:=-1;
  end;
end;

function TCardShoe.GetTopCardAsStr;
var Card,Suit,Value:integer; S:string;
begin
  Card:=GetTopCard;
  if Card=-1 then Result:=LoadStr(StrBase) else
  begin
    Suit:=Card div 13;
    Value:=Card mod 13;
    Result:=LoadStr(FaceBase+Value)+LoadStr(StrBase+1)+LoadStr(SuitBase+Suit);
  end;
end;

procedure TCardShoe.SetNumberOfDecks(Decks:integer);
begin
  if Decks<1 then Decks:=1;
  if Decks>8 then Decks:=8;
  fNumOfDecks:=Decks;
  fNumOfCards:=Decks*52;
  Shuffle;
end;

procedure TCardShoe.Shuffle;
var H,I,J,K:integer; Temp:CardElement;
begin
  {Reset top of deck}
  fTopOfDeck:=1;

  {First fill the card array with random values}
  Randomize;
  for I:=1 to fNumOfCards do
  begin
    fCards[I].CardID:=(I-1) mod 52;    {Initialize the card values}
    fCards[I].RndValue:=Random(32000); {Give the sort routine some room}
  end;

  {Do a Shell sort and the deck is ready}
  H:=1; while H<=fNumOfCards div 9 do H:=H*3+1;
  while H>0 do
  begin
    for J:=H to fNumOfCards do
    begin
      Temp:=fCards[J];
      K:=J-H;
      while K>=0 do
      begin
        if Temp.RndValue < fCards[K].RndValue then
        begin
          fCards[K+H]:=fCards[K];
          K:=K-H;
        end else break;
      end;
      fCards[K+H]:=Temp;
    end;
    H:=H div 3;
  end;

end;

end.
