// MyOle ver. 0.91
// Author: Matej Trampus, Slovenija
// E-MAIL:
//   - Matej.Trampus@kiss.uni-lj.si
// See readme.txt for details.
//
unit ClassFactory;

// BUGS:
//  - TClassFactory.LockServer not implemented
//  - TClassFactory.Destroy does not care for FRefCount
//  - We should count, how many objects are active in TDelhiOle.Create, TDelphiOle.Destroy
//    and allow shoutDown of classFactory when count is 0 (see online help)
//    However forms.DoneApplication should be called *before* destroying class factories
//    to destory any TDelphiOle objects, that are still referenced by not yet destroyed forms
//    If there would be still some TDelphiObjects left, the application should hide itself, waiting
//    for remaining objects to be destroyed...
//  - I am not shure which flag to use in CoRegisterClassObject
//
// HISTORY:
//--- ver 0.91
//  26.10.1996 Added automatic registration
//             Added procedures for entering registration data in Registry
//  25.10.1996 Added clsid2TMyOle,iOleObject2TDelphiOle


interface
uses ole2,windows,myOle;

type
  IUnknownClass=class of IUnknown;

  TMyClassFactory = class(IClassFactory)
  private
    FRefCount: Integer;
    FClassRef: TDelphiOleClass;
    regNr:     LongInt; // returned by CoRegisterClassObject
  protected
    function  isInRegistry(cls:TDelphiOleClass):boolean;
    procedure enterInRegistry(cls:TDelphiOleClass);
  public
    constructor Create(ClassRef: TDelphiOleClass);
    destructor Destroy; override;
    function QueryInterface(const iid: TIID; var obj): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;
    function CreateInstance(unkOuter: IUnknown; const iid: TIID;
      var obj): HResult; override;
    function LockServer(fLock: BOOL): HResult; override;
  end;

procedure RegisterOleObjects(objects:array of TDelphiOleClass); // creates class factories
procedure UnRegisterOleObjects; // destroys all class factories
function  clsid2TMyOle(const CLSID:TClsid):TDelphiOleClass; // successful only if class has been registered with RegisterOleObjects
function  iOleObject2TDelphiOle(obj:IOleObject):TDelphiOle; // nil, if it is not a Delphi object

//  WARNING:
//    clsid2TMyOleClass can find class of object only if the object has been registered with
//    RegisterOleObjects in *this* instance. It does not work with object types, that have been
//    registered in other Delphi applications.

implementation
uses debugMemo1,dataObject,sysutils,classes,registry;

var factories:TList;

//var classFactoryCount:integer;

constructor TMyClassFactory.Create(ClassRef:TDelphiOleClass);
begin
  debug2(self,'Create');
//  Inc(ClassFactoryCount);
  FRefCount := 1;
  FClassRef := ClassRef; // OleCheck!
  debug2(self,'RegisterClassObject ' + intToStr(
           coRegisterClassObject(fClassRef.GetCLSID,self,CLSCTX_LOCAL_SERVER{CLSCTX_INPROC_SERVER},REGCLS_MULTIPLEUSE,regNr)
         )); // is CLSCTX_LOCAL_SERVER correct value???
  if not isInRegistry(classRef) then EnterInRegistry(classRef);       
end;

function TMyClassFactory.isInRegistry(cls:TDelphiOleClass):boolean;
var reg:TRegistry;
begin
   result:=false;
   reg:=TRegistry.Create;
   try
     reg.RootKey:=HKEY_CLASSES_ROOT;
     result:=reg.KeyExists('\CLSID\'+iid2String(cls.GetCLSID));
   finally  reg.Free;  end;
end;

procedure TMyClassFactory.enterInRegistry(cls:TDelphiOleClass);
var reg:TRegistry;  clsId:string;
begin
  reg:=TRegistry.Create;
  try
    with reg do begin
      RootKey:=HKEY_CLASSES_ROOT;
      clsid:=iid2String(cls.GetClsid);
      OpenKey('\CLSID\'+clsid,TRUE);  WriteString('',cls.GetUserType);
        OpenKey('ProgID',TRUE);         WriteString('',cls.GetProgID);

      OpenKey('\'+cls.GetProgID,TRUE); writeString('',cls.GetUserType);
        OpenKey('CLSID',True);           WriteString('',clsid)
     // This is the minimum amount of information which must be entered in
     // registry. If this information is absent, drag'n'drop and QueryInsertObject
     // will not work properlly.
   end;
  finally
    reg.Free;
  end;
end;

destructor TMyClassFactory.Destroy;
begin
  debug2(self,'destroying refCount='+intToStr(fRefCount));
  debug('Calling CoRevokEClassObject....'+intToStr(CoRevokeClassObject(regNr)));
  debug2(self,'... refCount='+intToStr(fRefCount));
//  Dec(ClassFactoryCount);
end;

function TMyClassFactory.QueryInterface(const iid: TIID; var obj): HResult;
begin
  if IsEqualIID(iid, IID_IUnknown) or IsEqualIID(iid, IID_IClassFactory) then begin
    Pointer(obj) := Self;
    AddRef;
    Result := S_OK;
    debug2(self,'QueryInterface + '+iid2String(iid)+' '+self.ClassName);
  end else  begin           
    Pointer(obj) := nil;
    Result := E_NOINTERFACE;
    debug2(self,'not suported: '+iid2String(iid));
  end;
end;

function TMyClassFactory.AddRef: Longint;
begin
  debug2(self,'AddRef');
  Inc(FRefCount);
  Result := FRefCount;
end;

function TMyClassFactory.Release: Longint;
begin
  Dec(FRefCount);
  debug2(self,'Release - count''='+intToStr(FRefCount));
  Result := FRefCount;
  if FRefCount = 0 then Free;
end;

function TMyClassFactory.CreateInstance(unkOuter: IUnknown; const iid: TIID;
  var obj): HResult;
var
  anObject: IUnknown;
begin
  debug2(self,'CreateInstance '+iid2String(iid));
  Pointer(obj) := nil;
  if unkOuter <> nil then begin
    debug2(self,'CreateInstance - aggregation not supported');
    Result := CLASS_E_NOAGGREGATION;
    Exit;
  end;

  try
   anObject := TMyOle.Create(fClassRef);
  except
    on e:exception do begin
      debug2(self,'CreateInstance - creation failed '+e.Message);
      Result := E_UNEXPECTED;
      Exit;
    end;
  end;
  Result := anObject.QueryInterface(iid, obj);
  anObject.Release;
end;

function TMyClassFactory.LockServer(fLock: BOOL): HResult;
begin
  debug2(self,'LockServer - E_FAIL (NOT (yet) IMPLEMENTED!!!!'); // IMPLEMENTIRAJ!!!
  //Automation.CountAutoObject(fLock);
  Result := E_FAIL;
end;

procedure RegisterOleObjects(objects:array of TDelphiOleClass); // creates class factories
var i:integer; f:TMyClassFactory;
begin
  for i:=low(objects) to high(objects) do begin
    f:=TMyClassFactory.Create(objects[i]); // Create class factory
    factories.Add(f);                      // add to the list
  end;
end;

procedure UnRegisterOleObjects; // destroys all class factories
var i:integer;
begin
  debug('DESTROYING FACTORIES. TMyOle OBJECTS LEFT: '+intToStr(_TMyOleCount));
  // we will run into problems if _TMyOleCount>0
  for i:=0 to factories.count-1 do begin
    TMyClassFactory(factories.items[i]).Free; factories.items[i]:=nil;
  end;
end;

function  clsid2TMyOle(const CLSID:TClsid):TDelphiOleClass;
var i:integer;
begin
  for i:=0 to factories.count-1 do begin
    result:=TMyClassFactory(factories.items[i]).FClassRef;
    if IsEqualCLSID(clsid,result.GetClsId) then exit;
  end;
  result:=nil;
end;

function  iOleObject2TDelphiOle(obj:IOleObject):TDelphiOle; // nil, if it is not a Delphi object
var clsid:Tclsid;
begin
  result:=nil; if obj=nil then exit;
  // see note about emulating in online help: IOleObject::GetUserClassID
  if (obj.GetUserClassId(clsid)=S_OK) and (clsid2TmyOle(clsid)<>nil) then
    result:=TMyOle(obj).theObject;
    // only we know, how to create objects of our CLSID, so we can typecast IOleObject
    // to TMyOle. If obj were not created by us, typecast would not be possible
    // (and Delphi would crash trying referencig some other IOleObject as TMyOle)
end;

initialization
  factories:=TList.Create;

finalization
  if factories<>nil then unregisterOleObjects; // see comment at the begining of the file
  factories.Free;
end.
