unit DblList;

interface

uses Objects, DlstElem, Deque, Contain;


{------------- TDoubleList (Deque)----------------
								 Init
								 InitIterator
								 AddAt
								 DetachAtInc,Dec
								 DeleteAtInc,Dec}

{------------- TDoubleListIterator ---------------
								 Init
								 Done
								 This
								 IsFinished
								 Inc,Dec
								 Reset,ResetToHead,ResetToTail}
{------------------------------------------------}


type	PDoubleListIterator=^TDoubleListIterator;
			TDoubleListIterator=object(TContainerIterator)
				constructor Init(TheHead,TheTail:PDoubleListElement);
				destructor Done; virtual;
				function This:Pointer; virtual;
				function Finished:Boolean; virtual;
				function Inc:Boolean; virtual;
				function Dec:Boolean; virtual;
				procedure Reset; virtual;
				procedure ResetToHead; virtual;
				procedure ResetToTail; virtual;
			private
				Head,Tail,Current:PDoubleListElement;
				end;

type	PDoubleList=^TDoubleList;
			TDoubleList=object(TDeque)
				constructor Init;
				function InitIterator:PContainerIterator; virtual;
				procedure AddAt(Iterator:PDoubleListIterator;ToAdd:PObject); virtual;
				function DetachAtInc(Iterator:PDoubleListIterator):Pointer; virtual;
        function DetachAtDec(Iterator:PDoubleListIterator):Pointer; virtual;
				procedure DeleteAtInc(Iterator:PDoubleListIterator); virtual;
        procedure DeleteAtDec(Iterator:PDoubleListIterator); virtual;
				end;


implementation

{-----------------------------------------------}
{-------------DoubleList------------------------}
{--Init-----------------------------------------}

constructor TDoubleList.Init;
begin
TDeque.Init
end;

{--InitIterator---------------------------------}

function TDoubleList.InitIterator:PContainerIterator;
begin
InitIterator:=PContainerIterator(new(PDoubleListIterator, Init(Head,Tail)));
end;

{--AddAt----------------------------------------}

procedure TDoubleList.AddAt(Iterator:PDoubleListIterator;ToAdd:PObject);
var Temp,NewListElem:PDoubleListElement;
begin
if Iterator^.Current=Head then
	begin
	AddAtHead(ToAdd);
	Iterator^.Inc;
	end
else
	begin
	new(NewListElem, Init(ToAdd));
	NewListElem^.Next:=Iterator^.Current;
	NewListElem^.Previous:=Iterator^.Current^.Previous;
	Iterator^.Current^.Previous^.Next:=NewListElem;
	Iterator^.Current^.Previous:=NewListElem;
	Iterator^.Dec;
	end;
Size:=Size-1;
end;

{--DetachAt-------------------------------------}

function TDoubleList.DetachAtInc(Iterator:PDoubleListIterator):Pointer;
var Temp:PDoubleListElement;
begin
with Iterator^ do
	begin
	with Current^ do
		begin
		Temp:=Next;
		DetachAtInc:=Data;
		Data:=nil;
		Previous^.Next:=Next;
		Next^.Previous:=Previous;
		end;
	if not Finished then
		begin
		dispose(Current, Done);
		Size:=Size-1;
		end;
	Current:=Temp;
	end;
end;

function TDoubleList.DetachAtDec(Iterator:PDoubleListIterator):Pointer;
begin
DetachAtDec:=DetachAtInc(Iterator);
Iterator^.Dec;
end;

{--DeleteAt-------------------------------------}

procedure TDoubleList.DeleteAtInc(Iterator:PDoubleListIterator);
var Temp:PObject;
begin
Temp:=DetachAtInc(Iterator);
if Temp<>nil then dispose(Temp, Done);
end;

procedure TDoubleList.DeleteAtDec(Iterator:PDoubleListIterator);
var Temp:PObject;
begin
Temp:=DetachAtDec(Iterator);
if Temp<>nil then dispose(Temp, Done);
end;


{-----------------------------------------------}
{------------TDoubleListIterator----------------}
{--Init-----------------------------------------}

constructor TDoubleListIterator.Init(TheHead,TheTail:PDoubleListElement);
begin
TContainerIterator.Init;
Head:=TheHead;
Tail:=TheTail;
Reset;
Inc;
end;

{--Done-----------------------------------------}

destructor TDoubleListIterator.Done;
begin
TContainerIterator.Done;
end;

{--This-----------------------------------------}

function TDoubleListIterator.This:Pointer;
begin
This:=Current^.Data;
end;

{--IsFinished-----------------------------------}

function TDoubleListIterator.Finished:Boolean;
begin
Finished:=((Current=Tail) or (Current=Head));
end;

{--Inc,Dec--------------------------------------}

function TDoubleListIterator.Inc:Boolean;
begin
Current:=Current^.Next;
Inc:=Finished;
end;

function TDoubleListIterator.Dec:Boolean;
begin
Current:=Current^.Previous;
Dec:=Finished;
end;

{--Reset----------------------------------------}

procedure TDoubleListIterator.Reset;
begin ResetToHead end;
procedure TDoubleListIterator.ResetToHead;
begin Current:=Head end;
procedure TDoubleListIterator.ResetToTail;
begin Current:=Tail end;

{-----------------------------------------------}

end.