unit DBVRead;  { TDBVReader component. }
{ Created 3/16/96 8:57:56 AM }
{ Eagle Software CDK, Version 1.01 Rev. X-2 }
{ Modified 3/17/96 10:06:07 AM by the CDK, Version 1.01 Rev. X-2 }

{ DBV Reader - Data Aware DBV Reader Component}
{
	Author: Bill Wehnert
	Date:		17 March 1996
	(c) copyright 1996, Bill Wehnert - all rights reserved
	(c) copyright 1996, Kyle Poole - all rights reserved

	This component is a wrapper for the DBV Reader that Kyle Poole wrote
	to allow Delphi the ability to read and extract data from a DBV.

	I added to that so that I can specify a datasource and field, and have the
	component handle the minutia.

	To use the component:

	1) Drop a TTable, TDataSource, & TDBVReader on your form.
	2) Select your database
	3) Attach your DataSource to your Table
	4) Attach your DBVReader to your DataSource
	5) Pick your field from the DataField Drop Down
	6) Set FileName in the DBVReader to your DBV File
			- full filename & extension!!!
	7) When you want to extract the data in the DBV, call
		DBVReader.SaveToFile('filename.out')

	This process will create a file called "filename.out" that will hold
	the data that was in the DBV at the position of the assigned
	pointer field.


	This code is freely given into the public domain.  You may use it as you
	see fit.  If you distribute an app with source code, please provide this
	unit with the comments intact.

	If you find this unit usefull, drop me a line:

	CIS: 74603, 3153
	Internet: wehnertb@moodys.com

	ENJOY!
}

{  SHAMEFULL PLUG !!!!

	 As you may have noticed, this was written with Eagle Software's CDK
	 utility.  If you are going to do any Control Development, RUN - don't walk
	 to the nearest phone and order this product.  It makes component creation
	 a piece of cake!  Especially if you are unfamiliar with how database aware
	 components are created.  With their "canned" template definitions, it handled
	 all the things I would not have found right up front.

	 Definately worth the price!

}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Menus, DB, DbTables;


type

  TDBVStru = record
    Size : longint;
    Signiture : byte;
    Compression : byte;
  end;

  {Exception class}
	EDBVError = class(Exception);

  TDBVReader = class(TComponent)
  private
    FFieldDataLink: TFieldDataLink;
    { Private declarations }
    FFileName: TFileName;
    FDBVFile: TFileStream;
  	function ConvertPointer(const pointer : string) : longint;
    function GetFileName: TFileName;
    procedure SetFileName(newValue: TFileName);
		procedure VLF2Stream(stream : tstream; const fptr : string);
    procedure DataChange(Sender: TObject);  { Triggered when data changes in DataSource. }
    function GetDataField: string;  { Returns data field name. }
    function GetDataSource: TDataSource;  { Returns linked data source. }
    procedure SetDataField(const newFieldName: string);  { Assigns new field. }
    procedure SetDataSource(newSource: TDataSource);  { Assigns new data source. }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SaveToFile(AFilename: TFileName);
  published
    { Published properties and events }
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property FileName: TFileName read GetFileName write SetFileName;
  end;  { TDBVReader }

procedure Register;

implementation

procedure TDBVReader.DataChange(Sender: TObject);  { Triggered when data changes in DataSource. }
begin
  if FFieldDataLink.Field = nil then
  begin
    { CDK: Update your control to show there is no data link (optional).
    For example, if your control is has a caption property, you might
    use the following line:

    Caption := '*No Data Link*';
    }
    exit;
  end;
  { CDK: Update your control to reflect data change. For example, if this control
    were a descendant of TCalendar, you could use the following line:

      CalendarDate := FFieldDataLink.Field.AsDate;

    Other ways to look at data:

      AsBoolean
      AsDateTime
      AsFloat
      AsInteger
      AsString
      AsDate
      AsTime
      AsSmallInt
      AsWord
      AsCurrency
      AsBCD
  }
end;

function TDBVReader.GetDataField: string;  { Returns data field name. }
begin
  { FFieldDataLink is built in TDBVReader.Create; there's no need to check to see if it's assigned. }
  GetDataField := FFieldDataLink.FieldName;
end;		{ GetDataField }

function TDBVReader.GetDataSource: TDataSource;  { Returns linked data source. }
begin
  { FFieldDataLink is built in TDBVReader.Create; there's no need to check to see if it's assigned. }
  GetDataSource := FFieldDataLink.DataSource;
end;		{ GetDataSource }

procedure TDBVReader.SetDataField(const newFieldName: string);  { Assigns new field. }
begin
  { FFieldDataLink is built in TDBVReader.Create; there's no need to check to see if it's assigned. }
  FFieldDataLink.FieldName := newFieldName;
end;		{ SetDataField }

procedure TDBVReader.SetDataSource(newSource: TDataSource);  { Assigns new data source. }
begin
  { FFieldDataLink is built in TDBVReader.Create; there's no need to check to see if it's assigned. }
  FFieldDataLink.DataSource := newSource;
end;		{ SetDataSource }


function TDBVReader.ConvertPointer(const pointer : string) : longint;
var work : array[0..5] of byte;
   	x : integer;
begin
	move(pointer[1], work, 6);
  for x := 0 to 5 do begin
  	work[x] := (work[x] and 127) or (((work[5] shr x) and 1) shl 7);
  end;
  move(work,result,4);
end;

function TDBVReader.GetFileName: TFileName;
begin
  result := FFileName;
end;  { GetFileName }

procedure TDBVReader.SetFileName(newValue: TFileName);
begin
  if (FFileName <> newValue) then begin
    FFileName := newValue;
  end;  { if }
end;  { SetFileName }

procedure TDBVReader.SaveToFile(AFilename: TFileName);  { private }
var output : TFileStream;
begin
	if FFieldDataLink.Field <> nil then begin
    try
      FDBVFile := TFileStream.Create(FFilename, fmOpenRead or fmShareDenyNone);
    except
      if not (csDesigning in ComponentState) then
        raise EDBVError.Create('Unable to Open ' + FFilename);
    end;
    try
      Output := TFileStream.Create(AFilename, fmCreate);
      (FFieldDataLink.Field as TStringField).Transliterate := False;
      VLF2Stream(output, FFieldDataLink.Field.AsString);
    finally
      Output.Free;
    end;
  end;
end;  { SaveToFile }


procedure TDBVReader.VLF2Stream(stream : tstream; const fptr : string);
var offset : longint;
		Info : TDBVStru;
begin
	offset := ConvertPointer(fptr);
  FDBVFile.Seek(offset, soFromBeginning);
  FDBVFile.Read(Info, SizeOf(Info));
  Stream.CopyFrom(FDBVFile, Info.Size);
end;


destructor TDBVReader.Destroy;
begin
  FFieldDataLink.Free;
	if Assigned(FDBVFile) then
  	FDBVFile.Free;
  inherited Destroy;
end;  { Destroy }

constructor TDBVReader.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {FActive := False;}
  FFieldDataLink := TFieldDataLink.Create;
  FFieldDataLink.OnDataChange := DataChange;  { So we can respond to changes in data. }
end;  { Create }

procedure Register;
begin
  RegisterComponents('Data Access', [TDBVReader]);
end;  { Register }

initialization
end.
