unit Dialogs;

interface

uses ODialogs,OWindows,WinProcs,WinTypes,Strings,IntEdit;

type	ImageInfo=record
				MinDepth:longint;
				MaxHeight:longint;
				Period:longint;
				Brightness:integer;
				Filename:array[0..255] of char;
				PatternOption:integer;
				end;

const id_Dialog='Settings';
			id_OK=1;
			id_Cancel=2;
			id_Help=3;
			id_Defaults=201;
			id_Browse=202;
			id_View=203;
			id_MinDepth=101;
			id_MaxHeight=102;
			id_Period=103;
			id_Static=104;
			id_File=105;
			id_Brightness=106;
			id_Filename=107;
			UsingStatic=1;
			UsingFile=0;

var	CurrentImageSettings,Temporary,Default:ImageInfo;

type	PDSettings=^TDSettings;
			TDSettings=object(TDialog)
				constructor Init(AParent:PWindowsObject);
				procedure SetupWindow; virtual;
				procedure Ok(var Msg:TMessage); virtual id_First+id_Ok;
				procedure IGHelp(var Msg:TMessage); virtual id_First+id_Help;
				procedure IGDefaults(var Msg:TMessage); virtual id_First+id_Defaults;
				procedure IGBrowse(var Msg:TMessage); virtual id_First+id_Browse;
				procedure IGView(var Msg:TMessage); virtual id_First+id_View;
			private
				EditMinDepth:PIntegerEdit;
				EditMaxHeight:PIntegerEdit;
				EditPeriod:PIntegerEdit;
				EditBrightness:PIntegerEdit;
				EditFilename:PEdit;
				RadioStatic:PRadioButton;
				RadioFile:PRadioButton;
				procedure ReadDialog;
				procedure DisplayText;
				end;

implementation

uses Validate,FileDlgs,CommDlg,SaveBmp,GraphWin;

procedure WriteToIniFile(filename,Sec:pchar; ToWrite:Imageinfo); forward;

constructor TDSettings.Init;
var j:PButton;
begin
TDialog.Init(AParent,id_Dialog);
RadioStatic :=new(PRadioButton, InitResource(@Self,id_Static));
RadioFile   :=new(PRadioButton, InitResource(@Self,id_File));
EditMinDepth	:=new(PIntegerEdit, InitResource(@Self,id_MinDepth,15));
EditMaxHeight	:=new(PIntegerEdit, InitResource(@Self,id_MaxHeight,15));
EditPeriod		:=new(PIntegerEdit, InitResource(@Self,id_Period,15));
EditBrightness:=new(PIntegerEdit, InitResource(@Self,id_Brightness,15));
EditFilename	:=new(PEdit, InitResource(@Self,id_Filename,255));
EditMinDepth^. 	Validator:=new(PRangeValidator, Init(-1000,1000));
EditMaxHeight^.	Validator:=new(PRangeValidator, Init(-1000,1000));
EditPeriod^.		Validator:=new(PRangeValidator, Init(1,10000));
EditBrightness^.Validator:=new(PRangeValidator, Init(0,100));
j:=new(PButton, InitResource(@Self,id_OK));
j:=new(PButton, InitResource(@Self,id_Cancel));
j:=new(PButton, InitResource(@Self,id_Help));
j:=new(PButton, InitResource(@Self,id_Defaults));
end;

procedure TDSettings.SetupWindow;
begin
TDialog.SetupWindow;
Temporary:=CurrentImageSettings;
DisplayText;
end;

procedure TDSettings.DisplayText;
begin
EditMinDepth^.SetValue(Temporary.MinDepth);
EditMaxHeight^.SetValue(Temporary.MaxHeight);
EditPeriod^.SetValue(Temporary.Period);
EditBrightness^.SetValue(Temporary.Brightness);
EditFilename^.SetText(Temporary.Filename);
RadioStatic^.SetCheck(Temporary.PatternOption);
RadioFile^.SetCheck(1-Temporary.PatternOption);
end;

procedure TDSettings.ReadDialog;
begin
Temporary.MinDepth:=EditMinDepth^.GetValue;
Temporary.MaxHeight:=EditMaxHeight^.GetValue;
Temporary.Period:=EditPeriod^.GetValue;
Temporary.Brightness:=EditBrightness^.GetValue;
EditFilename^.GetText(Temporary.Filename,255);
Temporary.PatternOption:=RadioStatic^.GetCheck;
end;

procedure TDSettings.Ok;
begin
ReadDialog;
if (Temporary.PatternOption=UsingFile) and (not FileExists(Temporary.Filename)) then
	MessageBox(HWindow,'File does not exist','Error',mb_IconStop or mb_OK)
else
	begin
	CurrentImageSettings:=Temporary;
	WriteToIniFile('rds.ini','CurrentSettings',CurrentImageSettings);
	TDialog.OK(Msg);
	end;
end;

procedure TDSettings.IGDefaults;
begin
Temporary:=Default;
DisplayText;
end;

procedure TDSettings.IGHelp(var Msg:TMessage);
begin
WinHelp(Application^.MainWindow^.HWindow,'rds.hlp',help_context,2);
end;

procedure TDSettings.IGBrowse;
var	FileStruct:TOpenFilename;
		Filename:array[0..255] of char;
const Filters='Bitmap (*.bmp)'#0'*.bmp'#0'All files (*.*)'#0'*.*'#0#0;
begin
Filename[0]:=#0;
FillFileStructure(FileStruct,HWindow,Filename,filters,255);
FileStruct.lpstrTitle:='Select a file';
if GetOpenFilename(FileStruct) then
	begin
	StrCopy(Temporary.Filename,Filename);
	Temporary.PatternOption:=UsingFile;
	DisplayText;
	end;
end;

procedure TDSettings.IGView;
var	Bitmap:PSaveableBitmap;
		Window:PGraphicsWindow;
begin
ReadDialog;
if not FileExists(Temporary.Filename) then
	MessageBox(HWindow,'File does not exist','Error',mb_IconStop or mb_OK)
else
	begin
	new(Bitmap, Load(Temporary.Filename));
	new(Window, Init(@Self,'Viewing',Bitmap));
	Application^.MakeWindow(Window);
	end;
end;


function WritePrivateProfileInt(ApplicationName,KeyName:PChar; Int:integer;	FileName:PChar):Bool;
var	s:array[0..20] of char;
begin
Str(int,s);
WritePrivateProfileString(ApplicationName,KeyName,s,filename);
end;

procedure WriteToIniFile(filename,Sec:PChar; ToWrite:ImageInfo);
begin
WritePrivateProfileInt(Sec,'MinDepth',ToWrite.MinDepth,filename);
WritePrivateProfileInt(Sec,'MaxHeight',ToWrite.MaxHeight,filename);
WritePrivateProfileInt(Sec,'Period',ToWrite.Period,filename);
WritePrivateProfileInt(Sec,'Brightness',ToWrite.Brightness,filename);
WritePrivateProfileInt(Sec,'PatternOption',ToWrite.PatternOption,filename);
WritePrivateProfileString(Sec,'Filename',ToWrite.Filename,filename);
end;

procedure ReadFromIniFile(filename,Sec:PChar; var ToRead:ImageInfo);
begin
ToRead.MinDepth:=	 GetPrivateProfileInt(Sec,'MinDepth',0,filename);
ToRead.MaxHeight:= GetPrivateProfileInt(Sec,'MaxHeight',16,filename);
ToRead.Period:=    GetPrivateProfileInt(Sec,'Period',60,filename);
ToRead.Brightness:=GetPrivateProfileInt(Sec,'Brightness',50,filename);
ToRead.PatternOption:=GetPrivateProfileInt(Sec,'PatternOption',1,filename);
GetPrivateProfileString(Sec,'Filename','',ToRead.Filename,256,filename);
end;

begin
ReadFromIniFile('rds.ini','Defaults', Default);
ReadFromIniFile('rds.ini','CurrentSettings',CurrentImageSettings);
end.