unit DIBitmap;

interface

uses Bitmap,WinTypes;

type	TColourArray=array[0..255] of TRGBQuad;
			PColourArray=^TColourArray;

type	PDIBitmap=^TDIBitmap;
			TDIBitmap=object(TFileableBitmap)
				LogPalette:PLogPalette;
				constructor Load(Filename:PChar);
				constructor Copy(SourceBitmap:HBitmap;NewColourDepth:integer);
				constructor Create(NewWidth,NewHeight,NewColourDepth:integer);
				destructor Done; virtual;
				procedure Save(Filename:PChar); virtual;
				procedure Lock;
				procedure Unlock;
				function Locked:Boolean;
				function GetPixelRGB(x,y:integer):longint;
				procedure RedrawHBitmap(IntoDC:HDC);
				function ColourDepth:integer;
			private
				PaletteEntrys:Longint;
				UsingApproxPalette:boolean;
				BitsSize:longint;
				LockCount:integer;
				LockedBits:PChar;
				LineWidth:longint;
				BitmapFileHeader:TBitmapFileHeader;
				BitmapInfo:PBitmapInfo;
				BitmapInfoHeader:PBitmapInfoHeader;
				BitmapInfoColours:PColourArray;
				BitmapBits:THandle;
				function BitmapInfoSize:longint;
				function LogPaletteSize:longint;
				function CreateLogPalette:PLogPalette;
				function CreateApproxLogPalette:PLogPalette;
				procedure SetVariables;
				procedure SetBitmapInfoHeader(XSize,YSize,Bitcount:longint; var Into:TBitmapInfoHeader);
				procedure CopyBitmap(SourceBitmap:HBitmap; BitsPixel:integer);
				procedure LoadBitmap(Filename:PChar);
				procedure GeneralInit;
				end;

implementation

uses HugeFile,WinProcs;

const	BMType = $4D42;  { = 'BM' }

constructor TDIBitmap.Create(NewWidth,NewHeight,NewColourDepth:integer);
begin
TFileableBitmap.Create(NewWidth,NewHeight);
CopyBitmap(TheBitmap,NewColourDepth);
GeneralInit;
end;

constructor TDIBitmap.Copy(SourceBitmap:HBitmap; NewColourDepth:integer);
begin
TFileableBitmap.Copy(SourceBitmap);
CopyBitmap(SourceBitmap,NewColourDepth);
GeneralInit;
end;

constructor TDIBitmap.Load(Filename:PChar);
var ScreenDC:HDC;
begin
LoadBitmap(Filename);
ScreenDC:=GetDC(0);
RedrawHBitmap(ScreenDC);
ReleaseDC(0,ScreenDC);
TFileableBitmap.Copy(TheBitmap);
GeneralInit;
end;

procedure TDIBitmap.GeneralInit;
begin
if BitmapInfoHeader^.biBitCount=24 then
	LogPalette:=CreateApproxLogPalette
else
	LogPalette:=CreateLogPalette;
LockCount:=0;
end;

destructor TDIBitmap.Done;
begin
FreeMem(BitmapInfo,BitmapInfoSize);
FreeMem(LogPalette,LogPaletteSize);
GlobalFree(BitmapBits);
TFileableBitmap.Done;
end;

procedure TDIBitmap.Lock;
begin
LockedBits:=GlobalLock(BitmapBits);
LockCount:=LockCount+1;
end;

procedure TDIBitmap.Unlock;
begin
GlobalUnlock(BitmapBits);
if LockCount>0 then LockCount:=LockCount-1;
end;

function TDIBitmap.Locked:Boolean;
begin
Locked:=(LockCount=0);
end;

procedure TDIBitmap.RedrawHBitmap(IntoDC:HDC);
begin
DeleteObject(TheBitmap);
Lock;
TheBitmap:=CreateDIBitmap(IntoDC,BitmapInfoHeader^,cbm_Init,LockedBits,BitmapInfo^,dib_RGB_Colors);
Unlock;
end;

function TDIBitmap.GetPixelRGB(x,y:integer):longint;
	function ReadPixel:Longint;
	var	PalEntry:byte;
			off:longint;
			Colour:TRGBQuad;
			Ad:PByte;
	begin
	with BitmapInfoHeader^ do
		case biBitCount of
		1:  begin
				Off:=(Height-1-y)*LineWidth+(x div 8);
				Ad:=HugePtr(LockedBits,Off);
				PalEntry:=(Ad^ and (1 shl (x mod 8))) shr (x mod 8);
				{$R-}Colour:=BitmapInfoColours^[PalEntry];{$R+}
				ReadPixel:=longint(Colour);
				end;
		4:  begin
				Off:=(Height-1-y)*LineWidth+(x div 2);
				Ad:=HugePtr(LockedBits,Off);
				PalEntry:=(Ad^ and (15 shl ((x mod 2) shl 2))) shr ((x mod 2) shl 2);
				{$R-}Colour:=BitmapInfoColours^[PalEntry];{$R+}
				ReadPixel:=longint(Colour);
				end;
		8:  begin
				Off:=(Height-1-y)*LineWidth+x;
				Ad:=HugePtr(LockedBits,Off);
				PalEntry:=Ad^;
				{$R-}Colour:=BitmapInfoColours^[PalEntry];{$R+}
				ReadPixel:=longint(Colour);
				end;
		24: begin
				Off:=(Height-1-y)*LineWidth+(x*3);
				Ad:=HugePtr(LockedBits,Off);
				ReadPixel:=PLongint(Ad)^ and $00FFFFFF;
				end;
		end;
	end;
begin
if (y>=Height) or (x>=Width) then GetPixelRGB:=0 else
	begin
	if not Locked then GetPixelRGB:=ReadPixel else
		begin
		Lock; GetPixelRGB:=ReadPixel; Unlock;
		end;
	end;
end;

procedure TDIBitmap.SetVariables;
begin
BitmapInfoHeader:=PBitmapInfoHeader(BitmapInfo);
BitmapInfoColours:=@BitmapInfo^.bmiColors;
with BitmapInfoHeader^ do
	LineWidth:=(biWidth*biBitCount+31) div 32 * 4; {aligned to a longint boundary}
BitsSize:=BitmapInfoHeader^.biSizeImage;
end;

function TDIBitmap.ColourDepth:integer;
begin
ColourDepth:=BitmapInfoHeader^.biBitCount;
end;

function TDIBitmap.BitmapInfoSize;
begin
BitmapInfoSize:=SizeOf(TBitmapInfoHeader)+PaletteEntrys*SizeOf(TRGBQuad);
end;

function TDIBitmap.LogPaletteSize;
begin
if UsingApproxPalette then
	LogPaletteSize:=SizeOf(TLogPalette)+256*SizeOf(TPaletteEntry)
else
	LogPaletteSize:=SizeOf(TLogPalette)+PaletteEntrys*SizeOf(TPaletteEntry);
end;

function TDIBitmap.CreateLogPalette:PLogPalette;
var	NewPalette:PLogPalette; {TLogPalette}
		i:integer;
begin
UsingApproxPalette:=false;
GetMem(NewPalette,LogPaletteSize);
if PaletteEntrys>0 then
	for i:=0 to PaletteEntrys-1 do {$R-}
		with NewPalette^.palPalEntry[i],BitmapInfoColours^[i] do
			begin
			peRed:=rgbRed;
			peGreen:=rgbGreen;
			peBlue:=rgbBlue;
			peFlags:=0;
			end; {$R+}
NewPalette^.palVersion:=$300; {for Windows 3.0+}
NewPalette^.palNumEntries:=PaletteEntrys;
CreateLogPalette:=NewPalette;
end;

function TDIBitmap.CreateApproxLogPalette:PLogPalette;
var	NewPalette:PLogPalette;
		i:integer;
begin
UsingApproxPalette:=true;
GetMem(NewPalette,LogPaletteSize);
NewPalette^.palVersion:=$300; {for Windows 3.0+}
NewPalette^.palNumEntries:=256;
for i:=0 to 255 do
	{$R-}with NewPalette^.palPalEntry[i] do {$R+}
		begin
		peRed:=i; peGreen:=i; peBlue:=i; peFlags:=0;
		end;
CreateApproxLogPalette:=NewPalette;
end;


procedure TDIBitmap.LoadBitmap(Filename:PChar);
var TheFile:Integer;
		BitmapInfoStart:Integer;
		TempBitmapInfoHeader:TBitmapInfoHeader;
		Bits:PChar;
begin
TheFile:=_LOpen(FileName,of_Read);
{Read in TBitmapFileHeader}
_LRead(TheFile,@BitmapFileHeader,SizeOf(TBitmapFileHeader));
{Read temporarily TBitmapInfoHeader, tofind PaletteEntrys and correct BitmapInfoSize}
	BitmapInfoStart:=_llSeek(TheFile,0,1);
	_LRead(TheFile,@TempBitmapInfoHeader,SizeOf(TBitmapInfoHeader));
	PaletteEntrys:=(1 shl TempBitmapInfoHeader.biBitCount);
	GetMem(BitmapInfo,BitmapInfoSize);
{Read in TBitmapInfo, and set TBitmapInfoHeader and BitmapInfoColours}
_LlSeek(TheFile,BitmapInfoStart,0);
_LRead(TheFile,PChar(BitmapInfo),BitmapInfoSize);
SetVariables;
{Now need to load bitmap bits}
BitsSize:=_LFileSize(TheFile)-SizeOf(TBitmapFileHeader)-BitmapInfoSize;
BitmapBits:=GlobalAlloc(gmem_Moveable,BitsSize);
Bits:=GlobalLock(BitmapBits);
HugeIO(_LRead,TheFile,Bits,BitsSize);
{Clean up}
GlobalUnlock(BitmapBits);
_LClose(TheFile);
end;

procedure TDIBitmap.Save(Filename:PChar);
var	TheFile:Integer;
		Bits:PChar;
begin
TheFile:=_lcreat(FileName,0);
_lwrite(TheFile,@BitmapFileHeader,SizeOf(TBitmapFileHeader));
_lwrite(TheFile,PChar(BitmapInfo),BitmapInfoSize);
	Bits:=GlobalLock(BitmapBits);
HugeIO(_lwrite,TheFile,Bits,BitsSize);
	GlobalUnlock(BitmapBits);
_lclose(TheFile);
end;

procedure TDIBitmap.CopyBitmap(SourceBitmap:HBitmap; BitsPixel:integer);
var	BitmapData:TBitmap;
		Palette:array[0..255] of TPaletteEntry;
		DC:HDC;
		i:integer;
		Bits:PChar;
		black,white:longint;
begin
{A TBitmap record containing information about height etc}
GetObject(TheBitmap,SizeOf(TBitmap),@BitmapData);
{Prepare TBitmapInfo structure}
{allows us to force colour depth}
if BitsPixel<>0 then BitmapData.bmBitsPixel:=BitsPixel;
with BitmapData do
	begin
	PaletteEntrys:=(1 shl bmBitsPixel);
	GetMem(BitmapInfo,BitmapInfoSize);
	SetBitmapInfoHeader(bmWidth,bmHeight,bmBitsPixel,BitmapInfo^.bmiHeader);
	SetVariables;
	end;
BitsSize:=BitmapInfoHeader^.biSizeImage;
{Create TBitmapFileHeader}
with BitmapFileHeader do
	begin
	bfType:=BMType;
	bfOffBits:=SizeOf(TBitmapFileHeader)+BitmapInfoSize;
	bfReserved1:=0;
	bfReserved2:=0;
	bfSize:=bfOffBits+BitsSize;
	end;
{Create the BitmapInfoColours palette structure}
DC:=GetDC(0);
GetSystemPaletteEntries(DC,0,PaletteEntrys,Palette);
{if all we know is that the bitmap has two colours, a sensible choice would seem to be
to use the window background and foreground colours, rather than picking two aribitrary
elements from the system palette.}
if PaletteEntrys=2 then
	begin
	black:=GetSysColor(Color_WindowText);
	white:=GetSysColor(Color_Window);
	with BitmapInfoColours^[0] do begin
		rgbRed:=GetRValue(black); rgbGreen:=GetGValue(black); rgbBlue:=GetBValue(black); rgbReserved:=0 end;
	with BitmapInfoColours^[1] do begin
		rgbRed:=GetRValue(white); rgbGreen:=GetGValue(white); rgbBlue:=GetBValue(white); rgbReserved:=0 end;
	end;
if PaletteEntrys>2 then for i:=0 to PaletteEntrys-1 do
	with BitmapInfoColours^[i],Palette[i] do
		begin rgbRed:=peRed; rgbGreen:=peGreen; rgbBlue:=peBlue; rgbReserved:=0; end;
{Create the bits}
BitmapBits:=GlobalAlloc(gmem_Moveable,BitsSize);
Bits:=GlobalLock(BitmapBits);
GetDIBits(DC,SourceBitmap,0,Height,Bits,BitmapInfo^,dib_RGB_Colors);
GlobalUnlock(BitmapBits);
ReleaseDC(0,DC);
end;

procedure TDIBitmap.SetBitmapInfoHeader(XSize,YSize,Bitcount:longint; var Into:TBitmapInfoHeader);
begin
with Into do
	begin
	biSize:=SizeOf(TBitmapInfoHeader);
	biWidth:=XSize;
	biHeight:=YSize;
	biPlanes:=1;
	biBitCount:=Bitcount;
	biCompression:=0;
	biSizeImage:=((XSize*Bitcount+31) div 32 *4)*YSize; {to align each line on a longint boundary}
	biXPelsPerMeter:=0;
	biYPelsPerMeter:=0;
	biClrUsed:=0;
	biClrImportant:=0;
	end;
end;

end.