// MyOle ver. 0.91      
// Author: Matej Trampus, Slovenija
// E-MAIL:
//   - Matej.Trampus@kiss.uni-lj.si
// See readme.txt for details.
//
// BUGS:
// - We claim, that we support CfDelphiObjectData (see TMyDataObject.EnumFormatEtc), but
//   we do not.
// - What happens if I copy the object to clipboard and then exit the program. OLE will
//   still hold on IDataObject we provided, but the object will not exist anymore (see also
//   comment in ClassFactory.pas)
//
// History:
// -- ver 0.91
// 26.10.1996  Added clipoard fromats: Object descriptor, Embedded object
// -- ver 0.90
// - 23.10.1996 TMyDataObject.GetData, TMyDataObject.supportsFormat commented out

unit DataObject;

interface

uses windows,ole2,classes;

// coments starting with '//**' are from Inside Ole 2 (IOL2)
// IOL2 sets refCount to 0 in object's constructor, so object that called the other
// object's constructor must explicitly call otherObject.AddRef to hold onto otherObject
// I am using different approach (as in Delphi's oleCtnrs.dcu). I set the RefCount to 1
// in constructor and thus eliminating the need for explicit AddRef after calling the cunstructor.
// Again: Do not call otherObject.Addref after you created other object with otherObject:=TIxxxx.Create


type
  PFormatList = ^TFormatList;
  TFormatList = array[0..255] of TFormatEtc; // WARNING: ONLY 256 FORMATS!!!

type

// There is no support for aggregation for TMyEnumFormatEtc. Enumerators are seperate objects.
// (see TMyEnumFormatEtc.Query.Interface) fUnkRef is used only to control life time of outer object
  TMyEnumFormatEtc = class(IEnumFormatEtc)
  private
    FRefCount: Integer;       // object reference counting
    fUnkRef:IUnknown;         // IUnknown for ref. counting
    FIndex:integer;           // current element
    FFormatList: PFormatList;
    FFormatCount: Integer;
  public
    constructor Create(UnkRef:IUnknown;FormatCount:Integer;FormatList: PFormatList;index:integer);
    destructor Destroy; override;
    function QueryInterface(const iid: TIID; var obj): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;
    function Next(celt: Longint; var elt;pceltFetched: PLongint): HResult; override;
    function Skip(celt: Longint): HResult; override;
    function Reset: HResult; override;
    function Clone(var enum: IEnumFormatEtc): HResult; override;
  end;


// TMyData can be only used as part of another object. If you would like to change this,
// you have to change following functions: QueyInterface, AddRef and Relase

  TMyDataObject = class(IDataObject)
  private
    FRefCount: Integer;
    fUnkOuter:IOleObject;             // pointer to outer object
    fDataAdviseHolder:IDataAdviseHolder; // NOT TESTED
    function SupportsFormat(const formatEtcIn:TFormatEtc):HResult;
  public
    constructor Create(UnkOuter: IOleObject);
    destructor Destroy; override;
    function QueryInterface(const iid: TIID; var obj): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;
    function GetData(var formatetcIn: TFormatEtc;var medium: TStgMedium): HResult; override;
    function GetDataHere(var formatetc: TFormatEtc;var medium: TStgMedium): HResult; override;
    function QueryGetData(var formatetc: TFormatEtc): HResult; override;
    function GetCanonicalFormatEtc(var formatetc: TFormatEtc;var formatetcOut: TFormatEtc): HResult; override;
    function SetData(var formatetc: TFormatEtc; var medium: TStgMedium;fRelease: BOOL): HResult; override;
    function EnumFormatEtc(dwDirection: Longint; var enumFormatEtc:IEnumFormatEtc): HResult; override;
    // Data advisory dunctions were never tested  !!! - because they were never called from RIchEdit ;-)
    function DAdvise(var formatetc: TFormatEtc; advf: Longint;advSink: IAdviseSink; var dwConnection: Longint): HResult; override;
    function DUnadvise(dwConnection: Longint): HResult; override;
    function EnumDAdvise(var enumAdvise: IEnumStatData): HResult; override;
  end;


  // Delphi stream that uses IStream. It can be used for saving data to compond documents file
  TOleStream=class(TStream)
   private
    fIStream:IStream;
   public
    constructor Create(oleStream:IStream);
    destructor  Destroy; override;
    function Read(var Buffer; Count: Longint): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    procedure AssignTo(oleStream:IStream);
  end;



function iid2String(const iid:TGUid):string;

var CfDelphiObjectData:integer;
    cfEmbeddedObject:integer;
    cfObjectDescriptor:integer;
    cfEmbedSource:integer;

const cfDelphiObjectDataDescr='Matra Delphi Object2';
      cfEmbeddedObjectDescr='Embedded Object';
      cfObjectDescriptorDescr='Object Descriptor';
      cfEmbedSourceDescr='Embed Source';

implementation

uses sysutils,debugMemo1,dialogs;

const errSeekError='Stream seek error';
var   DataFormats:array[0..2] of TFormatEtc; // zaenkrat je to globalno


function GetFormatName(cfFormat:TClipFormat):string;
var formatName:array[0..200] of char;
begin
  GetClipBoardFormatName(cfFormat,@formatName,200);
  result:=formatName;
end;


function iid2String(const iid:TGUid):string;
var s:array[0..100] of WideChar;
begin
  stringFromGuid2(iid,@s,90);
  result:=wideCharToString(s);
end;

////////////////////////////////////////////////////////////////////////////////
/////   TMyEnumFormatEtc
////////////////////////////////////////////////////////////////////////////////
constructor TMyEnumFormatEtc.Create(UnkRef:IUnknown;FormatCount:Integer;FormatList: PFormatList;index:integer);
begin
  debug2(self,'Create');
  fRefCount:=1; // Object who created us wit this Delphi Constructor *must not*
                // explicitly call AddRef. IOL2 uses different technique
  fUnkRef:=UnkRef; fIndex:=index;
  fUnkRef.AddRef; // (as in AddRef) keep the data objects, that defines the contex of the enumerator alive

  fFormatCount:=formatCount;
  // TRY.. FINALLY TO TRAP out of memory? (No need if called from Delphi)
  GetMem(fFormatList,fFormatCount*sizeof(TFormatEtc));
  move(FormatList^,fFormatList^,fFormatCount*sizeof(TFormatEtc)); //copy format array
  // ok,ok, it is not nessessary to copy the array, but I did that, because that is the
  // way it is done in IOL2

end;

destructor TMyEnumFormatEtc.destroy;
begin
 debug2(self,'Destroy');
 freeMem(fFormatList);
end;

function TMyEnumFormatEtc.QueryInterface(const iid: TIID; var obj): HResult;
begin
  pointer(obj):=nil;
  //** Enumerators are seperate objects, not the data objects, so we only need
  //** to support our IUnknown and IEnumFORMATETC interfaces here, with no
  //** concern for agregation
  if IsEqualIID(iid, IID_IUnknown) or IsEqualIID(iid, IID_IEnumFormatEtc) then begin
    debug2(self,'QueryInterface + '+iid2String(iid));
    Pointer(obj) := Self;  AddRef;
    Result := S_OK;
  end else begin
    debug2(self,'QueryInterface + '+iid2String(iid)+' NOT SUPPORTED!');
    Result := E_NOINTERFACE;
  end;
end;

function TMyEnumFormatEtc.AddRef:Longint;
begin
  debug2(self,'AddRef');
  inc(FRefCount);
  fUnkRef.AddRef; // keep the data objects, that defines the contex of the enumerator alive
  result:=FRefCount;
end;

function TMyEnumFormatEtc.Release:Longint;
begin
  debug2(self,'Release ');
  dec(fRefCount);
  fUnkRef.release;
  result:=fRefCount;
  if fRefCount=0 then free;
end;

function TMyEnumFormatEtc.Next(celt: Longint; var elt;pceltFetched: PLongint): HResult;
var
  I: Integer;
begin
  debug2(self,'Next '+intToStr(celt));
  if fformatList=nil then begin result:=S_FALSE; exit; end; // no formats to enumerate
  if (pceltFetched=nil) and (celt<>1) then begin result:=E_POINTER; exit; end;
  if pCeltfetched<>nil then pCeltFetched^:=0;
  if (pointer(elt)=nil) or (FIndex>=FFormatCount) then begin result:=S_FALSE; exit; end;

  I := 0; // from olectnrs.TEnumFormatEtc.Next
  while (I < celt) and (FIndex < FFormatCount) do
  begin
    TFormatList(elt)[I] := FFormatList[FIndex];
    Inc(FIndex);
    Inc(I);
  end;
  if pceltFetched <> nil then pceltFetched^ := I;
  if I = celt then Result := S_OK else Result := S_FALSE;
  if result=S_OK then debug2(self,'Next returning format '+GetFormatName(TFormatList(elt)[0].cfFormat));
  // IOL2 returns NOERROR. win32sdk states that S_OK should be returned
  // if the number of elements returned is celt; S_FALSE otherwise.
end;

function TMyEnumFormatEtc.Skip(celt: Longint): HResult;
begin
  debug2(self,'Skip '+intToStr(celt));
  if fFormatList=nil then begin result:=S_FALSE; exit; end;
  if celt <= FFormatCount - FIndex then  begin // from olectnrs.TEnumFormatEtc.Skip
    FIndex := FIndex + celt;
    Result := S_OK;
  end else
  begin
    FIndex := FFormatCount;
    Result := S_FALSE;
  end;
  // IOL2 returns NOERROR. win32sdk states that S_OK should be returned
  // if the number of elements skipped is celt; otherwise S_FALSE.
end;

function TMyEnumFormatEtc.Reset: HResult;
begin
  debug2(self,'Reset');
  FIndex := 0; Result := S_OK;
end;

function TMyEnumFormatEtc.Clone(var enum: IEnumFormatEtc): HResult;
begin
  debug2(self,'Clone');
  try
    enum := TMyEnumFormatEtc.Create(fUnkRef,FFormatCount,FFormatList,fIndex);
    // AddRef is called in .Create
  except // out of memory
    result:=E_OUTOFMEMORY; exit;
  end;
  Result := S_OK;
end;

////////////////////////////////////////////////////////////////////////////////
/////   TMyDataObject
////////////////////////////////////////////////////////////////////////////////

function GetObjectDescriptor(OleObject:IOleObject): HGlobal;
var
  DescSize: Integer;
  Descriptor: PObjectDescriptor;
begin // BTW: there is a bug in olectnrs.TDataObject.GetObjectDescriptor -. FillChar has wrong order parameters order
  debug('getting object descritor');
  DescSize := SizeOf(TObjectDescriptor);
  Result := GlobalAlloc(GMEM_MOVEABLE, DescSize);
  if Result <> 0 then
  begin
    Descriptor := GlobalLock(Result);
    FillChar(Descriptor^, DescSize,0);
    with Descriptor^ do
    begin
      cbSize := DescSize;
      OleObject.GetUserClassID(clsid);
      dwDrawAspect := DVASPECT_CONTENT;
      OleObject.GetExtent(dwDrawAspect,size);
      // pointl=(0,0)
      OleObject.GetMiscStatus(DVASPECT_CONTENT, dwStatus);
      // fullUserTypeName=nil;
      // dwSrcOfCopy=nil;
    end;
    GlobalUnlock(Result);
  end;
end;

constructor TMyDataObject.Create(UnkOuter: IOleObject);
begin
 debug2(self,'Create');
 fRefCount:=1; // .. the outer object should not explicitly call addref after TMyDataObject.Create
 fUnkOuter:=UnkOuter;
end;

destructor TMyDataObject.Destroy;
begin
  if fDataAdviseHolder<>nil then fDataAdviseHolder.release;
  debug2(self,'Destroy');
end;

function TMyDataObject.QueryInterface(const iid: TIID; var obj): HResult;
begin
  debug2(self,'QueryInterface -delegating to outer');
  result:=fUnkOuter.QueryInterface(iid,obj); // delegates call to outer object
end;

function TMyDataObject.AddRef: Longint;
begin
  debug2(self,'AddRef - delegating to outer');
  Inc(FRefCount);
  result:=fUnkOUter.AddRef; // delegates call to outer object
end;

function TMyDataObject.Release: Longint;
begin
  debug2(self,'Release- delegating to outer');
  Dec(FRefCount);
  Result := fUnkOuter.Release; // delegates call to outer object
end;

function TMyDataObject.SupportsFormat(const formatEtcIn:TFormatEtc):HResult;
  function CheckMediumAndAspect(medium:dword;Aspect:dword;var res:hresult):boolean;
  begin
    res:=S_OK;
    if (formatEtcIn.tymed and medium)=0 then begin
      debug2(self,'supports format: unsupported medium'); res:=DV_E_TYMED;
    end else if aspect<>formatEtcIn.dwAspect then begin
        debug2(self,'supports format: unsupported aspect'); res:=DV_E_DVASPECT;
    end;
    result:=res=S_OK;
  end;

begin
  result:=E_FAIL;

  debug2(self,Format('supportsFormat: cfFormat=%s(%d), tyMed=%d, dwAspect=%d',[GetformatName(formatEtcIn.cfFormat),formatEtcIn.cfFormat,formatEtcIn.tyMed,formatEtcIn.dwAspect]));
  with formatEtcIn do begin
   if (cfFormat=cfEmbeddedObject) and
      checkMediumAndAspect(TYMED_ISTORAGE,DVASPECT_CONTENT,result) then exit;
   if (cfFormat=cfObjectDescriptor) and
      checkMediumAndAspect(TYMED_HGLOBAL,DVASPECT_CONTENT,result)  then exit;
  end;
  // MISSING support for cfDelphiObject
end;

function TMyDataObject.GetData(var formatetcIn: TFormatEtc;
  var medium: TStgMedium): HResult;
begin
  debug2(self,'Get Data');
  result:=E_UNEXPECTED;
  medium.tymed := 0;  medium.hGlobal := 0;  medium.unkForRelease := nil; // init [out] parameter

  result:=supportsFormat(formatEtcIn);   // check formats we support

  if result<>S_OK then exit;

  if formatEtcIn.cfFormat=cfObjectDescriptor then begin
    medium.tymed := TYMED_HGLOBAL;
    medium.hglobal:=GetObjectDescriptor(fUnkOuter);
  end;

  if formatEtcIn.cfFormat=cfEmbeddEdObject then
     result:=E_UNEXPECTED; // embedded object supported only in GetDataHere
end;

function TMyDataObject.GetDataHere(var formatetc: TFormatEtc;  var medium: TStgMedium): HResult;
var PersistStorage: IPersistStorage;
begin // from olectnrs.pas
  debug2(self,'GetDataHere '+GetFormatName(formatEtc.cfFormat));
  result:=supportsFormat(formatEtc); if result<>S_OK then exit;
  with formatetc do
    if (cfFormat = CFEmbeddedObject) then begin
      medium.unkForRelease := nil;
      fUnkOuter.QueryInterface(IID_IPersistStorage, PersistStorage);
      if PersistStorage <> nil then
      begin
        Result := OleSave(PersistStorage, medium.stg, False);
        PersistStorage.SaveCompleted(nil);
        PersistStorage.Release;
      end;
    end;
end;

function TMYDataObject.QueryGetData(var formatetc: TFormatEtc): HResult;
begin
  debug2(self,'QueryGetData');
  Result := SupportsFormat(FormatEtc);
end;

function TMyDataObject.GetCanonicalFormatEtc(var formatetc: TFormatEtc;
  var formatetcOut: TFormatEtc): HResult;
begin
  debug2(self,'GetCanonicalFormatEtc - DATA_S_SAMEFORMATETC');
  formatEtcOut:=formatEtc; // we do not provide device-specific renderings
  formatetcOut.ptd := nil;
  Result := DATA_S_SAMEFORMATETC;
end;

function TMyDataObject.SetData(var formatetc: TFormatEtc; var medium: TStgMedium;fRelease: BOOL): HResult;
begin
  debug2(self,'SetData - E_NOTIMPL');
  Result := E_NOTIMPL; // not implemented
end;

function TMyDataObject.EnumFormatEtc(dwDirection: Longint; var enumFormatEtc:IEnumFormatEtc): HResult;
begin
  debug2(self,'EnumFormatEtc');
  if dwDirection = DATADIR_GET then
  begin
    enumFormatEtc := TMyEnumFormatEtc.Create(self,High(DataFormats)-low(DataFormats)+1,@DataFormats, 0);
    Result := S_OK;
  end else begin
    enumFormatEtc := nil; Result := E_NOTIMPL;
  end;
end;

// WARNING: This function was never tested!!!
function TMyDataObject.DAdvise(var formatetc: TFormatEtc; advf: Longint;advSink: IAdviseSink; var dwConnection: Longint): HResult;
begin
  if fDataAdviseHolder=nil then begin
    result:=CreateDataAdviseHolder(fDataAdviseHolder);
    debug2(self,'DAdvise - Creating default data advise holder');
    if result<>S_OK then exit;
  end else debug2(self,'DAdvise');

  result:=fDataAdviseHolder.Advise(self,formatEtc,advf,advSink,dwConnection);
  debug2(self,'DAdvise - Delegated handler returned '+intToStr(result));
end;

// WARNING: This function was never tested!!!
function TMyDataObject.DUnadvise(dwConnection: Longint): HResult;
begin
  debug2(self,'DUnAdvise');
  if fDataAdviseHolder<>nil then
    result:=fDataAdviseHolder.Unadvise(dwConnection)
  else result:=E_FAIL;
end;

// WARNING: This function was never tested!!!
function TMyDataObject.EnumDAdvise(var enumAdvise: IEnumStatData): HResult;
begin
  debug2(self,'EnumDAdvise');
  if fDataAdviseHolder<>nil then
    result:=fDataAdviseHolder.EnumAdvise(enumAdvise)
  else result:=E_FAIL;
end;


////////////////////////////////////////////////////////////////////////////////
/////   TOleStream
////////////////////////////////////////////////////////////////////////////////

constructor TOleStream.Create(oleStream:IStream);
begin
  inherited Create; AssignTo(oleSTream);
end;

procedure TOleStream.AssignTo(oleStream:IStream);
begin
   if fIstream<>nil then fIstream.Release; // release old stream, if any
   fIStream:=oleStream; if fIstream<>nil then fIStream.AddRef;
end;

destructor  TOleStream.Destroy;
begin
  if fIstream<>nil then fIstream.Release;
  inherited destroy;
end;

function TOleStream.Write(const Buffer; Count: Longint): Longint;
begin
  fIStream.Write(@Buffer,Count,@Result);
end;

function TOleStream.Read(var Buffer; Count: Longint): Longint;

begin
 fIstream.read(@Buffer,Count,@result);
end;

function TOleStream.Seek(Offset: Longint; Origin: Word): Longint;
var temp:LargeInt;
begin
 // actually, this case statement is not needed, because values are identical (soFromBeginning=STREAM_SEEK_SET,etc.)
 case origin of
  soFromBeginning: origin:=STREAM_SEEK_SET;
  soFromCurrent:   origin:=STREAM_SEEK_CUR;
  soFromEnd:       origin:=STREAM_SEEK_END;
 end;
 if fIStream.Seek(Offset,origin,temp)<>S_OK then raise eStreamError.create(errSeekError);
 result:=Trunc(temp);
end;

initialization

  CfDelphiObjectData:=RegisterClipBoardFormat(cfDelphiObjectDataDescr);
  CfEmbeddedObject:=RegisterClipBoardFormat(cfEmbeddedObjectDescr);
  cfObjectDescriptor:=RegisterClipBoardFormat(cfObjectDescriptorDescr);
  cfEmbedSource:=RegisterClipBoardFormat(cfEmbedSourceDescr);

  DataFormats[0].cfFormat := CfDelphiObjectData;
  DataFormats[0].dwAspect := DVASPECT_CONTENT;
  DataFormats[0].lIndex := -1;
  DataFormats[0].tymed := TYMED_HGLOBAL;

  DataFormats[1].cfFormat := CfEmbeddedObject;
  DataFormats[1].dwAspect := DVASPECT_CONTENT;
  DataFormats[1].lIndex := -1;
  DataFormats[1].tymed := TYMED_ISTORAGE;

  DataFormats[2].cfFormat := CfObjectDescriptor;
  DataFormats[2].dwAspect := DVASPECT_CONTENT;
  DataFormats[2].lIndex := -1;
  DataFormats[2].tymed := TYMED_HGLOBAL;

end.
