(*
 *  TExplorerButton component version 2.3
 *  (c)1996 Fabrice Deville
 *  fdev@tornado.be
 *  http://www.tornado.be/~fdev/
 *
 *  Accelerator support thanks to Fred Schetterer
 *
 *  Freeware - compiles under Delphi 1.0 & 2.0
 *
 *  Source code provided under some conditions:
 *
 *  - if you use significant parts of the code, please credit me
 *  - do not release a new version of this component without
 *    significant enhancements
 *
 *  Note: this file is best viewed with tabstops set at 4 8 12 16 20 24 28
 *)

unit explbtn;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, dsgnIntf, ExtCtrls;

type
  TAboutExpButtonProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

 TExplorerButtonLayout = (blBitmapTop, blBitmapBottom, blBitmapLeft, blBitmapRight);
 TExplorerButtonOption = (boMonoDisplay, boIconOnly, boPopupMark);
 TExplorerButtonOptions = set of TExplorerButtonOption;
 TExplorerButtonShadingType = (stLight, stMedium, stDark);

  TExplorerButton = class(TGraphicControl)
  private
  	(* properties storage *)
   FAbout: TAboutExpButtonProperty;
   FAlignment: TAlignment;
   FAllowAllUp: Boolean;
   FBevelStyle: TBevelStyle;
   FBitmap, IBitmap, PMBitmap, FNoFocusBitmap,
   FDisabledBitmap: TBitmap; (* bitmap and shaded bitmap *)
   FCaption: TCaption;
   FDown: Boolean;
   FDropDown: TPopupMenu;
   FEnabled: Boolean;
   FGroupIndex: Integer;
   FLayout: TExplorerButtonLayout;
   FOptions: TExplorerButtonOptions;
   FShadingType: TExplorerButtonShadingType;
   FUnselectedFontColor: TColor;
   (* state flags *)
   Pushed, MouseIn: Boolean;
  protected
   procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
	procedure CreateGrayscaleBitmap(outputbmp, bmp: TBitmap);
	procedure DrawPopupMark(x, y: Integer);
   procedure DrawOutline(pushed: Boolean);
	procedure GetLost;
   procedure Loaded; override;
	procedure Notification(AComponent: TComponent; Operation: TOperation); override ;
   procedure Paint; override;
   procedure SetAlignment(value: TAlignment);
   procedure SetAllowAllUp(value: Boolean);
   procedure SetBevelStyle(value: TBevelStyle);
   procedure SetBitmap(value: TBitmap);
   procedure SetCaption(value: TCaption);
   procedure SetDisabledBitmap(value: TBitmap);
   procedure SetDown(value: Boolean);
   procedure SetDropDown(value: TPopupMenu);
   procedure SetEnabled(value: Boolean);
   procedure SetGroupIndex(value: Integer);
   procedure SetLayout(value: TExplorerButtonLayout);
   procedure SetNoFocusBitmap(value: TBitmap);
   procedure SetOptions(value: TExplorerButtonOptions);
   procedure SetShadingType(value: TExplorerButtonShadingType);
   procedure SetUnselectedFontColor(value: TColor);
   procedure WMRButtonUp(var msg: TWMRButtonUp); message WM_RBUTTONUP;
   procedure WMLButtonDown(var msg: TWMLButtonDown); message WM_LBUTTONDOWN;
   procedure WMLButtonUp(var msg: TWMLButtonUp); message WM_LBUTTONUP;
   procedure WMMouseMove(var msg: TWMMouseMove); message WM_MOUSEMOVE;
  public
   constructor Create(aOwner: TComponent); override;
   destructor Destroy; override;
  published
   (* new properties *)
   property About: TAboutExpButtonProperty read FAbout write FAbout;
   property Alignment: TAlignment read FAlignment write SetAlignment stored True default taCenter;
   property AllowAllUp: Boolean read FAllowAllUp write SetAllowAllUp stored True default False;
   property BevelStyle: TBevelStyle read FBevelStyle write SetBevelStyle stored True default bsRaised;
   property Bitmap: TBitmap read FBitmap write SetBitmap stored True;
   property Caption: TCaption read FCaption write SetCaption stored True;
   property GroupIndex: Integer read FGroupIndex write SetGroupIndex stored True default 0;
   property DisabledBitmap: TBitmap read FDisabledBitmap write SetDisabledBitmap stored True;
   property Down: Boolean read FDown write SetDown stored True default False;
   property DropDownMenu: TPopupMenu read FDropDown write SetDropDown stored True;
   property Enabled: Boolean read FEnabled write SetEnabled default True;
   property Layout: TExplorerButtonLayout read FLayout write SetLayout stored True default blBitmapTop;
   property NoFocusBitmap: TBitmap read FNoFocusBitmap write SetNoFocusBitmap stored True;
   property Options: TExplorerButtonOptions read FOptions write SetOptions stored True;
   property ShadingType: TExplorerButtonShadingType read FShadingType write SetShadingType stored True default stMedium;
   property UnselectedFontColor: TColor read FUnselectedFontColor write SetUnselectedFontColor stored True
            default clWindowText;
   (* republished properties *)
   property Font;
   property OnClick;
   property ParentFont;
   property ParentShowHint;
   property ShowHint;
   property Visible;
  end;

procedure Register;

implementation

(*
 * About property
 *)

procedure TAboutExpButtonProperty.Edit;
begin
	Application.MessageBox('This component is freeware. (c)1996 Fabrice Deville',
   						'TExplorerButton component version 2.3', MB_OK+ MB_ICONINFORMATION);
end;

function TAboutExpButtonProperty.GetAttributes: TPropertyAttributes;
begin
 	Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutExpButtonProperty.GetValue: string;
begin
	Result := '(about)';
end;

(*
 * Graphic support routines - these are not part of the object
 *)
procedure DrawTheText(Canvas: TCanvas; TempRect: TRect; TempCaption: PChar);
begin
{$IFDEF WIN32}
	DrawTextEx(Canvas.handle, TempCaption, StrLen(TempCaption), TempRect, DT_CENTER or DT_LEFT or DT_END_ELLIPSIS, nil);
{$ELSE}
	DrawText(Canvas.handle, TempCaption, StrLen(TempCaption), TempRect, DT_CENTER or DT_LEFT);
{$ENDIF}

(* NT 3.51 users: delete the previous lines and
   uncomment the following one
	DrawText(Canvas.handle, TempCaption, StrLen(TempCaption), TempRect, DT_CENTER or DT_LEFT);
*)
end;

(*
 * Drawing of a disabled text (Win95 style)
 *)
procedure DrawDisabledText(Canvas: TCanvas; TempRect: TRect; TempCaption: PChar);
begin
	Canvas.Brush.Style := bsClear;
	Canvas.Font.Color := clBtnHighlight;
	with TempRect do
	begin
		left := left + 1;
   	top  := top + 1;
		right:= right + 1;
		bottom:= bottom + 1;
	end;
	DrawTheText(Canvas, TempRect, TempCaption);
   Canvas.Font.Color := clBtnShadow;
	with TempRect do
	begin
		left := left - 1;
   	top  := top - 1;
		right:= right - 1;
		bottom:= bottom - 1;
	end;
	DrawTheText(Canvas, TempRect, TempCaption);
end;

(*
 * Drawing of a disabled bitmap (Win95 style)
 *)

procedure DrawDisabledBitmap(Canvas: TCanvas; x, y: Integer; bmp: TBitmap);
var MonoBmp: TBitmap;
begin
	MonoBmp := TBitmap.Create;
	try
          MonoBmp.Assign(bmp);
          MonoBmp.Canvas.Brush.Color := clBlack;
          MonoBmp.Monochrome := True;
          Canvas.Brush.Color := clBtnHighlight;
          SetTextColor(Canvas.Handle, clBlack);
          SetBkColor(Canvas.Handle, clWhite);
          BitBlt(Canvas.Handle, x+1, y+1, bmp.Width, bmp.Height,
                 MonoBmp.Canvas.Handle, 0, 0, $00E20746);
          Canvas.Brush.Color := clBtnShadow;
          SetTextColor(Canvas.Handle, clBlack);
          SetBkColor(Canvas.Handle, clWhite);
          BitBlt(Canvas.Handle, x, y, bmp.Width, bmp.Height,
                 MonoBmp.Canvas.Handle, 0, 0, $00E20746);
	finally
          MonoBmp.Free;
	end
end;

(*
 * Drawing of a bitmap on a canvas, with transparency
 * (adapted from code found in the Microsoft Knowledge Base)
 *)

procedure DrawTransparentBitmap(canvas: TCanvas; bmp: TBitmap; xStart,
                           yStart: Integer; cTransparentColor: LongInt);
var
   bm: WinTypes.TBitmap;
   cColor: TColorRef;
   bmAndBack, bmAndObject, bmAndMem, bmSave: HBITMAP;
   bmBackOld, bmObjectOld, bmMemOld, bmSaveOld, hBmp: HBITMAP;
   hdcMem, hdcBack, hdcObject, hdcTemp, hdcSave, dc: HDC;
   ptSize: WinTypes.TPoint;
begin
   dc := canvas.Handle;
   hBmp := bmp.Handle;
   hdcTemp := CreateCompatibleDC(dc);
   SelectObject(hdcTemp, hBmp);

   GetObject(hBmp, SizeOf(bm), @bm);
   ptSize.x := bm.bmWidth;
   ptSize.y := bm.bmHeight;

   hdcBack   := CreateCompatibleDC(dc);
   hdcObject := CreateCompatibleDC(dc);
   hdcMem    := CreateCompatibleDC(dc);
   hdcSave   := CreateCompatibleDC(dc);

   bmAndBack   := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

   bmAndObject := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

   bmAndMem    := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);
   bmSave      := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);

   bmBackOld   := SelectObject(hdcBack, bmAndBack);
   bmObjectOld := SelectObject(hdcObject, bmAndObject);
   bmMemOld    := SelectObject(hdcMem, bmAndMem);
   bmSaveOld   := SelectObject(hdcSave, bmSave);

   SetMapMode(hdcTemp, GetMapMode(dc));

   BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

   cColor := SetBkColor(hdcTemp, cTransparentColor);

   BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0,
          SRCCOPY);

   SetBkColor(hdcTemp, cColor);

   BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0,
          NOTSRCCOPY);
   BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, dc, xStart, yStart,
          SRCCOPY);
   BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);
   BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);
   BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);
   BitBlt(dc, xStart, yStart, ptSize.x, ptSize.y, hdcMem, 0, 0,
          SRCCOPY);
   BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

   DeleteObject(SelectObject(hdcBack, bmBackOld));
   DeleteObject(SelectObject(hdcObject, bmObjectOld));
   DeleteObject(SelectObject(hdcMem, bmMemOld));
   DeleteObject(SelectObject(hdcSave, bmSaveOld));

   DeleteDC(hdcMem);
   DeleteDC(hdcBack);
   DeleteDC(hdcObject);
   DeleteDC(hdcSave);
   DeleteDC(hdcTemp);
end;

{$B-} (* Quick evaluation *)
(* Won't work properly in the case of 'text&' caption, but the one
   who defines such a caption has a really sick mind :)*)
function RemoveAmpersand(input: String): String;
var i: Integer;
begin
	Result := input;
   i := 1;
   while i < Length(Result) do
   begin
   	if (Result[i] = '&') then
       begin
       	if (Result[i+1] = '&') then
           	Delete(Result, i + 1, 1)
           else
           	Delete(Result, i, 1);
       end
       else
       	Inc(i);
   end;
end;

(*
 * TExplorerButton implementation
 *)

constructor TExplorerButton.Create(aOwner: TComponent);
begin
	inherited Create(aOwner);
	FBitmap := TBitmap.Create;
	FNoFocusBitmap := TBitmap.Create;
	IBitmap := TBitmap.Create;
	FDisabledBitmap := TBitmap.Create;
	ControlStyle := [csClickEvents, csCaptureMouse, csSetCaption];
	Pushed := False;
	MouseIn := False;
	Alignment := taCenter;
	Width := 50;
	Height := 40;
   if csDesigning in ComponentState then
		Caption := 'ExplorerButton';
	FDropDown := nil;
   FOptions := [boMonoDisplay, boPopupMark];
   FLayout := blBitmapTop;
   FAllowAllUp := False;
   FDown := False;
	FEnabled := True;
   FGroupIndex := 0;
   FBevelStyle := bsRaised;
   FShadingType := stMedium;
   FUnselectedFontColor := clWindowText;
end;

destructor TExplorerButton.Destroy;
begin
	inherited Destroy;
	FBitmap.Free;
	FNoFocusBitmap.Free;
	IBitmap.Free;
   FDisabledBitmap.Free;
end;

procedure TExplorerButton.CMDialogChar(var Message: TCMDialogChar);
var lpPoint : TPoint;
begin
	with Message do
   if IsAccel(CharCode, Caption)and Enabled and Visible then
	begin
		MouseIn := True;
		Pushed := False;
		Refresh;
		Application.ProcessMessages;
		WMLButtonDown( TWMLBUTTONDOWN(Message));
		Application.ProcessMessages;
		WMLButtonUp( TWMLBUTTONUP(Message));
		Application.ProcessMessages;
		GetCursorPos(lpPoint);
		lpPoint := GetParentForm(self).ScreenToClient(lpPoint);
		if not  ((lpPoint.y > top) and (lpPoint.y < top + height)
			and (lpPoint.x > left) and (lpPoint.x < left + width)) then
       begin
			MouseIn := False;
           Refresh;
       end;
		Result := 1;
	end;
end;

(*
 * These thresholds are used for the grayscaling and were experimentaly
 * determined
 *)
const THRESHOLD1_LIGHT = 205;
      THRESHOLD2_LIGHT = 127;
      THRESHOLD3_LIGHT = 68;
      THRESHOLD1_MEDIUM = 553;
      THRESHOLD2_MEDIUM = 231;
      THRESHOLD3_MEDIUM = 57;
      THRESHOLD1_DARK = 335;
      THRESHOLD2_DARK = 274;
      THRESHOLD3_DARK = 175;
(*
 * Creation of the grayscale bitmap (pretty dumb routine, isn't it?)
 *)
procedure TExplorerButton.CreateGrayscaleBitmap(outputbmp, bmp: TBitmap);
var x, y: Integer;
    col, TransparentColor: LongInt;
    r, g, b, sum, threshold1, threshold2, threshold3: Integer;
begin
	outputbmp.Assign(bmp);
	TransparentColor := ColorToRGB(bmp.Canvas.Pixels[0,0]);
   if FShadingType = stLight then
   begin
      	threshold1 := THRESHOLD1_LIGHT;
      	threshold2 := THRESHOLD2_LIGHT;
      	threshold3 := THRESHOLD3_LIGHT;
   end
   else
   if FShadingType = stMedium then
   begin
      	threshold1 := THRESHOLD1_MEDIUM;
      	threshold2 := THRESHOLD2_MEDIUM;
      	threshold3 := THRESHOLD3_MEDIUM;
   end
   else
   if FShadingType = stDark then
   begin
     	threshold1 := THRESHOLD1_DARK;
      	threshold2 := THRESHOLD2_DARK;
      	threshold3 := THRESHOLD3_DARK;
   end;
	for x := 0 to bmp.Width do
	for y := 0 to bmp.Height do
	begin
		col := ColorToRGB(bmp.Canvas.Pixels[x, y]);
		if col <> TransparentColor then
		begin
			r := col shr 16;
			g := (col shr 8) and $00FF;
           b := col and $0000FF;
           sum := r + g + b;
			if sum > THRESHOLD1 then
           	outputbmp.Canvas.Pixels[x, y] := clWhite
           else if sum > THRESHOLD2 then
           	outputbmp.Canvas.Pixels[x, y] := clBtnHighlight
           else if sum > THRESHOLD3 then
           	outputbmp.Canvas.Pixels[x, y] := clBtnShadow
           else
           	outputbmp.Canvas.Pixels[x, y] := clBlack;
       end;
	end;
end;

procedure TExplorerButton.DrawOutline(pushed: Boolean);
begin
	if pushed then
   begin
   	if (BevelStyle = bsLowered) then
			Canvas.Pen.Color := clBtnHighlight
		else
			Canvas.Pen.Color := clBtnShadow;
   end
   else
   begin
   	if (BevelStyle = bsLowered) then
			Canvas.Pen.Color := clBtnShadow
		else
			Canvas.Pen.Color := clBtnHighlight;
   end;
	Canvas.MoveTo(0, Height);
	Canvas.LineTo(0, 0);
	Canvas.LineTo(Width-1, 0);
	if pushed then
   begin
   	if (BevelStyle = bsLowered) then
			Canvas.Pen.Color := clBtnShadow
		else
			Canvas.Pen.Color := clBtnHighlight;
   end
   else
   begin
   	if (BevelStyle = bsLowered) then
			Canvas.Pen.Color := clBtnHighlight
		else
			Canvas.Pen.Color := clBtnShadow;
   end;
	Canvas.LineTo(Width-1, Height-1);
	Canvas.LineTo(0, Height-1);
end;

procedure TExplorerButton.DrawPopupMark(x, y: Integer);
var theColor: TColor;
begin
	(* loops were unrolled *)
   theColor := Canvas.Font.Color;
	Canvas.Pixels[x    , y - 1] := theColor;
	Canvas.Pixels[x + 1, y - 1] := theColor;
	Canvas.Pixels[x + 2, y - 1] := theColor;
	Canvas.Pixels[x + 3, y - 1] := theColor;
	Canvas.Pixels[x + 4, y - 1] := theColor;
	Canvas.Pixels[x + 5, y - 1] := theColor;
	Canvas.Pixels[x + 6, y - 1] := theColor;

	Canvas.Pixels[x + 1, y    ] := theColor;
	Canvas.Pixels[x + 2, y    ] := theColor;
	Canvas.Pixels[x + 3, y    ] := theColor;
	Canvas.Pixels[x + 4, y    ] := theColor;
	Canvas.Pixels[x + 5, y    ] := theColor;

	Canvas.Pixels[x + 2, y + 1] := theColor;
	Canvas.Pixels[x + 3, y + 1] := theColor;
	Canvas.Pixels[x + 4, y + 1] := theColor;

	Canvas.Pixels[x + 3, y + 2] := theColor;
end;

procedure TExplorerButton.GetLost;
begin
	if FGroupIndex = 0 then
   	FDown := False;
	Pushed := False;
	MouseIn := False;
	Refresh;
   (* Thank you, Steve Pinneo ! *)
end;

(*
 * After the loading of the properties, the grayscale version can be
 * safely created
 *)
procedure TExplorerButton.Loaded;
begin
	inherited Loaded;
	if (not FBitmap.Empty) and FNoFocusBitmap.Empty then
		CreateGrayscaleBitmap(IBitmap, FBitmap);
end;

procedure TExplorerButton.Notification(AComponent: TComponent; Operation:
TOperation);
begin
    if (Operation = opRemove) and (AComponent = FDropDown) then
       FDropDown := nil ;
end ;

const PADDING = 2; (* Space between the bitmap and the caption *)

(*
 * Painting of the button
 *)
procedure TExplorerButton.Paint;
var xt, yt, wt, _wt, ht, xb, yb, wb, hb, thePadding: Integer;
	TempCaption : Array[0..256] of char;
	TempRect     : TRect;
begin
	if visible or (csDesigning in ComponentState) then
	begin
   	(* Just to stop compiler warnings *)
   	xt := 0;
       yb := 0;
       xb := 0;
		Canvas.Brush.Style := bsClear;
		Canvas.Font := Self.Font;

		if not (boIconOnly in FOptions) and (Length(FCaption) > 0) then
       begin
       	thePadding := PADDING;
			wt := Canvas.TextWidth(Caption);
			ht := Canvas.TextHeight(Caption)
       end
       else
       begin
       	thePadding := 0;
           wt := 0;
       	ht := 0;
       end;

       if wt > Width - 4 then
       	wt := Width - 4;

      	wb := FBitmap.Width;
       (* Reserve place for the drawing of the popup mark *)
       if (boPopupMark in FOptions) and Assigned(FDropDown) then
       	wb := wb + 10;
      	hb := FBitmap.Height;

       if FBitmap.Empty then
       begin
			if Assigned(FDropDown) and (boPopupMark in FOptions) then
           begin
           	_wt := wt + 10;
   	       	if wt > Width - 14 then
					wt := Width - 14;
           end
           else
           	_wt := wt;

           yt := (Height - ht) div 2;
       	case Alignment of
				taLeftJustify:	xt := 3;
				taRightJustify:	xt := Width - _wt - 3;
				taCenter:		xt := (Width - _wt) div 2
       	end;
       end
		else
       if (Layout = blBitmapTop) or (Layout = blBitmapBottom) then
       begin
       	if Layout = blBitmapTop then
           begin
          		yb := (Height - (ht + hb + thePadding)) div 2;
              	yt := yb + hb + thePadding
           end
           else
           begin
          		yt := (Height - (ht + hb + thePadding)) div 2;
              	yb := yt + ht + thePadding
           end;
			case Alignment of
				taLeftJustify:
				begin
   				xt := 3;
                   xb := 3
               end;
				taRightJustify:
               begin
               	xt := Width - wt - 3;
                   xb := Width - wb - 3
               end;
				taCenter:
               begin
               	xb := (Width - wb) div 2;
               	xt := (Width - wt) div 2
               end;
			end;
       end
       else
       if Layout = blBitmapLeft then
       begin
       	if wt + wb + thePadding > Width - 4 then
           	wt := Width - 4 - thePadding - wb;
     		yb := (Height - hb) div 2;
     		yt := (Height - ht) div 2;
			case Alignment of
				taLeftJustify:
               begin
  					xb := 3;
                   xt := xb + wb + thePadding
               end;
				taRightJustify:
               begin
               	xt := Width - wt - 3;
					xb := xt - wb - thePadding
               end;
				taCenter:
               begin
               	xb := (Width - (wb + wt + thePadding)) div 2;
              		xt := xb + wb + thePadding
               end;
          	end;
       end
       else (* blBitmapRight *)
       begin
       	if wt + wb + thePadding > Width - 4 then
           	wt := Width - 4 - thePadding - wb;
     		yb := (Height - hb) div 2;
     		yt := (Height - ht) div 2;
			case Alignment of
				taLeftJustify:
               begin
  					xt := 3;
                   xb := xt + wt + thePadding
               end;
				taRightJustify:
               begin
               	xb := Width - wb - 3;
					xt := xb - wt - thePadding
               end;
				taCenter:
               begin
               	xt := (Width - (wb + wt + thePadding)) div 2;
              		xb := xt + wt + thePadding
               end;
          	end;
       end;

		if csDesigning in ComponentState then
		begin
			Canvas.Pen.Color := clBlack;
			Canvas.Pen.Style := psSolid;
			Canvas.Brush.Style := bsClear;
			Canvas.Rectangle(0, 0, Width, Height);
		end;

       StrPCopy(TempCaption, Caption);
       with TempRect do
       begin
			left := xt;
			top  := yt;
			right:= xt + wt;
			bottom:= yt + ht;
		end;

		if Enabled then
		begin
			if not (Pushed and MouseIn) and not Down then
			begin
				(* Unpushed state - Mouse in or out *)
				if MouseIn then
					DrawOutline(False)
               else
               	Canvas.Font.Color := UnselectedFontColor;
               if BevelStyle = bsLowered then
               begin
               	Inc(TempRect.Left);
               	Inc(TempRect.Top);
               	Inc(TempRect.Right);
               	Inc(TempRect.Bottom);
               end;
				if not (boIconOnly in FOptions) and (Length(Caption) > 0) then
					DrawTheText(Canvas, TempRect, TempCaption);
				if not FBitmap.Empty then
				begin
					(* Draw the normal or shaded bitmap.
					 * Transparency color is at (0,0)
					 *)
					if MouseIn or not (boMonoDisplay in FOptions) then
						DrawTransparentBitmap(Canvas, FBitmap, xb, yb, ColorToRGB(FBitmap.Canvas.Pixels[0, 0]))
					else if FNoFocusBitmap.Empty then
						DrawTransparentBitmap(Canvas, IBitmap, xb, yb, ColorToRGB(IBitmap.Canvas.Pixels[0, 0]))
					else
						DrawTransparentBitmap(Canvas, FNoFocusBitmap, xb, yb, ColorToRGB(FNoFocusBitmap.Canvas.Pixels[0, 0]));
					if (boPopupMark in FOptions) and Assigned(FDropDown) then
						DrawPopupMark(xb + FBitmap.Width + 3, yb + (hb div 2));
				end
               else
					if (boPopupMark in FOptions) and Assigned(FDropDown) then
						DrawPopupMark(xt + wt + 3, yt + (ht div 2));
			end
			else
			begin
           	(* Pushed state *)
				DrawOutline(True);
               if BevelStyle = bsRaised then
               begin
               	Inc(TempRect.Left);
               	Inc(TempRect.Top);
               	Inc(TempRect.Right);
               	Inc(TempRect.Bottom);
               end;
				if not (boIconOnly in FOptions) and (Length(Caption) > 0) then
					DrawTheText(Canvas, TempRect, TempCaption);
				if not FBitmap.Empty then
               begin
					DrawTransparentBitmap(Canvas, FBitmap, xb+1, yb+1, ColorToRGB(FBitmap.Canvas.Pixels[0, 0]));
					if (boPopupMark in FOptions) and Assigned(FDropDown) then
						DrawPopupMark(xb + FBitmap.Width + 4, yb + (hb div 2) + 1);
               end
               else
					if (boPopupMark in FOptions) and Assigned(FDropDown) then
						DrawPopupMark(xt + wt + 4, yt + (ht div 2) + 1);
			end
		end
		else
		begin
  			(* Disabled state *)
			if Down then
				DrawOutline(True);
           if not FDisabledBitmap.Empty then
				DrawTransparentBitmap(Canvas, FDisabledBitmap, xb, yb, ColorToRGB(FDisabledBitmap.Canvas.Pixels[0, 0]))
           else
				DrawDisabledBitmap(Canvas, xb, yb, FBitmap);
			if not (boIconOnly in FOptions) and (Length(Caption) > 0) then
				DrawDisabledText(Canvas, TempRect, TempCaption);
		end
	end
end;

procedure TExplorerButton.SetAlignment(value: TAlignment);
begin
	FAlignment := value;
	Refresh;
end;

procedure TExplorerButton.SetAllowAllUp(value: Boolean);
var i: Integer;
	otherbutton: TExplorerButton;
begin
	FAllowAllUp := value;
   if FGroupIndex <> 0 then
		for i := 0 to Parent.ControlCount - 1 do
   	begin
   		if Parent.Controls[i] is TExplorerButton then
       	begin
       		otherbutton := (Parent.Controls[i] as TExplorerButton);
           	if (otherbutton <> Self) and (otherbutton.GroupIndex = GroupIndex) then
           		otherbutton.FAllowAllUp := value;
       	end
   	end;
end;

procedure TExplorerButton.SetBevelStyle(value: TBevelStyle);
begin
	FBevelStyle := value;
   Refresh;
end;

procedure TExplorerButton.SetBitmap(value: TBitmap);
begin
	FBitmap.Assign(value);
	if not FBitmap.Empty and FNoFocusBitmap.Empty then
		CreateGrayscaleBitmap(IBitmap, FBitmap);
	Refresh;
end;

procedure TExplorerButton.SetCaption(value: TCaption);
begin
	FCaption := value;
   if boIconOnly in FOptions then
   	Hint := RemoveAmpersand(FCaption)
   else
		Refresh;
end;

procedure TExplorerButton.SetDisabledBitmap(value: TBitmap);
begin
	FDisabledBitmap.Assign(value);
   if not FEnabled then
		Refresh;
end;

procedure TExplorerButton.SetDown(value: Boolean);
var i: Integer;
	otherbutton: TExplorerButton;
begin
   if FDown = value then
   	Exit;
	if FGroupIndex <> 0 then
   begin
		FDown := value;
		Refresh;
   	if FDown = True then
			for i := 0 to Parent.ControlCount - 1 do
   		begin
   			if Parent.Controls[i] is TExplorerButton then
       		begin
       			otherbutton := (Parent.Controls[i] as TExplorerButton);
           		if (otherbutton <> Self) and (otherbutton.GroupIndex = GroupIndex)
                      and (otherbutton.FDown = True) then
           		begin
           			otherbutton.FDown := False;
                   	otherbutton.Refresh;
           		end
       		end
   		end
   end
end;

procedure TExplorerButton.SetDropDown(value: TPopupMenu);
begin
	FDropDown := value;
   if boPopupMark in FOptions then
   	Refresh;
end;

procedure TExplorerButton.SetEnabled(value: Boolean);
begin
	if FEnabled = value then
   	Exit;
	FEnabled := value;
   GetLost;
end;

procedure TExplorerButton.SetGroupIndex(value: Integer);
begin
   if FGroupIndex = value then
   	Exit;
   FGroupIndex := value;
   if (FGroupIndex = 0) and FDown then
   begin
   	FDown := False;
   	Refresh;
   end;
end;

procedure TExplorerButton.SetLayout(value: TExplorerButtonLayout);
begin
	if FLayout = value then
   	Exit;
	FLayout := value;
   Refresh;
end;

procedure TExplorerButton.SetNoFocusBitmap(value: TBitmap);
begin
	FNoFocusBitmap.Assign(value);
	if not FBitmap.Empty and FNoFocusBitmap.Empty then
		CreateGrayscaleBitmap(IBitmap, FBitmap);
	Refresh;
end;

procedure TExplorerButton.SetOptions(value: TExplorerButtonOptions);
begin
	FOptions := value;
   if boIconOnly in FOptions then
   begin
   	Hint := RemoveAmpersand(FCaption);
       ShowHint := True;
   end
   else if Hint = RemoveAmpersand(FCaption) then
   begin
   	Hint := '';
       ShowHint := False;
   end;
   Refresh;
end;

procedure TExplorerButton.SetShadingType(value: TExplorerButtonShadingType);
begin
	FShadingType := value;
	if not FBitmap.Empty and FNoFocusBitmap.Empty then
   begin
		CreateGrayscaleBitmap(IBitmap, FBitmap);
   	Refresh;
   end;
end;

procedure TExplorerButton.SetUnselectedFontColor(value: TColor);
begin
	FUnselectedFontColor := value;
   Refresh;
end;

procedure TExplorerButton.WMLButtonDown(var msg: TWMLButtonDown);
var p: TPoint;
begin
	if Enabled and Visible then
  	begin
		Pushed := True;
       MouseIn := True;
		Refresh;
		(* If the popup is defined, display it under the button *)
		if FDropDown <> nil then
       begin
			p := ClientToScreen(Point(0, Height));
			FDropDown.Popup(p.x, p.y);
			Pushed := False;
			MouseIn := False;
   		Refresh;
		end;
	end;
end;

procedure TExplorerButton.WMLButtonUp(var msg: TWMLButtonUp);
var i: Integer;
	otherbutton: TExplorerButton;
begin
	if Enabled and Visible and Pushed then
	begin
		Pushed := False;
       if MouseIn and (GroupIndex <> 0) then
       begin
       	(* Was Down -> allow up state if AllowAllUp is True *)
           if FDown then
           begin
           	if FAllowAllUp then
           		FDown := False;
           end
           else
           (* Was Up *)
           begin
				(* Set 'up' all buttons having the same parent
                  and GroupIndex *)
               for i := 0 to Parent.ControlCount - 1 do
               begin
               	if Parent.Controls[i] is TExplorerButton then
                   begin
                   	otherbutton := (Parent.Controls[i] as TExplorerButton);
                   	if (otherbutton <> Self) and (otherbutton.GroupIndex = GroupIndex)
                          and (otherbutton.FDown = True) then
                       begin
                       	otherbutton.Down := False;
                           otherbutton.Refresh;
                       end
                   end
               end;
               FDown := True;
           end;
       end;
       (* BUG FIX 2.1 - Release the mouse and assume the focus goes out*)
       SetCaptureControl(nil);
       if MouseIn then
       begin
       	MouseIn := False;
       	Invalidate;
			(* If the popup is not defined, activate the click event *)
			if (FDropDown = nil) then
				Click;
       end
       (* END BUG FIX 2.1 *)
	end
end;

(* For the previous users: the implementation of the
 * MouseEnter and MouseLeave events has been rewritten.
 *
 * The previous version used the Borland CM_MOUSEENTER and
 * CM_MOUSELEAVE messages. Unfortunately, these messages
 * are not sent when the component is in a panel on a
 * MDI form.
 *
 * So this implementation works in all cases:
 *  - if the mouse goes over the button and the mouse is not
 *    yet captured, the mouse is captured and MouseEnter event occured
 *  - if the mouse goes beyond the boundaries of the button,
 *    the capture is released and the MouseLeave event occured
 *)
procedure TExplorerButton.WMMouseMove(var msg: TWMMouseMove);
begin
	if not MouseCapture then
	begin
   	SetCaptureControl(Self);
		if Enabled and Visible and GetParentForm(Self).Active then
		begin
			MouseIn := True;
           if not FDown then
				Refresh;
		end;
   end;
  	(* BUG FIX 2.1 *)
   if MouseCapture and ((msg.XPos < 0) or (msg.YPos < 0) or (msg.XPos > Width) or
   	(msg.YPos > Height)) and MouseIn then
   begin
   	MouseIn := False;
      	if not FDown then
			Refresh;
   	if not Pushed then
       	SetCaptureControl(nil);
   end;
   if MouseCapture and ((msg.XPos >= 0) and (msg.YPos >= 0) and (msg.XPos <= Width) and
   	(msg.YPos <= Height)) and not MouseIn then
   begin
   	MouseIn := True;
       if not FDown then
       	Refresh;
   end;
   (* END BUG FIX 2.1 *)
end;

procedure TExplorerButton.WMRButtonUp(var msg: TWMRButtonUp);
begin
     msg.Result := 0;
end;

procedure Register;
begin
	RegisterComponents('Freeware', [TExplorerButton]);
	RegisterPropertyEditor(TypeInfo(TAboutExpButtonProperty), TExplorerButton, 'ABOUT', TAboutExpButtonProperty);
end;

end.
