Unit ExtPage;

{***************************************************************}
{                                                               }
{   ExtPageControl component                                    }
{   is Freeware. You can always find the souce code at WWW      }
{   http://ourworld.compuserve.com/homepages/praxisservice/     }
{                                                               }
{   But it is not allowed to make this source code public       }
{   available anywhere else. Thank you!                         }
{                                                               }
{     Date:    Jan, 5th 1997                                    }
{     Version: 1.4d                                             }
{     Author:  Jan - M. Strube                                  }
{     System:  Delphi 2.01                                      }
{                                                               }
{     E-Mail:                                                   }
{         Compuserve: 100333,2744                               }
{         Internet:   100333.2744@Compuserve.com                }
{                                                               }
{***************************************************************}

{$DEFINE NODEBUG}       {DEBUG / NODEBUG}

{$ALIGN ON}
{$BOOLEVAL OFF}
{$EXTENDEDSYNTAX ON}
{$HINTS ON}
{$LONGSTRINGS ON}
{$OPTIMIZATION ON}
{$SAFEDIVIDE OFF}
{$STACKFRAMES OFF}
{$TYPEDADDRESS ON}
{$VARSTRINGCHECKS OFF}
{$WARNINGS ON}
{$WRITEABLECONST OFF}

{$IFDEF DEBUG}
  {$DEBUGINFO ON}
  {$LOCALSYMBOLS ON}
  {$OVERFLOWCHECKS ON}
  {$RANGECHECKS ON}
  {$REFERENCEINFO ON}
  {$STACKCHECKS ON}
{$ELSE}
  {$DEBUGINFO OFF}
  {$LOCALSYMBOLS OFF}
  {$OVERFLOWCHECKS OFF}
  {$RANGECHECKS OFF}
  {$REFERENCEINFO OFF}
  {$STACKCHECKS OFF}
{$ENDIF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  StdCtrls, ComCtrls;

type
  TExtNbDrawTabEvent = procedure(Control: TWinControl; Index: Integer;
                                 ActiveTab: Boolean; Rect: TRect;
                                 State: TOwnerDrawState) of object;

  TExtTabStyle = (tabStDefault, tabStButton);

  TExtPageControl = class(TPageControl)
    private
      FCanvas: TCanvas;
      FHintOrg: String;
      FLastTabHintIndex: Integer;
      FTabActiveColor: TColor;
      FTabActiveFont: TFont;
      FTabHints: TStringList;
      FTabInactiveColor: TColor;
      FTabInactiveFont: TFont;
      FTabStyle: TExtTabStyle;
      FOnDrawTab: TExtNbDrawTabEvent;
      InSwitching: Boolean;
      function GetActivePage: TTabSheet;
      procedure SetActivePage(Value: TTabSheet);
      procedure SetTabActiveColor(Value: TColor);
      procedure SetTabActiveFont(Value: TFont);
      procedure SetTabHints(Value: TStringList);
      procedure SetTabInactiveColor(Value: TColor);
      procedure SetTabInactiveFont(Value: TFont);
      procedure SetTabStyle(Value: TExtTabStyle);
      procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
      procedure CMHintShow(var Msg: TMessage); message CM_HINTSHOW;
      procedure CMParentColorChanged(var Msg: TMessage); message CM_PARENTCOLORCHANGED;
      procedure CNDrawTab(var Msg: TWMDrawItem); message CN_DRAWITEM;
      procedure WMEraseBkgnd(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
      procedure WMLButtonDown(var Msg: TWMLButtonDown); message WM_LBUTTONDOWN;
      procedure WMSetCursor(var Msg: TWMSetCursor); message WM_SETCURSOR;
    protected
      function CanChange: Boolean; override;
      procedure Change; override;
      procedure CreateParams(var Params: TCreateParams); override;
      function GetTabNumber(CurrentPos: TPoint): Integer; virtual;
      procedure Loaded; override;
      procedure TabFontChanged(Sender: TObject); virtual;
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure DrawTab(Index: Integer; ActiveTab: Boolean;
                        Rect: TRect; State: TOwnerDrawState); virtual;
      function GetPageIndexByTabIndex(CurrentTabIndex: Integer): Integer;
      procedure GetTabsRectArea(var AreaRect: TRect); virtual;
      property Canvas: TCanvas read FCanvas;
    published
      property ActivePage: TTabSheet
        read GetActivePage
        write SetActivePage;

      property Color;
      property ParentColor;

      property TabActiveColor: TColor
        read FTabActiveColor
        write SetTabActiveColor;

      property TabActiveFont: TFont
        read FTabActiveFont
        write SetTabActiveFont;

      property TabHints: TStringList
        read FTabHints
        write SetTabHints;

      property TabInactiveColor: TColor
        read FTabInactiveColor
        write SetTabInactiveColor;

      property TabInactiveFont: TFont
        read FTabInactiveFont
        write SetTabInactiveFont;

      property TabStyle: TExtTabStyle
        read FTabStyle
        write SetTabStyle
        default tabStDefault;

      property TabStop default False;

      property OnDrawTab: TExtNbDrawTabEvent
        read FOnDrawTab
        write FOnDrawTab;
  end;

procedure register;

implementation

uses
  CommCtrl, LibConst, Forms;

var
  IsNewWindowsVersion: Boolean;

constructor TExtPageControl.Create(AOwner: TComponent);
begin
  inherited;
  FCanvas:= TControlCanvas.Create;
  FTabHints:= TStringList.Create;
  FTabHints.Sorted:= False;
  FLastTabHintIndex:= -1;
  FTabActiveColor:= clBtnFace;
  FTabActiveFont:= TFont.Create;
  FTabActiveFont.OnChange:= TabFontChanged;
  FTabInactiveFont:= TFont.Create;
  FTabInactiveFont.OnChange:= TabFontChanged;
  FTabInactiveColor:= RGB(160, 160, 164);
  FTabStyle:= tabStDefault;
  InSwitching:= False;
  TabStop:= False;
end;

destructor TExtPageControl.Destroy;
begin
  FTabActiveFont.Free;
  FTabInactiveFont.Free;
  FCanvas.Free;
  FTabHints.Free;
  inherited;
end;

procedure TExtPageControl.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style:= Params.Style or TCS_OWNERDRAWFIXED;
  if FTabStyle = tabStButton then
    Params.Style:= Params.Style or TCS_BUTTONS;
end;

function TExtPageControl.CanChange: Boolean;
begin
  if inherited CanChange then begin
    CanChange:= True;
    InSwitching:= True;
  end else
    CanChange:= False;
end;

procedure TExtPageControl.Change;
begin
  InSwitching:= False;
  inherited;
end;

function TExtPageControl.GetTabNumber(CurrentPos: TPoint): Integer;
var
  HitTestInfo: TTCHitTestInfo;
begin
  if HandleAllocated then begin
    HitTestInfo.pt:= CurrentPos;
    Result:= Perform(TCM_HITTEST, 0, LParam(@HitTestInfo));
  end else
    Result:= -1;
end;

procedure TExtPageControl.Loaded;
begin
  inherited;
  FHintOrg:= Hint;
end;

procedure TExtPageControl.TabFontChanged(Sender: TObject);
begin
  // We let PageControl know what's the font of the tabs.
  // With this information it calculates height and width
  // itself, if TabHeight or TabWidth = 0
  if abs(FTabActiveFont.Height) > abs(FTabInActiveFont.Height) then
    SendMessage(Handle, WM_SETFONT, FTabActiveFont.Handle, 0)
  else
    SendMessage(Handle, WM_SETFONT, FTabInActiveFont.Handle, 0);
  Invalidate;
end;

function TExtPageControl.GetActivePage: TTabSheet;
begin
  GetActivePage:= inherited ActivePage;
end;

procedure TExtPageControl.SetActivePage(Value: TTabSheet);
begin
  if Value <> inherited ActivePage then begin
    if Value.Enabled or (csDesigning in ComponentState) then
      if CanChange then begin
        inherited ActivePage:= Value;
        Change;
      end;
  end;
end;

procedure TExtPageControl.SetTabActiveColor(Value: TColor);
begin
  if FTabActiveColor <> Value then begin
    FTabActiveColor:= Value;
    InvalidateRect(Handle, nil, False);
  end;
end;

procedure TExtPageControl.SetTabHints(Value: TStringList);
begin
  FTabHints.Assign(Value);
end;

procedure TExtPageControl.SetTabInactiveColor(Value: TColor);
begin
  if FTabInactiveColor <> Value then begin
    FTabInactiveColor:= Value;
    InvalidateRect(Handle, nil, False);
  end;
end;

procedure TExtPageControl.SetTabActiveFont(Value: TFont);
begin
  FTabActiveFont.Assign(Value);
end;

procedure TExtPageControl.SetTabInactiveFont(Value: TFont);
begin
  FTabInactiveFont.Assign(Value);
end;

procedure TExtPageControl.SetTabStyle(Value: TExtTabStyle);
begin
  if FTabStyle <> Value then begin
    FTabStyle:= Value;
    ReCreateWnd;
  end;
end;

procedure TExtPageControl.CMFontChanged(var Msg: TMessage);
begin
  Invalidate;
  NotifyControls(CM_PARENTFONTCHANGED);
end;

procedure TExtPageControl.CMHintShow(var Msg: TMessage);
var
  InfHint: ^THintInfo;
  TabIndex: Integer;
  CurrCurPos: TPoint;
  NewHintPos: TPoint;
  CurrTabRect: TRect;
begin
  if GetCursorPos(CurrCurPos) then begin
    Windows.ScreenToClient(Handle, CurrCurPos);
    TabIndex:= GetTabNumber(CurrCurPos);
    if TabIndex >= 0 then begin
      // Hint should appear at Tab
      if Perform(TCM_GETITEMRECT, TabIndex, LParam(@CurrTabRect)) <> 0 then begin
        InfHint:= Pointer(Msg.LParam);
        NewHintPos:= Point(CurrTabRect.Left + 3, CurrTabRect.Bottom + 1);
        Windows.ClientToScreen(Handle, NewHintPos);
        InfHint^.HintPos:= NewHintPos;
      end;
    end;
  end;
end;

procedure TExtPageControl.CMParentColorChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TExtPageControl.CNDrawTab(var Msg: TWMDrawItem);
var
  State: TOwnerDrawState;
  PaintRect: TRect;
  OldBitmap, MemBitmap: HBitmap;
  PaintDC, MemDC: HDC;
begin
  with Msg.DrawItemStruct^ do begin
    if Integer(itemID) < 0 then exit;
    PaintDC:= hDC;
    State:= TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);
    PaintRect:= Rect(0, 0, rcItem.Right - rcItem.Left, rcItem.Bottom - rcItem.Top);

    if IsNewWindowsVersion then begin
      // bug correction for NT 4: tab height is to small for inactive tabs
      if not (odSelected in State) then PaintRect.Bottom:= PaintRect.Bottom + 2;
    end;

    MemBitmap:= CreateCompatibleBitmap(PaintDC, PaintRect.Right, PaintRect.Bottom);
    try
      MemDC:= CreateCompatibleDC(PaintDC);
      try
        OldBitmap:= SelectObject(MemDC, MemBitmap);
        FCanvas.Handle:= MemDC;
        FCanvas.Brush:= Brush;
        try
          if odSelected in State then begin
            FCanvas.Brush.Color:= FTabActiveColor;
            FCanvas.Font:= FTabActiveFont;
            if not Pages[itemID].Enabled then
              FCanvas.Font.Color:= clGray;
            DrawTab(itemID, True, PaintRect, State);
          end else begin
            FCanvas.Brush.Color:= FTabInactiveColor;
            FCanvas.Font:= FTabInactiveFont;
            if not Pages[itemID].Enabled then
              FCanvas.Font.Color:= clGray;
            DrawTab(itemID, False, PaintRect, State);
          end;
          BitBlt(PaintDC, rcItem.Left, rcItem.Top, PaintRect.Right, PaintRect.Bottom,
                 MemDC, 0, 0, SRCCOPY);
        finally
          FCanvas.Handle:= 0;
          SelectObject(MemDC, OldBitmap);
        end;
      finally
        DeleteDC(MemDC);
      end;
    finally
      DeleteObject(MemBitmap);
    end;
  end;
end;

procedure TExtPageControl.DrawTab(Index: Integer; ActiveTab: Boolean;
                               Rect: TRect; State: TOwnerDrawState);
begin
  FillRect(FCanvas.Handle, Rect, FCanvas.Brush.Handle);
  if Assigned(FOnDrawTab) then
    FOnDrawTab(Self, GetPageIndexByTabIndex(Index), ActiveTab, Rect, State)
  else begin
    SetBkMode(FCanvas.Handle, Windows.TRANSPARENT);
    DrawText(FCanvas.Handle, PChar(Tabs[Index]), -1,
             Rect, DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  end;
end;

procedure TExtPageControl.WMEraseBkgnd(var Msg: TWMEraseBkgnd);
var
  WholeClientArea: TRect;
  TabsArea: TRect;
  TempRect: TRect;
  RgnAll, RgnTabs, RgnDest: HRgn;
  EmptyPoint: TPoint;
begin
  // less flicker, if we don't erase the
  // tabs area at WM_ERASEBKGND
  WholeClientArea:= ClientRect;
  EmptyPoint:= Point(0, 0);
  SetBrushOrgEx(Msg.DC, Left mod 8, Top mod 8, EmptyPoint);
  GetTabsRectArea(TabsArea);

  if InSwitching then begin
    RgnAll:= CreateRectRgnIndirect(WholeClientArea);
    try
      RgnTabs:= CreateRectRgnIndirect(TabsArea);
      try
        RgnDest:= CreateRectRgn(0, 0, 0, 0);
        try
          CombineRgn(RgnDest, RgnAll, RgnTabs, RGN_DIFF);
          FillRgn(Msg.DC, RgnDest, Parent.Brush.Handle);
        finally
          DeleteObject(RgnDest);
        end;
      finally
        DeleteObject(RgnTabs);
      end;
    finally
      DeleteObject(RgnAll);
    end;
  end else begin
    TempRect:= WholeClientArea;
    TempRect.Bottom:= TabsArea.Bottom;
    FillRect(Msg.DC, TempRect, Parent.Brush.Handle);
    TempRect:= WholeClientArea;
    TempRect.Top:= TabsArea.Bottom;
    FillRect(Msg.DC, TempRect, Brush.Handle);
  end;
  Msg.Result:= 1;
end;

procedure TExtPageControl.WMLButtonDown(var Msg: TWMLButtonDown);
var
  HitIndex: Integer;
begin
  // filter mouse clicks at disabled pages
  if csDesigning in ComponentState then
    inherited
  else begin
    HitIndex:= GetTabNumber(SmallPointToPoint(Msg.Pos));
    if HitIndex >= 0 then begin
     if Pages[HitIndex].Enabled then inherited;
    end else
      inherited;
  end;
end;

procedure TExtPageControl.WMSetCursor(var Msg: TWMSetCursor);
var
  TabIndexNew: Integer;
  CurrCurPos: TPoint;
begin
  inherited;
  // check if another Hint string necessary 
  if (not ShowHint) or (csDesigning in ComponentState) then exit;
  if Msg.CursorWnd <> Handle then begin
    if FLastTabHintIndex <> -1 then begin
      FLastTabHintIndex:= -1;
      Hint:= FHintOrg;
    end;
  end else
    if GetCursorPos(CurrCurPos) then begin
      Windows.ScreenToClient(Handle, CurrCurPos);
      TabIndexNew:= GetTabNumber(CurrCurPos);
      if TabIndexNew <> FLastTabHintIndex then begin
        FLastTabHintIndex:= TabIndexNew;
        if Application <> nil then begin
          Application.CancelHint;
          if (TabIndexNew >= 0) and (FTabHints.Count >= TabIndexNew+1) then
            Hint:= FTabHints[TabIndexNew]
          else
            Hint:= '';
        end;
      end;
    end;
end;

function TExtPageControl.GetPageIndexByTabIndex(CurrentTabIndex: Integer): Integer;
var
  LokZaehl: Integer;
begin
  // search the correct page within the Pages-Array
  for LokZaehl:= CurrentTabIndex to PageCount-1 do begin
    // Pages[LokZaehl].TabIndex is -1 if the Tab of the page
    // is invisible (TabVisible = False)
    if Pages[LokZaehl].TabIndex = CurrentTabIndex then begin
      GetPageIndexByTabIndex:= LokZaehl;
      exit;
    end;
  end;
  GetPageIndexByTabIndex:= -1;
end;

procedure TExtPageControl.GetTabsRectArea(var AreaRect: TRect);
var
  RFirstTab, RLastTab: TRect;
begin
  if Tabs.Count > 0 then begin
    Perform(TCM_GETITEMRECT, 0, LParam(@RFirstTab));
    Perform(TCM_GETITEMRECT, Tabs.Count-1, LParam(@RLastTab));
    SetRect(AreaRect, RFirstTab.Left, RFirstTab.Top,
            RLastTab.Right, RLastTab.Bottom);
  end else
    SetRect(AreaRect, 0, 0, 0, 0);
end;

procedure register;
begin
  RegisterComponents(LoadStr(srWin95), [TExtPageControl]);
end;

procedure GetIsNewWindowsVersion;
var
  OSVersionInfo: TOSVersionInfo;
begin
  OSVersionInfo.dwOSVersionInfoSize:= SizeOf(OSVersionInfo);
  IsNewWindowsVersion:= GetVersionEx(OSVersionInfo) and
                        (OSVersionInfo.dwPLatformId = VER_PLATFORM_WIN32_NT) and
                        (OSVersionInfo.dwMajorVersion >= 4);
end;

initialization
  GetIsNewWindowsVersion;
end.
