unit FancyMem;

interface

uses Controls,Classes,MemStr,MemBuf,SysUtils,WinTypes,WinProcs,
		 MethList,Measure,Graphics;

const BreakChars:set of char=[#13,' '];

type TFancyMemo = class(TGraphicControl)
		protected
			FMethodList:TMethodList;
			procedure DefineProperties(Filer:TFiler); override;
		public
			PenPos:TPoint;
			Indent:TRect;
			TextMetric:TTextMetric;
			Text:TDynamicMemoryStream;
			constructor Create(AOwner:TComponent); override;
			procedure Paint; override;
			procedure ProcessToken(Token:PChar; Length:longint);
			procedure WriteString(ToWrite:PChar; Length:longint);
			procedure NewLine; virtual;
			procedure Leading(Points:longint);
			procedure InitCanvas;
			function  GetWordWidth(W:PChar; L:longint):longint;
			procedure WriteWord(W:PChar; L:longint);
			property Canvas;
		published
			property MethodList:TMethodList read FMethodList;
			property indLeft:integer read Indent.Left write Indent.Left;
			property indRight:integer read Indent.Right write Indent.Right;
			property indTop:integer read Indent.Top write Indent.Top;
			property indBottom:integer read Indent.Bottom write Indent.Bottom;
			end;

procedure Register;

implementation

procedure Register;
begin
RegisterComponents('Additional',[TFancyMemo]);
end;

procedure TFancyMemo.DefineProperties(Filer:TFiler);
begin
inherited DefineProperties(Filer);
MethodList.DefineProperties(Filer);
end;

constructor TFancyMemo.Create(AOwner:TComponent);
var AParent:TWinControl;
begin
inherited Create(AOwner);
FMethodList:=TMethodList.Create(Owner,Self);
Text:=TDynamicMemoryStream.Create(0,256);
Width:=120;
Height:=80;
Indent.Left:=10;
Indent.Right:=10;
Indent.Top:=10;
Indent.Bottom:=10;
end;

procedure TFancyMemo.Paint;
var S:PChar;
		i,L:longint;
		C:Char;
	procedure ReadUpTo(Length:longint);
	begin
	L:=Length+1;
	GetMem(S,L);
	Text.Read(S^,Length,dmsUpdatePos);
	S[Length]:=#0;
	end;
var NextToken,EndToken:longint;
begin
Canvas.Rectangle(0,0,Width,Height);
InitCanvas;
PenPos.X:=Indent.Left;
PenPos.Y:=Indent.Top;
Canvas.Rectangle(0,0,Width,Height);
repeat
	NextToken:=Text.FindChar('<');
	if NextToken=1 then
		begin
		EndToken:=Text.FindChar('>');
		ReadUpTo(EndToken);
		ProcessToken(S,L);
		FreeMem(S,L);
		end
	else
		begin
		if NextToken>0 then
			begin
			ReadUpTo(NextToken-1);
			WriteString(S,L);
			FreeMem(S,L);
			end;
		end;
	until Text.AtEnd;
Text.Seek(0,stBeginning);
end;

procedure TFancyMemo.WriteString(ToWrite:PChar; Length:longint);
	procedure DoWord(W:PChar; L:longint);
	var WidthLeft,WordW:longint;
	const Space:PChar=' ';
	begin
	if L>0 then
		begin
		WidthLeft:=Width-Indent.Right-PenPos.X;
		WordW:=GetWordWidth(W,L);
		if WordW>WidthLeft then NewLine;
		Canvas.MoveTo(PenPos.X,PenPos.Y);
		WriteWord(W,L);
		TextOut(Canvas.Handle,0,0,Space,1);
		PenPos:=Canvas.PenPos;
		end;
	end;
var S:Pchar;
		i,j:longint;
begin
S:=ToWrite;
j:=0;
for i:=1 to Length-1 do
	begin
	if S[j] in BreakChars then
		begin
		DoWord(S,j);
		Inc(S,j+1);
		j:=0;
		end
	else
		Inc(j);
	end;
DoWord(S,j);
end;

procedure TFancyMemo.ProcessToken(Token:PChar; Length:longint);
var L,ArgsBegin:PChar;
		ArgsLength:longint;
		T:string;
		M:TTokenHandler;
begin
T:=StrPas(Token+1);
L:=StrScan(Token,' ');
if L=nil then
	begin
	ArgsBegin:=Token+Length-1;
	ArgsLength:=0;
	PByte(@T)^:=Length-3;
	end
else
	begin
	ArgsBegin:=L+1;
	ArgsLength:=Length-(L-Token)-3;
	PByte(@T)^:=L-Token-1;
	end;
M:=MethodList.GetMethod(T);
if Assigned(M) then
	M(T,ArgsBegin,ArgsLength);
end;

procedure TFancyMemo.InitCanvas;
begin
SetTextAlign(Canvas.Handle,ta_UpdateCP);
end;

procedure TFancyMemo.NewLine;
begin
GetTextMetrics(Canvas.Handle,TextMetric);
PenPos.X:=Indent.Left;
PenPos.Y:=PenPos.Y+TextMetric.tmHeight+TextMetric.tmExternalLeading;
end;

procedure TFancyMemo.Leading(Points:longint);
begin
NewLine;
PenPos.Y:=PenPos.Y+PointToPixels(Canvas,Points);
end;

procedure TFancyMemo.WriteWord(W:PChar; L:longint);
begin
TextOut(Canvas.Handle,PenPos.X,PenPos.Y,W,L);
end;

function TFancyMemo.GetWordWidth(W:PChar; L:longint):longint;
begin
Result:=LoWord(GetTextExtent(Canvas.Handle,W,L));
end;

end.
