unit FileDlgs;

interface

uses WinTypes,CommDlg;

function FillFileStructure(var ToFill:TOpenFilename; Parent:HWnd; Filename,Filters:PChar; FileLength:integer):POpenFilename;
function DoFileOpen(Window:HWnd; Filename,Default:PChar):Boolean;
function DoFileSave(Window:HWnd; Filename,Default:PChar):Boolean;

function FileExists(Filename:PChar):boolean;

implementation

uses WinProcs,Strings;

const	DefaultFileStructure:TOpenFilename =
			 (lStructSize: 			SizeOf(TOpenFilename);
				hwndOwner: 				0;
				hInstance: 				0;         {not needed generally}
				lpstrFilter: 			'All files (*.*)'#0'*.*'; {all filters, in pairs}
				lpstrCustomFilter:nil;  {initial filter, if desired to be different}
				nMaxCustFilter: 	0;    {size of previous field}
				nFilterIndex: 		1;    {0 means use custom filter, otherwise use nth filter from filter list}
				lpstrFile: 				nil;  {where the filename goes}
				nMaxFile: 				256;
				lpstrFileTitle: 	nil;   {can extract only filename, no path, to this buffer}
				nMaxFileTitle: 		0;     {size of above}
				lpstrInitialDir: 	nil;   {nil means use current directory}
				lpstrTitle: 			nil;   {dialog box title. Nil means default}
				Flags: 						ofn_HideReadOnly or ofn_PathMustExist or ofn_NoChangeDir or ofn_OverwritePrompt;
				nFileOffset: 			0;      {says where in full filename the actual filename begins}
				nFileExtension: 	0;     {says where extension begins}
				lpstrDefExt: 			'   ');  {default extension}

function FillFileStructure;
begin
ToFill:=DefaultFileStructure;
with ToFill do
	begin
	hwndOwner:=Parent;
	if Filters<>nil then lpstrFilter:=Filters;
	lpstrFile:=Filename;
	nMaxFile:=FileLength;
	lpstrDefExt:=StrScan(StrScan(lpstrFilter,#0)+1,'.')+1;
	end;
FillFileStructure:=@ToFill;
end;

procedure MakeFilterList(Default:PChar; Filter:PChar; length:integer);
var	NextF:PChar; NextLength:integer;
		NextD:PChar;
begin
StrLCopy(Filter,Default,length);
NextF:=StrEnd(Filter)+1; NextLength:=length-StrLen(Filter)-1;
if NextLength>0 then
	begin
	NextD:=StrEnd(Default)+1;
	StrLCopy(NextF,NextD,NextLength);
	NextF:=StrEnd(NextF)+1; NextLength:=NextLength-StrLen(Default)-1;
	if NextLength>0 then
		begin
		StrLCopy(NextF,'All files (*.*)',NextLength);
		NextF:=StrEnd(NextF)+1; NextLength:=NextLength-16;
		if NextLength>0 then
			begin
			StrLCopy(NextF,'*.*',NextLength);
			NextF:=StrEnd(NextF)+1; NextLength:=NextLength-4;
			if NextLength>0 then
				begin
				StrLCopy(NextF,#0,NextLength);
				end;
			end;
		end;
	end;
if NextLength<=0 then
	begin
	Filter[0]:=#0;
	Filter[1]:=#0;
	end;
end;

function DoFileOpen(Window:HWnd; Filename,Default:PChar):Boolean;
var FileStruct:TOpenFilename;
		Filter:array[0..80] of char;
begin
Filename[0]:=#0;
MakeFilterList(Default,Filter,SizeOf(Filter));
if Filter[1]=#0 then StrCopy(Filename,Default);
FillFileStructure(FileStruct,Window,Filename,Filter,30);
DoFileOpen:=GetOpenFilename(Filestruct);
end;

function DoFileSave(Window:HWnd; Filename,Default:PChar):Boolean;
var Filestruct:TOpenFilename;
		Filter:array[0..80] of char;
begin
Filename[0]:=#0;
MakeFilterList(Default,Filter,SizeOf(Filter));
if Filter[1]=#0 then StrCopy(Filename,Default);
FillFileStructure(FileStruct,Window,Filename,Filter,30);
DoFileSave:=GetSaveFilename(FileStruct);
end;

function FileExists;
var result:integer;
begin
result:=_lopen(Filename,of_Read);
if result=-1 then FileExists:=false {file does not exist}
else
	begin
	FileExists:=true;
	_lclose(result);
	end;
end;

end.
