unit Filter;

{ Table filter component for D 1.0 / D 2.0  Version 1.0
  copyright  Grahame Grieve, Kestral Computing}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, DBiTypes, DBiProcs, DBiErrs,
  propedit, dsgnintf;

type TOperator = (equals, greaterThan, LessThan, NotEqual);
     TCombMode = (cbAnd, cbOr);
type
  TTabFilter = class(TComponent)
  private
    { Private declarations }
    FTable: TTable;                              {store's tablename}
    FField1,FField2: string;                              {store's fieldname}
    FFilter1,FFilter2: string;                             {store's filter}
    FOp1,FOp2:Toperator;
    FCombMode:TCombMode;
{$IFNDEF WIN32}
    FFiltering: boolean;
    FWaitForLoaded: boolean;
{$ENDIF}
    procedure SetTablename(value: TTable);
    procedure SetFiltering(v:boolean);
    function GetFiltering:boolean;
    procedure SetField1(value: string);
    procedure SetOp1(v:TOPerator);
    procedure SetFilter1(value: string);
    procedure SetField2(value: string);
    procedure SetOp2(v:TOPerator);
    procedure SetFilter2(value: string);
    procedure SetCombMode(v:TCombMode);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor create(AOwner:TComponent); Override;
    procedure loaded; override;
    procedure SetFilter;                         {set filter on table}
    procedure RemoveFilter;                      {remove filter from table}
  published
    { Published declarations }
    property Table: TTable read FTable write SetTablename;
    property FirstField:string read FField1 write SetField1;
    property FirstOp:TOperator read FOp1 write SetOp1;
    property FirstFilter:string read FFilter1 write SetFilter1;
    property SecondField:string read FField2 write SetField2;
    property SecondOp:TOperator read FOp2 write SetOp2;
    property SecondFilter:string read FFilter2 write SetFilter2;
    Property CombMode:TCombMode read FCombMode write SetCombMode;
    Property Filtering: boolean read GetFiltering write SetFiltering;
  end;

procedure Register;

{******************************************************************************}
implementation

{register component}
procedure Register;
begin
 RegisterComponents('Samples', [TTabFilter]);
 RegisterPropertyEditor(TypeInfo(string), TTabFilter, 'FirstField', TmyFieldProperty);
 RegisterPropertyEditor(TypeInfo(string), TTabFilter, 'SecondField', TmyFieldProperty);
end;

constructor TTabFilter.create;
begin
  inherited create(AOwner);
  FTable := nil;
  FField1 := '';
  FOp1 := equals;
  FFilter1 := '';
  FField2 := '';
  FOp2 := equals;
  FFilter2 := '';
  FCombMode := cbOr;
  FFiltering := false;
  FWaitForloaded := false;
end;

procedure TTabFilter.loaded;
begin
 inherited loaded;
 if FWaitforloaded then SetFilter;
 FWaitForLoaded := false;
end;

procedure TTabFilter.SetTablename(value: TTable);
begin
 if value <> FTable then
   begin
   FTable := value;
   if GetFiltering then
     begin
     {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
     SetFilter;
     end;
   end;
end;

procedure TTabFilter.SetField1(value: string);
begin
 if value <> FField1 then
   begin
   FField1 := value;
   if GetFiltering then
     begin
     {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
     SetFilter;
     end;
   end;
end;

procedure TTabFilter.SetFilter1(value: string);
begin
 if value <> FFilter1 then
  begin
  FFilter1 := value;
  if GetFiltering then
    begin
    {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
    SetFilter;
    end;
  end;
end;

procedure TTabFilter.SetOp1(v:TOperator);
begin
 if v <> FOp1 then
  begin
  FOp1 := v;
  if GetFiltering then
    begin
    {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
    SetFilter;
    end;
  end;
end;

procedure TTabFilter.SetField2(value: string);
begin
 if value <> FField2 then
   begin
   FField2 := value;
   if GetFiltering then
     begin
     {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
     SetFilter;
     end;
   end;
end;

procedure TTabFilter.SetFilter2(value: string);
begin
 if value <> FFilter2 then
  begin
  FFilter2 := value;
  if GetFiltering then
    begin
    {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
    SetFilter;
    end;
  end;
end;

procedure TTabFilter.SetOp2(v:TOperator);
begin
 if v <> FOp2 then
  begin
  FOp2 := v;
  if GetFiltering then
    begin
    {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
    SetFilter;
    end;
  end;
end;

procedure TTabFilter.SetCombMode(v:TCombMode);
begin
 if v <> FCombMode then
  begin
  FCombMode := v;
  if GetFiltering then
    begin
    {$IFNDEF WIN32} RemoveFilter; {$ENDIF}
    SetFilter;
    end;
  end;
end;

function TTabFilter.GetFiltering;
begin
{$IFDEF WIN32}
  result := (FTable <> nil) and FTable.Filtered;
{$ELSE}
  result := FFiltering;
{$ENDIF}
end;


procedure TTabfilter.SetFiltering;
begin
  if v <> GetFiltering then
    if v
      then SetFilter
      else RemoveFilter;
end;

{set filter on table}
procedure TTabFilter.SetFilter;

{$IFDEF WIN32}
{delphi 2.0 use the ttable filtering routines }
begin
  if (FTable <> nil) and (FField <> '') and (FFilter <> '') then
    begin
    FTable.Filter := FField + ' = ' + FFilter;
    FTable.Filtered := true;
    end;
end;

{$ELSE}
{pascal version of expression tree}
type
  TmyFilter = record
   Expr: CANExpr;
   Nodes: array[0..6] of CANNode;
   literals: array[0..255] of char;
end;

procedure SetupFilter(var f:TMyFilter;i:integer);
begin
 f.Expr.iVer := 1;
 f.Expr.iTotalSize := sizeOf(f);
 if i = 1 then f.Expr.iNodes := 3 else f.Expr.iNodes := 7;
 f.Expr.iNodeStart := sizeOf(CANExpr);
 f.Expr.iLiteralStart := sizeOf(CANExpr) + 7 * sizeOf(CANNode);
end;

function getdatatype(dt:tfieldtype):integer;
begin
  case dt of
    ftString  :result := fldZSTRING;
    ftCurrency:result := fldstMONEY;
    ftDate    :result := fldDATE;
    ftTime    :result := fldTIME;
    ftFloat   :result := fldFLOAT;
    ftInteger :result := fldINT32;
    ftSmallInt:result := fldINT16;
    ftDateTime:result := fldTIMESTAMP;
 end;
end;

function addliteral(dt:integer; var s:string; fl:pchar; var litc,fc:integer):integer;
var li:longint;
    si:integer;
    d1:double;
    dtime: TDateTime;
    yr, mh, dy: word;        {used for date\time fields}
 begin
 inc(fc);
 case dt of
  fldZSTRING:begin
   result := length(s);
   move(s[1],fl[litc],result);
   inc(litc,result);
   fl[litc] := #0;
   inc(litc);
   end;
  fldINT32:begin
   li := StrToInt(s);
   move(li,fl[litc],sizeOf(li));
   inc(litc,sizeOf(li));
   result := sizeof(li);
   end;
  fldINT16:begin
   si := StrToInt(s);
   move(si,fl[litc],sizeOf(si));
   inc(litc,sizeOf(si));
   result := sizeof(si);
   end;
  fldFLOAT:begin
   d1 := StrToFloat(s);
   move(d1,fl[litc],sizeOf(d1));
   inc(litc,sizeOf(d1));
   result := sizeof(d1);
   end;
  fldDATE:begin
   dtime := StrToDate(s);
   DecodeDate(dtime,yr,mh,dy);
   check(DBiDateEncode(mh,dy,integer(yr),li));
   move(li,fl[litc],sizeOf(li));
   inc(litc,sizeOf(li));
   result := sizeof(li);
   end;
  else raise exception.create('UnSupported Field Type');
  end;
end;

var filtercount,literalscount,nodecount,i,fieldcount:integer;
    cOp:TOperator;
    cField,cFilter:string;
    myFilter: TmyFilter;
    hFilter: hDBiFilter;         {handle to filter}
begin
  if (csLoading in componentstate) then
    begin
    FWaitForloaded := true;
    exit;
    end;
  if (FTable = nil) or (FField1 = '') or (FFilter1 = '') then exit;
  if (FField2 <> '') and (FFilter2 <> '') then  filtercount := 2 else filtercount := 1;
  setupfilter(myfilter,filtercount);
  literalscount := 0;
  fieldcount := 0;
  nodecount := -1;
  if filtercount = 2 then
    begin
    inc(nodecount);
    myFilter.nodes[nodecount].canBinary.NodeClass  := nodeBinary;
    if FCombMode = cbOr then
      myFilter.nodes[nodecount].canBinary.canOP := canOR
    else
      myFilter.nodes[nodecount].canBinary.canOP := canAND;
    myFilter.nodes[nodecount].canBinary.iOperand1 := sizeOf(CANNode);
    myFilter.nodes[nodecount].canBinary.iOperand2 := 4 * sizeOf(CANNode);
    end;

  for i := 1 to filtercount do
    begin
    if i = 1 then
      begin
      cField := FField1;
      cOp := FOp1;
      cFilter := FFilter1;
      end
    else
      begin
      cField := FField2;
      cOp := FOp2;
      cFilter := FFilter2;
      end;

    inc(nodecount);
    myFilter.nodes[nodecount].canBinary.NodeClass  := nodeBinary;
    case cOp of
     equals     :myFilter.nodes[nodecount].canBinary.canOP := canEQ;
     greaterThan:myFilter.nodes[nodecount].canBinary.canOP := canGT;
     LessThan   :myFilter.nodes[nodecount].canBinary.canOP := canLT;
     NotEqual   :myFilter.nodes[nodecount].canBinary.canOP := canNE;
    end;
    myFilter.nodes[nodecount].canBinary.iOperand1 := (nodecount + 1) * sizeOf(CANNode);
    myFilter.nodes[nodecount].canBinary.iOperand2 := (nodecount + 2) * sizeOf(CANNode);

    inc(nodecount);
    myFilter.nodes[nodecount].canField.nodeClass := nodeField;
    myFilter.nodes[nodecount].canField.canOP := canField2;
    myFilter.nodes[nodecount].canField.iFieldNum := fieldcount;
    myFilter.nodes[nodecount].canField.iNameOffset := literalscount;
    addliteral(fldZSTRING,cfield,myFilter.literals,literalscount,fieldcount);

    inc(nodecount);
    myFilter.nodes[nodecount].canConst.nodeClass := nodeConst;
    myFilter.nodes[nodecount].canConst.canOP := canCONST2;
    myFilter.nodes[nodecount].canConst.iOffset := literalscount;
    myFilter.nodes[nodecount].canConst.iType :=
                    getdatatype(FTable.Fieldbyname(cField).DataType);
    myFilter.nodes[nodecount].canConst.iSize :=
     addliteral(myFilter.nodes[nodecount].canConst.iType,
        cFilter,myFilter.literals,literalscount,fieldcount);
    end;

  check(DBiAddFilter(FTable.handle,1,1,False,addr(myFilter),nil,hFilter));
  check(DBiActivateFilter(FTable.handle,hFilter));
  FTable.refresh;
  FFiltering := true;
end;
{$ENDIF}



{remove filter}
procedure TTabFilter.RemoveFilter;
{$IFDEF WIN32}
begin
 if FTable <> nil then FTable.filtered := false;
end;

{$ELSE}
var hFilter: hDBiFilter;  {handle to filter}
begin
 if (FTable = nil) then
 exit;
 check(DBiDeactivateFilter(FTable.handle,nil));
    {if filter handle is null, then all table filters are deactivated}
 check(DBiDropFilter(FTable.handle,nil));
    {if filter handle is null, then all table filters are dropped}
 FTable.refresh;
 FFiltering := false;
end;
{$ENDIF}

end.
