{**********************************************************************
 *  MS-Office95 like gradient window title 32-bit Delphi component. *
 **********************************************************************
 *                                                                    *
 *  Version 1.0 beta created by E. Mayer, last updated 21. 1. 1997    *
 *                                                                    *
 *  Portion of code used from TGradientFill component which has been  *
 *  created by Dennis M. Passmore, Greg Lief and Paul Dougherty.      *
 *  Thank you folks!                                                  *
 *                                                                    *
 *  This is the freeware component. Feel free to use it, upload it.   *
 *  Feel free to modify this sources, but please send me your         *
 *  extensions to my E-mail: inmay@fce.vutbr.cz                       *
 *                                                                    *
 *  While uploading modified sources of this component you should     *
 *  keep information about original author in your docs and comments! *
 *                                                                    *
 **********************************************************************} 

unit GradCapt;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry;

type
  TLoadColorsMode = (lmActive, lmInactive);

  TGradientCaption = class(TComponent)
  private
    { Private declarations }
    FGradColor: TColor;
    FProductCaption: string;
    FProductCaptionColor: TColor;
    FCaptionColor: TColor;
    FProductCaptionFontStyle: TFontStyles;
    FCaptionFontStyle: TFontStyles;
    procedure SetGradColor(Value: TColor);
    procedure SetProductCaption(Value: string);
    procedure SetProductCaptionColor(Value: TColor);
    procedure SetCaptionColor(Value: TColor);
    procedure SetProductCaptionFontStyle(Value: TFontStyles);
    procedure SetCaptionFontStyle(Value: TFontStyles);
  protected
    { Protected declarations }
    FrameWidth: integer;
    FrameHeight: integer;
    TitleHeight: integer;
    ButtonWidth: integer;
    IconWidth, IconHeight: Integer;
    DC: HDC;
    ActiveColor: TColor;
    InActiveColor: TColor;
    ActiveTable: array[0..255] of Longint;   { precalculated Brush.Colors for active title}
    InactiveTable: array[0..255] of Longint; { precalculated Brush.Colors for inactive title}
    procedure LoadColors(Mode: TLoadColorsMode); virtual;
    procedure PaintTitle(Mode: TLoadColorsMode); virtual;
    procedure FindColours(Str : String; var OutR : Integer; var OutG : Integer; var OutB : Integer); virtual;
    procedure GetFromReg(const Col : String; var R_G_B : String); virtual;
  public
    { Public declarations }
    TitleCanvas: TCanvas;
    constructor Create(Owner: TComponent); override;
    procedure DoMessage(var Message: TMessage); virtual;
  published
    { Published declarations }
    property GradientColor: TColor read FGradColor write SetGradColor;
    property ProductCaption: string read FProductCaption write SetProductCaption;
    property ProductCaptionColor: TColor read FProductCaptionColor write SetProductCaptionColor;
    property CaptionColor: TColor read FCaptionColor write SetCaptionColor;
    property ProductCaptionFontStyle: TFontStyles read FProductCaptionFontStyle write SetProductCaptionFontStyle;
    property CaptionFontStyle: TFontStyles read FCaptionFontStyle write SetCaptionFontStyle;
  end;

procedure Register;

implementation

constructor TGradientCaption.Create(Owner: TComponent);
var R, G, B    : integer;
    RegRGB     : string;
begin
  inherited Create(Owner);
  FrameWidth := GetSystemMetrics(SM_CXSIZEFRAME);
  FrameHeight := GetSystemMetrics(SM_CYSIZEFRAME);
  TitleHeight := GetSystemMetrics(SM_CYSIZE) + 4;
  ButtonWidth := GetSystemMetrics(SM_CXSIZE) * 3;
  IconWidth := GetSystemMetrics(SM_CXSMICON);
  IconHeight := GetSystemMetrics(SM_CYSMICON);
  GetFromReg('ActiveTitle', RegRGB);
  FindColours(RegRGB, R, G, B);
  ActiveColor := RGB(R, G, B);
  GetFromReg('InactiveTitle', RegRGB);
  FindColours(RegRGB, R, G, B);
  InActiveColor := RGB(R, G, B);
  LoadColors(lmActive);
  LoadColors(lmInactive);
  TitleCanvas := TCanvas.Create;
  TitleCanvas.Handle := GetWindowDC(TForm(Owner).Handle);
end;

procedure TGradientCaption.FindColours(Str : String; var OutR : Integer; var OutG : Integer; var OutB : Integer);
var
   Count : Integer;
   Ch : String;
begin
   Count := 0;
   OutR := 0;
   OutG := 0;
   OutB := 0;

   repeat
      Count := Count + 1;
      Ch := Copy(Str, Count, 1);
      if (Ch <> ' ') and (Ch <> '') then
         OutR := (OutR * 10) + StrToInt(Ch);
   until (Ch = ' ') or (Ch = '');
   repeat
      Count := Count + 1;
      Ch := Copy(Str, Count, 1);
      if (Ch <> ' ') and (Ch <> '') then
         OutG := (OutG * 10) + StrToInt(Ch);
   until (Ch = ' ') or (Ch = '');
   repeat
      Count := Count + 1;
      Ch := Copy(Str, Count, 1);
      if (Ch <> ' ') and (Ch <> '') then
         OutB := (OutB * 10) + StrToInt(Ch);
   until (Ch = ' ') or (Ch = '');
end;

procedure TGradientCaption.GetFromReg(const Col : String; var R_G_B : String);
begin
   with TRegistry.Create do
   try
      RootKey := HKEY_CURRENT_USER;
      OpenKey('Control Panel\Colors', False);
      R_G_B := ReadString(Col);
   finally
      Free;
   end;
end;

procedure TGradientCaption.LoadColors(Mode: TLoadColorsMode);
var
  X: Integer;
  SColor, EColor : TColor;
  DiffR, DiffG, DiffB : Currency;
  LastR, LastG, LastB : Currency;
  SC, EC : String;
  MySR, MySG, MySB : Integer;
  MyER, MyEG, MyEB : Integer;

  procedure SortOut(SelCol : TColor; var CRed : Integer;
                                     var CGreen : Integer;
                                     var CBlue : Integer);
  var
     RegRGB : String;
  begin
     if SelCol = clBackground then               GetFromReg('Background', RegRGB)
     else if SelCol = clActiveCaption then       GetFromReg('ActiveTitle', RegRGB)
     else if SelCol = clInactiveCaption then     GetFromReg('InactiveTitle', RegRGB)
     else if SelCol = clMenu then                GetFromReg('Menu', RegRGB)
     else if SelCol = clWindow then              GetFromReg('Window', RegRGB)
     else if SelCol = clWindowFrame then         GetFromReg('WindowFrame', RegRGB)
     else if SelCol = clMenuText then            GetFromReg('MenuText', RegRGB)
     else if SelCol = clWindowText then          GetFromReg('WindowText', RegRGB)
     else if SelCol = clCaptionText then         GetFromReg('TitleText', RegRGB)
     else if SelCol = clActiveBorder then        GetFromReg('ActiveBorder', RegRGB)
     else if SelCol = clInactiveBorder then      GetFromReg('InactiveBorder', RegRGB)
     else if SelCol = clAppWorkSpace then        GetFromReg('AppWorkspace', RegRGB)
     else if SelCol = clHighlight then           GetFromReg('Highlight', RegRGB)
     else if SelCol = clHighlightText then       GetFromReg('HighlightText', RegRGB)
     else if SelCol = clBtnFace then             GetFromReg('ButtonFace', RegRGB)
     else if SelCol = clBtnShadow then           GetFromReg('ButtonShadow', RegRGB)
     else if SelCol = clGrayText then            GetFromReg('GrayText', RegRGB)
     else if SelCol = clBtnText then             GetFromReg('ButtonText', RegRGB)
     else if SelCol = clInactiveCaptionText then GetFromReg('InactiveTitleText', RegRGB)
     else if SelCol = clBtnHighlight then        GetFromReg('ButtonHilight', RegRGB)
     else if SelCol = cl3DDkShadow then          GetFromReg('ButtonDkShadow', RegRGB)
     else if SelCol = cl3DLight then             GetFromReg('ButtonLight', RegRGB)
     else if SelCol = clInfoText then            GetFromReg('InfoText', RegRGB)
     else if SelCol = clInfoBk then              GetFromReg('InfoWindow', RegRGB);

     if RegRGB <> '' then
        FindColours(RegRGB, CRed, CGreen, CBlue)
     else begin
        CRed := GetRValue(SelCol);
        CGreen := GetGValue(SelCol);
        CBlue := GetBValue(SelCol);
     end
  end;

begin
   if Mode = lmActive then
     SortOut(ActiveColor, MySR, MySG, MySB)
   else
     SortOut(InactiveColor, MySR, MySG, MySB);

   SortOut(FGradColor, MyER, MyEG, MyEB);

   EColor := RGB(MySR, MySG, MySB); // set the colours normally
   SColor := RGB(MyER, MyEG, MyEB);

   DiffR := (GetRValue(SColor) - GetRValue(EColor)) / 255.0; // work out the difference between
   DiffG := (GetGValue(SColor) - GetGValue(EColor)) / 255.0; // each R G and B pair and divide
   DiffB := (GetBValue(SColor) - GetBValue(EColor)) / 255.0; // into 255 increments

   if Mode = lmActive then
     ActiveTable[0] := SColor // fix the start colour
   else
     InactiveTable[0] := SColor; // fix the start colour
   LastR := GetRValue(SColor); // remember the last colours used
   LastG := GetGValue(SColor); // i.e. the start colour
   LastB := GetBValue(SColor);

   for X := 1 to 254 do begin
      LastR := LastR - DiffR; // keep incrementing the colour stored
      LastG := LastG - DiffG;
      LastB := LastB - DiffB;
      if Mode = lmActive then
        ActiveTable[x] := RGB( Trunc(LastR), Trunc(LastG), Trunc(LastB) )
      else
        InactiveTable[x] := RGB( Trunc(LastR), Trunc(LastG), Trunc(LastB) );
   end;
   if Mode = lmActive then
     ActiveTable[255] := EColor // fix the end colour
   else
     InActiveTable[255] := EColor; // fix the end colour
end;

procedure TGradientCaption.PaintTitle(Mode: TLoadColorsMode);
var
  TempRect   : TRect   ;
  TempStepV  : Single  ;
  ColorCode  : Integer ;
  TempTop    : Integer ;
  TempHeight : Integer ;
  Rect       : TRect;
  CaptionWidth : integer;
  TempBitmap: TBitmap;
begin
  GetWindowRect(TForm(Owner).Handle, Rect);
  TitleCanvas.Brush.Style := bsSolid;

  if Rect.Right - Rect.Left - FrameWidth * 2 - ButtonWidth - 200 > 0 then begin
    TempBitmap := TBitmap.Create;
    TempBitmap.Width := Rect.Right - Rect.Left - FrameWidth * 2 - ButtonWidth - 50 ;
    TempBitmap.Height := TitleHeight - FrameHeight;

    TempRect.Left := 0;
    TempRect.Top := 0;
    TempRect.Right := TempRect.Left + 100;
    TempRect.Bottom := TitleHeight - FrameHeight;
    TempBitmap.Canvas.Brush.Color := FGradColor;
    TempBitmap.Canvas.FillRect(TempRect);

    TempStepV := (Rect.Right - Rect.Left - FrameWidth * 2 - ButtonWidth - 150) / 255;                     { Height of each color band }
    TempHeight := Trunc(TempStepV + 1);            { adjust up to ensure overlap }

    TempTop := 100;
    ColorCode := 0;
    TempRect.Top := 0;
    TempRect.Bottom := TitleHeight - FrameHeight;
    for ColorCode := 0 to 255 do begin
       if Mode = lmActive then
         TempBitmap.Canvas.Brush.Color := ActiveTable[ColorCode]
       else
         TempBitmap.Canvas.Brush.Color := InactiveTable[ColorCode];
       TempRect.Left  := TempTop;
       TempRect.Right := TempTop + TempHeight;
       TempBitmap.Canvas.FillRect(TempRect);
       TempTop := 100 + Trunc(TempStepV * ColorCode);
    end;

    TempBitmap.Canvas.Font.Assign(TitleCanvas.Font);
    TempBitmap.Canvas.Font.Style := ProductCaptionFontStyle;
    TempBitmap.Canvas.Font.Color := FProductCaptionColor;
    TempBitmap.Canvas.Brush.Style := bsClear;
    CaptionWidth := TempBitmap.Canvas.TextWidth(ProductCaption);
    TempBitmap.Canvas.TextOut(FrameWidth + 2 + IconWidth + 3,
                              TempBitmap.Height div 2 - TempBitmap.Canvas.TextHeight('X') div 2,
                              ProductCaption);
    TempBitmap.Canvas.Font.Style := CaptionFontStyle;
    TempBitmap.Canvas.Font.Color := FCaptionColor;
    TempBitmap.Canvas.TextOut(FrameWidth + 2 + IconWidth + 3 + CaptionWidth + 20,
                              TempBitmap.Height div 2 - TempBitmap.Canvas.TextHeight('X') div 2,
                              TForm(Owner).Caption);
    TitleCanvas.Draw(FrameWidth, FrameHeight, TempBitmap);
    DrawIconEx(TitleCanvas.Handle, FrameWidth + 1, FrameHeight + 1, Application.Icon.Handle,
               IconWidth, IconHeight, 0, 0, DI_NORMAL);
    TempBitmap.Free;
  end;
end;

procedure TGradientCaption.DoMessage(var Message: TMessage);
begin
  with Message do begin
    DC := GetWindowDC(TForm(Owner).Handle);
    if Msg = WM_ACTIVATE then begin
      if WParamLo = WA_INACTIVE then PaintTitle(lmInactive);
      if WParamLo = WA_ACTIVE then PaintTitle(lmActive);
    end else begin
      if TForm(Owner).Handle = GetActiveWindow then
        PaintTitle(lmActive)
      else
        PaintTitle(lmInactive);
    end;
    ReleaseDC(TForm(Owner).Handle, DC);
  end;
end;

procedure TGradientCaption.SetGradColor(Value: TColor);
begin
  if FGradColor <> Value then begin
    FGradColor := Value;
    if not (csDesigning in ComponentState) then begin
      LoadColors(lmActive);
      LoadColors(lmInactive);
      if TForm(Owner).Handle = GetActiveWindow then
        PaintTitle(lmActive)
      else
        PaintTitle(lmInactive);
    end;
  end;
end;

procedure TGradientCaption.SetProductCaption(Value: string);
begin
  if FProductCaption <> Value then begin
    FProductCaption := Value;
    if not (csDesigning in ComponentState) then begin
      if TForm(Owner).Handle = GetActiveWindow then
        PaintTitle(lmActive)
      else
        PaintTitle(lmInactive);
    end;
  end;
end;

procedure TGradientCaption.SetProductCaptionColor(Value: TColor);
begin
  if FProductCaptionColor <> Value then begin
    FProductCaptionColor := Value;
    if not (csDesigning in ComponentState) then begin
      if TForm(Owner).Handle = GetActiveWindow then
        PaintTitle(lmActive)
      else
        PaintTitle(lmInactive);
    end;
  end;
end;

procedure TGradientCaption.SetCaptionColor(Value: TColor);
begin
  if FCaptionColor <> Value then begin
    FCaptionColor := Value;
    if not (csDesigning in ComponentState) then begin
      if TForm(Owner).Handle = GetActiveWindow then
        PaintTitle(lmActive)
      else
        PaintTitle(lmInactive);
    end;
  end;
end;

procedure TGradientCaption.SetProductCaptionFontStyle(Value: TFontStyles);
begin
  if FProductCaptionFontStyle <> Value then begin
    FProductCaptionFontStyle := Value;
    if not (csDesigning in ComponentState) then begin
      if TForm(Owner).Handle = GetActiveWindow then
        PaintTitle(lmActive)
      else
        PaintTitle(lmInactive);
    end;
  end;
end;

procedure TGradientCaption.SetCaptionFontStyle(Value: TFontStyles);
begin
  if FCaptionFontStyle <> Value then begin
    FCaptionFontStyle := Value;
    if not (csDesigning in ComponentState) then begin
      if TForm(Owner).Handle = GetActiveWindow then
        PaintTitle(lmActive)
      else
        PaintTitle(lmInactive);
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Win95', [TGradientCaption]);
end;

end.
