unit	GraphPalWin;

interface

uses GraphFileWin,WinTypes,OWindows,DIBitmap,GraphWin;

type	PGraphicsPaletteWindow=^TGraphicsPaletteWindow;
			TGraphicsPaletteWindow=object(TGraphicsFileWindow)
				DIContents:PDIBitmap; {only for referring to what GraphWindow stores already}
				constructor Init(AParent:PWindowsObject;ATitle:PChar;NewContents:PDIBitmap);
				destructor Done; virtual;
				procedure Reinitialize(NewContents:PDIBitmap);
				procedure SelectInto(TheDC:HDC); virtual;
				procedure Deselect(TheDC:HDC); virtual;
				procedure WMPaletteChanged (var Msg:TMessage); virtual wm_First+wm_PaletteChanged;
				procedure WMQueryNewPalette(var Msg:TMessage); virtual wm_First+wm_QueryNewPalette;
				procedure WMClose(var Msg:TMessage); virtual wm_First+wm_Close;
				procedure Paint(PaintDC:HDC; var PaintInfo:TPaintStruct); virtual;
			private
				UsingPalette:Boolean;
				MyPalette:HPalette;
				OldPalette:HPalette;
				NeedPaletteRedrawn:boolean;
				Closing:boolean;
				end;

implementation

uses WinProcs;

constructor TGraphicsPaletteWindow.Init;
var BitCount:integer;
begin
TGraphicsFileWindow.Init(AParent,ATitle,NewContents);
DIContents:=NewContents;
Closing:=false;
MyPalette:=CreatePalette(DIContents^.LogPalette^);
GetDC(DisplayDC); BitCount:=GetDeviceCaps(DC,BitsPixel); ReleaseDC;
if BitCount=8 then UsingPalette:=true else UsingPalette:=false;
if UsingPalette then NeedPaletteRedrawn:=true else NeedPaletteRedrawn:=false;
end;

destructor TGraphicsPaletteWindow.Done;
begin
DeleteObject(MyPalette);
TGraphicsFileWindow.Done;
end;

procedure TGraphicsPaletteWindow.Reinitialize;
var	DesiredPalette:PLogPalette;
begin
DIContents:=NewContents;
if UsingPalette then
	begin
	DesiredPalette:=NewContents^.LogPalette;
	ResizePalette(MyPalette,DesiredPalette^.PalNumEntries);
	SetPaletteEntries(MyPalette,0,DesiredPalette^.PalNumEntries,DesiredPalette^.palPalEntry);
	GetDC(DisplayDC);
	UnrealizeObject(MyPalette);
	RealizePalette(DC);
	DIContents^.RedrawHBitmap(DC);
	ReleaseDC;
	end
else
	begin
	GetDC(DisplayDC);
	DIContents^.RedrawHBitmap(DC);
	ReleaseDC;
	end;
TGraphicsFileWindow.Reinitialize(NewContents);
end;

procedure TGraphicsPaletteWindow.SelectInto(TheDC:HDC);
begin
OldPalette:=SelectPalette(TheDC,MyPalette,false);
TGraphicsFileWindow.SelectInto(TheDC);
end;

procedure TGraphicsPaletteWindow.Deselect(TheDC:HDC);
begin
TGraphicsFileWindow.Deselect(TheDC);
SelectPalette(TheDC,OldPalette,false);
end;

procedure TGraphicsPaletteWindow.Paint;
var OldPal:HPalette;
begin
OldPal:=SelectPalette(PaintDC,MyPalette,false);
TGraphicsFileWindow.Paint(PaintDC,PaintInfo);
if NeedPaletteRedrawn then UpdateColors(PaintDC);
SelectPalette(PaintDC,OldPal,false);
end;

procedure TGraphicsPaletteWindow.WMPaletteChanged;
begin
if (Msg.wParam<>HWindow) and (not Closing) then
	begin
	Msg.wParam:=0;
	if UsingPalette then
		begin
		NeedPaletteRedrawn:=true;
		GetDC(DisplayDC);
		RealizePalette(DC);
		UpdateColors(DC);
		ReleaseDC;
		end;
	end;
end;

procedure TGraphicsPaletteWindow.WMQueryNewPalette;
begin
if NeedPaletteRedrawn then
	begin
	NeedPaletteRedrawn:=false;
	Msg.wParam:=1;
	UnrealizeObject(MyPalette);
	GetDC(DisplayDC);
	RealizePalette(DC);
	DIContents^.RedrawHBitmap(DC);
	ReleaseDC;
	Redraw;
	end
else
	Msg.wParam:=0;
end;

procedure TGraphicsPaletteWindow.WMClose;
begin
Closing:=true;
TGraphicsWindow.WMClose(Msg);
end;

end.
