unit GraphWin;

{$R GraphWin.Res}

interface

uses ScrolWin,Bitmap,WinTypes,OWindows;

const	DisplayDC=0;
			MemoryDC=1;

type	PGraphicsWindow=^TGraphicsWindow;
			TGraphicsWindow=object(TScrollingWindow)
				Contents:PBitmapObject;
				CurrentDC:HDC;
				constructor Init(AParent:PWindowsObject;ATitle:PChar; NewContents:PBitmapObject);
				procedure ReInitialize(NewContents:PBitmapObject);
				destructor Done; virtual;
				procedure Paint(PaintDC:HDC; var PaintInfo:TPaintStruct); virtual;
				procedure GetWindowClass(var AWndClass:TWndClass); virtual;
				function GetClassName:PChar; virtual;
				procedure Redraw; virtual;
				function 	GetDC(DCType:integer):HDC; virtual;
				procedure ReleaseDC; virtual;
				function 	DC:HDC; virtual;
				function 	GotDC:Boolean; virtual;
				procedure SelectInto(TheDC:HDC); virtual;
				procedure Deselect(TheDC:HDC); virtual;
			private
				GetDCCount:integer; GotDisplayDC:boolean;
				OldBitmap:HBitmap;
				end;

implementation

uses WinProcs;

constructor TGraphicsWindow.Init;
begin
Contents:=NewContents;
TScrollingWindow.Init(AParent,ATitle,Contents^.Width,Contents^.Height);
GetDCCount:=0;
end;

destructor TGraphicsWindow.Done;
begin
while GotDC do ReleaseDC;
dispose(Contents, Done);
TScrollingWindow.Done;
end;

procedure TGraphicsWindow.ReInitialize;
begin
while GotDC do ReleaseDC;
dispose(Contents, Done);
Contents:=NewContents;
TScrollingWindow.Resize(Contents^.Width,Contents^.Height);
Redraw;
end;

function TGraphicsWindow.GetDC(DCType:integer):HDC;
var TheDC:HDC;
begin
if not GotDC then
	begin
	TheDC:=WinProcs.GetDC(HWindow);
	if (DCType=DisplayDC) then
		begin
		CurrentDC:=TheDC;
		GotDisplayDC:=true;
		end
	else
		begin
		CurrentDC:=CreateCompatibleDC(TheDC);
		WinProcs.ReleaseDC(HWindow,TheDC);
		GotDisplayDC:=false;
		end;
	GetDC:=CurrentDC;
	SelectInto(CurrentDC);
	end
else
	if GotDisplayDC<>(DCType=DisplayDC) then Application^.Error(51);
GetDC:=CurrentDC;
GetDCCount:=GetDCCount+1;
end;

procedure TGraphicsWindow.ReleaseDC;
begin
if GetDCCount=1 then
	begin
	Deselect(CurrentDC);
	if GotDisplayDC then
		WinProcs.ReleaseDC(HWindow,CurrentDC)
	else
		DeleteDC(CurrentDC);
	end;
if GetDCCount>0 then GetDCCount:=GetDCCount-1;
end;

function TGraphicsWindow.DC:HDC;
begin
if not GotDC then Application^.Error(50);
DC:=CurrentDC;
end;

function TGraphicsWindow.GotDC:Boolean;
begin
GotDC:=(GetDCCount<>0);
end;

procedure TGraphicsWindow.SelectInto(TheDC:HDC);
begin
OldBitmap:=SelectObject(TheDC,Contents^.TheBitmap);
end;

procedure TGraphicsWindow.Deselect(TheDC:HDC);
begin
SelectObject(TheDC,OldBitmap);
end;

procedure TGraphicsWindow.GetWindowClass;
begin
TScrollingWindow.GetWindowClass(AWndClass);
AWndClass.HIcon:=LoadIcon(HInstance,'graphicon');
end;

function TGraphicsWindow.GetClassName;
begin
GetClassName:='GraphicsWindow';
end;

procedure TGraphicsWindow.Redraw;
begin
InvalidateRect(HWindow,nil,false);
UpdateWindow(HWindow);
end;

procedure TGraphicsWindow.Paint;
begin
if not GotDC then
	begin
	GetDC(MemoryDC); BitBlt(PaintDC,0,0,width,height,CurrentDC,0,0,srcCopy); ReleaseDC;
	end
else
	BitBlt (PaintDC,0,0,width,height,CurrentDC,0,0,srcCopy);
end;

end.