unit HugeFile;

interface

const OneIO=32768;  { Number bytes handled per huge IO operation }
type	PtrRec=record Lo,Hi:Word end;
			IOFunction=function(FP:integer;Buf:PChar;Size:Integer):Word;
function _LFileSize(F:integer):longint;
function HugeIO(IOFunc:IOFunction; F:Integer; P:Pointer; Size:Longint):Word;
function HugePtr(Start:Pointer; Increment:Longint):Pointer;

implementation

uses WinProcs;

procedure AHIncr; far; external 'KERNEL' index 114;

{Perform IO operation in chunks to avoid memory segment crossings.
 Returns 0 if error else non-zero}
function HugeIO(IOFunc:IOFunction; F:Integer; P:Pointer; Size:Longint):Word;
var
	L, N: Longint;	       	 { L maintains total bytes }
begin   			 { N maintains bytes for current pass }
	HugeIO := 1;
	L := 0;
	while L < Size do
	begin
		N := Size - L;
		if N > OneIO then N := OneIO;
		if IOFunc(F,
		{ Compute the segment and offset reached.
			The Hi word of P contains the initial segment.
			Think of the following as performing arithmetic
				modulo segment-size, since the size of a segment
				fills one word:
			The Hi word of L contains the number of segments crossed
				(the size of one segment fills the Lo word, so Hi word
				will roll over as segments are filled).
				Multiply by Ofs(AHIncr) to obtain the number used to
				indicate this number of segments.
			The Lo word of L contains the number of bytes already
				passed in the present segment.
		 }
				 Ptr(PtrRec(P).Hi + PtrRec(L).Hi * Ofs(AHIncr),
							 PtrRec(L).Lo),
							 Integer(N))     { Guaranteed to be in Integer range }
			 <> N then
		begin
			HugeIO := 0;
			Exit; { abnormal termination }
		end;
		Inc(L, N);
	end;
end;

function HugePtr(Start:Pointer; Increment:longint):Pointer;
begin
HugePtr:=Ptr(PtrRec(Start).Hi+PtrRec(Increment).Hi*Ofs(AHIncr),PtrRec(Increment).Lo);
end;

function _LFileSize(F : integer) : longint;
{- an equivalent to TP's FileSize() function }
var
	CurPos : longint;
begin
	CurPos := _llseek(F,0,1);
	_LFileSize := _llseek(F,0,2);
	_llseek(F,CurPos,0);
end;

end.