unit JPRSplit;

interface

uses Windows,Messages,SysUtils,Classes,Controls,Graphics,Forms,
     StdCtrls,ExtCtrls;

type
  TOrientation = (spVertical,spHorizontal);
  TSplitter = class(TCustomPanel)
  private
     fPriorControl:TControl;
     fNextControl:TControl;
     fOrientation:TOrientation;
     fSplitBar:TPanel;
     fOnStartSplit:TNotifyEvent;
     fOnEndSplit:TNotifyEvent;
     procedure DoAlignment;
  protected
     procedure Loaded; override;
     procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer); override;
     procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
     procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer); override;
     procedure Notification(AComponent: TComponent;
       Operation: TOperation); override;
     procedure SetOrientation(Value:TOrientation);
     procedure SetPriorControl(Value:TControl);
     procedure SetNextControl(Value:TControl);
  public
     constructor Create(AOwner:TComponent); override;
     destructor Destroy; override;
  published
     property BevelInner;
     property BevelOuter;
     property Color;
     property Ctl3D;
     property ParentCtl3D;
     property Orientation:TOrientation read fOrientation write SetOrientation;
     property ControlLeftTop:TControl read fPriorControl write SetPriorControl;
     property ControlRightBottom:TControl read fNextControl write SetNextControl;
     property OnStartSplit:TNotifyEvent read fOnStartSplit write fOnStartSplit;
     property OnEndSplit:TNotifyEvent read fOnEndSplit write fOnEndSplit;
     end;


implementation

function Min(V1,V2:integer):integer;
begin
  if V1<V2 then Result := V1 else Result := V2;
end;

function Max(V1,V2:integer):integer;
begin
  if V1>V2 then Result := V1 else Result := V2;
end;


constructor TSplitter.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle-[csSetCaption,csAcceptsControls];
  fSplitBar := TPanel.Create(Self);
  with fSplitBar do begin
    Color := clGray;
    Visible := false;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    BorderStyle := bsNone;
    Caption := '';
    end;
  Width := 4;
  Cursor := crHSplit;
  BorderWidth := 1;
  BorderStyle := bsNone;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  Height := 50;
end;

destructor TSplitter.Destroy;
begin
  inherited Destroy;
end;

procedure TSplitter.Loaded;
begin
  inherited Loaded;
end;

procedure TSplitter.MouseDown(
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   { Begin Splitting }
   if not (csDesigning in ComponentState) then
     fSplitBar.Parent := Parent;
   fSplitBar.BoundsRect := BoundsRect;
   fSplitBar.Visible := true;
   fSplitBar.BringToFront;
   inherited MouseDown(Button,Shift,X,Y);
   if Assigned(fOnStartSplit) then
      fOnStartSplit(Self);
end;

procedure TSplitter.MouseMove(
  Shift: TShiftState; X, Y: Integer);
var
  Temp:integer;
begin
   { Move Splitter }
   if fSplitBar.Visible then
      if Orientation = spVertical then
         begin
         Temp := Min(Max(Left+X,0),Parent.ClientWidth-(Width*2));
         fSplitBar.Left := Temp;
         end
      else
         begin
         Temp := Min(Max(Top+Y,0),Parent.ClientHeight-(Height*2));
         fSplitBar.Top := Temp;
         end;
   inherited MouseMove(Shift,X,Y);
end;


procedure TSplitter.MouseUp(
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  { Anchor everything }
  if Orientation = spVertical then
     fPriorControl.Width := fSplitBar.Left-fPriorControl.Left
  else
     fPriorControl.Height := fSplitBar.Top-fPriorControl.Top;
  fSplitBar.Visible := false;
  inherited MouseUp(Button,Shift,X,Y);
  if Assigned(fOnEndSplit) then
      fOnEndSplit(Self);
end;

procedure TSplitter.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if AComponent <> Self then
    if Operation = opRemove then
       begin
       if (AComponent = fPriorControl) then fPriorControl := nil;
       if (AComponent = fNextControl) then fNextControl := nil;
       DoAlignment;
       end;
  inherited Notification(AComponent,Operation);
end;

procedure TSplitter.DoAlignment;
var
  PWidth,PHeight,LWidth,LHeight,RWidth,RHeight:integer;
begin
   if (Parent <> nil) and ((ControlRightBottom=nil) or (ControlLeftTop=nil)) then
        begin
        Parent.DisableAlign;
        try
          if ControlLeftTop <> nil then ControlLeftTop.Align := alNone;
          if ControlRightBottom <> nil then ControlRightBottom.Align := alNone;
          Align := alNone;
        finally
           Parent.EnableAlign;
           end;
        exit;
        end;
   if Parent = nil then exit;
   PWidth := Parent.Width;
   PHeight := Parent.Height;
   if Orientation = spVertical then
      begin
      Parent.DisableAlign;
      ControlRightBottom.Align := alNone;
      Align := alNone;
      ControlLeftTop.Align := alNone;
      Parent.EnableAlign;
      {}
      ControlLeftTop.SetBounds(0,0,(PWidth div 2)-(Width+2),PHeight);
      SetBounds((PWidth div 2),0,Width,Height);
      With ControlRightBottom do
         SetBounds((PHeight div 2)+(Self.Height+2),0,Width,Height);
      Parent.DisableAlign;
      ControlLeftTop.Align := alLeft;
      Align := alLeft;
      ControlRightBottom.Align := alClient;
      Parent.EnableAlign;
      Cursor := crHSplit;
      end
   else
      begin
      Parent.DisableAlign;
      ControlRightBottom.Align := alNone;
      Align := alNone;
      ControlLeftTop.Align := alNone;
      Parent.EnableAlign;
      ControlLeftTop.SetBounds(0,0,PWidth,(PHeight div 2)-(Height+2));
      SetBounds(0,(PHeight div 2),Width,Height);
      With ControlRightBottom do
         SetBounds(0,(PHeight div 2)+(Self.Height+2),Width,Height);
      Parent.DisableAlign;
      ControlLeftTop.Align := alTop;
      Align := alTop;
      ControlRightBottom.Align := alClient;
      Parent.EnableAlign;
      Cursor := crVSplit;
      end;
end;

procedure TSplitter.SetOrientation(Value:TOrientation);
var
  Temp:integer;
begin
  if Value <> fOrientation then
     begin
     Parent.DisableAlign;
     if ControlRightBottom <> nil then
        ControlRightBottom.Align := alNone;
     Align := alNone;
     if ControlLeftTop <> nil then
        ControlLeftTop.Align := alNone;
     if not (csLoading in ComponentState) then
        begin
        Temp := Width;
        Width := Height;
        Height := Temp;
        end;
     fOrientation := Value;
     Parent.EnableAlign;
     DoAlignment;
     end;
end;

procedure TSplitter.SetPriorControl(Value:TControl);
begin
  fPriorControl := Value;
  DoAlignment;
end;

procedure TSplitter.SetNextControl(Value:TControl);
begin
  fNextControl := Value;
  DoAlignment;
end;

end.
