{*****************************************************}
{       TLineList 1.0 for Delphi 1.0/2.0              }
{                                                     }
{       Copyright (c) 1997 Tibor F. Liska             }
{       Tel/Fax:    +36-1-165-2019                    }
{       Office:     +36-1-269-8284                    }
{       E-mail: liska@sztaki.hu                       }
{*****************************************************}
unit LineList;

interface
uses SysUtils, Classes;

type
  TLineList = class(TStringList)
  public
      FilePos,
      FileSize: Longint;
    constructor Create(const Fn: TFileName; Ahead: Integer);
    destructor Destroy; override;
  private
      ReadAhead : Integer;
      From : TFileStream;
      Buffer,
      BufEnd,
      BufTop : PChar;
    procedure BufferFree;
    procedure AfterChange(Sender: TObject);
  end;

implementation

const
      BufSize = 4096;

function GetString(BufPtr: PChar; var S: string): PChar; assembler;
asm
        PUSH    DS
        LDS     SI,BufPtr
        LES     DI,S
        XOR     BX,BX
        CLD
@@1:    LODSB
        CMP     AL,' '
        JB      @@3
@@2:    CMP     BX,255
        JE      @@1
        INC     BX
        MOV     ES:[DI+BX],AL
        JMP     @@1
@@3:    OR      AL,AL
        JE      @@4
        CMP     AL,1AH
        JE      @@4
        CMP     AL,0AH
        JE      @@5
        CMP     AL,0DH
        JNE     @@2
        LODSB
        CMP     AL,0AH
        JE      @@5
@@4:    DEC     SI
@@5:    MOV     ES:[DI],BL
        MOV     AX,SI
        MOV     DX,DS
        POP     DS
end;

constructor TLineList.Create(const Fn: TFileName; Ahead: Integer);
begin
  inherited Create;
  OnChange := AfterChange;
  From := TFileStream.Create(Fn, fmOpenRead);
  From.Seek(0, 2);           { End of file }
  FileSize := From.Position;
  From.Seek(0, 0);           { Begining of file }
  ReadAhead := Ahead;
  GetMem(Buffer, BufSize);
  BufEnd := Buffer + BufSize;
  BufTop := BufEnd;
  Changed;
end;

destructor TLineList.Destroy;
begin
  BufferFree;
  inherited Destroy;
end;

procedure TLineList.BufferFree;
begin
  FileSize := 0;
  FilePos := 0;
  if From <> nil then From.Free;
  if Buffer <> nil then FreeMem(Buffer, BufSize);
  From := nil;
  Buffer := nil;
end;

procedure TLineList.AfterChange(Sender: TObject);
  const
       EOF = Chr($1A);
  var
      n : Integer;
      BufPtr : PChar;
      S : string;
begin
  if (From = nil) or (Count >= ReadAhead) then Exit;
  BeginUpdate;
try
  repeat
    n := BufEnd - BufTop;
    if n <> 0 then System.Move(BufTop[0], Buffer[0], n);
    BufTop := Buffer + n;
    BufTop := BufTop + From.Read(BufTop[0], BufEnd - BufTop);
    if BufTop < BufEnd then BufTop[0] := EOF else
    begin
      BufTop := LineStart(Buffer, BufTop);
      if BufTop = Buffer then
        raise Exception.Create('Too long line');
    end;
    BufPtr := Buffer;
    while (BufPtr < BufTop) and (BufPtr[0] <> EOF) do
    begin
      BufPtr := GetString(BufPtr, S);
      Add(S);
    end;
  until (Count > ReadAhead) or
        (BufPtr < BufEnd) and (BufPtr[0] = EOF);
  if Count <= ReadAhead then BufferFree
                        else FilePos := From.Position;
finally
  EndUpdate;
end; end;

end.
