unit MemBuf;

interface

uses Classes,SysUtils;

const	stBeginning=0;
			stRelative=1;
			stEnd=2;

type  EEndOfStream=class(Exception);

type  TMemoryBlock=class(TObject)
				Data:PChar;
				Next:TMemoryBlock;
				Size:longint;
				constructor Create(BlockSize:longint);
				destructor  Destroy; override;
				end;

type	TPosition=record
				Block:TMemoryBlock;
				Offset:longint;
				end;

type  TMemoryBuffer=class(TStream)
			protected
				BlockSize:longint;
				StartPos:TPosition;
				WritePos:TPosition;
				function GetSize:longint; virtual;
			public
				constructor Create(OrigSize,NewBlockSize:Longint);
				destructor  Destroy; override;
				function Write(const Buf; Count:longint):longint; override;
				property Size:longint read GetSize;
				function IsEmpty:boolean;
				end;

implementation

{-----------------------------------------------------------------------}
{------------------ TMemoryBlock ---------------------------------------}
{-----------------------------------------------------------------------}

constructor TMemoryBlock.Create(BlockSize:longint);
begin
inherited Create;
Size:=BlockSize;
GetMem(Data,Size);
Next:=nil;
end;

destructor TMemoryBlock.Destroy;
begin
FreeMem(Data,Size);
inherited Destroy;
end;

{-----------------------------------------------------------------------}
{------------------ TMemoryBuffer --------------------------------------}
{-----------------------------------------------------------------------}

constructor TMemoryBuffer.Create(OrigSize,NewBlockSize:longint);
var NewBlock:TMemoryBlock;
begin
inherited Create;
BlockSize:=NewBlockSize;
if OrigSize=0 then OrigSize:=BlockSize;
NewBlock:=TMemoryBlock.Create(OrigSize);
WritePos.Block:=NewBlock; WritePos.Offset:=0;
StartPos.Block:=NewBlock; StartPos.Offset:=0;
end;

destructor TMemoryBuffer.Destroy;
var	TempBlock:TMemoryBlock;
begin
while StartPos.Block<>nil do
	begin
	TempBlock:=StartPos.Block;
	StartPos.Block:=StartPos.Block.Next;
	TempBlock.Free;
	end;
inherited Destroy;
end;

function TMemoryBuffer.Write(const Buf; Count:longint):longint;
var LastBlockSpaceLeft:longint;
		i,BlocksRequired:longint;
		CountLeft:word;
		StartWrite,BufRead:PChar;
begin
with WritePos do
	begin
	LastBlockSpaceLeft:=Block.Size-Offset;
	StartWrite:=Block.Data;
	Inc(StartWrite,Offset);
	BufRead:=PChar(@Buf);
	if Count<=LastBlockSpaceLeft then
		begin
		Move(BufRead^,StartWrite^,Count);
		Offset:=Offset+Count;
		end
	else
		begin
		Move(BufRead^,StartWrite^,LastBlockSpaceLeft);
		Inc(BufRead,LastBlockSpaceLeft);
		CountLeft:=Count-LastBlockSpaceLeft;
		BlocksRequired:=(CountLeft+BlockSize-1) div BlockSize;
		for i:=1 to BlocksRequired do
			begin
			Block.Next:=TMemoryBlock.Create(BlockSize);
			Block:=Block.Next;
			if CountLeft<=BlockSize then
				begin
				Move(BufRead^,Block.Data^,CountLeft);
				Offset:=CountLeft;
				end
			else
				begin
				Move(BufRead^,Block.Data^,BlockSize);
				Inc(BufRead,BlockSize);
				Dec(CountLeft,BlockSize);
				end;
			end;
		end;
	end;
Result:=Count;
end;

function TMemoryBuffer.GetSize:longint;
var Size:longint;
		Block:TMemoryBlock;
begin
Size:=0;
Block:=StartPos.Block;
repeat
	Size:=Size+Block.Size;
	Block:=Block.Next;
	until Block=nil;
Size:=Size-(WritePos.Block.Size-WritePos.Offset);
GetSize:=Size;
end;

function TMemoryBuffer.IsEmpty:boolean;
begin
IsEmpty:= (StartPos.Block=WritePos.Block)
					and (StartPos.Offset=WritePos.Offset);
end;

end.
