unit MemStr;

interface

uses MemBuf;

type TReadType=(dmsUpdatePos,dmsNoUpdatePos);

type TDynamicMemoryStream = class(TMemoryBuffer)
		private
			ReadPos:TPosition;
			function GetPosition:longint;
		public
			constructor Create(OrigSize,NewBlockSize:longint);
			function Read(var Buf; Count:longint; UpdateQ:TReadType):longint; virtual;
			function Seek(Offset:longint; Origin:word):longint; override;
			function FindChar(ToFind:Char):longint; virtual;
			property Position:longint read GetPosition;
			function AtEnd:Boolean;
		end;

implementation

constructor TDynamicMemoryStream.Create(OrigSize,NewBlockSize:longint);
begin
inherited Create(OrigSize,NewBlockSize);
ReadPos:=StartPos;
end;

function TDynamicMemoryStream.Read(var Buf; Count:longint; UpdateQ:TReadType):longint;
var CountLeft:longint;
		BufWrite:PChar;
		CurrentPos:TPosition;
	procedure ReadBlock;
	var CanRead,WillRead:longint;
			Read:PChar;
	begin
	CanRead:=CurrentPos.Block.Size-CurrentPos.Offset;
	if CurrentPos.Block=WritePos.Block then
		CanRead:=CanRead-WritePos.Block.Size+WritePos.Offset;
	if CanRead<CountLeft then WillRead:=CanRead else WillRead:=CountLeft;
	Read:=CurrentPos.Block.Data;
	Inc(Read,CurrentPos.Offset);
	Move(Read^,BufWrite^,WillRead);
	Inc(BufWrite,WillRead);
	Dec(CountLeft,WillRead);
	Inc(CurrentPos.Offset,WillRead);
	end;
var Failed:boolean;
begin
CountLeft:=Count;
BufWrite:=@Buf;
CurrentPos:=ReadPos;
Failed:=false;
repeat
	ReadBlock;
	if CountLeft>0 then
		begin
		if CurrentPos.Block.Next<>nil then
			begin
			CurrentPos.Block:=CurrentPos.Block.Next;
			CurrentPos.Offset:=0;
			end
		else
			Failed:=true;
		end
	until (CountLeft=0) or Failed;
if Failed then BufWrite^:=#0;
Result:=(Count-CountLeft);
if (UpdateQ=dmsUpdatePos) then ReadPos:=CurrentPos;
end;

function TDynamicMemoryStream.Seek(Offset:longint; Origin:word):longint;
var Buf:Pointer;
begin
case Origin of
	stRelative: Result:=Seek(Position+Offset,stBeginning);
	stEnd:			Result:=Seek(Size+Offset,stBeginning);
	stBeginning:
		begin
		ReadPos:=StartPos;
		GetMem(Buf,Offset);
		Read(Buf,Offset,dmsUpdatePos);
		FreeMem(Buf,Offset);
		Result:=Offset;
		end;
	end;
end;

function TDynamicMemoryStream.FindChar(ToFind:Char):longint;
var TempPos:TPosition;
		Buf:Char;
		NumRead,Count:longint;
begin
TempPos:=ReadPos;
Count:=0;
repeat
	NumRead:=Read(Buf,1,dmsUpdatePos);
	Inc(Count,NumRead);
	until (Buf=ToFind) or (NumRead=0);
ReadPos:=TempPos;
if NumRead=0 then Result:=0 else Result:=Count;
end;


function TDynamicMemoryStream.GetPosition:longint;
var Block:TMemoryBlock;
begin
Block:=StartPos.Block;
while Block<>ReadPos.Block do
	begin
	Result:=Result+Block.Size;
	Block:=Block.Next;
	end;
Result:=Result+ReadPos.Offset;
end;

function TDynamicMemoryStream.AtEnd:boolean;
begin
AtEnd:=(ReadPos.Block=WritePos.Block) and (ReadPos.Offset=WritePos.Offset);
end;

end.
