unit MethEd;

interface

uses MethList,Classes,DsgnIntf,WinProcs,TypInfo,SysUtils;

type TTokenProcessorProperty=class(TPropertyEditor)
		protected
			TokenWrapper:TTokenWrapper;
			Designer:TFormDesigner;
			procedure Modified;
		public
			Name:string;
			constructor Create(ADesigner:TFormDesigner;
												 AName:string; AWrapper:TTokenWrapper);
			function  GetPropTypeData:PTypeData;
			function  GetName:string; override;
			function  GetAttributes:TPropertyAttributes; override;
			function  AllEqual:boolean; override;
			function  GetEditLimit:integer; override;
			procedure GetValues(Proc:TGetStrProc); override;
			function  GetValue:string; override;
			procedure SetValue(const AValue:string); override;
			procedure Edit; override;
			function  GetMethodValue:TMethod;
			procedure SetMethodValue(AMethod:TMethod);
		end;

procedure Register;

implementation

procedure Register;
begin
RegisterPropertyEditor(TypeInfo(TTokenHandler),nil,'',TTokenProcessorProperty);
end;

constructor TTokenProcessorProperty.Create;
begin
inherited Create;
Name:=AName;
Designer:=ADesigner;
TokenWrapper:=AWrapper;
end;

procedure TTokenProcessorProperty.Modified;
begin
Designer.Modified;
end;

function TTokenProcessorProperty.GetName:string;
begin
Result:=Name;
end;

function TTokenProcessorProperty.GetPropTypeData:PTypeData;
begin
Result:=GetTypeData(TypeInfo(TTokenHandler));
end;

function TTokenProcessorProperty.GetAttributes:TPropertyAttributes;
begin
Result:=[paValueList,paSortList];
end;

function TTokenProcessorProperty.AllEqual:Boolean;
begin
AllEqual:=true;
end;

procedure TTokenProcessorProperty.Edit;
var MethodName:string[63];
begin
MethodName:=GetValue;
if MethodName='' then
	MethodName:='On'+GetName;
SetMethodValue(Designer.CreateMethod(MethodName,GetPropTypeData));
Designer.ShowMethod(MethodName);
end;

function TTokenProcessorProperty.GetEditLimit:integer;
begin
Result:=64;
end;

function TTokenProcessorProperty.GetValue:string;
begin
Result:=Designer.GetMethodName(GetMethodValue);
end;

procedure TTokenProcessorProperty.GetValues(Proc:TGetStrProc);
begin
Designer.GetMethods(GetPropTypeData,Proc);
end;

procedure TTokenProcessorProperty.SetValue(const AValue:string);
var NewMethod:boolean;
		CurValue:string[63];
begin
CurValue:=GetValue;
if (CurValue<>'') and (AValue<>'') and
	((CompareText(CurValue,AValue)=0) or not Designer.MethodExists(AValue))
	then
		Designer.RenameMethod(CurValue,AValue)
	else
		begin
		NewMethod:=(AValue<>'') and not Designer.MethodExists(AValue);
		SetMethodValue(Designer.CreateMethod(AValue,GetPropTypeData));
		if NewMethod then Designer.ShowMethod(AValue);
		end;
end;

function TTokenProcessorProperty.GetMethodValue:TMethod;
begin
Result.Data:=TokenWrapper.Owner;
Result.Code:=@TokenWrapper.Method;
end;

procedure TTokenProcessorProperty.SetMethodValue(AMethod:TMethod);
begin
TokenWrapper.Owner:=TObject(AMethod.Data);
@TokenWrapper.Method:=AMethod.Code;
Modified;
end;

end.

