unit MethList;

interface

uses Classes,SysUtils,WinProcs;

type TTokenHandler=procedure(Token:string; Args:PChar; Length:longint) of object;

type TTokenWrapper=class
		public
			Owner:TObject;
			Method:TTokenHandler;
			constructor Create(AOwner:TObject; AMethod:TTokenHandler);
		end;

type TMethodList=class(TStringList)
		private
			OldFindMethod:TFindMethodEvent;
			procedure IgnoreFindMethod(Reader:TReader;
								const MethodName:string; var Address:Pointer; var Error:Boolean);
		protected
			Form,Memo:TComponent;
			procedure ReadTokens(Reader:TReader);
			procedure WriteTokens(Writer:TWriter);
			function AddAMethod(Token:string; Owner:TObject; Method:TTokenHandler):integer;
		public
			constructor Create(AForm,AMemo:TComponent);
			procedure DefineProperties(Filer:TFiler);
			function GetMethod(Token:string):TTokenHandler;
			function GetMethodWrapper(Token:string):TTokenWrapper;
			function AddMethod(Token:string; Method:TTokenHandler):integer;
			function AddExternalMethod(Token:string; Method:TTokenHandler):integer;
		end;

implementation

type TEasyReader=class(TReader) end;

constructor TTokenWrapper.Create(AOwner:TObject; AMethod:TTokenHandler);
begin
inherited Create;
Owner:=AOwner;
Method:=AMethod;
end;


constructor TMethodList.Create(AForm,AMemo:TComponent);
begin
inherited Create;
Form:=AForm;
Memo:=AMemo;
end;

procedure TMethodList.DefineProperties(Filer:TFiler);
begin
Filer.DefineProperty('Token handlers',ReadTokens,WriteTokens,True);
end;

procedure TMethodList.WriteTokens(Writer:TWriter);
var	i:integer;
		W:TTokenWrapper;
		Meth:TTokenHandler;
		MethName:string;
begin
Writer.WriteListBegin;
for i:=0 to Count-1 do
	begin
	W:=Objects[i] as TTokenWrapper;
	if Assigned(W.Owner) and (W.Owner<>Memo) then
		begin
		MethName:=W.Owner.MethodName(@W.Method);
		Writer.WriteString(Strings[i]);
		if W.Owner=Form then Writer.WriteString('Form') else
												 Writer.WriteString('Unknown');
		Writer.WriteIdent(MethName);
		end;
	end;
Writer.WriteListEnd;
end;

procedure TMethodList.ReadTokens(Reader:TReader);
type TVarMethod=record
			case integer of
			1: (Meth:TTokenHandler);
			2: (T:TMethod);
			end;
var M:TVarMethod;
		OwnerStr:string;
		MethName:string;
		Token:string;
begin
OldFindMethod:=Reader.OnFindMethod;
Reader.OnFindMethod:=IgnoreFindMethod;
Reader.ReadListBegin;
while not Reader.EndOfList do
	begin
	Token:=Reader.ReadString;
	OwnerStr:=Reader.ReadString;
	MethName:=Reader.ReadIdent;
	M.T.Code:=TEasyReader(Reader).FindMethod(Form,MethName);
	M.T.Data:=Form;
	AddExternalMethod(Token,M.Meth);
	end;
Reader.ReadListEnd;
Reader.OnFindMethod:=OldFindMethod;
end;

procedure TMethodList.IgnoreFindMethod;
begin
if Assigned(OldFindMethod) then
	OldFindMethod(Reader,MethodName,Address,Error);
if Error then
	begin
	Error:=false;
	end;
end;

function TMethodList.AddAMethod(Token:string; Owner:TObject; Method:TTokenHandler):integer;
var AWrapper:TTokenWrapper;
		i:integer;
begin
i:=IndexOf(Token);
if i=-1 then
	begin
	AWrapper:=TTokenWrapper.Create(Owner,Method);
	Result:=AddObject(Token,AWrapper);
	end
else
	begin
	AWrapper:=Objects[i] as TTokenWrapper;
	AWrapper.Owner:=Owner;
	AWrapper.Method:=Method;
	end;
end;

function TMethodList.AddMethod(Token:string; Method:TTokenHandler):integer;
begin
Result:=AddAMethod(Token,Memo,Method);
end;

function TMethodList.AddExternalMethod(Token:string; Method:TTokenHandler):integer;
begin
Result:=AddAMethod(Token,Form,Method);
end;

function TMethodList.GetMethodWrapper(Token:string):TTokenWrapper;
var i:integer;
		W:TTokenWrapper;
begin
i:=IndexOf(Token);
if i=-1 then
	begin
	W:=TTokenWrapper.Create(Form,nil);
	AddObject(Token,W);
	Result:=W;
	end
else
	Result:=(Objects[i] as TTokenWrapper);
end;

function TMethodList.GetMethod(Token:string):TTokenHandler;
var i:integer;
begin
i:=IndexOf(Token);
if i=-1 then
	Result:=nil
else
	Result:=(Objects[i] as TTokenWrapper).Method;
end;

end.
