
//
// DebugLog - Debug output logging utility.
//
// Simple but very useful demonstration of OLE Automation in Delphi 2.0.
// This is the automation server. It is a simple application containing
// a TMemo that logs debugging strings sent to it by an automation client.
//
// See the TestLog sample application to see how you use how you use
// this automation client to log debugging information.
//
// Copyright  1996 Mike Scott All Rights Reserved
// 100140.2420@compuserve.com -- 10th January 1996
//

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OLEAuto, Menus, StdCtrls;

type
  TLogWindow = class ;

  TMainForm = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    SaveDialog1: TSaveDialog;
    SaveAs1: TMenuItem;
    Options1: TMenuItem;
    ConfirmClear: TMenuItem;
    Saveonexit: TMenuItem;
    Clear2: TMenuItem;
    ReadOnly: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure ClearClick(Sender: TObject);
    procedure OptionClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    { Private declarations }
    FLogWindow : TLogWindow ;
    LastSize   : TRect ;
    Shown      : boolean ;
    function  Save : boolean ;
    procedure NoteSize ;
    procedure wmMove( var Msg : TWMMove ) ; message WM_MOVE ;
    procedure wmSize( var Msg : TWMSize ) ; message WM_SIZE ;
  public
    { Public declarations }
  end;

  // Automation Server
  TLogWindow = class(TAutoObject)
  private
    { Private declarations }
    FMainForm : TMainForm ;
  automated
    { Automated declarations }
    procedure Log( const s : string ) ;
    procedure Clear ;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses IniFiles, About ;

procedure RegisterLogWindow;
const
  AutoClassInfo: TAutoClassInfo = (
    AutoClass: TLogWindow;
    ProgID: 'DebugLog.LogWindow';
    ClassID: '{2BA2A220-4B71-11CF-9324-00608C71DCE6}';
    Description: 'Debug Logging Window';
    Instancing: acMultiInstance);
begin
  Automation.RegisterClass(AutoClassInfo);
end;

{ TLogWindow }

procedure TLogWindow.Log( const s : string ) ;
begin
  MainForm.Memo1.Lines.Add( s ) ;
end ;

procedure TLogWindow.Clear ;
begin
  MainForm.Memo1.Clear ;
end ;

{ TMainForm }

const
  IniFile       = 'DebugLog.ini' ;

  sOptions      = 'Options' ;      // INI section heading
  sConfirmClear = 'ConfirmClear' ;
  sReadOnly     = 'ReadOnly' ;
  sSaveOnExit   = 'SaveOnExit' ;

  sPosition     = 'Position' ;     // INI section heading
  sLeft         = 'Left' ;
  sRight        = 'Right' ;
  sTop          = 'Top' ;
  sBottom       = 'Bottom' ;


procedure TMainForm.FormCreate(Sender: TObject);
begin
  // create the automation object and output date and time.
  FLogWindow := TLogWindow.Create ;
  FLogWindow.FMainForm := Self ;
  FLogWindow.Log( Format( 'Shields up! Stardate %s at %s',
                          [ DateToStr( Date ), TimeToStr( Time ) ] ) ) ;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  // remember to release the automation server object
  FLogWindow.Release ;

  // remember the form position
  with TIniFile.Create( IniFile ), LastSize do try
    WriteInteger( sPosition, sLeft, Left ) ;
    WriteInteger( sPosition, sTop, Top ) ;
    WriteInteger( sPosition, sRight, Right ) ;
    WriteInteger( sPosition, sBottom, Bottom ) ;
  finally
    Free ;
  end ;
end;

function  TMainForm.Save : boolean ;
begin
  // save the log to disk
  Result := SaveDialog1.Execute ;
  if Result then Memo1.Lines.SaveToFile( SaveDialog1.FileName ) ;
end ;

procedure TMainForm.NoteSize ;

begin
  // note the current size if not maximised or minimized
  if Shown and not IsZoomed( Handle ) and not IsIconic( Handle ) then
    LastSize := BoundsRect ;
end ;

procedure TMainForm.wmMove( var Msg : TWMMove ) ;
begin
  inherited ;
  NoteSize ;
end ;

procedure TMainForm.wmSize( var Msg : TWMSize ) ;
begin
  inherited ;
  NoteSize ;
end ;

procedure TMainForm.Exit1Click(Sender: TObject);
var Option : TModalResult ;
begin
  // bring up save option and close if OK
  Option := mrNo ;
  if SaveOnExit.Checked then begin
     Option := MessageDlg( 'Save Debug Log?', mtConfirmation,
                           [ mbYes, mbNo, mbCancel ], 0 ) ;
     if ( Option = mrYes ) and not Save then Option := mrCancel ;
  end ;
  if Option <> mrCancel then Close ;
end;

procedure TMainForm.SaveAs1Click(Sender: TObject);
begin
  Save ;
end;

procedure TMainForm.ClearClick(Sender: TObject);
begin
  // clear log window if OK
  if not ConfirmClear.Checked or
     ( MessageDlg( 'Are you sure you want to clear the log?',
                   mtConfirmation, [ mbYes, mbNo ], 0 ) = idYes ) then
    FLogWindow.Clear ;
end;

procedure TMainForm.OptionClick(Sender: TObject);
begin
  // toggle meny item state
  with ( Sender as TMenuItem ) do Checked := not Checked ;
  if Sender = ReadOnly then Memo1.ReadOnly := ReadOnly.Checked ;

  // save the options in the INI file
  with TIniFile.Create( IniFile ) do try
    WriteBool( sOptions, sConfirmClear, ConfirmClear.Checked ) ;
    WriteBool( sOptions, sReadOnly, ReadOnly.Checked ) ;
    WriteBool( sOptions, sSaveOnExit, SaveOnExit.Checked ) ;
  finally
    Free ;
  end ;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  // restore the previous options and form position & size
  with TIniFile.Create( IniFile ) do try
    ConfirmClear.Checked := ReadBool( sOptions, sConfirmClear, false ) ;
    ReadOnly.Checked := ReadBool( sOptions, sReadOnly, false ) ;
    SaveOnExit.Checked := ReadBool( sOptions, sSaveOnExit, true ) ;
    LastSize := BoundsRect ;
    with LastSize do begin
      Left   := ReadInteger( sPosition, sLeft, Left ) ;
      Top    := ReadInteger( sPosition, sTop, Top ) ;
      Right  := ReadInteger( sPosition, sRight, Right ) ;
      Bottom := ReadInteger( sPosition, sBottom, Bottom ) ;
    end ;
    BoundsRect := LastSize ;
  finally
    Free ;
  end ;
  Memo1.ReadOnly := ReadOnly.Checked ;
  Shown := true ;
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
  TAboutBox.Create( Application ).ShowModal ;
end;

initialization
  RegisterLogWindow;
end.
