(*******************************************************************************
**
**  NoTask
**
**	Prevents a program's icons from showing on the Win95 TASKBAR
**  while the program is running and the form is visible.  Works
**  only in Delphi 2.0. Simply drop the component on the form, and
**  set the HideForm property to TRUE to hide the TASKBAR icons.
**  Set to FALSE to show the TASKBAR icons.
**
**  The outline of this code was shamelessly stolen from the most
**  Excellent (Freeware!!) TPREVCHEK component which prevents a second
**  instance of an application.  Ironically enough, PREVCHEK only works
**  in DELPHI 1.0, whereas NOTASK only works in Delphi 2.0...  Go
**  figure.  Anyway, try this out, lemme know what you think.  This is
**  generally intended to be used with one of the TASKTRAY components.
**  Feedback to:
**	Eric Lawrence
**	Lead Programmer
**	Delta Programming Group
**
**	deltagrp@juno.com    or  deltagrp@keynetcorp.net	
**
**  This component should be dropped on either the main form of the
**  application or on the Splash screen form, whichever shows first.
**
**  Controls:
**  This code may be used, modified, included in applications without any
**  license agreements as long as the disclaimers are accepted, and the
**  comments are left intact ( Some Hope! <g> ),
**
**  Disclaimer:
**  This software is released into the public domain on the strict understanding
**  that neither myself nor any associates or companies I work for have any
**  liability explictly or implied.
**
**  Possible Enhancements:
**	Beats me.  Any ideas?
*******************************************************************************)
unit NoTask;

interface
uses
  WinTypes, WinProcs, Classes, SysUtils,Forms, Controls;

type
  TDuplicateComponent = class(Exception);
  { Define a Form not Owner object exception }
  TFormNotOwner = class(Exception);
  { The NoTask Class declaration }
  TNoTask = class(TComponent)
  private
    { Private declarations - Attributes }
    FHideForm: boolean;
    { Private declarations - Methods }
    Function IsIt:boolean;
    Procedure SetIt (value:boolean);
  public
    { Public declarations - Methods }
    constructor Create(AOwner: TComponent); override;
    procedure Loaded; override;
    procedure ProcessEnabled;

  published
    { Published declarations }
    property HideForm: boolean read IsIt write SetIt stored true;
  end;

procedure Register;                     { Register for Object Inspector }




implementation

(*
**  Overridden constructor for our component.  Creates underlying TComponent,
**  sets our own method pointer to an unassigned state, and checks that there
**  is only one TNoTask Component on the form.  If there is another one,
**  this one is destroyed by virtue of the exception handler within the design
**  mode.
*)
constructor TNoTask.Create(AOwner: TComponent);
var
  i: word;				{ General Loop Counter }
  CompCount: byte;			{ TNoTask Component count }
begin
  inherited Create(AOwner);             { Call TComponent Constructor }
  CompCount := 0;                       { Initialise Component Count to zero }
  { If we are designing at present }
  if (csDesigning in ComponentState) then
    if (AOwner is TForm) then
      with (AOwner as TForm) do
      begin
        for i := 0 to ComponentCount - 1 do { Check if there is already one of us! }
          if Components[i] is TNoTask then inc(CompCount);

if CompCount > 1 then raise TDuplicateComponent.Create ('There is already a TNoTask component on this Form');
     end
      else
        raise TFormNotOwner.Create
              ('The owner of TNoTask Component is not a TForm');
end;

(*
**  Overridden LOADED method.  This ensures that the check for a previous
**  instance of our Application is done immediately after the TNoTask
**  components properties have been read in from the form, and initialised.
*)
procedure TNoTask.Loaded;
begin
  inherited Loaded;                     { Always call inherited Loaded method }
 if not (csDesigning in ComponentState) then ProcessEnabled;                     { May not return from here! }
end;


Procedure TNoTask.ProcessEnabled;
begin
if fHideform then
showwindow(FindWindow (nil,@Application.Title[1]),sw_hide)
else
showwindow(FindWindow (nil,@Application.Title[1]),sw_restore);
end;

Function TNoTask.IsIt:boolean;
begin
result:=fHideform;
end;

Procedure TNoTask.SetIt(value:boolean);
begin
fhideform:=value;
processenabled;
end;


{Registration of us for the Object Inspector & Component Palette}
procedure Register;
begin
  RegisterComponents('Additional', [TNoTask]);
end;

end.
