(*=============================================================================
QueryQBE VCL 2.0
Allows use of QBE query language and is TDBDataSet compatible
===============================================================================
This VCL was adapted from the original author of the TQBE component (UtQBE)
which worked so well with Delphi 1.0. I used it in a lot of projects, being
a Paradox convert and when we upgraded to 2.0 I tried to recompile it and
kept getting a wierd declaration error. Apparently the error stemmed from the
uses clause in the implementation section. In any event, after waiting for a
couple months, a new version didn't appear anywhere. I posted a couple times
and didn't get many responses. Essentially, I got frustrated and set out to
fix the code so it would work. After some trial and error, I got this version
to work. It's essentially the same component as before. My changes were limited
to getting it to work with 2.0 and cleaning up the syntax, varnames a bit. I
don't want anyone to construe this as my own original work. I would give credit
to the original author, credit being due, but their name wasn't included in
the original version and I'm at a loss as to who it was originally.

Anyway, it was freeware and always will be. I make no claims to ownership of
this code. If it's up to me, use it as you will. Likewise, I take no
responsibility for any problems you might encounter. It works for me on local
Paradox and DBF tables, but I haven't tested it with SQLLinks or ODBC tables.
But there's no warranty or guarantee implied anywhere in any of this, so don't
construe one.

I would like to make some changes to the component, particularly in the way
params are handled. If anyone has additional suggestions please feel free to
contact me.

Grant Carpenter
grant@tiac.net

===============================================================================
*ORIGINAL COMMENTS* (edited down slightly)
Answer table name can be specified with or without alias. Eg: :dbdemos:void.db

Assumed answer table Driver is PARADOX
The AnswerType property allows PARADOX, DBASE and ASCII table types

If the answer table exists, an attempt to delete it is made before copying the
result cursor. The phisical answer table must be opened in exclusive mode and
all related family files are erased together with the table.

Other functions in this unit:

  **  Function GetAliasPath(Const Alias:String):String;
      Returns the path for the "alias" or a empty string if not found.
      Eg: ('dbdemos') returns 'c:\delphi\demos\data'

  **  Function GetDBTablePath(Const TableName:String):String;
      Returns the TableName with path instead of alias (if it has an alias).
      Eg: (':dbdemos:customer.db') returns 'c:\delphi\demos\data\customer.db'
=============================================================================*)

unit QueryQBE;

interface

uses Classes, DBTables, DB, SysUtils, DBConsts, LibConst, DBIProcs, DBITypes,
     DsgnIntf, Dialogs, Forms, DBIErrs;

{ uses Dialogs, DBIErrs, Forms; }
{ This was the original line that caused it not to compile under 2.0--it was  }
{ under the implementation, so I moved the units referenced up here to the    }
{ interface section                                                           }

const
  MAX_PARAMS = 10;
  MAX_PARAM_LEN = 50;

type
  TParamArray = array[0..MAX_PARAMS] of String[MAX_PARAM_LEN];

type TQueryQBE = class(TDBDataSet)
     private
       FAnswerTable: String;
       FAnswerType: TTableType;
       FAuxTables: Boolean;
       FBlankAsZero: Boolean;
       FQBE: TStrings;
       FRequestLive: Boolean;
       NumParams: Integer;
       Param: TParamArray;
       Subst: TParamArray;
       procedure SetQBE(QBE: TStrings);
     protected
       function CreateHandle: HDBICur; override;
       procedure AddParam(const TempParam, TempSubst: String);
       function ReplaceString(S: String): String;
       procedure ClearParams;
     public
       constructor Create(AOwner:TComponent); override;
       destructor Destroy; override;
     published
       property AnswerTable: String read FAnswerTable write FAnswerTable;
       property AnswerType:TTableType read FAnswerType write FAnswerType;
       property AuxTables: Boolean read FAuxTables write FAuxTables;
       property BlankasZero: Boolean read FBlankasZero write FBlankAsZero;
       property QBE: TStrings read FQBE write SetQBE;
     end;

procedure register;
function GetAliasPath(const Alias: String): String;
function GetDBTablePath(const TableName: String): String;

implementation

procedure register;
begin
  RegisterComponents(LoadStr(srDAccess), [TQueryQBE]);
end;

constructor TQueryQBE.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FQBE := TStringList.Create;
  NumParams := 0;
  FAnswerType := ttParadox;
end;

destructor TQueryQBE.Destroy;
begin
  FQBe.Free;
  inherited Destroy;
end;

function TQueryQBE.CreateHandle: HDBICur;
var
  P: HDbiCur;
begin
  Check(DbiQExecDirect(DBHandle, qryLangQBE, FQBE.GetText, @P));
  Result:=p;
end;

procedure TQueryQBE.SetQBE(QBE: TStrings);
begin
  FQBE.Assign(QBE);
end;

procedure TQueryQBE.ClearParams;
begin
  NumParams := 0;
end;

procedure TQueryQBE.AddParam(const TempParam, TempSubst: String);
begin
  if TempParam <> '' then begin
    if NumParams < MAX_PARAMS then begin
      Inc(NumParams);
      Param[NumParams] := TempParam;
      Subst[NumParams] := TempSubst;
    end else
      raise Exception.Create('Maximum number of query parameters achieved');
  end;
end;

function TQueryQBE.ReplaceString(S: String): String;
var
  T, I: Integer;
begin
  for T := 1 to NumParams do
  repeat
    I := Pos(Param[T], S);
    if I > 0 then
      S := Copy(S, 1, I-1) + Subst[T] + Copy(S, I + Length(Param[T]), 255);
  until I = 0;
  Result := S;
End;

function HasAlias(const TableName: String): Boolean;
begin
  Result := Pos(':', TableName) > 0;
end;

function GetAliasPath(const Alias: String): String;
var
  AliasList: TStringList;
  I: Longint;
  DBPath: String;
begin
  Result := '';
  AliasList := TStringList.Create;
  try
    Session.GetAliasNames(AliasList);
    I := AliasList.IndexOf(Alias);
    if I < 0 then
      raise EDatabaseError.Create('Alias '+Alias+' doesnt exist')
    else begin
      Session.GetAliasParams(Alias,AliasList);
      DBPath := AliasList.Values['PATH'];
      if DBPath='' then
        raise EDatabaseError.Create('Alias path from ' + Alias + ' invalid')
      else
        Result := DBPath;
    end;
  finally
    AliasList.Free;
  end;
end;

procedure SplitTableName(const TableName: String; var Alias, Name: String);
var
  P1, P2: Integer;
begin
  Name := TableName;
  Alias := '';
  P1 := Pos(':', TableName);
  if P1 > 0 then begin
    P2:=Pos(':', Copy(TableName, P1+1, 255));
    if P2>0 then begin
      Alias := Copy(TableName, P1+1, P2-1);
      Name := Copy(TableName, P1+P2+1, 255);
    end;
  end;
end;

function GetDBTablePath(const TableName: String): String;
var
  Alias, Name: String;
begin
  if not HasAlias(TableName) then
    Result := TableName
  else begin
    SplitTableName(TableName, Alias,Name);
    if Alias <> '' then
      Result := GetAliasPath(Alias) + '\' + Name
    else
      Result := TableName;
  end;
end;

end.
