program rds;

{$R rds}

uses	OWindows,ODialogs,FileDlgs,WinTypes,WinProcs,Dialogs,
			GraphPalWin,GraphWin,GraphFileWin,SaveBmp,DIBitmap;

const Colour_Blue=$00FF0000;
			Colour_Black=$00000000;
			Colour_White=$00FFFFFF;
			id_Menu='rds_menu';
			cm_FileOpen=101;
			cm_ImageSettings=201;
			cm_ImageGenerate=202;
			cm_HelpContents=901;
			cm_HelpAbout=902;

type	RDSApplication=object(TApplication)
				procedure InitMainWindow; virtual;
				end;

type	PMainWindow=^MainWindow;
			MainWindow=object(TGraphicsPaletteWindow)
				constructor Init(AParent:PWindowsObject);
				destructor Done; virtual;
				procedure Generate(DestImage:PGraphicsWindow; Pattern:PDIBitmap);
				procedure CMImageSettings(var Msg:TMessage); virtual cm_First+cm_ImageSettings;
				procedure CMImageGenerate(var Msg:TMessage); virtual cm_First+cm_ImageGenerate;
				procedure CMFileOpen(var Msg:TMessage); virtual cm_First+cm_FileOpen;
				procedure CMHelpContents(var Msg:TMessage); virtual cm_First+cm_HelpContents;
				procedure CMHelpAbout(var Msg:TMessage); virtual cm_First+cm_HelpAbout;
				procedure PermitClosing; virtual;
				procedure DenyClosing; virtual;
			private
				GotSource:Boolean;
				end;

procedure RDSApplication.InitMainWindow;
begin
MainWindow:=new(PMainWindow, Init(nil));
end;

constructor MainWindow.Init;
var OpenBitmap:PDIBitmap;
begin
new(OpenBitmap,Create(250,250,1));
TGraphicsPaletteWindow.Init(AParent,'Repeating Dot Stereogram',OpenBitmap);
DestroyMenu(Attr.Menu);
Attr.Menu:=LoadMenu(HInstance,'rds_menu');
Attr.Style:=Attr.Style and (not ws_Popup);
GotSource:=false;
end;

destructor MainWindow.Done;
begin
TGraphicsPaletteWindow.Done;
end;

procedure MainWindow.CMHelpContents;
begin
WinHelp(HWindow,'rds.hlp',help_index,0);
end;

procedure MainWindow.CMHelpAbout;
begin
Application^.ExecDialog(new(PDialog, Init(@Self,'About')));
end;

procedure MainWindow.CMFileOpen;
var Name:array[0..250] of Char;
		Bitmap:PDIBitmap;
begin
if DoFileOpen(HWindow,Name,'Bitmap (*.bmp)'#0'*.bmp') then
	begin
	new(Bitmap,Load(Name));
	if Bitmap^.TheBitmap=0 then
		messagebox(HWindow,'Can not read Bitmap','Error',mb_OK)
	else
		begin
		TGraphicsPaletteWindow.ReInitialize(Bitmap);
		GotSource:=True;
		end;
	end;
end;

procedure MainWindow.PermitClosing;
begin
TGraphicsPaletteWindow.PermitClosing;
EnableMenuItem(Attr.Menu,cm_FileOpen,mf_ByCommand or mf_Enabled);
EnableMenuItem(Attr.Menu,cm_ImageGenerate,mf_ByCommand or mf_Enabled);
end;

procedure MainWindow.DenyClosing;
begin
TGraphicsPaletteWindow.DenyClosing;
EnableMenuItem(Attr.Menu,cm_FileOpen,mf_ByCommand or mf_Grayed);
EnableMenuItem(Attr.Menu,cm_ImageGenerate,mf_ByCommand or mf_Grayed);
end;

procedure Yield;
var Msg:TMsg;
begin
while PeekMessage(Msg,0,0,0,pm_Remove) do
	begin
	TranslateMessage(Msg);
	DispatchMessage(Msg);
	end;
end;

procedure MainWindow.CMImageSettings;
begin
Application^.ExecDialog(new(PDSettings, Init(@Self)));
end;

procedure MainWindow.CMImageGenerate;
var	DestBitmap:PSaveableBitmap;
		Pattern:PDIBitmap;
		DestImage:PGraphicsFileWindow;
		ColDepth:integer;
begin
if GotSource then
	begin
	if CurrentImageSettings.PatternOption=UsingFile then
		begin
		new(Pattern, Load(CurrentImageSettings.Filename));
		ColDepth:=Pattern^.ColourDepth;
		end
	else ColDepth:=1;
	new(DestBitmap, Create(Width,Height,ColDepth));
	new(DestImage, Init(@Self,'Image',DestBitmap));
	Application^.MakeWindow(DestImage);
	DenyClosing;
	DestImage^.DenyClosing;
	DestImage^.GetDC(MemoryDC);
	Generate(DestImage,Pattern);
	DestImage^.ReleaseDC;
	DestImage^.Redraw;
	DestImage^.PermitClosing;
	PermitClosing;
	if CurrentImageSettings.PatternOption=UsingFile then dispose(Pattern, done);
	end
else
	messagebox(HWindow,'Load Source image first','Error',mb_Ok);
end;

procedure MainWindow.Generate(DestImage:PGraphicsWindow; Pattern:PDIBitmap);

	function GeneratePixel(X,Y:longint):longint;
	var NewX,NewY:longint;
	begin
	if CurrentImageSettings.PatternOption=UsingStatic then
		begin
		if Random(100)>CurrentImageSettings.Brightness then
			GeneratePixel:=Colour_Black
		else GeneratePixel:=Colour_White;
		end
	else
		begin
		NewX:=(X mod CurrentImageSettings.Period) mod Pattern^.Width;
		NewY:=y mod Pattern^.Height;
		GeneratePixel:=Pattern^.GetPixelRGB(NewX,NewY);
		end;
	end;

	function dHeight(X,Y:integer):longint;
	var Pixcol:longint;
	begin
	PixCol:=DIContents^.GetPixelRGB(X,Y);
	with TRGBQuad(PixCol),CurrentImageSettings do
		dHeight:=round((rgbRed+rgbBlue+rgbGreen)*(MaxHeight-MinDepth) div 765)+MinDepth;
	end;

type SetInfo=array[0..MaxInt] of boolean;
		 ColourInfo=array[0..16000] of longint;
var Coord:TPoint;
		T:longint;
		XShift:integer;
		ImageWidth:longint;
		IsSet:^SetInfo;
		Colour:^ColourInfo;
begin
ImageWidth:=Width-1+CurrentImageSettings.Period;
DIContents^.Lock;
GetMem(IsSet ,(ImageWidth+1)*SizeOf(boolean));
GetMem(Colour,(ImageWidth+1)*SizeOf(longint));

for Coord.Y:=0 to Height-1 do
	begin
	for Coord.X:=0 to ImageWidth do IsSet^[Coord.X]:=false;
	for Coord.X:=0 to ImageWidth do
		begin
		if not IsSet^[Coord.X] then
			begin
			T:=GeneratePixel(Coord.X,Coord.Y);
			Colour^[Coord.X]:=T;
			end
		else
			T:=Colour^[Coord.X];
		XShift:=Coord.X+CurrentImageSettings.Period+dHeight(Coord.X,Coord.Y);
		if (XShift<ImageWidth) and (XShift>=0) then
			begin
			Colour^[XShift]:=T;
			IsSet^[XShift]:=true;
			end;
		end;
	for Coord.X:=0 to Width-1 do
		SetPixel(DestImage^.DC,Coord.X,Coord.Y,Colour^[Coord.X+CurrentImageSettings.Period]);
	if (Coord.Y mod 20)=0 then
		DestImage^.Redraw;
	Yield;
	end;

messagebeep(0);
DIContents^.Unlock;
FreeMem(IsSet, (ImageWidth+1)*SizeOf(boolean));
FreeMem(Colour,(ImageWidth+1)*SizeOf(longint));
end;


var MyApp:RDSApplication;
begin
MyApp.Init('RDS');
MyApp.Run;
MyApp.Done;
end.