unit ScrolWin;

{$R scrolwin.res}

interface

uses WinTypes,OWindows;

type	PScrollingWindow=^TScrollingWindow;
			TScrollingWindow=object(TWindow)
				Width,Height:integer;
				constructor Init(AParent:PWindowsObject;ATitle:PChar;W,H:integer);
				procedure SetupWindow; virtual;
				procedure WMGetMinMaxInfo(var Msg:TMessage); virtual wm_First+wm_GetMinMaxInfo;
				procedure WMSize(var Msg:TMessage); virtual wm_First+wm_Size;
				procedure WMWinIniChange(var Msg:TMessage); virtual wm_First+wm_WinIniChange;
        procedure GetWindowClass(var AWndClass:TWndClass); virtual;
        function GetClassName:PChar; virtual;
				procedure Resize(NewWidth,NewHeight:integer);
				procedure PermitClosing; virtual;
				procedure DenyClosing; virtual;
				function CanClose:Boolean; virtual;
			private
				FullSizeSet:boolean;
				FullW,FullH:integer;
				MinW,MinH,MaxW,MaxH:integer;
				AllowedToClose:Boolean;
				procedure CalculateSize;
				end;


implementation

uses WinProcs,Strings;

constructor TScrollingWindow.Init;
begin
TWindow.Init(AParent,ATitle);
Width:=W; Height:=H;
with Attr do
	begin
	W:=Width;
	Style:=Style or ws_Popup or ws_Visible or ws_OverlappedWindow or ws_HScroll or ws_VScroll;
	end;
Scroller:=new(PScroller, Init(@Self,1,1,1,1));
FullSizeSet:=false;
PermitClosing;
end;

procedure TScrollingWindow.SetupWindow;
begin
TWindow.SetupWindow;
CalculateSize;
end;

procedure TScrollingWindow.WMWinIniChange;
begin
if StrComp(PChar(Msg.lParam),'Windows')=0 then
	Resize(Width,Height);
end;

procedure TScrollingWindow.GetWindowClass;
begin
TWindow.GetWindowClass(AWndClass);
AWndClass.HIcon:=LoadIcon(HInstance,'scrolicon');
end;

function TScrollingWindow.GetClassName;
begin
GetClassName:='ScrollingWindow';
end;


procedure TScrollingWindow.Resize;
var Area:TRect;
begin
GetWindowRect(HWindow,Area);
Width:=NewWidth;
Height:=NewHeight;
CalculateSize;
end;

procedure TScrollingWindow.CalculateSize;
var Inside,Outside,Desk:TRect;
		OldW,OldH,NewW,NewH:integer;
begin
FullSizeSet:=False;
Scroller^.SetRange(0,0);
GetClientRect(GetDesktopWindow,Desk);
MaxW:=Desk.Right-Desk.Left;
MaxH:=Desk.Bottom-Desk.Top;

NewH:=0; NewW:=0;

repeat
	OldW:=NewW;
	GetClientRect(HWindow,Inside);
	GetWindowRect(HWindow,Outside);
	NewW:=Width+(Outside.Right-Outside.Left)-(Inside.Right-Inside.Left);
	if NewW<MinW then NewW:=MinW;
	if NewW>MaxW then NewW:=MaxW;

	repeat
		OldH:=NewH;
		GetClientRect(HWindow,Inside);
		GetWindowRect(HWindow,Outside);
		NewH:=Height+(Outside.Bottom-Outside.Top)-(Inside.Bottom-Inside.Top);
		if NewH<MinH then NewH:=MinH;
		if NewH>MaxH then NewH:=MaxH;

		MoveWindow(HWindow,Outside.left,Outside.Top,NewW,NewH,false);

	until OldH=NewH;
until OldW=NewW;

FullW:=Width+(Outside.Right-Outside.Left)-(Inside.Right-Inside.Left);
FullH:=Height+(Outside.Bottom-Outside.Top)-(Inside.Bottom-Inside.Top);
if FullW<MinW then FullW:=MinW;
if FullH<MinH then FullH:=MinH;
FullSizeSet:=true;
end;


procedure TScrollingWindow.WMGetMinMaxInfo;
type InfoArray=Array[0..4] of TPoint;
var Info:^InfoArray;
begin
Info:=Pointer(Msg.lParam);
if FullSizeSet then
	begin
	Info^[4].X:=FullW;
	Info^[4].Y:=FullH;
	end
else
	begin
	MinW:=Info^[3].X;
	MinH:=Info^[3].Y;
	end;
end;

procedure TScrollingWindow.WMSize;
var W,H:integer;
		Area:TRect;
		HasScroll:Record V,H:Boolean end;
begin
TWindow.WMSize(Msg);
W:=Msg.lParamlo;
H:=Msg.lParamHi;
HasScroll.H:=Scroller^.HasHScrollBar;
HasScroll.V:=Scroller^.HasVScrollBar;
if FullSizeSet and (Attr.W=FullW) and (Attr.H=FullH) then
	Scroller^.SetRange(0,0)
else
	Scroller^.SetRange(Width-W,Height-H);
end;

procedure TScrollingWindow.PermitClosing;
begin
AllowedToClose:=true;
end;

procedure TScrollingWindow.DenyClosing;
begin
AllowedToClose:=false;
end;

function TScrollingWindow.CanClose;
begin
TWindow.CanClose;
CanClose:=AllowedToClose;
end;

end.