{*******************************************************}
{                                                       }
{       Component's Hierarhy Browser                    }
{                                                       }
{       Builded with XDK (eXpert Development Kit)       }
{                                                       }
{       Copyright (c) 1996,97 Sergey Orlic              }
{       - product manager of Borland's Moscow Office    }
{                                                       }
{*******************************************************}
unit CHform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, Buttons, ToolWin,
  TypInfo, LibIntf, ExptIntf,
  XDKMain, XDKMenu, XDKNotif, StdCtrls, XDKGen, XDKSnips;

type
  TCHBrowser = class(TForm)
    Status: TStatusBar;
    CompTree: TTreeView;
    Splitter: TSplitter;
    PropTree: TTreeView;
    ToolBar: TToolBar;
    ToolBtnHelp: TToolButton;
    ToolBtnSave: TToolButton;
    ToolBtnFind: TToolButton;
    ImgList: TImageList;
    ImgList2: TImageList;
    FindComboBox: TComboBox;
    CHBMenu: TxdkAddInMenu;
    CHBMenuItem: TxdkAddInMenuItem;
    ToolBtnNewComp: TToolButton;
    SaveDialog: TSaveDialog;
    ToolBtnOnTop: TToolButton;
    ToolBtnSplit: TToolButton;
    ComponentCreator: TxdkModuleCreator;
    ComponentSnippet: TxdkSnippet;
    ToolBtnRefresh: TToolButton;
    procedure CHBMenuItemClick(Sender: TObject);
    procedure ChangePackageSet(Sender: TObject;
      const FileName: String);
    procedure CompTreeChange(Sender: TObject; Node: TTreeNode);
    procedure ToolBtnHelpClick(Sender: TObject);
    procedure FindClass;
    procedure ToolBtnFindClick(Sender: TObject);
    procedure FindComboBoxEnter(Sender: TObject);
    procedure FindComboBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FindComboBoxClick(Sender: TObject);
    procedure ToolBtnSaveClick(Sender: TObject);
    procedure ToolBtnNewCompClick(Sender: TObject);
    procedure ToolBtnOnTopClick(Sender: TObject);
    procedure ToolBtnSplitClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    function ComponentCreatorSourceWriting(UnitIdent, FormIdent,
      AncestorIdent: String): String;
    procedure ToolBtnRefreshClick(Sender: TObject);
  public
    procedure FillCompTree;
    procedure ClearCompTree;
    procedure RefreshCompTree;
    procedure ShowProps;
    procedure ShowStatus;
  end;

var
  CHBrowser: TCHBrowser;

implementation

{$R *.DFM}

const
  SOrdType   : array [0..4] of string =('ShortInt', 'Byte', 'Integer', 'Word', 'Longint');
  SFloatType : array [0..3] of string =('Single', 'Double', 'Extended', 'Comp');

//=========================================================
// TClassInfo

type
  TClassInfo = class
    Ins         : boolean;
    iRef        : TClass;
    iName       : string;
    iParentName : string;
    iModuleName : string;
    iUnit       : string;
    constructor Create;
  end;
  
constructor TClassInfo.Create;
begin
  Ins:=false;
end;

//=========================================================
// TCHBrowser

procedure TCHBrowser.CHBMenuItemClick(Sender: TObject);
begin
  if Visible then
    BringToFront
  else
    Show;
end;

procedure TCHBrowser.ChangePackageSet(Sender: TObject;
  const FileName: String);
begin
  // we need refresh components hierarhy
  // for case of installing/uninstalling components
  // from other packages
  if (not Application.Terminated) and (AnsiCompareText(ExtractFileName(FileName),'CHB.dpl')<>0) then
    RefreshCompTree;
end;

function GetTextNode(TV:TTreeView; Value:string):TTreeNode;
begin
  Result := TV.Items.GetFirstNode;
  while (Result <> nil) do
  begin
    if Result.Text=Value then Break;
    Result:=Result.GetNext;
  end;
end;

procedure TCHBrowser.FillCompTree;
var
  DraftList : TStringList; // temporeray list
  ic,                      // classes counter
  im        : integer;     // modules counter
  CRef      : TClass;
  CInfo     : TClassInfo;
  idx       : longint;
  Node      : TTreeNode;
  Cur  : TCursor;
begin
  Cur:=Screen.Cursor;
  Screen.Cursor:=crAppStart;

  DraftList:=TStringList.Create;

 {extract internal information about registered classes }

  for im:=0 to ToolServices.GetModuleCount-1 do
  begin
    for ic:=0 to ToolServices.GetComponentCount(im)-1 do
    begin
    //================================================================
    // following string of code demonstrate using of GetClass function
    // from standard VCL module Classes.pas
    // Please, note that list classes was filled by IDE
    // via RegisterComponents function, when packages had been loading
    //================================================================
      CRef:=TClass(GetClass(ToolServices.GetComponentName(im,ic)));
      while CRef<>nil do
      begin
        CInfo:=TClassInfo.Create;
        CInfo.iRef:=CRef;
        CInfo.iName:=CRef.ClassName;
        CInfo.iModuleName:=ToolServices.GetModuleName(im);
        CInfo.iUnit:=GetTypeData(PTypeInfo(CRef.ClassInfo))^.UnitName;
        if CInfo.iName<>'TComponent' then
          begin
            CRef:=CRef.ClassParent;
            if CRef<>nil then
              CInfo.iParentName:=CRef.ClassName
            else
              CInfo.iParentName:=EmptyStr;
          end
        else
          CRef:=nil;
        if DraftList.IndexOf(CInfo.iName)=-1 then  // check duplicates CInfo
          DraftList.AddObject(CInfo.iName,CInfo)
        else
          CInfo.Free;      // exclude duplicates CInfo with free it
      end;
    end;
  end;
  DraftList.Sorted:=true;

 // building hierarchy tree
  ic:=DraftList.IndexOf('TComponent');
  CompTree.Items.AddObjectFirst(nil,DraftList.Strings[ic],DraftList.Objects[ic]);
  DraftList.Delete(ic);

  while DraftList.Count>CompTree.Items.Count+1 do
  begin
    for ic:=0 to DraftList.Count-1 do
    begin
      if TClassInfo(DraftList.Objects[ic]).Ins then
        continue;
      Application.ProcessMessages;

      if GetTextNode(CompTree,DraftList.Strings[ic])=nil then
      begin
        Node:=GetTextNode(CompTree,TClassInfo(DraftList.Objects[ic]).iParentName);
        if Node<>nil then
        begin
          CompTree.Items.AddChildObject(Node,DraftList.Strings[ic],DraftList.Objects[ic]);
          TClassInfo(DraftList.Objects[ic]).Ins:=true;
        end;
      end;
    end;
  end;
  DraftList.Free;  // freeing temporary list

  CompTree.Items[0].Expand(true);
  CompTree.TopItem:=CompTree.Items[0];
  CompTree.Items[0].Selected:=true;
  CompTree.Items[0].Focused:=true;
  ShowProps;
  Screen.Cursor:=Cur;
end;

procedure TCHBrowser.ClearCompTree;
var
  i : integer;
  Cur : TCursor;
begin
  Cur:=Screen.Cursor;
  Screen.Cursor:=crHourGlass;
  for i:=0 to CompTree.Items.Count-1 do
    if CompTree.Items[i].Data<>nil then
      TClassInfo(CompTree.Items[i].Data).Free;
  CompTree.Items.Clear;
  Screen.Cursor:=Cur;
end;

procedure TCHBrowser.RefreshCompTree;
begin
  CompTree.Items.BeginUpdate;
  CHBrowser.CompTree.OnChange:=nil;
  ClearCompTree;
  FillCompTree;
  CHBrowser.CompTree.OnChange:=CompTreeChange;
  CompTree.Items.EndUpdate;
end;

procedure TCHBrowser.ShowProps;
var
  TypeInf  : PTypeInfo;
  TypeData : PTypeData;
  i        : Integer;
  aName    : string;
  PropList : pPropList;
  NumProps : Word;
  FirstNode: TTreeNode;

  Procedure AddToOutline(ATypeInfo:PTypeInfo;Node:TTreeNode);
  // Recursive routine
  var
    i         : integer;
    aPropList : pPropList;
    aNumProps : Word;
    TmpNode    : TTreeNode;
  begin
    TypeData := GetTypeData(aTypeInfo);
    With ATypeInfo^ do
      Case Kind of
        tkUnknown       :TmpNode := PropTree.Items.AddChild(Node,'UNKNOWN - '+aName);
        tkInteger       :TmpNode := PropTree.Items.AddChild(Node,
                                                      aName+' : '+
                                                        SOrdType[Integer(TypeData^.OrdType)]);
        tkChar          :TmpNode := PropTree.Items.AddChild(Node,
                                                      aName+' : '+
                                                      System.Copy(GetEnumName(TypeInfo(TTypeKind),
                                                      Integer(Kind)),
                                                           3,255)
                                                      );
        tkEnumeration   :TmpNode := PropTree.Items.AddChild(Node,
                                                      aName+' : '+aTypeInfo^.Name);
        tkFloat         :TmpNode := PropTree.Items.AddChild(Node,
                                                      aName+' : '+
                                                      SFloatType[Integer(TypeData^.FloatType)]);
        tkString        :TmpNode := PropTree.Items.AddChild(Node,
                                                      aName+' : '+aTypeInfo^.Name+
                                                      ' ( '+
                                                      Format('String[%d]',[TypeData^.MaxLength])
                                                      +' )');
        tkSet           :TmpNode := PropTree.Items.AddChild(Node,
                                                      aName+' : '+aTypeInfo^.Name);
        tkClass         : begin
                            TmpNode := PropTree.Items.AddChild(Node,aName+' : '+
                              TypeData^.ClassType.ClassName+' ( Unit : '+TypeData^.UnitName+' )');
                            aNumProps := TypeData^.PropCount;
                            GetMem(aPropList,aNumProps*sizeof(pointer));
                            try
                              GetPropInfos(ATypeInfo,aPropList);
                              For I := 0 to aNumProps-1 do
                              begin
                                AName := aPropList^[I]^.Name;
                                AddToOutline(aPropList^[I]^.PropType^,TmpNode);
                              end;
                            finally
                              FreeMem(aPropList,aNumProps*sizeof(pointer));
                            end;
                          end;
        tkMethod        :TmpNode := PropTree.Items.AddChild(Node,
                                                      AName+' : '+ Name + ' ( ' +
                                                      System.Copy(GetEnumName(TypeInfo(TMethodKind),
                                                        Integer(TypeData^.MethodKind))
                                                        +' )',3,255)
                                                      );
      end;
  end;

begin
  if CompTree.Selected=nil then
    Exit;
  PropTree.Items.Clear;
  try
    TypeInf :=GetClass(CompTree.Selected.Text).ClassInfo;
    AName := TypeInf^.Name;
    TypeData := GetTypeData(TypeInf);
    NumProps := TypeData^.PropCount;
    GetMem(PropList,NumProps*sizeof(pointer));

  // adding class name as root node to ClassTree
    FirstNode:=PropTree.Items.AddChild(nil,CompTree.Selected.Text);
    try
      GetPropInfos(TypeInf,PropList);
      For I := 0 to NumProps-1 do
      begin
        AName := PropList^[I]^.Name;
        AddToOutline(PropList^[I]^.PropType^,FirstNode);
      end;
    finally
      FreeMem(PropList,NumProps*sizeof(pointer));
    end;
  except
    on E:Exception do
      MessageDlg('Internal Error for '+AName+' : '+E.Message, mtError,[mbOk],0);
  end;
// with 'false' as parameter for non-recursive expanding of tree items
  PropTree.Items.GetFirstNode.Expand(false);
end;

procedure TCHBrowser.ShowStatus;
var
  CInfo : TClassInfo;
begin
  CInfo:=TClassInfo(CompTree.Selected.Data);
  if CInfo<>nil then
    begin
      if CInfo.iParentName<>EmptyStr then
        Status.Panels[0].Text:='Ancestor: '+CInfo.iParentName
      else
        Status.Panels[0].Text:=EmptyStr;
      Status.Panels[1].Text:='Package: '+TClassInfo(CompTree.Selected.Data).iModuleName;
      Status.Panels[2].Text:='Unit: '+TClassInfo(CompTree.Selected.Data).iUnit;
    end
  else
    begin
      Status.Panels[0].Text:=EmptyStr;
      Status.Panels[1].Text:=EmptyStr;
      Status.Panels[2].Text:=EmptyStr;
    end
end;

procedure TCHBrowser.CompTreeChange(Sender: TObject; Node: TTreeNode);
begin
  ShowStatus;
  ShowProps;
end;

procedure TCHBrowser.ToolBtnHelpClick(Sender: TObject);
begin
  if CompTree.Focused then
    DelphiIDE{AppBuilder}.ShowClassHelp(CompTree.Selected.Text)
  else
    DelphiIDE{AppBuilder}.ShowClassHelp(
      System.Copy(PropTree.Selected.Text,1,System.Pos(' :',PropTree.Selected.Text)-1)
      +','+CompTree.Selected.Text);
end;

procedure TCHBrowser.FindClass;
var
  Node : TTreeNode;
begin
  if FindComboBox.Text=EmptyStr then Exit;
  Node:=CompTree.Items.GetFirstNode;
  while (Node<>nil) do
      if AnsiCompareText(Node.Text,FindComboBox.Text)=0 then
      begin
        CompTree.TopItem:=Node;
        Node.Selected:=true;
        Node.Focused:=true;
        if FindComboBox.Items.IndexOf(FindComboBox.Text)=-1 then
          FindComboBox.Items.Insert(0,FindComboBox.Text);
        Break;
      end
    else
      Node:=Node.GetNext;
end;

procedure TCHBrowser.ToolBtnFindClick(Sender: TObject);
begin
  FindClass;
end;

procedure TCHBrowser.FindComboBoxEnter(Sender: TObject);
begin
  FindClass;
end;

procedure TCHBrowser.FindComboBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_Return then
    FindClass;
end;

procedure TCHBrowser.FindComboBoxClick(Sender: TObject);
begin
  FindClass;
end;

procedure TCHBrowser.ToolBtnSaveClick(Sender: TObject);
begin
  if SaveDialog.Execute then
    CompTree.SaveToFile(SaveDialog.FileName);
end;

procedure TCHBrowser.ToolBtnNewCompClick(Sender: TObject);
var
  CName: string;
begin
  if InputQuery('New component','Component name :',CName) then
    if (CName<>EmptyStr) and (CName[1] in ['A'..'Z','a'..'z','_']) then
    begin
      ComponentCreator.AncestorName:=CName;
      ComponentCreator.ModuleCreate;        
    end;
end;

procedure TCHBrowser.ToolBtnOnTopClick(Sender: TObject);
begin
  if TToolButton(Sender).Down then
    FormStyle:=fsStayOnTop
  else
    FormStyle:=fsNormal;
  CompTree.Items[0].Expand(true);
  CompTree.TopItem:=CompTree.Items[0];
  PropTree.Items[0].Expand(false);
end;

procedure TCHBrowser.ToolBtnSplitClick(Sender: TObject);
begin
  if TToolButton(Sender).ImageIndex=6 then
    begin
      TToolButton(Sender).ImageIndex:=5;
      CompTree.Align:=alTop;
      CompTree.Height:=Height div 2;
      Splitter.Align:=alTop;
      Splitter.Cursor:=crVSplit;
    end
  else
    begin
      TToolButton(Sender).ImageIndex:=6;
      Splitter.Align:=alLeft;
      Splitter.Cursor:=crHSplit;
      CompTree.Align:=alLeft;
      CompTree.Width:=(Width*2) div 3
    end;
end;

procedure TCHBrowser.FormResize(Sender: TObject);
begin
  if ToolBtnSplit.ImageIndex=6 then
    CompTree.Width:=(Width*2) div 3
  else
    CompTree.Height:=Height div 2
end;

function TCHBrowser.ComponentCreatorSourceWriting(UnitIdent, FormIdent,
  AncestorIdent: String): String;
var
  CInfo : TClassInfo;
begin
  CInfo:=TClassInfo(CompTree.Selected.Data);
  ComponentSnippet.ParamValues['UnitName']:=UnitIdent;
  if (CInfo.iUnit='Graphics') or (CInfo.iUnit='Controls')
    or (CInfo.iUnit='Forms') or (CInfo.iUnit='Dialogs')
    or ((CInfo.iUnit='Classes'))
  then
    ComponentSnippet.ParamValues['ToUses']:=' '
  else
    ComponentSnippet.ParamValues['ToUses']:=', '+CInfo.iUnit;
  ComponentSnippet.ParamValues['AncestorName']:=CInfo.iName;
  ComponentSnippet.ParamValues['ClassName']:=AncestorIdent;
  ComponentSnippet.DoResultCode;
  Result:=ComponentSnippet.ResultCodeText;
end;

procedure TCHBrowser.ToolBtnRefreshClick(Sender: TObject);
begin
  RefreshCompTree;
end;

initialization
  xdkInitModule(TCHBrowser,CHBrowser);
  CHBrowser.FillCompTree;
  CHBrowser.Show;
finalization
  if Assigned(CHBrowser) then
  begin
    CHBrowser.CompTree.OnChange:=nil;
    CHBrowser.ClearCompTree;
  end;
  xdkDoneModule(CHBrowser);
end.
