unit Sql11;

{ ****************************************************************************
  Example of using direct api to access MS MSQL server 6.0
  (This should also work with 6.5)
  Ed Lyk 1997-08-27

  All imported functions are in MSSQL16.PAS
  ****************************************************************************}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, mssql16;

type
  TForm1 = class(TForm)
    BTN_Connect: TButton;
    BTN_Query: TButton;
    ListBox1: TListBox;
    Label1: TLabel;
    BTN_Stored: TButton;
    BTN_Proc2: TButton;
    procedure BTN_ConnectClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BTN_QueryClick(Sender: TObject);
    procedure BTN_StoredClick(Sender: TObject);
    procedure BTN_Proc2Click(Sender: TObject);
  private
    { Private declarations }
    login: LOGINREC;  {near pointer to login record}
  public
    { Public declarations }
  end;


var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.BTN_ConnectClick(Sender: TObject);
var
  szVer: PChar;
  dbproc:  DBPROCESS;
  ret: Integer;
  MyKey: ^Integer;
begin

  {first initialize the db dlls.  returns null string if error}
  szVer := dbinit;
  MessageBox(Handle, szVer, 'Opened', mb_ok);

  {create a login record}
  login := dblogin;

  {set the user id}
  ret := dbsetlname(login, 'sa', DBSETUSER);
  if ret = DBFAIL then
    begin
    MessageDlg('set user failed', mtError, [mbOk], 0);
    dbfreelogin(login);
    Exit;
    end;

  {set the password}
  ret := dbsetlname(login, 'xyzzy', DBSETPWD);
  if ret = DBFAIL then
    begin
    MessageDlg('set password failed', mtError, [mbOk], 0);
    dbfreelogin(login);
    Exit;
    end;

  {create db process, connection to server}
  dbproc := dbopen(login, 'DEV-SERVER');

  if dbproc <> 0 then
    begin
    {set sql statement}
    dbcmd(dbproc, 'SELECT * FROM DIVBRAZ.DBO.KEYID');
    {execute the sql}
    dbsqlexec(dbproc);
    {see if we have results}
    if dbresults(dbproc) = DBSUCCEED then
      begin
      {get each row of result set}
      while dbnextrow(dbproc) = REG_ROW do
        begin
        {set data from row 1..  note: couldn't get binding to work in 16 bit windows}
        MyKey := dbdata(dbproc, 1);
        MessageDlg('Keyid from table is: ' + IntToStr(MyKey^), mtInformation, [mbOk], 0);
        end;

      end;
    {close the connection, frees memory and resources...}
    dbclose(dbproc);
    end;

  BTN_Query.Enabled := True;
  BTN_Stored.Enabled := True;
  BTN_Proc2.Enabled := True;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  {free up the dll stuff, important...}
  dbexit;
end;

procedure TForm1.BTN_QueryClick(Sender: TObject);
var
  sTemp: String;
  szName, szBaseNo: PChar;
  dbproc:  DBPROCESS;
  ret: Integer;
  t: TDateTime;
  wHour, wMinute, wSec, wMSec: Word;
begin

  ListBox1.Items.Clear;

  {create connection to server}
  dbproc := dbopen(login, 'DEV-SERVER');

  if dbproc <> 0 then
    begin

    t := Time;

    {set sql statement}
    dbcmd(dbproc, 'SELECT CLIENT_NAME, CLIENT_BASE_NUMBER FROM DIVBRAZ.DBO.CLIENTS');
    {execute sql}
    dbsqlexec(dbproc);
    {check for results}
    if dbresults(dbproc) = DBSUCCEED then
      begin

      szName := StrAlloc(255);
      szBaseNo := StrAlloc(255);

      {get each row of the result set}
      while dbnextrow(dbproc) = REG_ROW do
        begin
        {get data from each column...
         note: strings are not null terminated, thus you must use
               StrLCopy and use dbdatlen to get the actual length
               of the string}
        StrLCopy(szName, dbdata(dbproc, 1), dbdatlen(dbproc,1));
        StrLCopy(szBaseNo, dbdata(dbproc, 2), dbdatlen(dbproc, 2));
        sTemp := strPas(szBaseNo) + ' : ' + strPas(szName);
        ListBox1.Items.Add(sTemp);
        end;

      StrDispose(szName);
      StrDispose(szBaseNo);

      end;

    {close the connection}
    dbclose(dbproc);

    t := Time - t;

    end;

  DecodeTime(t, wHour, wMinute, wSec, wMSec);
  Label1.Caption := 'Rows loaded: ' + IntToStr(ListBox1.Items.Count) + '  Time: ' +
  Format('%.2d:%.2d:%.2d.%d', [wHour, wMinute, wSec, wMSec]);


end;

procedure TForm1.BTN_StoredClick(Sender: TObject);
var
  ret: Integer;
  dbproc:  DBPROCESS;
  MyKey: ^Longint;
  RealKey: Longint;
begin

  {note please: I never did get this routine to work quite right,
                that is why the routine under this one exists...
                If anyone can get stored procedures returning an
                output variable to work, please e-mail me your code... Ed}

  MyKey := Addr(RealKey);
  RealKey := 12;

  {make connection to server}
  dbproc := dbopen(login, 'DEV-SERVER');

  if dbproc <> 0 then
    begin

    {use DIVBRAZ for default database, otherwise have to prefix table and procedures
     with full location: i.e.  DIVBRAZ.DBO.CLIENTS...}
    if dbuse(dbproc, 'DIVBRAZ') = DBFAIL then
      begin
      MessageDlg('couldn''t use DIVBRAZ', mtError, [mbOk], 0);
      dbclose(dbproc);
      Exit;
      end;

    {initialize the stored procedure}
    ret := dbrpcinit(dbproc, 'MAKEKEYID', 0);
    if ret = DBSUCCEED then
      begin
      {create parameter for sp}
      ret := dbrpcparam(dbproc, '@NEWKEY', DBRPCRETURN, SQLINTN, -1, 0, nil);
      if ret = DBSUCCEED then
        begin
        {execute stored procedure}
        ret := dbrpcsend(dbproc);
        if ret = DBSUCCEED then
          begin
          {wait for sp to end}
          ret := dbsqlok(dbproc);
          if ret = DBSUCCEED then
            begin
            {get output parameter data}
            MyKey := dbretdata(dbproc, 1);
            MessageDlg('Keyid from stored proc: ' + IntToStr(Mykey^), mtInformation, [mbOk], 0);
            end
          else
            MessageDlg('sql not ok.', mtError, [mbOk], 0);
          end
        else
          MessageDlg('Error sending stored proc.', mtError, [mbOk], 0);
        end
      else
        MessageDlg('error setting param', mtError, [mbOk], 0);
      end
    else
      MessageDlg('error init stored proc', mtError, [mbOk], 0);
    {close connection}
    dbclose(dbproc);
    end;

    {defination of procedure I'm trying to run:

     CREATE PROCEDURE MAKEKEYID(@NEWKEY INT OUTPUT) AS
     BEGIN
       BEGIN TRAN

       SELECT @NEWKEY = KEYID FROM DBO.KEYID TABLOCKX

       UPDATE DBO.KEYID
       SET KEYID = (@NEWKEY + 1)

       COMMIT TRAN
     END

    }
end;

procedure TForm1.BTN_Proc2Click(Sender: TObject);
var
  NewKey: ^Longint;
  dbproc:  DBPROCESS;
begin

  ListBox1.Items.Clear;

  {this is an example of calling a stored procedure with an output
   variable without using the stored procedure routines....}

  {create the connection}
  dbproc := dbopen(login, 'DEV-SERVER');

  if dbproc <> 0 then
    begin

    {use DIVBRAZ database}
    dbuse(dbproc, 'DIVBRAZ');

    {set sql statement.  note the use of multiple calls to dbcmd,
     this will be simply concatenated together, so leave a blank
     space at beginning or end of line to prevent errors...

     also to the tricky creating of a variable and doing a select
     on it to cause a result set to read it out...}
    dbcmd(dbproc, 'declare @newkey int');
    dbcmd(dbproc, ' execute dbo.makekeyid @newkey output');
    dbcmd(dbproc, ' select "out" = @newkey');
    {execute the sql}
    dbsqlexec(dbproc);
    {check for results}
    if dbresults(dbproc) = DBSUCCEED then
      begin

      {get each row, even though there should only be one}
      while dbnextrow(dbproc) = REG_ROW do
        begin
        {get data from column 1}
        NewKey := dbdata(dbproc, 1);
        MessageDlg('Next Key from Stored Proc: ' + IntToStr(NewKey^), mtInformation, [mbOk], 0);
        end;

      end;

    {close the connection}
    dbclose(dbproc);

    end;

end;

{I hope you've enjoyed this...  now program something... }

end.
