{ File: Unit1.Pas                                                    }
{ Description: Test ODBC Class Library                               }
{ Author: Pieter A. Myburgh                                          }
{ Updated: 01/10/96                                                  }
{                                                                    }
{ Notes:                                                             }
{                                                                    }
{ 1.  This testing program demonstrates general ODBCExpress          }
{     functionality.                                                 }
{ 2.  See README.TXT for information on which ODBC data sources to   }
{     set up for this testing program.                               }
{ 3.  This program was run against MS SQLServer and MS Access.       }

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtGrid, OCIH, OCI, ODI, OCL;

type
  TForm1 = class(TForm)
    ListBox1: TListBox;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    procedure BitBtn1Click(Sender: TObject);
  private
    { Private declarations }
    Core: Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
{$R GENDER.RES}

uses
  OETree, Unit2, Unit3, Unit4, Unit5;

procedure TForm1.BitBtn1Click(Sender: TObject);
var
  ListCount: Integer;

{ Utilities }

procedure AddList(s: String);
begin
  { Add an Item to the ListBox }
  Form1.ListBox1.Items.Add('  '+s);
  Form1.ListBox1.ItemIndex:= ListCount;
  Inc(ListCount);
  Form1.ListBox1.Refresh;
end;

procedure UpdateList;
var
  S: String;
begin
  { Update an Item in the ListBox }
  S:= Form1.ListBox1.Items.Strings[Form1.ListBox1.ItemIndex];
  S[1]:= Chr(251);
  Form1.ListBox1.Items.Strings[Form1.ListBox1.ItemIndex]:= S;
  Form1.ListBox1.Refresh;
end;

{ Test Functions }

procedure TestInsert;
var
  { Students Table }
  SNo: NullString;
  SName: NullString;
  SGender: Byte;

  { Courses Table }
  CCode: NullString;
  CDescr: NullString;
  CNumStuds: Smallint;

  { Takes Table }
  TMark: Single;
begin
  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Students (SNo, SName, SGender) VALUES (?, ?, ?)';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  if Core then
  begin
    Tree.Hstmt1.BindParamCore(1, SQL_C_CHAR, @SNo, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(2, SQL_C_CHAR, @SName, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(3, SQL_C_BIT, @SGender, SQL_BIT);
  end
  else
  begin
    Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
    Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @SName);
    Tree.Hstmt1.BindParam(3, SQL_C_BIT, @SGender);
  end;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700001';
  SName:= 'Van der Merwe, J';
  SGender:= 1;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700002';
  SName:= 'Schwarzenegger, A';
  SGender:= 1;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700003';
  SName:= 'Hamilton, L';
  SGender:= 0;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Courses (CCode, CDescr, CNumStuds) VALUES (?, ?, ?)';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  if Core then
  begin
    Tree.Hstmt1.BindParamCore(1, SQL_C_CHAR, @CCode, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(2, SQL_C_CHAR, @CDescr, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(3, SQL_C_SSHORT, @CNumStuds, SQL_SMALLINT);
  end
  else
  begin
    Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @CCode);
    Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @CDescr);
    Tree.Hstmt1.BindParam(3, SQL_C_SSHORT, @CNumStuds);
  end;
  UpdateList;

  AddList('Assigning Parameters...');
  CCode:= 'WR1';
  CDescr:= 'Computer Science 1';
  CNumStuds:= 80;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Assigning Parameters...');
  CCode:= 'WR2';
  CDescr:= 'Computer Science 2';
  CNumStuds:= 40;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Assigning Parameters...');
  CCode:= 'WR3';
  CDescr:= 'Computer Science 3';
  CNumStuds:= 20;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Takes (SNo, CCode, TMark) VALUES (?, ?, ?)';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  if Core then
  begin
    Tree.Hstmt1.BindParamCore(1, SQL_C_CHAR, @SNo, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(2, SQL_C_CHAR, @CCode, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(3, SQL_C_FLOAT, @TMark, SQL_REAL);
  end
  else
  begin
    Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
    Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @CCode);
    Tree.Hstmt1.BindParam(3, SQL_C_FLOAT, @TMark);
  end;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700001';
  CCode:= 'WR1';
  TMark:= 80.6;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700002';
  CCode:= 'WR2';
  TMark:= 70.7;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700003';
  CCode:= 'WR3';
  TMark:= 60.8;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;
end;

procedure TestInsert2;
var
  BytesRead: Longint;

  SNo: NullString;
  SName: NullString;
  SGender: Byte;
  SBirth: TTimeStamp;
  SPic: TMemoryStream;
  SComment: TMemoryStream;
begin
  SPic:= TMemoryStream.Create;
  SComment:= TMemoryStream.Create;

  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Students (SNo, SName, SGender, SBirth, SPic) '+
              'VALUES (?, ?, ?, ?, ?)';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
  Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @SName);
  Tree.Hstmt1.BindParam(3, SQL_C_BIT, @SGender);
  Tree.Hstmt1.BindParam(4, SQL_C_TIMESTAMP, @SBirth);
  SPic.LoadFromFile('WEAVER.BMP');
  Tree.Hstmt1.BindParam(5, SQL_C_BINARY, SPic.Memory);
  Tree.Hstmt1.ParamSize[5]:= SPic.Size;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700004';
  SName:= 'Weaver, S';
  SGender:= 0;
  SBirth:= TimeStamp(1961,2,3,0,0,0,0);
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Students (SNo, SName, SGender, SBirth, SPic) '+
              'VALUES (?, ?, ?, ?, ?)';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
  Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @SName);
  Tree.Hstmt1.BindParam(3, SQL_C_BIT, @SGender);
  Tree.Hstmt1.BindParam(4, SQL_C_TIMESTAMP, @SBirth);
  SPic.LoadFromFile('GIBSON.BMP');
  Tree.Hstmt1.BindParam(5, SQL_C_BINARY, SPic.Memory);
  Tree.Hstmt1.ParamSize[5]:= SPic.Size;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700005';
  SName:= 'Gibson, M';
  SGender:= 1;
  SBirth:= TimeStamp(1962,3,4,0,0,0,0);
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Students (SNo, SName, SGender, SBirth, SPic, SComment) '+
              'VALUES (?, ?, ?, ?, ?, ?)';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
  Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @SName);
  Tree.Hstmt1.BindParam(3, SQL_C_BIT, @SGender);
  Tree.Hstmt1.BindParam(4, SQL_C_TIMESTAMP, @SBirth);
  SPic.LoadFromFile('TRAVOLTA.BMP');
  Tree.Hstmt1.BindParam(5, SQL_C_BINARY, SPic.Memory);
  Tree.Hstmt1.ParamSize[5]:= SPic.Size;
  SComment.LoadFromFile('README.TXT');
  Tree.Hstmt1.BindParam(6, SQL_C_CHAR, SComment.Memory);
  Tree.Hstmt1.ParamSize[6]:= SComment.Size;
  UpdateList;

  AddList('Assigning Parameters...');
  SNo:= '9700006';
  SName:= 'Travolta, J';
  SGender:= 1;
  SBirth:= TimeStamp(1963,4,5,0,0,0,0);
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  SPic.Free;
  SComment.Free;
end;

procedure TestSelect;
var
  SNo: NullString;
begin
  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'SELECT * FROM Students WHERE SNo = ?';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameter...');
  Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
  UpdateList;

  AddList('Assigning Parameter...');
  SNo:= '9700004';
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  if Tree.Hstmt1.FetchNext then
  begin
    Tree.Hstmt1.ColMemory(5).SaveToFile('WEAVER.OUT');
    AddList('Student''s Picture Saved: '+Tree.Hstmt1.ColString(2));
  end;

  AddList('Assigning Parameter...');
  SNo:= '9700005';
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  if Tree.Hstmt1.FetchNext then
  begin
    Tree.Hstmt1.ColMemory(5).SaveToFile('GIBSON.OUT');
    AddList('Student''s Picture Saved: '+Tree.Hstmt1.ColString(2));
  end;

  AddList('Assigning Parameter...');
  SNo:= '9700006';
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  if Tree.Hstmt1.FetchNext then
  begin
    Tree.Hstmt1.ColMemory(6).SaveToFile('TRAVOLTA.OUT');
    AddList('Student''s Comment Saved: '+Tree.Hstmt1.ColString(2));
  end;
end;

procedure TestDelete(SNo: String);
begin
  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'DELETE FROM Students WHERE SNo = ?';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameter...');
  if Core then
    Tree.Hstmt1.BindParamCore(1, SQL_C_CHAR, PChar(SNo), SQL_CHAR)
  else
    Tree.Hstmt1.BindParam(1, SQL_C_CHAR, PChar(SNo));
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;
end;

procedure TestInsertBulk(NumRows: Longint);
var
  i: Longint;

  SNo: NullString;
  SName: NullString;
  SGender: Byte;
begin
  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Students (SNo, SName, SGender) VALUES (?, ?, ?)';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  if Core then
  begin
    Tree.Hstmt1.BindParamCore(1, SQL_C_CHAR, @SNo, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(2, SQL_C_CHAR, @SName, SQL_CHAR);
    Tree.Hstmt1.BindParamCore(3, SQL_C_BIT, @SGender, SQL_CHAR);
  end
  else
  begin
    Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
    Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @SName);
    Tree.Hstmt1.BindParam(3, SQL_C_BIT, @SGender);
  end;
  UpdateList;

  for i:= 1 to NumRows do
  begin
    StrPCopy(SNo, '98'+LeadingZeros(i, 4));
    StrPCopy(SName, 'Surname, '+LeadingZeros(i, 4));
    SGender:= i mod 2;

    AddList('Executing Statement #'+LeadingZeros(i, 4)+'...');
    Tree.Hstmt1.Execute;
    UpdateList;
  end;
end;

procedure TestPopulate;
begin
  Tree.Hstmt1.PopulateOn(Form2, False);

  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'SELECT SNo, SName, SGender, SBirth FROM Students';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Test Populate...');
  Form2.ShowModal;
  UpdateList;

  Tree.Hstmt1.PopulateOff;
end;

procedure TestListBox;
begin
  AddList('Populating ListBox...');
  Form3.ShowModal;
  UpdateList;
end;

procedure TestGrid;
begin
  AddList('Populating Grid...');
  Form4.ShowModal;
  UpdateList;
end;

procedure TestBlobs;
var
  SNo: NullString;
begin
  Tree.Hstmt1.PopulateOn(Form5, False);

  AddList('Preparing Statement...');
  Tree.Hstmt1.SQL:= 'SELECT SNo, SName, SGender, SPic, SComment FROM Students WHERE SNo = ?';
  Tree.Hstmt1.Prepare;
  UpdateList;

  AddList('Binding Parameter...');
  SNo:= '9700006';
  Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo);
  UpdateList;

  AddList('Executing Statement...');
  Tree.Hstmt1.Execute;
  UpdateList;

  AddList('Fetching Result...');
  Tree.Hstmt1.FetchNext;
  UpdateList;

  AddList('Test Memo and Image...');
  Form5.ShowModal;
  UpdateList;

  Tree.Hstmt1.PopulateOff;
end;

procedure TestTransactions;
var
  SNo1, SNo2: NullString;
  SName1, SName2: NullString;
  SGender1, SGender2: Byte;

procedure PrintOptions(var Hdbc: THdbc;
                       var Hstmt: THstmt);
begin
  { Isolation Level }
  AddList('  Isolation Level:');
  case Hdbc.IsolationLevel of
    SQL_TXN_READ_UNCOMMITTED:
      AddList('    Read Uncommitted');
    SQL_TXN_READ_COMMITTED:
      AddList('    Read Committed');
    SQL_TXN_REPEATABLE_READ:
      AddList('    Repeatable Read');
    SQL_TXN_SERIALIZABLE:
      AddList('    Serializable');
    SQL_TXN_VERSIONING:
      AddList('    Versioning');
  end;

  { Concurrency Type }
  AddList('  Concurrency Type:');
  case Hstmt.ConcurrencyType of
    SQL_CONCUR_READ_ONLY:
      AddList('    Read Only');
    SQL_CONCUR_LOCK:
      AddList('    Lock');
    SQL_CONCUR_ROWVER:
      AddList('    RowVer');
    SQL_CONCUR_VALUES:
      AddList('    Values');
  end;

  { Cursor Type }
  AddList('  Cursor Type:');
  case Hstmt.CursorType of
    SQL_CURSOR_FORWARD_ONLY:
      AddList('    Forward Only');
    SQL_CURSOR_STATIC:
      AddList('    Static');
    SQL_CURSOR_KEYSET_DRIVEN:
      AddList('    Keyset Driven');
    SQL_CURSOR_DYNAMIC:
      AddList('    Dynamic');
  end;
end;

begin
  Tree.Hdbc1.Terminate;
  Tree.Hdbc2.Terminate;

  Tree.Hstmt1.ConcurrencyType:= SQL_CONCUR_ROWVER;
  Tree.Hstmt1.CursorType:= SQL_CURSOR_KEYSET_DRIVEN;

  Tree.Hstmt2.ConcurrencyType:= SQL_CONCUR_ROWVER;
  Tree.Hstmt2.CursorType:= SQL_CURSOR_KEYSET_DRIVEN;

  AddList('Options Hdbc1, Hstmt1:');
  PrintOptions(Tree.Hdbc1, Tree.Hstmt1);

  AddList('Options Hdbc2, Hstmt2:');
  PrintOptions(Tree.Hdbc2, Tree.Hstmt2);

  { *** Start Transactions *** }

  AddList('Starting Transactions...');
  Tree.Hdbc1.StartTransact;
  Tree.Hdbc2.StartTransact;
  UpdateList;

  AddList('Preparing Statements...');
  Tree.Hstmt1.SQL:= 'INSERT INTO Students (SNo, SName, SGender) VALUES (?, ?, ?)';
  Tree.Hstmt1.Prepare;

  Tree.Hstmt2.SQL:= 'INSERT INTO Students (SNo, SName, SGender) VALUES (?, ?, ?)';
  Tree.Hstmt2.Prepare;
  UpdateList;

  AddList('Binding Parameters...');
  if Core then
  begin
    SNo1:= '9700011';
    Tree.Hstmt1.BindParamCore(1, SQL_C_CHAR, @SNo1, SQL_CHAR);
    SName1:= 'Name1';
    Tree.Hstmt1.BindParamCore(2, SQL_C_CHAR, @SName1, SQL_CHAR);
    SGender1:= 0;
    Tree.Hstmt1.BindParamCore(3, SQL_C_BIT, @SGender1, SQL_BIT);

    SNo2:= '9700012';
    Tree.Hstmt2.BindParamCore(1, SQL_C_CHAR, @SNo2, SQL_CHAR);
    SName2:= 'Name2';
    Tree.Hstmt2.BindParamCore(2, SQL_C_CHAR, @SName2, SQL_CHAR);
    SGender2:= 0;
    Tree.Hstmt2.BindParamCore(3, SQL_C_BIT, @SGender2, SQL_BIT);
  end
  else
  begin
    SNo1:= '9700011';
    Tree.Hstmt1.BindParam(1, SQL_C_CHAR, @SNo1);
    SName1:= 'Name1';
    Tree.Hstmt1.BindParam(2, SQL_C_CHAR, @SName1);
    SGender1:= 0;
    Tree.Hstmt1.BindParam(3, SQL_C_BIT, @SGender1);

    SNo2:= '9700012';
    Tree.Hstmt2.BindParam(1, SQL_C_CHAR, @SNo2);
    SName2:= 'Name2';
    Tree.Hstmt2.BindParam(2, SQL_C_CHAR, @SName2);
    SGender2:= 0;
    Tree.Hstmt2.BindParam(3, SQL_C_BIT, @SGender2);
  end;
  UpdateList;

  AddList('Executing Statement 1...');
  Tree.Hstmt1.Execute;
  Tree.Hdbc1.Commit;  //if this line is commented out, the app will hang on the next execute
  UpdateList;

  AddList('Executing Statement 2...');
  Tree.Hstmt2.Execute;
  Tree.Hdbc2.Commit;
  UpdateList;

  AddList('Ending Transactions...');
  Tree.Hdbc1.EndTransact;
  Tree.Hdbc2.EndTransact;
  UpdateList;

  TestDelete('9700011');
  TestDelete('9700012');
end;

procedure TestSQLServer;
begin
  AddList('Test SQLServer...');
  Core:= False;
  Tree.Hdbc1.Terminate;

  AddList('Connect to Data Source...');
  Tree.Hdbc1.DataSource:= 'TESTSQLSERVER32';
  Tree.Hdbc1.UserName:= 'SA';
  Tree.Hdbc1.Password:= '';
  Tree.Hdbc2.DataSource:= Tree.Hdbc1.DataSource;
  Tree.Hdbc2.UserName:= Tree.Hdbc1.UserName;
  Tree.Hdbc2.Password:= Tree.Hdbc1.Password;

  AddList('Run Test...');
  try
    Tree.OECreateTable1.Drop;
  except
    On Exception do;
  end;
  Tree.OECreateTable1.Load;
  TestInsert;
  TestInsert2;
  TestInsertBulk(1000);

  TestSelect;
  TestPopulate;
  TestListBox;
  TestGrid;
  TestBlobs;
  TestTransactions;
end;

procedure TestAccess;
begin
  AddList('Test Access...');
  Core:= True;
  Tree.Hdbc1.Terminate;

  AddList('Connect to Data Source...');
  Tree.Hdbc1.DataSource:= 'TESTACCESS32';
  Tree.Hdbc1.UserName:= '';
  Tree.Hdbc1.Password:= '';
  Tree.Hdbc2.DataSource:= Tree.Hdbc1.DataSource;
  Tree.Hdbc2.UserName:= Tree.Hdbc1.UserName;
  Tree.Hdbc2.Password:= Tree.Hdbc1.Password;

  AddList('Run Test...');
  try
    Tree.OECreateTable2.Drop;
  except
    On Exception do;
  end;
  Tree.OECreateTable2.Load;
  TestInsert;
  TestInsertBulk(1000);

  TestPopulate;
  TestListBox;
  TestGrid;
  TestTransactions;
end;

begin
  Form1.ListBox1.Items.Clear;
  ListCount:= 0;
  AddList('Start Test...');

  try

    TestSQLServer;
    TestAccess;

  except
    on ODBCError do
      Tree.Henv1.Error.Display;
  end;

  AddList('Test Completed');  //terminations and disconnections are done automatically
end;

end.

