unit ThreeD;

interface

uses Classes,ExtCtrls,Matrices,SysUtils,Graphics,DsgnIntf,Forms,Controls,
     Matrixed;

type

TProjector=class;
TViewport=class;

TLine=class(TObject)
private
   FStartPoint:TVector;
   FEndPoint:TVector;
   procedure FSetStartPoint(const Value:TVector); virtual;
   procedure FSetEndPoint(const Value:TVector); virtual;
public
   constructor Create(CDimensions:integer);
   constructor CreateWithCoords(S,E:array of double);
   procedure Free;
   property StartPoint:TVector read FStartPoint write FSetStartPoint;
   property EndPoint:TVector read FEndPoint write FSetEndPoint;
end;

TWorld3d=class(TComponent)
private
   FLines:TList;
   FPos:TVector;
   FProjectors:TList;
   FOnCreate:TNotifyEvent;
   FOnDestroy:TNotifyEvent;
   FOnChange:TNotifyEvent;
   function FGetLine(Index:integer):TLine;
public
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   property Position:TVector read FPos;
   procedure MoveTo(const Point:TVector);
   procedure LineTo(const Point:TVector);
   procedure Line(const Point1,Point2:TVector);
   property Lines[Index:integer]:TLine read FGetLine;
   function NumberOfLines:integer;
   procedure Clear;
   procedure AddProjector(Sender:TProjector);
   procedure RemoveProjector(Sender:TProjector);
published
   property OnCreate:TNotifyEvent read FOnCreate write FOnCreate;
   property OnDestroy:TNotifyEvent read FOnDestroy write FOnDestroy;
   property OnChange:TNotifyEvent read FOnChange write FOnChange;
end;

TProjector=class(TComponent)
private
   FLines:TList;
   FWorld:TWorld3d;
   FViewports:TList;
   FEnabled:boolean;
   FOnCreate:TNotifyEvent;
   FOnDestroy:TNotifyEvent;
   FOnChange:TNotifyEvent;
   FOnRegenerate:TNotifyEvent;
   procedure FSetWorld(const Value:TWorld3d);
   function FGetLine(Index:integer):TLine;
   function Convert(const Line3d:TLine):TLine;
   function ConvertPoint(Point3d:TVector):TVector; virtual; abstract;
   procedure FSetEnabled(const Value:boolean);
public
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   procedure Invalidate; virtual;
   procedure AddViewport(Sender:TViewport);
   procedure RemoveViewport(Sender:TViewport);
   property Lines[Index:integer]:TLine read FGetLine;
   function NumberOfLines:integer;
published
   property World:TWorld3d read FWorld write FSetWorld;
   property Enabled:boolean read FEnabled write FSetEnabled;
   property OnCreate:TNotifyEvent read FOnCreate write FOnCreate;
   property OnDestroy:TNotifyEvent read FOnDestroy write FOnDestroy;
   property OnChange:TNotifyEvent read FOnChange write FOnChange;
   property OnRegenerate:TNotifyEvent read FOnRegenerate write FOnRegenerate;
end;

TStereoProjector=class(TProjector)
private
   FRLines:TList;
   function FGetRLine(Index:integer):TLine;
   function RConvert(const Line3d:TLine):TLine;
   function RConvertPoint(Point3d:TVector):TVector; virtual; abstract;
public
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   procedure Invalidate; override;
   property RLines[Index:integer]:TLine read FGetRLine;
published
end;

   TCamera=class(TProjector)
   private
      FAngle:double;
      FElevation:double;
      FDistance:double;
      FPosition:TVector;
      FDirection:TVector;
      FTarget:TVector;
      FRoll:double;
      FFieldOfView:double;
      FTransfMatrix:TMatrix;
      procedure FSetAngle(const Value:double);
      procedure FSetElevation(const Value:double);
      procedure FSetDistance(const Value:double);
      procedure FSetPosition(const Value:TVector);
      procedure FSetTarget(const Value:TVector);
      procedure FSetFieldOfView(const Value:double);
      procedure FSetRoll(const Value:double);
      function ConvertPoint(Point3d:TVector):TVector; override;
   public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
      procedure Recalculate;
      procedure RecalculateAED;
   published
      property FieldOfView:double read FFieldOfView write FSetFieldOfView;
      property Roll:double read FRoll write FSetRoll;
      property Angle:double read FAngle write FSetAngle;
      property Elevation:double read FElevation write FSetElevation;
      property Distance:double read FDistance write FSetDistance;
      property Position:TVector read FPosition write FSetPosition;
      property Target:TVector read FTarget write FSetTarget;
   end;

   TParallelProjector=class(TProjector)
   private
      FAngle:double;
      FElevation:double;
      FRoll:double;
      FTransfMatrix:TMatrix;
      procedure FSetAngle(const Value:double);
      procedure FSetElevation(const Value:double);
      procedure FSetRoll(const Value:double);
      function ConvertPoint(Point3d:TVector):TVector; override;
   public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
      procedure Recalculate;
   published
      property Angle:double read FAngle write FSetAngle;
      property Elevation:double read FElevation write FSetElevation;
      property Roll:double read FRoll write FSetRoll;
   end;

   TIsometricProjector=class(TProjector)
   private
      FTransfMatrix:TMatrix;
      FXVector:TVector;
      FYVector:TVector;
      FZVector:TVector;
      procedure FSetXVector(const Value:TVector);
      procedure FSetYVector(const Value:TVector);
      procedure FSetZVector(const Value:TVector);
      function ConvertPoint(Point3d:TVector):TVector; override;
   public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
      procedure Recalculate;
   published
      property XVector:TVector read FXVector write FSetXVector;
      property YVector:TVector read FYVector write FSetYVector;
      property ZVector:TVector read FZVector write FSetZVector;
   end;

   TStereoCamera=class(TStereoProjector)
   private
      FAngle:double;
      FElevation:double;
      FDistance:double;
      FPosition:TVector;
      FLPosition:TVector;
      FRPosition:TVector;
      FTarget:TVector;
      FRoll:double;
      FFieldOfView:double;
      FEyeDistance:double;
      FLTransfMatrix:TMatrix;
      FRTransfMatrix:TMatrix;
      procedure FSetAngle(const Value:double);
      procedure FSetElevation(const Value:double);
      procedure FSetDistance(const Value:double);
      procedure FSetPosition(const Value:TVector);
      procedure FSetTarget(const Value:TVector);
      procedure FSetFieldOfView(const Value:double);
      procedure FSetRoll(const Value:double);
      procedure FSetEyeDistance(const Value:double);
      function ConvertPoint(Point3d:TVector):TVector; override;
      function RConvertPoint(Point3d:TVector):TVector; override;
   public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
      procedure Recalculate;
      procedure RecalculateAED;
   published
      property FieldOfView:double read FFieldOfView write FSetFieldOfView;
      property Roll:double read FRoll write FSetRoll;
      property Angle:double read FAngle write FSetAngle;
      property Elevation:double read FElevation write FSetElevation;
      property Distance:double read FDistance write FSetDistance;
      property Position:TVector read FPosition write FSetPosition;
      property Target:TVector read FTarget write FSetTarget;
      property EyeDistance:double read FEyeDistance write FSetEyeDistance;
   end;

TViewport=class(TCustomPanel)
private
   FProjector:TProjector;
   FPan:TVector;
   FScale:double;
   FEnabled:boolean;
   FOnCreate:TNotifyEvent;
   FOnDestroy:TNotifyEvent;
   FOnChange:TNotifyEvent;
   FOnRedraw:TNotifyEvent;
   procedure FSetPan(const Value:TVector);
   procedure FSetScale(const Value:double);
   procedure FSetProjector(const Value:TProjector); virtual;
   procedure FSetEnabled(const Value:boolean);
public
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   procedure Invalidate; override;
   procedure Paint; override;
published
   property Align;
   property Projector:TProjector read FProjector write FSetProjector;
   property Pan:TVector read FPan write FSetPan;
   property Scale:double read FScale write FSetScale;
   property Enabled:boolean read FEnabled write FSetEnabled;
   property OnCreate:TNotifyEvent read FOnCreate write FOnCreate;
   property OnDestroy:TNotifyEvent read FOnDestroy write FOnDestroy;
   property OnChange:TNotifyEvent read FOnChange write FOnChange;
   property OnRedraw:TNotifyEvent read FOnRedraw write FOnRedraw;
end;

TStereoViewport=class(TViewport)
private
   FProjector:TStereoProjector;
   procedure FSetProjector(const Value:TProjector); override;
public
   constructor Create(AOwner:TComponent); override;
   procedure Paint; override;
published
end;

implementation

constructor TLine.Create(CDimensions:integer);
begin
inherited Create;
FStartPoint:=TVector.Create(CDimensions);
FEndPoint:=TVector.Create(CDimensions);
end;

constructor TLine.CreateWithCoords(S,E:array of double);
begin
inherited Create;
if High(S)<>High(E) then
   raise EMatrixError.Create('Vectors have unequal size in TLine.CreateWithCoords')
else begin
   FStartPoint:=TVector.CreateWithCoords(S);
   FEndPoint:=TVector.CreateWithCoords(E);
   end;
end;

procedure TLine.Free;
begin
FStartPoint.Free;
FEndPoint.Free;
inherited Free;
end;

procedure TLine.FSetStartPoint(const Value:TVector);
begin
FStartPoint.Assign(Value);
end;

procedure TLine.FSetEndPoint(const Value:TVector);
begin
FEndPoint.Assign(Value);
end;

constructor TWorld3d.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FLines:=TList.Create;
FPos:=TVector.Create(3);
FProjectors:=TList.Create;

FLines.Add(TLine.CreateWithCoords([0,0,0],[1,0,0]));
FLines.Add(TLine.CreateWithCoords([0,0,0],[0,1,0]));
FLines.Add(TLine.CreateWithCoords([0,0,0],[0,0,1]));
FLines.Add(TLine.CreateWithCoords([1,0,0],[1,1,0]));
FLines.Add(TLine.CreateWithCoords([1,0,0],[1,0,1]));
FLines.Add(TLine.CreateWithCoords([0,1,0],[0,1,1]));
FLines.Add(TLine.CreateWithCoords([1,1,1],[0,1,1]));
FLines.Add(TLine.CreateWithCoords([1,1,1],[1,0,1]));
FLines.Add(TLine.CreateWithCoords([1,1,1],[1,1,0]));
FLines.Add(TLine.CreateWithCoords([0,1,1],[0,0,1]));
FLines.Add(TLine.CreateWithCoords([0,0,1],[1,0,1]));
FLines.Add(TLine.CreateWithCoords([1,1,0],[0,1,0]));

if Assigned(FOnCreate) then FOnCreate(self);
end;

destructor TWorld3d.Destroy;
var i:integer;
begin
if Assigned(FOnDestroy) then FOnDestroy(self);
FLines.Free;
FPos.Free;
i:=0;
while i<FProjectors.Count do begin
      TProjector(FProjectors.Items[i]).World:=nil;
      Inc(i);
      end;
FProjectors.Free;
inherited Destroy;
end;

procedure TWorld3d.MoveTo(const Point:TVector);
begin
FPos.Assign(Point);
end;

procedure TWorld3d.LineTo(const Point:TVector);
var Line:TLine;
begin
Line:=TLine.Create(3);
Line.StartPoint:=FPos;
Line.EndPoint:=Point;
FLines.Add(Line);
FPos.Assign(Point);
if Assigned(FOnChange) then FOnChange(self);
end;

procedure TWorld3d.Line(const Point1,Point2:TVector);
var Line:TLine;
begin
Line:=TLine.Create(3);
Line.StartPoint:=Point1;
Line.EndPoint:=Point2;
FLines.Add(Line);
FPos.Assign(Point2);
if Assigned(FOnChange) then FOnChange(self);
end;

procedure TWorld3d.Clear;
begin
FLines.Clear;
if Assigned(FOnChange) then FOnChange(self);
end;

function TWorld3d.FGetLine(Index:integer):TLine;
begin
Result:=TLine(FLines.Items[Index]);
end;

function TWorld3d.NumberOfLines:integer;
begin
Result:=FLines.Count;
end;

procedure TWorld3d.AddProjector(Sender:TProjector);
begin
if FProjectors.IndexOf(Sender)=-1 then
   FProjectors.Add(Sender);
if Sender.World<>self then Sender.World:=self;
end;

procedure TWorld3d.RemoveProjector(Sender:TProjector);
begin
if Sender.World=self then Sender.World:=nil;
FProjectors.Remove(Sender);
end;

constructor TProjector.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FWorld:=nil;
FLines:=TList.Create;
FViewports:=TList.Create;
FEnabled:=True;
if Assigned(FOnCreate) then FOnCreate(self);
end;

destructor TProjector.Destroy;
var i:integer;
begin
if Assigned(FOnDestroy) then FOnDestroy(self);
if World<>nil then World.RemoveProjector(self);
i:=0;
while i<FViewports.Count do begin
      TViewport(FViewports.Items[i]).Projector:=nil;
      Inc(i);
      end;
FViewports.Free;
FLines.Free;
inherited Destroy;
end;

procedure TProjector.FSetWorld(const Value:TWorld3d);
begin
FWorld:=Value;
if Value<>nil then Value.AddProjector(self);
Invalidate;
end;

procedure TProjector.Invalidate;
var i,n:integer;
begin
if Enabled then begin
   FLines.Clear;
   if World<>nil then begin
      n:=World.NumberOfLines;
      i:=0;
      while i<n do begin
         FLines.Add(Convert(World.Lines[i]));
         Inc(i);
         end;
      i:=0;
      while i<FViewports.Count do begin
         TViewport(FViewports.Items[i]).Invalidate;
         Inc(i);
         end;
      end;
   end;
if Assigned(FOnRegenerate) then FOnRegenerate(self);
end;

procedure TProjector.FSetEnabled(const Value:boolean);
begin
FEnabled:=Value;
if Value then Invalidate;
end;

function TProjector.NumberOfLines:integer;
begin
Result:=FLines.Count;
end;

function TProjector.FGetLine(Index:integer):TLine;
begin
Result:=TLine(FLines.Items[Index]);
end;

procedure TProjector.AddViewport(Sender:TViewport);
begin
if FViewports.IndexOf(Sender)=-1 then
FViewports.Add(Sender);
if Sender.Projector<>self then Sender.Projector:=self;
end;

procedure TProjector.RemoveViewport(Sender:TViewport);
begin
if Sender.Projector=self then Sender.Projector:=nil;
FViewports.Remove(Sender);
end;

function TProjector.Convert(const Line3d:TLine):TLine;
begin
Result:=TLine.Create(2);
Result.StartPoint:=ConvertPoint(Line3d.StartPoint);
Result.endPoint:=ConvertPoint(Line3d.EndPoint);
end;


constructor TStereoProjector.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FRLines:=TList.Create;
end;

destructor TStereoProjector.Destroy;
begin
FRLines.Free;
inherited Destroy;
end;

procedure TStereoProjector.Invalidate;
var i,n:integer;
begin
if Enabled then begin
   FRLines.Clear;
   if World<>nil then begin
      n:=World.NumberOfLines;
      i:=0;
      while i<n do begin
         FRLines.Add(RConvert(World.Lines[i]));
         Inc(i);
         end;
      end;
   end;
   inherited Invalidate;
end;

function TStereoProjector.FGetRLine(Index:integer):TLine;
begin
Result:=TLine(FRLines.Items[Index]);
end;

function TStereoProjector.RConvert(const Line3d:TLine):TLine;
begin
Result:=TLine.Create(2);
Result.StartPoint:=RConvertPoint(Line3d.StartPoint);
Result.EndPoint:=RConvertPoint(Line3d.EndPoint);
end;

function ConvertGeneral(const Transf:TMatrix; const Position:TVector;
                        const Point3d:TVector; const FOV:double):TVector;
var v:TVector;
    w:TMatrix;
    fact:double;
begin
Result:=TVector.Create(2);
v:=TVector.Create(3);
v.Assign(Point3d);
v.Sub(Position);
w:=TMatrix.Multiply(Transf,v);
fact:=w[1,0]*sin((FOV/180*PI))/cos(FOV/180*PI);
if abs(fact)>1E-5 then begin
   Result[0,0]:=w[0,0]/fact;
   Result[1,0]:=w[2,0]/fact;
   end
else begin
   end;
v.Free;
w.Free;
end;

function CalcAngle(v:TVector):double;
var w:TVector;
begin
if v.Norm=0 then Result:=0
else begin
   if abs(v[0,0])/v.norm>0.001 then begin
      Result:=ArcTan(v[1,0]/v[0,0]);
      if v[0,0]<0 then Result:=Result+PI;
      end
   else begin
      w:=TVector.CreateWithCoords([-v[1,0],v[0,0]]);
      Result:=CalcAngle(w)-PI/2;
      w.Free;
      end
   end;
end;

constructor TCamera.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FAngle:=0;
FElevation:=0;
FDistance:=1.5;
FPosition:=TVector.CreateWithCoords([2,0.5,0.5]);
FDirection:=TVector.Create(3);
FTarget:=TVector.CreateWithCoords([0.5,0.5,0.5]);
FDirection.Assign(FTarget);
FDirection.Sub(FPosition);
FFieldOfView:=45;
FRoll:=0;
FTransfMatrix:=TMatrix.Create(3,3);
Recalculate;
end;

destructor TCamera.Destroy;
begin
FPosition.Free;
FDirection.Free;
FTarget.Free;
FTransfMatrix.Free;
inherited Destroy;
end;


procedure TCamera.Recalculate;
var v:TVector;
    M,Rx,Ry,Rz:TMatrix;
    r:double;
begin
FTransfMatrix.Free;

v:=TVector.CreateWithCoords([FDirection[0,0],FDirection[1,0]]);
FAngle:=CalcAngle(v)/PI*180{+180};
r:=v.Norm;
v[0,0]:=r;
v[1,0]:=FDirection[2,0];
FElevation:=CalcAngle(v)/PI*180;
FDistance:=FDirection.Norm;

Rz:=TMatrix.RotZ(-PI/2-FAngle/180*PI);
Rx:=TMatrix.RotX(FElevation/180*PI);
Ry:=TMatrix.RotY(-(FRoll/180*PI));

M:=TMatrix.Multiply(Rx,Rz);
FTransfMatrix:=TMatrix.Multiply(Ry,M);

v.Free;
M.Free;
Rz.Free;
Rx.Free;
Ry.Free;
Invalidate;
if Assigned(FOnChange) then FOnChange(self);
end;

procedure TCamera.RecalculateAED;
var M,Rx,Ry,Rz:TMatrix;
    r:double;
begin
FTransfMatrix.Free;

Rz:=TMatrix.RotZ(-PI/2-FAngle/180*PI);
Rx:=TMatrix.RotX(FElevation/180*PI);
Ry:=TMatrix.RotY(-(FRoll/180*PI));

M:=TMatrix.Multiply(Rx,Rz);
FTransfMatrix:=TMatrix.Multiply(Ry,M);

r:=FDistance*cos(FElevation/180*PI);
FDirection[0,0]:=r*cos(FAngle/180*PI);
FDirection[1,0]:=r*sin(FAngle/180*PI);
FDirection[2,0]:=FDistance*sin(FElevation/180*PI);
FPosition.Assign(FTarget);
FPosition.Sub(FDirection);

M.Free;
Rz.Free;
Rx.Free;
Ry.Free;
Invalidate;

if Assigned(FOnChange) then FOnChange(self);
end;

procedure TCamera.FSetAngle(const Value:double);
begin
FAngle:=Value;
ReCalculateAED;
end;

procedure TCamera.FSetElevation(const Value:double);
begin
FElevation:=Value;
ReCalculateAED;
end;

procedure TCamera.FSetDistance(const Value:double);
begin
FDistance:=Value;
ReCalculateAED;
end;

procedure TCamera.FSetPosition(const Value:TVector);
begin
FPosition.Assign(Value);
FDirection.Assign(FTarget);
FDirection.Sub(FPosition);
Recalculate;
end;

procedure TCamera.FSetTarget(const Value:TVector);
begin
FTarget.Assign(Value);
FDirection.Assign(FTarget);
FDirection.Sub(FPosition);
Recalculate;
end;

procedure TCamera.FSetFieldOfView(const Value:double);
begin
if (Value<1) or (Value>89) then raise Exception.Create('Field of view out of range')
else begin
   FFieldOfView:=Value;
   Recalculate;
   end;
end;

procedure TCamera.FSetRoll(const Value:double);
begin
FRoll:=Value;
Recalculate;
end;

function TCamera.ConvertPoint(Point3d:TVector):TVector;
begin
Result:=ConvertGeneral(FTransfMatrix,FPosition,Point3d,FFieldOfView);
end;

constructor TStereoCamera.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FAngle:=0;
FElevation:=0;
FDistance:=1.5;
FEyeDistance:=0.1;
FPosition:=TVector.CreateWithCoords([2,0.5,0.5]);
FLPosition:=TVector.Create(3);
FRPosition:=TVector.Create(3);
FTarget:=TVector.CreateWithCoords([0.5,0.5,0.5]);
FFieldOfView:=45;
FRoll:=0;
FLTransfMatrix:=TMatrix.Create(3,3);
FRTransfMatrix:=TMatrix.Create(3,3);
Recalculate;
end;

destructor TStereoCamera.Destroy;
begin
FPosition.Free;
FLPosition.Free;
FRPosition.Free;
FTarget.Free;
FLTransfMatrix.Free;
FRTransfMatrix.Free;
inherited Destroy;
end;

procedure TStereoCamera.FSetAngle(const Value:double);
begin
FAngle:=Value;
ReCalculateAED;
end;

procedure TStereoCamera.FSetElevation(const Value:double);
begin
FElevation:=Value;
ReCalculateAED;
end;

procedure TStereoCamera.FSetDistance(const Value:double);
begin
FDistance:=Value;
ReCalculateAED;
end;

procedure TStereoCamera.FSetPosition(const Value:TVector);
begin
FPosition.Assign(Value);
Recalculate;
end;

procedure TStereoCamera.FSetTarget(const Value:TVector);
begin
FTarget.Assign(Value);
Recalculate;
end;

procedure TStereoCamera.FSetFieldOfView(const Value:double);
begin
if (Value<1) or (Value>89) then raise Exception.Create('Field of view out of range')
else begin
   FFieldOfView:=Value;
   Recalculate;
   end;
end;

procedure TStereoCamera.FSetRoll(const Value:double);
begin
FRoll:=Value;
Recalculate;
end;

procedure TStereoCamera.FSetEyeDistance(const Value:double);
begin
FEyeDistance:=Value;
Recalculate;
end;


function TStereoCamera.ConvertPoint(Point3d:TVector):TVector;
begin
Result:=ConvertGeneral(FLTransfMatrix,FLPosition,Point3d,FFieldOfView);
end;

function TStereoCamera.RConvertPoint(Point3d:TVector):TVector;
begin
Result:=ConvertGeneral(FRTransfMatrix,FRPosition,Point3d,FFieldOfView);
end;

function CalcTransfMatrix(const Position:TVector; const Target:TVector;
                          const Roll:double):TMatrix;
var v:TVector;
    Ang,Elev,r:double;
    Rx,Ry,Rz,M:TMatrix;
begin
v:=TVector.CreateWithCoords([Target[0,0],Target[1,0]]);
v.Sub(Position);
Ang:=CalcAngle(v)/PI*180{+180};
r:=v.Norm;
v[0,0]:=r;
v[1,0]:=Target[2,0]-Position[2,0];
Elev:=CalcAngle(v)/PI*180;

Rz:=TMatrix.RotZ(-PI/2-Ang/180*PI);
Rx:=TMatrix.RotX(Elev/180*PI);
Ry:=TMatrix.RotY(-Roll/180*PI);

M:=TMatrix.Multiply(Rx,Rz);
Result:=TMatrix.Multiply(Ry,M);   (* World -> Cam *)

v.Free; Rx.Free; Ry.Free; Rz.Free; M.Free;
end;

procedure TStereoCamera.Recalculate;
var Direction,v:TVector;
    M,Mi,Rx,Rxi,Ry,Ryi,Rz,Rzi:TMatrix;
    r:double;
    T,C:TMatrix;
begin
FLTransfMatrix.Free;
FRTransfMatrix.Free;

Direction:=TVector.Create(3);
Direction.Assign(FTarget);
Direction.Sub(FPosition);

v:=TVector.Create(2);
v.Assign(Direction);

FAngle:=CalcAngle(v)/PI*180{+180};
r:=v.Norm;
v[0,0]:=r;
v[1,0]:=Direction[2,0];
FElevation:=CalcAngle(v)/PI*180;
FDistance:=Direction.Norm;

Rz:=TMatrix.RotZ(-PI/2-FAngle/180*PI);
Rzi:=TMatrix.RotZ(PI/2+FAngle/180*PI);
Rx:=TMatrix.RotX(FElevation/180*PI);
Rxi:=TMatrix.RotX(-FElevation/180*PI);
Ry:=TMatrix.RotY(-FRoll/180*PI);
Ryi:=TMatrix.RotY(FRoll/180*PI);

M:=TMatrix.Multiply(Rx,Rz);
Mi:=TMatrix.Multiply(Rxi,Ryi);
T:=TMatrix.Multiply(Ry,M);   (* World -> Cam *)
C:=TMatrix.Multiply(Rzi,Mi); (* Cam -> World *)

(* LPos:=Pos+C(-ED/2,0,0) *)
v.Free;
M.Free;
v:=TVector.CreateWithCoords([-FEyeDistance/2,0,0]);
M:=TMatrix.Multiply(C,v);
FLPosition.Assign(FPosition);
FLPosition.Add(M);

(* RPos:=Pos+C(-ED/2,0,0) *)
v.Free;
M.Free;
v:=TVector.CreateWithCoords([FEyeDistance/2,0,0]);
M:=TMatrix.Multiply(C,v);
FRPosition.Assign(FPosition);
FRPosition.Add(M);

FLTransfMatrix:=CalcTransfMatrix(FLPosition,FTarget,FRoll);
FRTransfMatrix:=CalcTransfMatrix(FRPosition,FTarget,FRoll);

Direction.Free;
T.Free;
C.Free;
v.Free;
M.Free;
Mi.Free;
Rz.Free;
Rzi.Free;
Rx.Free;
Rxi.Free;
Ry.Free;
Ryi.Free;
Invalidate;
if Assigned(FOnChange) then FOnChange(self);
end;

procedure TStereoCamera.RecalculateAED;
var T,C,M,Mi,Rx,Rxi,Ry,Ryi,Rz,Rzi:TMatrix;
    r:double;
    v:TVector;
begin
FLTransfMatrix.Free;
FRTransfMatrix.Free;

Rz:=TMatrix.RotZ(-PI/2-FAngle/180*PI);
Rzi:=TMatrix.RotZ(PI/2+FAngle/180*PI);
Rx:=TMatrix.RotX(FElevation/180*PI);
Rxi:=TMatrix.RotX(-FElevation/180*PI);
Ry:=TMatrix.RotY(-FRoll/180*PI);
Ryi:=TMatrix.RotY(FRoll/180*PI);

M:=TMatrix.Multiply(Rx,Rz);
Mi:=TMatrix.Multiply(Rxi,Ryi);
T:=TMatrix.Multiply(Ry,M);
C:=TMatrix.Multiply(Rzi,Mi);

r:=FDistance*cos(FElevation/180*PI);
FPosition[0,0]:=FTarget[0,0]-r*cos(FAngle/180*PI);
FPosition[1,0]:=FTarget[1,0]-r*sin(FAngle/180*PI);
FPosition[2,0]:=FTarget[2,0]-FDistance*sin(FElevation/180*PI);

(* LPos:=Pos+C(-ED/2,0,0) *)
M.Free;
v:=TVector.CreateWithCoords([-FEyeDistance/2,0,0]);
M:=TMatrix.Multiply(C,v);
FLPosition.Assign(FPosition);
FLPosition.Add(M);

(* RPos:=Pos+C(ED/2,0,0) *)
v.Free;
M.Free;
v:=TVector.CreateWithCoords([FEyeDistance/2,0,0]);
M:=TMatrix.Multiply(C,v);
FRPosition.Assign(FPosition);
FRPosition.Add(M);

FLTransfMatrix:=CalcTransfMatrix(FLPosition,FTarget,FRoll);
FRTransfMatrix:=CalcTransfMatrix(FRPosition,FTarget,FRoll);

M.Free;
T.Free;
C.Free;
Rz.Free;
Rx.Free;
Ry.Free;
Invalidate;
if Assigned(FOnChange) then FOnChange(self);
end;

constructor TParallelProjector.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FAngle:=0;
FElevation:=0;
FRoll:=0;
FTransfMatrix:=TMatrix.Create(3,3);
Recalculate;
end;

destructor TParallelProjector.Destroy;
begin
FTransfMatrix.Free;
inherited Destroy;
end;

procedure TParallelProjector.FSetAngle(const Value:double);
begin
FAngle:=Value;
ReCalculate;
end;

procedure TParallelProjector.FSetElevation(const Value:double);
begin
FElevation:=Value;
ReCalculate;
end;

procedure TParallelProjector.FSetRoll(const Value:double);
begin
FRoll:=Value;
Recalculate;
end;

function TParallelProjector.ConvertPoint(Point3d:TVector):TVector;
var v:TVector;
    w:TMatrix;
begin
Result:=TVector.Create(2);
v:=TVector.Create(3);
v.Assign(Point3d);
w:=TMatrix.Multiply(FTransfMatrix,v);
Result[0,0]:=w[0,0];
Result[1,0]:=w[2,0];
v.Free;
w.Free;
end;

procedure TParallelProjector.Recalculate;
var M,Rx,Ry,Rz:TMatrix;
    r:double;
begin
FTransfMatrix.Free;

Rz:=TMatrix.RotZ(-PI/2-FAngle/180*PI);
Rx:=TMatrix.RotX(FElevation/180*PI);
Ry:=TMatrix.RotY(-(FRoll/180*PI));

M:=TMatrix.Multiply(Rx,Rz);
FTransfMatrix:=TMatrix.Multiply(Ry,M);

M.Free;
Rz.Free;
Rx.Free;
Ry.Free;
Invalidate;
if Assigned(FOnChange) then FOnChange(self);
end;

constructor TIsometricProjector.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FTransfMatrix:=TMatrix.Create(2,3);
FXVector:=TVector.Create(2);
  FXVector[0,0]:=-cos(30/180*PI);
  FXVector[1,0]:=-sin(30/180*PI);
FYVector:=TVector.Create(2);
  FYVector[0,0]:=cos(30/180*PI);
  FYVector[1,0]:=-sin(30/180*PI);
FZVector:=TVector.Create(2);
  FZVector[0,0]:=0;
  FZVector[1,0]:=1;
Recalculate;
end;

destructor TIsometricProjector.Destroy;
begin
FTransfMatrix.Free;
FXVector.Free;
FYVector.Free;
FZVector.Free;
inherited Destroy;
end;

procedure TIsometricProjector.FSetXVector(const Value:TVector);
begin
if (Value.Columns<>1) or (Value.Rows<>2) then
   raise Exception.Create('Dimensions do not match in SetXVector')
else begin
   FXVector.Assign(Value);
   Recalculate;
   end;
end;

procedure TIsometricProjector.FSetYVector(const Value:TVector);
begin
if (Value.Columns<>1) or (Value.Rows<>2) then
   raise Exception.Create('Dimensions do not match in SetXVector')
else begin
   FYVector.Assign(Value);
   Recalculate;
   end;
end;

procedure TIsometricProjector.FSetZVector(const Value:TVector);
begin
if (Value.Columns<>1) or (Value.Rows<>2) then
   raise Exception.Create('Dimensions do not match in SetXVector')
else begin
   FZVector.Assign(Value);
   Recalculate;
   end;
end;

function TIsometricProjector.ConvertPoint(Point3d:TVector):TVector;
var v:TMatrix;
begin
v:=TMatrix.Multiply(FTransfMatrix,Point3d);
Result:=TVector.Create(2);
Result.Assign(v);
v.Free;
end;

procedure TIsometricProjector.Recalculate;
begin
FTransfMatrix[0,0]:=FXVector[0,0];
FTransfMatrix[1,0]:=FXVector[1,0];

FTransfMatrix[0,1]:=FYVector[0,0];
FTransfMatrix[1,1]:=FYVector[1,0];

FTransfMatrix[0,2]:=FZVector[0,0];
FTransfMatrix[1,2]:=FZVector[1,0];
Invalidate;
if Assigned(FOnChange) then FOnChange(self);
end;

constructor TViewport.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FPan:=TVector.Create(2);
FScale:=1;
FEnabled:=True;
if Assigned(FOnCreate) then FOnCreate(self);
end;

procedure TViewport.FSetEnabled(const Value:boolean);
begin
FEnabled:=Value;
if Value then Invalidate;
end;

procedure TViewport.Invalidate;
begin
if Assigned(FOnRedraw) then FOnRedraw(self);
inherited Invalidate;
end;

procedure TViewport.FSetPan(const Value:TVector);
begin
if (Value.Rows<>2) or (Value.Columns<>1) then
   raise Exception.Create('Vector dimensions do not match in SetPan')
else begin
   FPan.Assign(Value);
   if Assigned(FOnChange) then FOnChange(self);
   if Enabled then Invalidate;
   end;
end;

procedure TViewport.FSetScale(const Value:double);
begin
FScale:=Value;
if Assigned(FOnChange) then FOnChange(self);
if Enabled then Invalidate;
end;

procedure TViewport.FSetProjector(const Value:TProjector);
begin
FProjector:=Value;
if Value<>nil then begin
   Value.AddViewport(self);
   end;
if Assigned(FOnChange) then FOnChange(self);
if Enabled then Invalidate;
end;

destructor TViewport.Destroy;
begin
if Assigned(FOnDestroy) then FOnDestroy(self);
if Projector<>nil then Projector.RemoveViewport(self);
FPan.Free;
inherited Destroy;
end;

procedure TViewport.Paint;
var i,cx,cy:integer;
    Line:TLine;
    fact:double;
begin
if Projector=nil then
   inherited Paint
else begin
   with ClientRect do begin
        Canvas.Brush.Color:=clWhite;
        Canvas.FillRect(Rect(Left,Top,Right,Bottom));
        Canvas.Pen.Color:=clBlack;
        Canvas.Rectangle(Left,Top,Right,Bottom);
        end;
   cx:=ClientRect.Right;
   cy:=ClientRect.Bottom;
   if cx<cy then fact:=0.5*cx else fact:=0.5*cy;
   fact:=FScale*fact;

   cx:=cx div 2;
   cy:=cy div 2;

   i:=0;
   while i<Projector.NumberOfLines do begin
      Line:=Projector.Lines[i];
      with Line.StartPoint do
           Canvas.MoveTo(cx+round(fact*(Contents[0]-FPan[0,0])),
                         cy-round(fact*(Contents[1]-FPan[1,0])));
      with Line.EndPoint do
           Canvas.LineTo(cx+round(fact*(Contents[0]-FPan[0,0])),
                         cy-round(fact*(Contents[1]-FPan[1,0])));
      Inc(i);
      end;
   end;
end;

constructor TStereoViewport.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
end;

procedure TStereoViewport.FSetProjector(const Value:TProjector);
begin
if Value<>nil then begin
   if not (Value is TStereoProjector) then
      raise Exception.Create('A stereo-viewport needs a stereo-projector')
   end;
inherited FSetProjector(Value);
end;

procedure TStereoViewport.Paint;
var i,cx,cy:integer;
    LLine,RLine:TLine;
    fact:double;
    SP:TStereoProjector;
begin
if Projector=nil then
   inherited Paint
else begin
   SP:=Projector as TStereoProjector;
   with ClientRect do begin
        Canvas.Brush.Color:=clBlack;
        Canvas.FillRect(Rect(Left,Top,Right,Bottom));
        Canvas.Pen.Color:=clBlack;
        Canvas.Rectangle(Left,Top,Right,Bottom);
        end;
   cx:=ClientRect.Right;
   cy:=ClientRect.Bottom;
   if cx<cy then fact:=0.5*cx else fact:=0.5*cy;
   fact:=FScale*fact;

   cx:=cx div 2;
   cy:=cy div 2;

   i:=0;
   while i<SP.NumberOfLines do begin
      LLine:=SP.Lines[i];
      RLine:=SP.RLines[i];
      Canvas.Pen.Color:=$0000FF00;
      with LLine.StartPoint do
           Canvas.MoveTo(cx+round(fact*(Contents[0]-FPan[0,0])),
                         cy-round(fact*(Contents[1]-FPan[1,0])));
      with LLine.EndPoint do
           Canvas.LineTo(cx+round(fact*(Contents[0]-FPan[0,0])),
                         cy-round(fact*(Contents[1]-FPan[1,0])));
      Canvas.Pen.Color:=$000000FF;
      with RLine.StartPoint do
           Canvas.MoveTo(cx+round(fact*(Contents[0]-FPan[0,0])),
                         cy-round(fact*(Contents[1]-FPan[1,0])));
      with RLine.EndPoint do
           Canvas.LineTo(cx+round(fact*(Contents[0]-FPan[0,0])),
                         cy-round(fact*(Contents[1]-FPan[1,0])));
      Inc(i);
      end;
   end;
end;



procedure Register;
begin
RegisterComponents('3DComp',[TWorld3d,TCamera,TParallelProjector,
   TIsometricProjector,TStereoCamera,TViewport,TStereoViewport]);
end;

end.
