unit Timegrph;

interface

uses Controls,ExtCtrls,Classes,Graphics,SysUtils;

type
  TMemArray=array[0..5000] of double;
  PMemArray=^TMemArray;

  TSampleEvent=procedure (Sender:TObject; var Sample:double) of object;

  TTimeGraph=class(TCustomPanel)
  private
     FTimer:TTimer;
     FMemArray:PMemArray;
     FPosition:integer;
     FInterval:word;
     FNumberOfSamples:word;
     FSamplesInMemory:integer;
     FGetSample:TSampleEvent;
     FEnabled:boolean;
     FMinimum:double;
     FMaximum:double;
     FLineColor:TColor;
     procedure FSetInterval(const Value:word);
     function  FGetHistory:longint;
     procedure FSetHistory(const Value:longint);
     procedure FSetEnabled(const Value:boolean);
     function  FGetMemory(Index:integer):double;
     procedure FSetGetSample(const Value:TSampleEvent);
     procedure FSetMinimum(const Value:double);
     procedure FSetMaximum(const Value:double);
  public
     constructor Create(AOwner:TComponent); override;
     destructor  Destroy; override;
     procedure   Free;
     procedure   Paint; override;
     property    Memory[Index:integer]:double read FGetMemory;
     procedure   TakeSample(Sender:TObject);
  published
     property  OnGetSample:TSampleEvent read FGetSample write FSetGetSample;
     property  Interval:word read FInterval write FSetInterval;
     property  History:longint read FGetHistory write FSetHistory;
     property  Enabled:boolean read FEnabled write FSetEnabled;
     property  Minimum:double read FMinimum write FSetMinimum;
     property  Maximum:double read FMaximum write FSetMaximum;
     property  Color;
     property  LineColor:TColor read FLineColor write FLineColor;
  end;

  TimeGraphException=class(Exception);

  procedure Register;

implementation

constructor TTimeGraph.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FInterval:=1000;
FNumberOfSamples:=100;
GetMem(FMemArray,FNumberOfSamples*sizeof(double));
FPosition:=0;
FSamplesInMemory:=0;
FGetSample:=nil;
FEnabled:=True;
FMinimum:=0;
FMaximum:=1;
FTimer:=TTimer.Create(self);
FTimer.Interval:=FInterval;
FTimer.Enabled:=FEnabled;
FTimer.OnTimer:=TakeSample;
Color:=clBlack;
FLineColor:=clGreen;
end;

destructor TTimeGraph.Destroy;
begin
FTimer.Free;
FreeMem(FMemArray,FNumberOfSamples*sizeof(double));
inherited Destroy;
end;

procedure TTimeGraph.Free;
begin
if Assigned(self) then Destroy;
end;

procedure TTimeGraph.Paint;
var v,w,h,hfact:double;
    px,py,i:integer;
begin
Canvas.Pen.Color:=clBlack;
with ClientRect do begin
     Canvas.Brush.Color:=Color;
     Canvas.Rectangle(Left,Top,Right,Bottom);
     w:=Right-Left;
     h:=Bottom-Top;
     hfact:=(Bottom-Top)/(FMaximum-FMinimum);
     end;

Canvas.Pen.Color:=LineColor;
i:=0;
while i<FSamplesInMemory do begin
      v:=Memory[i];
      px:=round(w-w*(i/FNumberOfSamples));
      py:=round(h-(v-FMinimum)*hfact);
      if i=0 then Canvas.MoveTo(px,py)
      else Canvas.LineTo(px,py);
      Inc(i);
      end;
end;

procedure TTimeGraph.TakeSample;
var Sample:double;
begin
if Assigned(FGetSample)
then begin
   If FSamplesInMemory<FNumberOfSamples then Inc(FSamplesInMemory);
   Sample:=0;
   FGetSample(self,Sample);
   FMemArray^[FPosition]:=Sample;
   FPosition:=(FPosition+1) mod FNumberOfSamples;
   Paint;
   end;
end;

procedure TTimeGraph.FSetInterval(const Value:word);
var hist:word;
    OldEnabled:boolean;
begin
OldEnabled:=Enabled;
Enabled:=False;
hist:=History;
FreeMem(FMemArray,FNumberOfSamples*sizeof(double));
FTimer.Interval:=Value;
FInterval:=Value;
FNumberOfSamples:=Hist div FInterval;
GetMem(FMemArray,FNumberOfSamples*sizeof(double));
FPosition:=0;
FSamplesInMemory:=0;
Enabled:=OldEnabled;
Paint;
end;

function  TTimeGraph.FGetHistory:longint;
begin
Result:=longint(FNumberOfSamples)*longint(FInterval);
end;

procedure TTimeGraph.FSetHistory(const Value:longint);
begin
if Value<FInterval then
   raise TimeGraphException.Create('History must be greater then Interval')
else begin
   FTimer.Enabled:=False;
   FreeMem(FMemArray,FNumberOfSamples*sizeof(double));
   FNumberOfSamples:=Value div FInterval;
   GetMem(FMemArray,FNumberOfSamples*sizeof(double));
   FPosition:=0;
   FSamplesInMemory:=0;
   FTimer.Enabled:=FEnabled
   end;
end;

procedure TTimeGraph.FSetEnabled(const Value:boolean);
begin
FTimer.Enabled:=Value;
FEnabled:=Value;
Paint;
end;

function  TTimeGraph.FGetMemory(Index:integer):double;
var p:integer;
begin
if (Index<0) or (Index>=FNumberOfSamples) then
   raise TimeGraphException.Create('Index out of range')
else begin
   p:=FPosition-1-Index;
   p:=p mod FNumberOfSamples;
   if p<0 then p:=p+FNumberOfSamples;
   Result:=FMemArray^[p];
   end;
end;

procedure TTimeGraph.FSetGetSample(const Value:TSampleEvent);
begin
FGetSample:=Value;
end;

procedure TTimeGraph.FSetMinimum(const Value:double);
begin
if Value>=FMaximum then
   raise TimeGraphException.Create('Minimum must be smaller than maximum')
else FMinimum:=Value
end;

procedure TTimeGraph.FSetMaximum(const Value:double);
begin
if Value<=FMinimum then
   raise TimeGraphException.Create('Maximum must be greater than minimum')
else FMaximum:=Value
end;

procedure Register;
begin
RegisterComponents('TimeGraph',[TTimeGraph]);
end;

end.
