// MyOle ver. 0.91
// Author: Matej Trampus, Slovenija
// E-MAIL:
//   - Matej.Trampus@kiss.uni-lj.si
// See readme.txt for details.
//

unit TestObjects;

interface
uses myOle,graphics,ole2,windows,classes;

type

  TOleCircle=class(TDelphiOle)
    public
      class function  GetCLSID:TCLSID; override;
      class function  GetUserType:string; override;
      class function  GetProgID:string; override;
      procedure   Draw(dwDrawAspect: Longint;const bounds:TRect;Canvas:TCanvas); override;
      procedure   Save(str:TStream); override;
      procedure   Load(str:TStream); override;
      function    DoVerb(iVerb: Longint): HResult; override;
      function    GetExtent(dwDrawAspect: Longint):TPoint; override; // result in HIMETRIC
  end;


  TTestOle=class(TDelphiOle)
    private
      fIcon:TIcon;  //we could use DVASPECT_ICON but using VCL is much easier
      fShowAsIcon:boolean;
      procedure AssignToContainerFont(DestFont:TFont);
    protected
      data:string; // data v property, on change klici viewChanged
      procedure   SetShowAsIcon(asIcon:boolean);
      procedure   SetIcon(newIcon:TIcon);
    public
      class function  GetCLSID:TCLSID; override;
      class function  GetUserType:string; override;
      class function  GetProgID:string; override;

      constructor Create; override;
      destructor  destroy; override;
      procedure   Draw(dwDrawAspect: Longint;const bounds:TRect;Canvas:TCanvas); override;
      procedure   Save(str:TStream); override;
      procedure   Load(str:TStream); override;
      function    DoVerb(iVerb: Longint): HResult; override;
      function    GetExtent(dwDrawAspect: Longint):TPoint; override;
      property    showAsIcon:boolean read fShowAsIcon write setShowAsIcon;
      property    Icon:TIcon read fIcon write SetIcon;
  end;


implementation
uses controls,ComCtrls,debugMemo1,dialogs;

////////////////////////////////////////////////////////////////////////////////
// TOleCircle
////////////////////////////////////////////////////////////////////////////////
procedure TOleCircle.Draw(dwDrawAspect: Longint;const bounds:TRect;Canvas:TCanvas);
begin
  canvas.pen.color:=clBlue;
  canvas.brush.color:=clBlue;
  canvas.brush.style:=bsFDiagonal;
  with bounds do canvas.ellipse(left,top,right,bottom);
end;

procedure TOleCircle.Save(str:TStream);
begin end;

procedure TOleCircle.Load(str:TStream);
begin end;

function  TOleCircle.DoVerb(iVerb: Longint): HResult;
begin
  result:=S_OK;
end;

function  TOleCircle.GetExtent(dwDrawAspect: Longint):TPoint;  // result in HIMETRIC
begin
  result.x:=500; result.y:=500;
end;

class function TOleCircle.GetCLSID:TCLSID;
// {7CA25701-0DFB-11CF-85B0-444553540000}
const CIRCLE_CLSID:TGUID = (D1:$7CA25701;D2:$0DFB;D3:$11CF;D4:($85,$B0,$44,$45,$53,$54,$00,$00));
begin result:=CIRCLE_CLSID; end;

class function  TOleCircle.GetUserType:string; begin result:='OleDemo Circle example'; end;
class function  TOleCircle.GetProgID:string; begin result:='OleDemo.'+className; end;

////////////////////////////////////////////////////////////////////////////////
// TTestOle
////////////////////////////////////////////////////////////////////////////////

constructor TTestOle.Create;
begin
  inherited Create; data:='Delphi OLE!';
  fIcon:=TIcon.Create;
end;

destructor TTestOle.destroy;
begin
  fIcon.Free;
  inherited Destroy;
end;

class function  TTestOle.GetCLSID:TCLSID;
const MY_CLSID:TGUID = (D1:$42BFA701;D2:$EC57;D3:$0000;D4:($C0,$00,$00,$00,$00,$00,$00,$46));
begin  result:=MY_CLSID; end;

class function  TTestOle.GetUserType:string; begin result:='OleDemo text/icon example'; end;
class function  TTestOle.GetProgID:string; begin result:='OleDemo.'+className; end;


procedure TTestOle.AssignToContainerFont(DestFont:TFont);
var win:TWinControl;
begin
  win:=owner.ContainerControl;
  if win is TRichEdit then destFont.Assign(TRichEdit(win).font);
  // we use the same font as our Delphi container uses.
  // BTW: font property is declared in TWinControl, but is inaccessible, because it is protected
end;

procedure TTestOle.Draw(dwDrawAspect: Longint;const bounds:TRect;Canvas:TCanvas);

begin
  if fShowAsIcon then begin
    canvas.draw(bounds.left,bounds.top,icon);
  end else begin
    assignToContainerFont(canvas.font); // use same font as container
    canvas.font.style:=canvas.font.style+[fsUnderline];
    canvas.TextOut(bounds.left,bounds.top,data);
 end;
end;

procedure TTestOle.Save(str:TStream);
  procedure WriteInt(l:integer);
  begin str.writeBuffer(l,sizeof(l)); end;
var l:integer;
begin
  l:=length(data);
  writeInt(l); str.writeBuffer(PChar(data)^,l);
  // write icon data only if not empty
  if icon.empty then writeInt(0) else begin writeInt(1); icon.SaveToStream(str); end;
end;

procedure TTestOle.Load(str:TStream);
  function ReadInt:integer;
  begin   str.ReadBuffer(result,sizeof(result)); end;
var l:integer;
begin
  l:=readInt; //str.ReadBuffer(l,sizeof(l));
  SetString(data, PChar(nil), l);
  str.ReadBuffer(PChar(data)^,l);
  showAsIcon:=data='';
  debug2(self,'Load: data='+data);
  if readInt=1 then icon.LoadFromStream(str);
end;

function TTestOle.GetExtent(dwDrawAspect: Longint):TPoint;
var cnv:TCanvas; dc:HDC;
begin
  if not fShowAsIcon then begin
    cnv:=TCanvas.Create;  // I AM NOT SHURE, THAT WHAT I AM DOING HERE IS LEGAL...
      assignToContainerFont(cnv.Font); // use same font as container (this line is correct)
      // we should properly modify font attributes here
      DC := GetDC(0);
        cnv.handle:=dc;
        result.x:=cnv.TextWidth(Data);
        result.y:=cnv.TextHeight(Data);
      ReleaseDC(0, DC);
    cnv.Free;      // ...(END OF SUSPICIOUS BLOCK)
    result:=PixelsToHiMetric(result);
  end else begin // showAsIcon
    result.x:=icon.width; result.y:=icon.height;
    result:=PixelsToHiMetric(result);
  end;
end;

function TTestOle.DoVerb(iVerb: Longint): HResult;
begin
  data:=inputBox(ClassName,'Enter empty string for displaying Icon',data);
  showAsIcon:=data='';
  viewChanged(DVASPECT_CONTENT);
  // potrebno je poklicati se advise za Data
  // flag dirty na TRUE ipd
  result:=S_OK;
end;

// property acces methods
procedure TTestOle.SetShowAsIcon(asIcon:boolean);
begin
  if asIcon<>fShowAsIcon then begin fShowAsIcon:=asIcon; viewChanged(DVASPECT_CONTENT); end;
end;

procedure TTestOle.SetIcon(newIcon:TIcon);
begin
  fIcon.Assign(newIcon); if fShowAsIcon then viewChanged(DVASPECT_CONTENT);
end;


end.
