unit trayicfm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls;

type
  TTrayIconForm = class(TForm)
    procedure FormClose(Sender: TObject; var Action:
      TCloseAction);
  private
    { Private declarations }
    fTrayIcon : hIcon;
    fTip      : String;
    fID       : Integer;
    fReady    : Boolean;
    procedure SetTrayIcon(value: hIcon);
    procedure SetTip(const value: String);
    function  ShellNotify(Msg, Flags: Integer): Boolean;
  protected
    procedure DefaultHandler(var Message); override;
  public
    { Public declarations }
    property TrayIcon : hIcon Write SetTrayIcon;
    property Tip      : String Write SetTip;
    procedure SetupTrayIcon(vID: Integer; vTrayIcon: hIcon;
      const vTip : String);
  end;

implementation
USES ShellApi, Unit1;
{$R *.DFM}

function TTrayIconForm.ShellNotify(Msg, Flags: Integer) : Boolean;
VAR tnd : TNotifyIconData;
BEGIN
  Result := False;
  IF fID = 0 THEN Exit;
  FillChar(tnd, SizeOf(tnd), 0);
  WITH tnd DO
    BEGIN
      Wnd := Self.Handle;
      uFlags := Flags;
      IF Msg = NIM_ADD THEN
        uFlags := uFlags OR NIF_MESSAGE OR NIF_ICON OR NIF_TIP;
      cbSize := SizeOf(tnd);
      uID := fID;
      uCallbackMessage:= WM_USER + fID;
      IF fTrayIcon <> 0 THEN hIcon:= fTrayIcon
      ELSE hIcon := LoadIcon(hInstance, IDI_WINLOGO);
      StrLCopy(szTip, PChar(fTip), 64);
      Result := Shell_NotifyIcon(msg, @tnd);
      IF Result THEN DestroyIcon(fTrayIcon);
    END;
END;

procedure TTrayIconForm.SetupTrayIcon(vID: Integer;
  vTrayIcon: hIcon; const vTip : String);
BEGIN
  fID       := vID;
  fTrayIcon := vTrayIcon;
  fTip      := vTip;
  fReady    := True;
  ShellNotify(NIM_ADD, NIF_TIP OR NIF_MESSAGE OR NIF_ICON);
END;

procedure TTrayIconForm.DefaultHandler(var Message);
VAR CP : TPoint;
begin
  Inherited DefaultHandler(Message);
  WITH TMessage(Message) DO
    IF (fID <> 0) AND (Msg = WM_USER + fID) AND (wParam = fID) THEN
      CASE lParam OF
        WM_RBUTTONDOWN : IF Assigned(PopupMenu) THEN
          BEGIN
            SetForegroundWindow(Handle);
            GetCursorPos(CP);
            PopupMenu.Popup(CP.X, CP.Y);
            PostMessage(Handle, WM_USER, 0, 0);
          END
        ELSE Close;
      END;
end;

procedure TTrayIconForm.SetTrayIcon(value: hIcon);
begin
  IF NOT fReady THEN Exit;
  IF Value = fTrayIcon THEN Exit;
  fTrayIcon := Value;
  ShellNotify(NIM_MODIFY, NIF_ICON)
end;

procedure TTrayIconForm.SetTip(const value: String);
begin
  IF NOT fReady THEN Exit;
  IF Value = fTip THEN Exit;
  fTip := Value;
  ShellNotify(NIM_MODIFY, NIF_TIP)
end;

procedure TTrayIconForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ShellNotify(NIM_DELETE, 0);
end;

end.
