{--------------------------------------------------------------------
 Super Boolean Combo Box (SBCB)
 ==============================
 YSNCOMBO - Yes/No combo box control for Borland's Delphi product.
 v1.0 August 27, 1996 (for Delphi 1 and 2)
 Freeware!
---------------------------------------------------------------------
 Copyright  1996 Eric G.V. Fookes
 - All Rights Reserved -
 Address:  Av. Eugne-Pittard 22Ter, CH - 1206 Geneva, Switzerland
 Internet: fookes@sc2a.unige.ch
 Interested in my other components or updates? Visit my web page...
 http://www.unige.ch/sciences/terre/geologie/fookes/delphi.htm
---------------------------------------------------------------------}
unit YsnCombo;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

type
  TBoolFormat = (bfTrueFalse,bfYesNo,bfOnOff,bfByte);
  TBoolean = (bFalse,bTrue,bNull);

  TBoolComboBox = class(TComboBox)
    private
     FBoolFormat:TBoolFormat;
     FValue:TBoolean;
     FAllowNull:Boolean;
    protected
     procedure CnCommand(var Message: TWMCommand); message CN_COMMAND;
     procedure DoExit; override;
     procedure KeyPress(var Key: Char); override;
     procedure DblClick; override;
     procedure Change; override;
     procedure FillList(AFormat:TBoolFormat);
     procedure SetParent(AParent:TWinControl); override;
     procedure SetAllowNull(SetOn:Boolean);
     procedure SetBoolFormat(AFormat:TBoolFormat);
     procedure SetValue(AValue:TBoolean);
    public
     constructor Create(AOwner:TComponent); override;
     function ValueToText(AFormat:TBoolFormat):string;
     function IsTrue:Boolean;
     function IsNull:Boolean;
    published
     property AllowNull:Boolean read FAllowNull write SetAllowNull;
     property BoolFormat:TBoolFormat read FBoolFormat write SetBoolFormat
              default bfTrueFalse;
     property Value:TBoolean read FValue write SetValue
              default bFalse;
     property Width default 65;
  end;

function IndexOfStart(const SubString:string; AStringList:TStrings; IsSorted:Boolean):Integer;

procedure Register;

implementation

function IndexOfStart(const SubString:string; AStringList:TStrings; IsSorted:Boolean):Integer;
   function Dichotomic(SubString:string; AStringList:TStrings):Integer;
   var Left,Right,L,I:Integer; S:string; Found:Boolean;
   begin
      L := Length(SubString);
      Left := 0;
      Right := AStringList.Count-1;
      I := (Left+Right) div 2;
      SubString := Uppercase(SubString);
      S := Uppercase(Copy(AStringList.Strings[I],1,L));
      Found := S = SubString;
      while (not Found) and ((Right-Left) > 1) do
         begin
            if S < SubString then
               begin
                  Left := I;
                  I := (Succ(I)+Right) div 2;
               end
            else
               begin
                  Right := I;
                  I := (Pred(I)+Left) div 2;
               end;
            S := Uppercase(Copy(AStringList.Strings[I],1,L));
            Found := S = SubString;
         end;
      if Found then
         Result := I
      else
         Result := -1;
   end;
   function Sequential(SubString:string; AStringList:TStrings):Integer;
   var MaxIndex,L,I:Integer; Found:Boolean;
   begin
      SubString := Uppercase(SubString);
      L := Length(SubString);
      MaxIndex := AStringList.Count-1;
      I := 0;
      Found := False;
      while (not Found) and (I <= MaxIndex) do
         if Uppercase(Copy(AStringList.Strings[I],1,L)) = SubString then
            Found := True
         else
            Inc(I);
      if Found then
         Result := I
      else
         Result := -1;
   end;
begin
   if (SubString = '') or (AStringList = nil) or (AStringList.Count = 0) then
      Result := -1
   else if IsSorted then
      Result := Dichotomic(SubString,AStringList)
   else
      Result := Sequential(SubString,AStringList)
end;

constructor TBoolComboBox.Create(AOwner:TComponent);
begin
   inherited Create(AOwner);
   FBoolFormat := bfTrueFalse;
   FValue := bFalse;
   Width := 65;
end;

procedure TBoolComboBox.SetParent(AParent:TWinControl);
begin
   inherited SetParent(AParent);
   if AParent <> nil then FillList(FBoolFormat);
end;

procedure TBoolComboBox.CnCommand(var Message: TWMCommand);
var I:Integer;
begin
  if Message.NotifyCode = CBN_SELCHANGE then
     begin
        I := ItemIndex;
        if I = -1 then
           if FAllowNull then
              FValue := bNull
           else
              begin
                 ItemIndex := 0;
                 FValue := bFalse;
              end
        else
           FValue := TBoolean(ItemIndex);
     end;
  inherited;
end;

procedure TBoolComboBox.DoExit;
begin
   if Text = '' then
      if FAllowNull then
         begin
            if FValue <> bNull then
               begin
                  SetValue(bNull);
                  inherited Change;
               end;
         end
      else
         begin
            SetValue(bFalse);
            inherited Change;
         end;
   inherited DoExit;
end;

procedure TBoolComboBox.DblClick;
begin
   inherited DblClick;
   case ItemIndex of
     -1 : SetValue(bFalse);
      0 : SetValue(bTrue);
      1 : if FAllowNull then
             SetValue(bNull)
          else
             SetValue(bFalse);
   end;
   inherited Change;
end;

procedure TBoolComboBox.Change;
begin
   inherited Change;
   if (not FAllowNull) and (Text = '') then
      SetValue(bFalse);
end;

procedure TBoolComboBox.KeyPress(var Key: Char);
var NewString:string; I:Integer;
begin
   Inherited KeyPress(Key);
   if (Key > #31) and (Key <> #127) then
      begin
         NewString := Text;
         if SelLength > 0 then
            Delete(NewString,SelStart+1,SelLength);
         Insert(Key,NewString,SelStart+1);
         I := IndexOfStart(NewString,Items,Sorted);
         Key := #0;
         if I = -1 then
            MessageBeep(MB_ICONEXCLAMATION)
         else
            begin
               ItemIndex := I;
               SetValue(TBoolean(I));
               inherited Change;
            end;
      end;
end;

procedure TBoolComboBox.FillList(AFormat:TBoolFormat);
begin
   Text := '';
   Items.Clear;
   with Items do
      case AFormat of
         bfTrueFalse :
            begin
               Add('False');
               Add('True');
               MaxLength := 5;
            end;
         bfYesNo :
            begin
               Add('No');
               Add('Yes');
               MaxLength := 3;
            end;
         bfOnOff :
            begin
               Add('Off');
               Add('On');
               MaxLength := 3;
            end;
         bfByte :
            begin
               Add('0');
               Add('1');
               MaxLength := 1;
            end;
      end;
   if FValue = bNull then
      ItemIndex := -1
   else
      ItemIndex := Ord(FValue);
end;

procedure TBoolComboBox.SetAllowNull(SetOn:Boolean);
begin
   if FAllowNull <> SetOn then
      begin
         FAllowNull := SetOn;
         if FAllowNull and (ItemIndex = 0) and (csDesigning in ComponentState) then
            SetValue(bNull)
         else
         if (not FAllowNull) and (ItemIndex = -1) then
            SetValue(bFalse);
      end;
end;

procedure TBoolComboBox.SetBoolFormat(AFormat:TBoolFormat);
begin
   if FBoolFormat <> AFormat then
      begin
         FBoolFormat := AFormat;
         FillList(FBoolFormat);
      end;
end;

procedure TBoolComboBox.SetValue(AValue:TBoolean);
begin
   if (AValue = bNull) and not FAllowNull then
      Exit;
   FValue := AValue;
   Text := '';
   if FValue = bNull then
      ItemIndex := -1
   else
      ItemIndex := Ord(FValue);
end;

function TBoolComboBox.ValueToText(AFormat:TBoolFormat):string;
begin
   if FAllowNull and (FValue = bNull) then
      Result := ''
   else
      case AFormat of
         bfTrueFalse :
            begin
               if FValue = bTrue then
                  Result := 'True'
               else
                  Result := 'False';
            end;
         bfYesNo :
            begin
               if FValue = bTrue then
                  Result := 'Yes'
               else
                  Result := 'No';
            end;
         bfOnOff :
            begin
               if FValue = bTrue then
                  Result := 'On'
               else
                  Result := 'Off';
            end;
         bfByte :
            begin
               if FValue = bTrue then
                  Result := '1'
               else
                  Result := '0';
            end;
      end;

end;

function TBoolComboBox.IsTrue:Boolean;
begin
   Result := FValue = bTrue;
end;

function TBoolComboBox.IsNull:Boolean;
begin
   Result := FAllowNull and (FValue = bNull) or (Text = '');
end;


procedure Register;
begin
  RegisterComponents('Samples', [TBoolComboBox]);
end;

end.
