unit Rprev;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, Printers, Dialogs, ExtCtrls;

type
  TRyPrinter = class;
  TRyPreviewFrm = class(TForm)
    Panel1: TPanel;
    PrintBtn: TButton;
    NextBtn: TButton;
    LastBtn: TButton;
    FirstBtn: TButton;
    PriorBtn: TButton;
    Button6: TButton;
    Panel2: TPanel;
    Button2: TButton;
    Button7: TButton;
    sb: TScrollBox;
    PaintBox1: TPaintBox;
    Button1: TButton;
    Button3: TButton;
    Button4: TButton;
    procedure PrintBtnClick(Sender: TObject);
    procedure FirstBtnClick(Sender: TObject);
    procedure PriorBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure LastBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure ShwGrdBtnClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
    Ry : TRyPrinter;
    PageDisplaying : Integer;
  public
    { Public declarations }
  end;

  TRyPrinter = class(TObject)
  private
    { Private declarations }
    FCurrentPage : Integer;
    FCanvases : TList;
    FDonePrinting : Boolean;
    FMetafiles : TList;
    FRyPreviewForm : TRyPreviewFrm;
    FOrientation : TPrinterOrientation;

    function GetPageCount : Integer;
    procedure SetCurrentPage(Index : Integer);
    function GetCanvas(Index : Integer): TMetafileCanvas;
    function GetMetafile(Index : Integer): TMetafile;
    function GetFont : TFont;
    procedure SetFont(Value : TFont);
    function GetPen : TPen;
    procedure SetPen(Value : TPen);
    procedure SetOrientation(Value : TPrinterOrientation);
  protected
    { Protected declarations }
  public
    { Public declarations }
    pXppi : Integer;
    pYppi : Integer;

    XOff : Integer;
    YOff : Integer;
    Title : String;

    function PageSize : TPoint;
    function PrintSize : TPoint;
    procedure PrintPage(const PageNum : Integer);
    procedure PrintIt;
    procedure Preview;
    procedure SaveToFile(const TheFile : String);

    procedure DisplayPage(Page : Integer);

    function XInch(const aSingle : Single): Integer;
    function YInch(const aSingle : Single): Integer;

    procedure TextOut(const X, Y : Integer; const Text : String);
    procedure TextOutRight(const X, Y : Integer; const Text : String);
    procedure TextOutCenter(const X, Y : Integer; const Text : String);
    procedure ExtTextOut(const X, Y : Integer; const Right : Integer;
      const Text : String);
    procedure ExtTextOutRight(const X, Y : Integer; const Left : Integer;
      const Text : String);
    procedure ExtTextOutCenter(const X, Y : Integer; const Left, Right : Integer;
      const Text : String);
    procedure TextRect(const aRect : TRect; const X, Y : Integer;
      const Text : String);
    function TextWidth(const Text : String): Integer;
    function TextHeight: Integer;
    procedure Line(const a, b : TPoint);
    procedure Rectangle(const aRect : TRect);
    function MemoOut(const aRect : TRect; p : PChar; DisposePChar : Boolean;
      DontPrint : Boolean): Integer;
    procedure PutPageNums(const X, Y : Integer; Alignment : TAlignment);

    procedure NewJob;
    procedure DonePrinting;
    function NewPage : Integer;
    constructor Create;
    destructor Destroy; override;

    property PageCount : Integer  read GetPageCount;
    property CurrentPage : Integer read FCurrentPage write
      SetCurrentPage;
    property Canvases[Index : Integer] : TMetafileCanvas read GetCanvas;
    property Metafiles[Index : Integer] : TMetafile read GetMetafile;
    property Font : TFont read GetFont write SetFont;
    property Pen : TPen read GetPen write SetPen;
    property Orientation : TPrinterOrientation read FOrientation write
      SetOrientation;
  end;

implementation

{$R *.DFM}

uses cd2html;

function TRyPrinter.GetPageCount : Integer;
begin
Result := FCanvases.Count;
end;

procedure TRyPrinter.SetCurrentPage(Index : Integer);
begin
if (Index <= PageCount) AND (Index > 0) then
  FCurrentPage := Index;
end;

function TRyPrinter.GetCanvas(Index : Integer): TMetafileCanvas;
begin
if (Index > 0) AND (Index <= PageCount) then
  Result := TMetafileCanvas(FCanvases[Index - 1]) else
  Result := nil;
end;

function TRyPrinter.GetMetafile(Index : Integer): TMetafile;
begin
if (Index > 0) AND (Index <= PageCount) AND FDonePrinting then
  Result := TMetafile(FMetafiles[Index - 1]) else
  Result := nil;
end;

function TRyPrinter.GetFont : TFont;
begin
Result := Canvases[CurrentPage].Font;
end;

procedure TRyPrinter.SetFont(Value : TFont);
begin
Canvases[CurrentPage].Font := Value;
end;

function TRyPrinter.GetPen : TPen;
begin
Result := Canvases[CurrentPage].Pen;
end;

procedure TRyPrinter.SetPen(Value : TPen);
begin
Canvases[CurrentPage].Pen := Value;
end;

procedure TRyPrinter.SetOrientation(Value : TPrinterOrientation);
begin
FOrientation := Value;
Printer.Orientation := Value;
end;

function TRyPrinter.PageSize : TPoint;
begin
Escape(Printer.Handle, GETPHYSPAGESIZE, 0, nil, @Result);
end;

function TRyPrinter.PrintSize : TPoint;
begin
Result.X := Printer.PageWidth;
Result.Y := Printer.PageHeight;
end;

procedure TRyPrinter.PrintPage(const PageNum : Integer);
var
 s : String;
begin
if not FDonePrinting then DonePrinting;
if (PageNum > 0) AND (PageNum <= PageCount) then begin
  Printer.Title := Title;
  Printer.BeginDoc;
  if Assigned(FRyPreviewForm) then s := FRyPreviewForm.Panel2.Caption;
  try
  if Assigned(FRyPreviewForm) then
    FRyPreviewForm.Panel2.Caption := Format('Printing page %d of %d',
      [PageNum, PageCount]);
  Printer.Canvas.StretchDraw(
    Rect(0,0,Printer.PageWidth, Printer.PageHeight), Metafiles[PageNum]);
  finally
  Printer.EndDoc;
  if Assigned(FRyPreviewForm) then FRyPreviewForm.Panel2.Caption := s;
  end;
  end;
end;

procedure TRyPrinter.PrintIt;
var
 i : Integer;
 s : String;
begin
if not FDonePrinting then DonePrinting;
if PageCount > 0 then begin
  Printer.Title := Title;
  if not Printer.Printing then Printer.BeginDoc;
  i := 1;
  if Assigned(FRyPreviewForm) then
    s := FRyPreviewForm.Panel2.Caption;
  try
  if Assigned(FRyPreviewForm) then
    FRyPreviewForm.Panel2.Caption := Format('Printing page %d of %d',
      [1, PageCount]);
  Printer.Canvas.StretchDraw(
    Rect(0,0,Printer.PageWidth, Printer.PageHeight), Metafiles[i]);
  for i := 2 to PageCount do begin
    if Assigned(FRyPreviewForm) then
      FRyPreviewForm.Panel2.Caption := Format('Printing page %d of %d',
        [i, PageCount]);
    Printer.NewPage;
    Printer.Canvas.StretchDraw(
      Rect(0,0,Printer.PageWidth, Printer.PageHeight), Metafiles[i]);
    end;
  finally
  Printer.EndDoc;
  if Assigned(FRyPreviewForm) then
    FRyPreviewForm.Panel2.Caption := s;
  end;
  end;
end;

procedure TRyPrinter.Preview;
begin
if not FDonePrinting then DonePrinting;
FRyPreviewForm := TRyPreviewFrm.Create(Application);
FRyPreviewForm.Ry := Self;
FRyPreviewForm.ShowModal;
FRyPreviewForm.Release;
FRyPreviewForm := nil;
end;

procedure TRyPrinter.SaveToFile(const TheFile : String);
var
 i : integer;
begin
if FDonePrinting then
  for i := 1 to PageCount do begin
    Metafiles[i].SaveToFile(Format('%6s%.2d', [TheFile, i]));
    end;
end;

procedure TRyPrinter.DisplayPage(Page : Integer);
var
  p : TPoint;
  Sc : Single;
  r : TRect;
  i : integer;
begin
if (Page > 0) AND (Page <= PageCount) then begin
  Sc := FRyPreviewForm.PaintBox1.Width / PageSize.X;

  FRyPreviewForm.PaintBox1.Canvas.Rectangle(
    0, 0, FRyPreviewForm.PaintBox1.Width,
    FRyPreviewForm.PaintBox1.Height);
  FRyPreviewForm.PaintBox1.Canvas.FillRect(
    Rect(1, 1, FRyPreviewForm.PaintBox1.Width - 2,
    FRyPreviewForm.PaintBox1.Height - 2));

  Escape(Printer.Handle, GETPRINTINGOFFSET, 0, nil, @p);

  r.Left := Trunc(p.X * Sc);
  r.Top := Trunc(p.Y * Sc);
  r.Right := r.Left + Trunc(Printer.PageWidth * Sc);
  r.Bottom := r.Top + Trunc(Printer.PageHeight * Sc);

  if FRyPreviewForm.Button4.Caption[1] = 'H' then begin
    with FRyPreviewForm.PaintBox1.Canvas do begin
      Pen.Style := psDash;
      Rectangle(r.Left, r.Top, r.Right, r.Bottom);
      for i := 1 to Self.PageSize.X div Self.pXppi do begin
        MoveTo(Trunc(i * Self.pXppi * Sc), 0);
        LineTo(Trunc(i * Self.pXppi * Sc), Trunc(PageSize.Y * Sc));
        end;
      for i := 1 to Self.PageSize.Y div Self.pYppi do begin
        MoveTo(0, Trunc(i * Self.pYppi * Sc));
        LineTo(Trunc(PageSize.X * Sc), Trunc(i * Self.pYppi * Sc));
        end;
      Pen.Style := psSolid;
      end;
    end;

  FRyPreviewForm.PaintBox1.Canvas.StretchDraw(r, Metafiles[Page]);

  FRyPreviewForm.Panel2.Caption :=
    Format('Page %d of %d', [Page, PageCount]);

  FRyPreviewForm.PageDisplaying := Page;

  if Page = 1 then begin
    FRyPreviewForm.FirstBtn.Enabled := False;
    FRyPreviewForm.PriorBtn.Enabled := False;
    end else begin
    FRyPreviewForm.FirstBtn.Enabled := True;
    FRyPreviewForm.PriorBtn.Enabled := True;
    end;

  if PageCount > Page then begin
    FRyPreviewForm.NextBtn.Enabled := True;
    FRyPreviewForm.LastBtn.Enabled := True;
    end else begin
    FRyPreviewForm.NextBtn.Enabled := False;
    FRyPreviewForm.LastBtn.Enabled := False;
    end;
  end;
end;

function TRyPrinter.XInch(const aSingle : Single): Integer;
begin
Result := Trunc(pXppi * aSingle);
end;

function TRyPrinter.YInch(const aSingle : Single): Integer;
begin
Result := Trunc(pYppi * aSingle);
end;

procedure TRyPrinter.TextOut(const X, Y : Integer; const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextOut(X - XOff, Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.TextOutRight(const X, Y : Integer; const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextOut(X - Self.TextWidth(Text) - XOff, Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.TextOutCenter(const X, Y : Integer; const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextOut(X - (Self.TextWidth(Text) div 2) - XOff, Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.ExtTextOut(const X, Y, Right : Integer;
 const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextRect(Rect(X - XOff, Y - YOff, Right - XOff,
      Y + (Self.TextHeight * 2) - YOff), X - XOff, Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.ExtTextOutRight(const X, Y, Left : Integer;
 const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextRect(Rect(Left - XOff, Y - YOff, X - XOff,
      Y + (Self.TextHeight * 2) - YOff), X - Self.TextWidth(Text) - XOff,
      Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.ExtTextOutCenter(const X, Y, Left, Right : Integer;
 const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextRect(Rect(Left - XOff, Y - YOff, Right - XOff,
      Y + (Self.TextHeight * 2) - YOff), X - (Self.TextWidth(Text) div 2) - XOff,
      Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.TextRect(const aRect : TRect; const X, Y : Integer;
  const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextRect(Rect(aRect.Left - XOff, aRect.Top - YOff, aRect.Right - XOff,
      aRect.Bottom - YOff), X - XOff, Y - YOff, Text);
    end;
  end;
end;

function TRyPrinter.TextWidth(const Text : String): Integer;
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  Printer.Canvas.Font := Canvases[FCurrentPage].Font;
  Result := Printer.Canvas.TextWidth(Text);
  end else Result := -1;
end;

function TRyPrinter.TextHeight: Integer;
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  Printer.Canvas.Font := Canvases[FCurrentPage].Font;
  Result := Printer.Canvas.TextHeight('X');
  end else Result := -1;
end;

procedure TRyPrinter.Line(const a, b : TPoint);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    MoveTo(a.X - XOff, a.Y - YOff);
    LineTo(b.X - XOff, b.Y - YOff);
    end;
  end;
end;

procedure TRyPrinter.Rectangle(const aRect : TRect);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    Rectangle(aRect.Left - XOff, aRect.Top - YOff, aRect.Right - XOff,
       aRect.Bottom - YOff);
    end;
  end;
end;

function TRyPrinter.MemoOut(const aRect : TRect; p : PChar; DisposePChar : Boolean;
  DontPrint : Boolean): Integer;
var
  r : TRect;
begin
r := aRect;
Dec(r.Left, XOff);
Dec(r.Right, XOff);
Dec(r.Top, YOff);
Dec(r.Bottom, YOff);
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  if DontPrint then
    Result := DrawText(
      Canvases[FCurrentPage].Handle,
      p, Length(p), R, DT_CALCRECT or DT_EXPANDTABS or
        DT_NOPREFIX or DT_WORDBREAK or DT_NOCLIP) else
    Result := DrawText(
      Canvases[FCurrentPage].Handle,
      p, Length(p), R, DT_EXPANDTABS or
        DT_NOPREFIX or DT_WORDBREAK or DT_NOCLIP);
  end;

if DisposePChar then StrDispose(p);
end;

procedure TRyPrinter.PutPageNums(const X, Y : Integer;
  Alignment : TAlignment);
var
  i : Integer;
  s : String;
  o : Integer;
begin
o := CurrentPage;
for i := 1 to PageCount do begin
  s := Format('Page %d of %d', [i, PageCount]);
  CurrentPage := i;
  case Alignment of
    taLeftJustify : TextOut(X, Y, s);
    taRightJustify : TextOutRight(X, Y, s);
    taCenter : TextOutCenter(X, Y, s);
    end;
  end;
CurrentPage := o;
end;

procedure TRyPrinter.NewJob;
var
 i : Integer;
 p : TPoint;
begin
for i := FMetafiles.Count - 1 downto 0 do begin
  TMetafile(FMetafiles[i]).Free;
  end;
FCanvases.Clear;
FMetafiles.Clear;
FCurrentPage := 0;
FDonePrinting := False;
Escape(Printer.Handle, GETPRINTINGOFFSET, 0, nil, @p);
XOff := p.X;
YOff := p.Y;
pXppi := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
pYppi := GetDeviceCaps(Printer.Handle, LOGPIXELSY);
Printer.BeginDoc;
NewPage;
end;

procedure TRyPrinter.DonePrinting;
var
  i : integer;
begin
if not FDonePrinting then begin
  for i := 1 to PageCount do Canvases[i].Free;
  FDonePrinting := True;
  end;
end;

function TRyPrinter.NewPage : Integer;
begin
if not FDonePrinting then begin
  Result := FMetafiles.Add(TMetafile.Create) + 1;
  FCanvases.Add(TMetafileCanvas.Create(TMetafile(FMetafiles[Result - 1]),
     Printer.Canvas.Handle));
  FCurrentPage := Result;
//these aren't needed in EMF's
//  SetMapMode(Canvases[Result].Handle, MM_ANISOTROPIC);
//  SetWindowExtEx(Canvases[Result].Handle, Printer.PageWidth,
//    Printer.PageHeight, nil);
  with Canvases[Result] do begin
    Brush.style := bsClear;
    Font.Name := 'Arial';
    Font.PixelsPerInch := pYppi;
    Font.Size := 10;
    end;
  end;
end;

constructor TRyPrinter.Create;
begin
inherited Create;
FCanvases := TList.Create;
FMetafiles := TList.Create;
FCurrentPage := 0;
FDonePrinting := False;
FOrientation := poPortrait;
end;

destructor TRyPrinter.Destroy;
var
 i : Integer;
begin
for i := FMetafiles.Count - 1 downto 0 do begin
  TMetafile(FMetafiles[i]).Free;
  end;
FCanvases.Free;
FMetafiles.Free;
if Printer.Printing then Printer.Abort;
inherited Destroy;
end;

{***************** this is for the preview form ****************}

procedure TRyPreviewFrm.PrintBtnClick(Sender: TObject);
begin
Ry.PrintIt;
end;

procedure TRyPreviewFrm.FirstBtnClick(Sender: TObject);
begin
Ry.DisplayPage(1);
end;

procedure TRyPreviewFrm.PriorBtnClick(Sender: TObject);
begin
Ry.DisplayPage(PageDisplaying - 1);
end;

procedure TRyPreviewFrm.NextBtnClick(Sender: TObject);
begin
Ry.DisplayPage(PageDisplaying + 1);
end;

procedure TRyPreviewFrm.LastBtnClick(Sender: TObject);
begin
Ry.DisplayPage(Ry.PageCount);
end;

procedure TRyPreviewFrm.FormCreate(Sender: TObject);
begin
PageDisplaying := 1;
Button7Click(Self);
end;

procedure TRyPreviewFrm.Button7Click(Sender: TObject);
begin
PaintBox1.Visible := False;
PaintBox1.Top := 15;
PaintBox1.Height := Sb.height - 30;
PaintBox1.Width := (Longint(PaintBox1.Height) * Longint(Ry.PageSize.X)) div
   Longint(Ry.PageSize.Y);
PaintBox1.Left := (Width div 2) - (PaintBox1.Width div 2);
PaintBox1.Visible := True;
end;

procedure TRyPreviewFrm.PaintBox1Paint(Sender: TObject);
begin
Ry.DisplayPage(PageDisplaying);
end;

procedure TRyPreviewFrm.Button2Click(Sender: TObject);
begin
PaintBox1.Visible := False;
PaintBox1.Top := Panel1.Height + 15;
PaintBox1.Left := 15;
PaintBox1.Width := ClientWidth - 45;
PaintBox1.Height := (Longint(PaintBox1.Width) * Longint(Ry.PageSize.Y)) div
   Longint(Ry.PageSize.X);
PaintBox1.Visible := True;
end;

procedure TRyPreviewFrm.ShwGrdBtnClick(Sender: TObject);
begin
if Button4.Caption[1] = 'H' then
  Button4.Caption := 'Show Grid' else
  Button4.Caption := 'Hide Grid';
PaintBox1.Refresh;
end;

procedure TRyPreviewFrm.Button1Click(Sender: TObject);
var
 s : String;
begin
s := '1';
if InputQuery('Print Page', 'Enter page number to print', s) then begin
  StrToInt(s);
  Ry.PrintPage(StrToInt(s));
  end;
end;

procedure TRyPreviewFrm.PaintBox1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 i : Integer;
begin
case Button of
  mbLeft : begin
    PaintBox1.Visible := False;
    PaintBox1.Top := (ClientHeight div 2) - Y;
    PaintBox1.Left := (ClientWidth div 2) - X;
    i := Trunc(PaintBox1.Width * 0.35) div 2;
    PaintBox1.Width := PaintBox1.Width + i;
    PaintBox1.Left := PaintBox1.Left - i;
    i := Trunc(PaintBox1.Height * 0.35) div 2;
    PaintBox1.Height := PaintBox1.Height + i;
    PaintBox1.Top := PaintBox1.Top - i;
    PaintBox1.Visible := True;
    end;
  mbRight : begin
    PaintBox1.Visible := False;
    i := Trunc(PaintBox1.Width * 0.35) div 2;
    PaintBox1.Width := PaintBox1.Width - i;
    PaintBox1.Left := PaintBox1.Left + i;
    i := Trunc(PaintBox1.Height * 0.35) div 2;
    PaintBox1.Height := PaintBox1.Height - i;
    PaintBox1.Top := PaintBox1.Top + i;
    PaintBox1.Top := (ClientHeight div 2) - Y;
    PaintBox1.Left := (ClientWidth div 2) - X;
    PaintBox1.Visible := True;
    end;
  end;
end;

procedure TRyPreviewFrm.Button3Click(Sender: TObject);
var
  s : String;
begin
s := 'REPORT';
if InputQuery('Save Page as EMF', 'Enter filename', s) then
  Ry.SaveToFile(s);
end;

end.
