// MyOle ver. 0.90
// Author: Matej Trampus, Slovenija
// E-MAIL:
//   - Matej.Trampus@kiss.uni-lj.si
// See readme.txt for details.
//
// HISTORY:
//  25.10.1996
//   - Added notification, when debugMemo gets destroyed, so we do not have
//     problems when application is shutting down. Added SetDebugMemo procedure
//   - now, we do not close stream in finalization setion of DebugMemo1.pas, so
//     that we can also log debug output from other finalization sections
//     (windows? or will close files for us???)

unit debugMemo1;
// TO DO:
// - Create TCOmponent, call _debugMemo.FreeNotification to ensure, that we are notifed
//   when _debugMemo is going to be destroyed... !!! (now we are checking only for csDestroying)

interface

uses stdctrls,classes,consts;
{xx$DEFINE DIRECTWRITE} // write-through debug information

procedure SetDebugMemo(newMemo:TMemo);
procedure Debug(s:string);
procedure Debug2(p:TObject;const s:string);

{type TWriteThroughStream=class(TFileStream) // not used
       constructor Create(const FileName: string; Mode: Word);
     end;
}

implementation
uses sysutils,windows,myOle,dialogs;

Type TNotifyComponent=class(TComponent)
       procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     end;

var fs:THandleStream;
    buffer:string;
    designing:boolean;
   _debugMemo:Tmemo;
   notifyComponent:TNotifyComponent;

procedure TNotifyComponent.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent,Operation);
  if (aComponent=_debugMemo) and (Operation=opRemove) then _debugMemo:=nil;

end;

procedure SetDebugMemo(newMemo:TMemo);
begin
  if newMemo<>nil then newMemo.FreeNotification(notifyComponent); // when memo will be destroyed, we'll be notified
  _debugMemo:=newMemo;
end;

procedure Debug(s:string);
var s2:string[200];
begin
 if designing then exit;
 s2:=s; s:=s+#13#10; // add CR/LF
 if (_debugMemo<>nil) and (csDestroying in _debugMemo.ComponentState) then SetDebugMemo(nil);


 if (_debugMemo=nil) or ((_debugMemo<>nil) and not _debugMemo.HandleAllocated) then begin // memo not allocated
   buffer:=buffer+s; // buffer data for displaying in Memo
   if fs<>nil then fs.WriteBuffer(s[1],length(s)); // direct write to file
 end else begin
   // only write to memo if it has handle allocated. We must not force lcreation of momo's handle
   // if we do, we run into problems when we are calling debug() from CreateHandle procedure (VCl uses some global variables)
   if buffer<>'' then begin _debugMemo.Lines.Add(buffer); buffer:=''; end; // write buffered data to Memo
   _debugMemo.Lines.Add(s2);
   fs.WriteBuffer(s[1],length(s));
 end;
end;

procedure Debug2(p:TObject;const s:string);
begin
  if p=nil then Debug('*NIL*: '+s)
           else if p is TMyOle then Debug(format('%s(%d):  %s',[p.ClassName,TMyOle(p).GetObjHandle,s]))
                               else Debug(format('%s(%p):  %s',[p.ClassName,pointer(p),s]))
end;


function FileCreateWT(const FileName: string): Integer;
begin
  Result := CreateFile(PChar(FileName), GENERIC_READ or GENERIC_WRITE,
    FILE_SHARE_READ, nil, CREATE_ALWAYS,
     {$IFDEF DIRECTWRITE} FILE_FLAG_WRITE_THROUGH {$ELSE} FILE_ATTRIBUTE_NORMAL {$ENDIF}, 0);
end;

function CreateWriteThroughStream(const filename:string):THandleStream;
var NewHandle:integer;
begin
 newHandle := FileCreateWT(FileName);
  if newHandle < 0 then
    raise EFCreateError.CreateResFmt(SFCreateError, [FileName]);
  result:=THandleStream.Create(NewHandle); // set handle

end;

(*
constructor TWriteThroughStream.Create(const FileName: string; Mode: Word);
var NewHandle:integer;
begin
  if Mode = fmCreate then begin
    newHandle := FileCreateWT(FileName);
    if newHandle < 0 then
      raise EFCreateError.CreateResFmt(SFCreateError, [FileName]);
    THandleStream.Create(NewHandle); // set handle
    // previus line sets handle to 0??????
  end else  begin
     abort; // not yet supported
{    FHandle := FileOpen(FileName, Mode);
    if FHandle < 0 then
      raise EFOpenError.CreateResFmt(SFOpenError, [FileName]);}
  end;
end;
*)

initialization
  //fs:=TWriteThroughStream {tFileStream}.Create('c:\brisi.txt',fmCreate);
  fs:=nil;
  notifyComponent:=TNotifyComponent.Create(nil);
  designing:=isLibrary; // COMPLIB.DCL is library ?
  if not designing then // log only at run-time
     fs:=CreateWriteThroughStream('c:\myole.log');
  buffer:='';
finalization
  // We do *not* close the stream, so that debug() will work also in other finalization
  // section
  //fs.Free; fs:=nil;
  notifyComponent.Free; notifyComponent:=nil;
end.
