unit gridsrch;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, DBGrids, forms,
  DBTables;

const
  dbgridSearsh_VK = VK_F3;

type
TDBGridSearch = class(TComponent)
private
  FDBGrid : TCustomDBGrid;
  FieldName : String;
  ObjectInstance : Pointer;
  FDBGridWndProcAdd : Pointer;

  procedure SetDBGrid(Value : TCustomDBGrid);
protected
  procedure DBGridWndProc(var Message: TMessage);
public
  constructor Create(AOwner : TComponent); override;
  destructor Destroy; override;
  procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  function Execute : Boolean;
published
  property DBGrid : TCustomDBGrid read FDBGrid write SetDBGrid;
  property DataField : string read FieldName write FieldName;
end;

implementation

uses grs_dlg;

{TDBGridSearch}
constructor TDBGridSearch.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  ObjectInstance := MakeObjectInstance(DBGridWndProc);
end;

destructor TDBGridSearch.Destroy;
begin
  if (ObjectInstance <> Nil) then
    FreeObjectInstance(ObjectInstance);

  inherited Destroy;
end;

procedure TDBGridSearch.DBGridWndProc(var Message: TMessage);
begin
  with TMessage(Message) do begin
    if (msg = WM_KEYDOWN) And (wparam = dbgridSearsh_VK) then
       Execute;
     Result := CallWindowProc(FDBGridWndProcAdd, FDBGrid.Handle, Msg, WParam, LParam);
   end;
end;

procedure TDBGridSearch.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if(AComponent = FDBGrid) And (Operation = opRemove) then
    FDBGrid := Nil;
end;

procedure TDBGridSearch.SetDBGrid(Value : TCustomDBGrid);
begin
  if(FDBGrid = Value) then exit;
  if(FDBGrid <> Nil) And (FDBGridWndProcAdd <> Nil) then begin
    SetWindowLong(FDBGrid.Handle, GWL_WNDPROC, LongInt(FDBGridWndProcAdd));
    FDBGridWndProcAdd := Nil;
  end;
  FDBGrid := Value;
  if(FDBGrid <> Nil) And  Not (csDesigning in FDBGrid.ComponentState) then begin
    FDBGridWndProcAdd := Pointer(GetWindowLong(FDBGrid.Handle, GWL_WNDPROC));
    SetWindowLong(FDBGrid.Handle, GWL_WNDPROC, LongInt(ObjectInstance));
  end;
end;

function TDBGridSearch.Execute : Boolean;
Var
  i : Integer;
  pUp, pDn : TPoint;
  form : TfdbgridSearchdlg;
  ds : TTable;
begin
  Result := False;
  if(TDBGrid(DBGrid).DataSource <> Nil) and (TDBGrid(DBGrid).DataSource.DataSet <> Nil) and
  (TDBGrid(DBGrid).DataSource.DataSet is TTable) and (TDBGrid(DBGrid).DataSource.DataSet.Active) and
  (TDBGrid(DBGrid).DataSource.DataSet.FindField(FieldName) <> Nil) then
    ds := TTable(TDBGrid(DBGrid).DataSource.DataSet)
  else  exit;

  Result := True;

  Form := TfdbgridSearchdlg.Create(Application);
  Form.Table := ds;
  Form.Field := ds.FindField(FieldName);

  form.Width := TDBGrid(DBGrid).Width;

  pDn.X := 0;
  pDn.Y := DBGrid.Height;
  pDn := TDBGrid(DBGrid).ClientToScreen(pDn);
  pUp.X := 0;
  pUp.Y := 0;
  pUp := TDBGrid(DBGrid).ClientToScreen(pUp);

  if(pDn.X > 0) then
    form.Left := pDn.X
  else form.Left := 0;
  if(pDn.Y < Screen.Height - form.Height) then
    form.Top := pDn.Y
  else begin
    if(pUp.Y > form.Height) then
      form.Top := pUp.Y - form.Height
    else form.Top := Screen.Height - form.Height;
  end;

  form.edit.Width := form.Panel.Width - form.panel.BorderWidth - 2*form.Edit.Left;
  form.ShowModal;

  form.free;
end;

end.
