unit gridsrch;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, DBGrids, forms,
  DB, Menus;
  
const
  dbgridSearsh_VK = VK_F3; 

type
TDBGridSearch = class(TComponent)
private
  FDBGrid : TCustomDBGrid;
  FCaseInsensitive : Boolean;
  ObjectInstance : Pointer;
  FDBGridWndProcAdd : Pointer;
  FHotKey: TShortCut;

  FieldName : String;

  procedure SetDBGrid(Value : TCustomDBGrid);
protected
  procedure DBGridWndProc(var Message: TMessage);
public
  constructor Create(AOwner : TComponent); override;
  destructor Destroy; override;
  procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  function Execute : Boolean;
published
  property CaseInsensitive : Boolean read FCaseInsensitive write FCaseInsensitive;
  property DataField : String read FieldName write FieldName;
  property DBGrid : TCustomDBGrid read FDBGrid write SetDBGrid;
  property HotKey: TShortCut read FHotKey write FHotKey;  
end;


implementation

uses grs_dlg;

{TDBGridSearch}
constructor TDBGridSearch.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  ObjectInstance := MakeObjectInstance(DBGridWndProc);
  FHotKey := dbgridSearsh_VK;
end;

destructor TDBGridSearch.Destroy;
begin
  if (ObjectInstance <> Nil) then
    FreeObjectInstance(ObjectInstance);
  inherited Destroy;
end;

procedure TDBGridSearch.DBGridWndProc(var Message: TMessage);
begin
  with TMessage(Message) do begin
    if (msg = WM_KEYDOWN) And (wparam = FHotKey) then
       Execute;
     Result := CallWindowProc(FDBGridWndProcAdd, FDBGrid.Handle, Msg, WParam, LParam);
   end;
end;


procedure TDBGridSearch.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if(AComponent = FDBGrid) And (Operation = opRemove) then
    FDBGrid := Nil;
end;

procedure TDBGridSearch.SetDBGrid(Value : TCustomDBGrid);
begin
  if(FDBGrid = Value) then exit;
  if(FDBGrid <> Nil) And (FDBGridWndProcAdd <> Nil) then begin
    SetWindowLong(FDBGrid.Handle, GWL_WNDPROC, LongInt(FDBGridWndProcAdd));
    FDBGridWndProcAdd := Nil;
  end;
  FDBGrid := Value;
  if(FDBGrid <> Nil) And  Not (csDesigning in FDBGrid.ComponentState) then begin
    FDBGridWndProcAdd := Pointer(GetWindowLong(FDBGrid.Handle, GWL_WNDPROC));
    SetWindowLong(FDBGrid.Handle, GWL_WNDPROC, LongInt(ObjectInstance));
  end;
end;


function TDBGridSearch.Execute : Boolean;
Var
  i : Integer;
  pUp, pDn : TPoint;
  form : TfdbgridSearchdlg;
  ds : TDataSet;
begin
  Result := False;
  if(TDBGrid(FDBGrid).DataSource <> Nil) and (TDBGrid(FDBGrid).DataSource.DataSet <> Nil) and
  (TDBGrid(FDBGrid).DataSource.DataSet.Active) and ((TDBGrid(FDBGrid).SelectedField <> Nil) or
  ((FieldName <> '') and (TDBGrid(FDBGrid).DataSource.DataSet.FindField(FieldName) <> Nil)))then
    ds := TDBGrid(FDBGrid).DataSource.DataSet
  else  exit;

  Result := True;

  Form := TfdbgridSearchdlg.Create(Application);
  Form.DataSet := ds;
  if(TDBGrid(FDBGrid).DataSource.DataSet.FindField(FieldName) <> Nil) then
    Form.Field := TDBGrid(FDBGrid).DataSource.DataSet.FindField(FieldName)
  else Form.Field := TDBGrid(FDBGrid).SelectedField;
  form.CaseInsensitive := CaseInsensitive;

  form.Width := 50;
  for i := 0 to TDBGrid(FDBGrid).Columns.Count - 1 do
    if(TDBGrid(FDBGrid).Columns[i].Field = Form.Field) then begin
      form.Width := TDBGrid(FDBGrid).Columns[i].Width + (form.panel.BorderWidth + form.Edit.Left)*2;
      break;
    end;
  if(form.Width < 50) then
    form.Width := 50;

  pDn.X := 0;
  pDn.Y := DBGrid.Height;
  pDn := DBGrid.ClientToScreen(pDn);
  pUp.X := 0;
  pUp.Y := 0;
  pUp := DBGrid.ClientToScreen(pUp);

  if(pDn.X > 0) then
    form.Left := pDn.X
  else form.Left := 0;
  if(pDn.Y < Screen.Height - form.Height) then
    form.Top := pDn.Y
  else begin
    if(pUp.Y > form.Height) then
      form.Top := pUp.Y - form.Height
    else form.Top := Screen.Height - form.Height;
  end;

  form.edit.Width := form.Panel.Width - form.panel.BorderWidth - 2*form.Edit.Left;
  form.ShowModal;

  form.free;
end;

end.
