unit uPentium;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, uMemObj;


type
  TPentiumTimer = class(TComponent)
  private
    { Private declarations }
    FActive : boolean;
    FM_time : integer;
    FbNoInterrupt : boolean;
    Fm_bColdCache : boolean;
    m_dwCount : DWORD;
    tmpMem : TmemBlock;

    FOnCacheHeath : TNotifyEvent;
    FOnTestFunction : TNotifyEvent;
    FOnEndtest : TNotifyEvent;

    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure SetActive(Value : boolean);
    procedure Chillcache;
    procedure HeathCache(mem : TMemBlock);
  protected
    { Protected declarations }
    m_startHi : Integer;
    m_StartLo : Integer;
    m_EndHi : Integer;
    m_EndLo : Integer;
  public
    { Public declarations }
    HotMemory : TList;
    function AllocateMemory(size : longint) : Pointer;
    procedure HeathVariable(var hvar; Size : integer);
    function GetPointer(index : integer) : pointer;
    function GetSize(index : integer) : longint;
    procedure FreeAll;
    procedure RunTest;
  published
    property Active : boolean read FActive write SetActive default false;
    property ColdCache : boolean read Fm_bColdCache write Fm_bColdCache default false;
    Property TimeCount : Integer read Fm_Time default 0;
    property NoInterrupt : Boolean read FbNoInterrupt write FbNoInterrupt default false;

    property OnCacheHeath : TNotifyEvent read FOnCacheHeath write FOnCacheHeath;
    property OnTestFunction : TNotifyEvent read FOnTestFunction write FOnTestFunction;
    property OnEndTest : TNotifyEvent read FOnEndTest write FOnEndTest;
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('System', [TPentiumTimer]);
end;

constructor TPentiumTimer.Create;
begin
     inherited;
     Fm_bColdCache := False;
     Fm_Time := 0;
     FActive := False;

     HotMemory := TList.Create;
     Hotmemory.Clear;
end;

destructor TPentiumTimer.Destroy;
begin
     inherited;
     FreeAll;
end;

function TPentiumTimer.AllocateMemory(size : longint):pointer;
var
   tmpPtr : pointer;
begin

     tmpMem := TMemBlock.Create;
     tmpPtr := tmpMem.Allocate(size);
     tmpMem.tag := 1;

     if (tmpptr <> nil) then
        HotMemory.Add(tmpMem);

     Result := tmpptr;
end;

procedure TPentiumTimer.HeathVariable(var hvar ; size : integer);
begin
     tmpMem := TmemBlock.Create;
     tmpMem.dwSize := size;
     tmpMem.ptrMem := Addr(hvar);
     tmpMem.Tag := 0;

     HotMemory.Add(tmpMem);
end;

procedure TPentiumTimer.FreeAll;
var
   i : integer;
begin
     i := 0;
     while (i < hotmemory.count) do
        begin
           TMemBlock(HotMemory[i]).free;
           i:=i+1;
        end;

     Hotmemory.clear;
end;

function TPentiumTimer.GetPointer(index : integer) : pointer;
begin
     if (index<0) or (index>hotmemory.count-1) then
        begin
             Result := nil;
             exit;
        end;

     Result := TMemBlock( HotMemory[index]).ptrMem;
end;

function TPentiumTimer.GetSize(index : integer) : Longint;
begin
     if (index<0) or (index>hotmemory.count-1) then
        begin
             Result := -1;
             exit;
        end;

     Result := TMemBlock( HotMemory[index]^).dwSize;
end;

procedure TPentiumTimer.RunTest;
var
   StartHi,StartLo,EndHi,EndLo,i : Integer;
begin
     FActive := True;
     if (Fm_bColdCache) then
        ChillCache
     else
         begin
              if (assigned(FOnCacheHeath)) then
                 FOnCacheHeath(Self);

              i := 0;
              while (i < HotMemory.count) do
                    begin
                         HeathCache(TMemBlock(HotMemory[i]));
                         i:=i+1;
                    end;

              if (assigned (FOnTestFunction)) then
                 FOnTestFunction(self);
         end;

     if (FbNoInterrupt) then
        begin
             asm
             cli
             end;
        end;
     if (assigned (FOnTestFunction)) then
      begin
       asm
        DB 0FH
        DB 031H
        mov StartHi,edx
        mov StartLo,eax
       end;
       FOnTestFunction(self);
       asm
        DB 0FH
        DB 031H
        mov EndHi,edx
        mov EndLo,eax
       end;
      end;
     if (FbNoInterrupt) then
        begin
             asm
             sti
             end;
        end;

     Fm_Time := EndLo - StartLo;

     if (assigned (FOnEndTest)) then
        FOnEndTest(self);
     FActive := False;
end;

procedure TPentiumTimer.ChillCache;
const
     JunkSize : integer = 16*1024;
var
   pJunk : pointer;
begin
     pJunk := allocmem(junksize);

     asm
        push esi

        mov ecx,junksize
        mov esi,pJunk
@MainLoop:
        mov eax,[esi+ecx]
        sub ecx,32
        jge @MainLoop

        pop esi
      end;

      FreeMem(pJunk);
end;


procedure TPentiumTimer.HeathCache(mem : TMemBlock);
var
   iSize : integer;
   pMem : Pointer;
begin
     iSize := mem.dwSize;
     pmem := mem.ptrMem;

     asm
        push esi

        mov ecx,iSize
        mov esi,pMem
@MainLoop:
        mov eax, [esi+ecx]
        sub ecx,32
        jge @MainLoop

        pop esi
      end;
end;

procedure TPentiumTimer.SetActive(Value : Boolean);
begin
     if (FActive) then exit;

     FActive := Value;

     if Value then
        RunTest;

end;


end.
