(*
 DESCRIPTION : This component is very simple panel-based-component witch can show
                points in a diagram
 AUTHOR      : Harm v. Zoest, email : 4923559@hsu1.fnt.hvu.nl
 VERSION     : 0.5 (alfa) 07-05-1996
 REMARKS     : If you have comments, found bugs, ore you have added some
               nice features, please mail me!
 *)
unit pntpanel;

interface

uses
  WinProcs, WinTypes, Classes, Graphics,Controls, ExtCtrls, forms;

const
  {Maximum number of points per diagram}
  MaxTPoints = 500;

type
  TPointPanel = class(TCustomPanel)
  private
    { Private declarations }
    FDataPen    : TPen;
    FDataBrush  : TBrush;
    FPoints     : array[0..MaxTPoints-1] of TPoint;
    FHorzUnit,
    FVertUnit   : Integer;
  protected
    { Protected declarations }
    function GetPoint(Index: Integer): TPoint; virtual;
    procedure SetPoint(Index: Integer;
      APoint: TPoint); virtual;
    procedure SetHorzUnit(AValue: Integer);
    procedure SetVertUnit(AValue: Integer);
    procedure SetDataBrush(value : TBrush);
    procedure SetDataPen(value : TPen);
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    procedure CtrlStyleChanged(Sender: TObject);
    property Points[Index: Integer]: TPoint  read GetPoint write SetPoint;
  published
    { Published declarations }

    { own properties}
    property DataBrush: TBrush read FDataBrush write SetDatabrush;
    property DataPen: TPen read FDataPen write SetDatapen;
    property UnitHorizontal: Integer
      read FHorzUnit write SetHorzUnit;
    property UnitVertical: Integer
      read FVertUnit write SetVertUnit;
    {enherited properties}
    property BevelInner;
    property BevelOuter;
    property Borderstyle;
    property Borderwidth;
    property Color;
    property Parentcolor;
  end;

procedure Register;

implementation

constructor TPointPanel.Create;
begin
  inherited Create(AnOwner);
  {Create pens and brushes}
  FDataPen := TPen.Create;
  FDataBrush := TBrush.Create;
  FDataPen.OnChange := CtrlStyleChanged;
  FDataBrush.OnChange := CtrlStyleChanged;
  { set defaults}
  Width := 150;
  Height := 150;
  FHorzUnit := 25;
  FVertUnit := 25;
  Fdatabrush.color := clblack;
end;{create}


destructor TPointPanel.Destroy;
begin
  FDataPen.Free;
  FDataBrush.Free;
  inherited Destroy;
end;

procedure TPointPanel.CtrlStyleChanged;
begin
  { force to repaint so soon as possible}
  Invalidate;
end;


procedure TPointPanel.SetDataBrush(value : TBrush);
begin
  FDataBrush := value;
end;

procedure TPointPanel.SetDataPen(value : TPen);
begin
  FDataPen := value;
end;


function  TPointPanel.GetPoint;
begin
  if Index < MaxTPoints then
    Result := FPoints[Index]
  else
    Result := Point(-1,-1);
end;{GetPoint}


procedure TPointPanel.SetPoint;
begin
  if Index < MaxTPoints then
    FPoints[Index] := APoint;
  Invalidate;
end;{SetPoint}


procedure TPointPanel.SetHorzUnit;
begin
  FHorzUnit := AValue;
  Invalidate;
end;{SetHorzUnit}


procedure TPointPanel.SetVertUnit;
begin
  FVertUnit := AValue;
  Invalidate;
end;{SetVertUnit}


procedure TPointPanel.Paint;
var
  i, X, Y : Integer;
begin
  with Canvas do begin
    inherited Paint;
    { draw the grid}
    i := UnitHorizontal;
    while I < Width do begin
      MoveTo(i,0);
      LineTo(i,Height);
      Inc(i,UnitHorizontal);
    end;
    {vert}
    i := Height;
    while i > 0 do begin
      MoveTo(0,i);
      LineTo(Width,i);
      Dec(i,UnitVertical);
    end;

    {plot  the points}
    Brush := FDataBrush;
    Pen := FDataPen;
    for i := 0 to MaxTPoints do
    begin
      X := Points[i].X;
      if (X <> -1) then
      begin
        Y := Points[i].Y;
        Ellipse(X-3,Height-(Y-3),X+3,Height-(Y+3));
      end;{if}
    end; {for}
  end;  {with}
end; {paint}

procedure Register;
begin
  RegisterComponents('Samples', [TPointPanel]);
end;

end.
