unit psiProgressBar;

{================================================================}
{ TpsiProgressBar - Version 1.0                                  }
{================================================================}
{                                                                }
{ DELPHI 2.0 COMPONENT                                           }
{ FREEWARE                                                       }
{                                                                }
{ TpsiProgressBar is a "Win95" style progress meter with a text  }
{ display (caption) of the meter's progress.                     }
{                                                                }
{ Installs to a "psi" tab                                        }
{                                                                }
{ 1) Control height is determined by the font you select for     }
{    the text (caption).                                         }
{ 2) Color properties include:                                   }
{    Background                                                  }
{    Progress Bar                                                }
{    Caption (via font setting)                                  }
{    3d Highlight                                                }
{    3d Shadow                                                   }
{ 3) Maintaining the "progress" can be handled a couple ways:    }
{    Fill "NumTodo", then update "NumDone" when appropriate      }
{         (fills "Progress" for you)                             }
{    Fill "Progress" with the percent to display (integer)       }
{                                                                }
{----------------------------------------------------------------}
{                                                                }
{ If it's helpful in some way, GREAT.                            }
{ As you'd expect, USE TpsiProgressBar AT YOUR OWN RISK.         }
{ If you use it, please send any comments.                       }
{ If you improve it, please let me know.                         }
{                                                                }
{ Enjoy it,                                                      }
{ Jim Albert                                                     }
{   PIN Systems, Inc.                                            }
{     Compuserve -- 102426,2527                                  }
{     Internet   -- PINSystems@mailhost.net                      }
{                                                                }
{================================================================}

interface

uses
  Graphics, Controls, Classes, SysUtils, Forms;

{var}

{type}

type
  TpsiProgressBar = class(TGraphicControl)
  private
    FBrush :TBrush;
    FCaptionPanelWidth: Integer;
    FColor3DHighlight: TColor;
    FColor3DShadow: TColor;
    FColorBack :TColor;
    FColorBar :TColor;
    FFontCaption :TFont;
    FLastProgress :integer;
    FNumDone :integer;
    FNumTodo :integer;
    FPen :TPen;
    FProgress :integer;
    FOnPaint: TNotifyEvent;
    procedure PaintMainPanel;
    procedure PaintCaption;
    procedure PaintProgressBar;
    procedure SetColor3DHighlight(Value: TColor);
    procedure SetColor3DShadow(Value: TColor);
    procedure SetColorBack(Value: TColor);
    procedure SetColorBar(Value: TColor);
    procedure SetFontCaption(Value: TFont);
    procedure SetNumDone(Value: integer);
    procedure SetNumTodo(Value: integer);
    procedure SetProgress(Value: integer);
     property CaptionPanelWidth: Integer read FCaptionPanelWidth write FCaptionPanelWidth;
     property LastProgress: integer read FLastProgress write FLastProgress;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas;
  published
    property Color3DHighlight: TColor read FColor3DHighlight write SetColor3DHighlight;
    property Color3DShadow: TColor read FColor3DShadow write SetColor3DShadow;
    property ColorBack: TColor read FColorBack write SetColorBack;
    property ColorBar: TColor read FColorBar write SetColorBar;
    property FontCaption: TFont read FFontCaption write SetFontCaption;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property NumDone: integer read FNumDone write SetNumDone;
    property NumTodo: integer read FNumTodo write SetNumTodo;
    property Progress: integer read FProgress write SetProgress;
    property Visible;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('psi', [TpsiProgressBar]);
end;

constructor TpsiProgressBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  CaptionPanelWidth :=0;
  Canvas.Font.Color :=clBlack;
  Canvas.Font.Height :=-11;
  Canvas.Font.Name :='Arial';
  Canvas.Font.Pitch :=fpDefault;
  Canvas.Font.Size :=8;
  Canvas.Font.Style :=[fsBold];
  Color3DHighlight :=clWhite;
  Color3DShadow :=clGray;
  ColorBack :=clSilver;
  ColorBar :=clNavy;
  Cursor :=crHourGlass;
  FFontCaption :=TFont.Create;
  with FFontCaption do
    begin
    Color :=clBlack;
    Height :=-11;
    Name :='Arial';
    Pitch :=fpDefault;
    Size :=8;
    Style :=[fsBold];
    end;
  SetFontCaption(FFontCaption);
  Height :=21;
  NumDone :=0;
  NumTodo :=0;
  Progress :=0;
  Visible :=True;
  Width :=261;
end;


destructor TpsiProgressBar.Destroy;
begin
  inherited Destroy;
end;


{===================================================================}
procedure TpsiProgressBar.Paint;
{===================================================================}
begin
  Canvas.Font.Color :=FFontCaption.Color;
  Canvas.Font.Height := FFontCaption.Height;
  Canvas.Font.Name := FFontCaption.Name;
  Canvas.Font.Pitch :=FFontCaption.Pitch;
  Canvas.Font.Size := FFontCaption.Size;
  Canvas.Font.Style := FFontCaption.Style;

  Self.Height :=((-Self.FontCaption.Height)+10);
  Self.CaptionPanelWidth :=Self.Canvas.TextWidth('100%') +11;
  if Self.Width <Self.CaptionPanelWidth +(Self.Height*2) then
    Self.Width :=Self.CaptionPanelWidth +(Self.Height*2);
  PaintMainPanel;
  PaintCaption;
  PaintProgressBar;
end;


{===================================================================}
procedure TpsiProgressBar.PaintMainPanel;
{===================================================================}
var
  InPen: TPen;
  InBrush: TBrush;
begin
  With Self.Canvas as TCanvas do
  begin
  InPen :=Pen;
  InBrush :=Brush;

  Brush.Color :=ColorBack;
  Brush.Style :=bsSolid;
  Pen.Color :=Color3DShadow;
  Pen.Width :=1;
  Rectangle(0,0,Width,Height);
  Pen.Color :=Color3DHighlight;
  MoveTo(0,0);
  LineTo(0,Height);
  MoveTo(0,0);
  LineTo(Width,0);

  Pen :=InPen;
  Brush :=InBrush;
  end;
end;


{===================================================================}
procedure TpsiProgressBar.PaintCaption;
{===================================================================}
var
  InPen: TPen;
  InBrush: TBrush;
  s: string;
begin
  InPen :=Canvas.Pen;
  InBrush :=Canvas.Brush;
  With Self.Canvas as TCanvas do
  begin
    {...............}
    { Caption Panel }
    {...............}
  Pen.Color :=ColorBack;
  Brush.Color :=ColorBack;
  Brush.Style :=bsSolid;
  Rectangle(2,2,CaptionPanelWidth,Height-3);

    {...............}
    { Caption Text  }
    {...............}
  Pen.Color :=ColorBack;
  Brush.Color :=ColorBack;

  Canvas.Font.Color :=FontCaption.Color;
  Canvas.Font.Height := FontCaption.Height;
  Canvas.Font.Name := FontCaption.Name;
  Canvas.Font.Pitch :=FontCaption.Pitch;
  Canvas.Font.Size := FontCaption.Size;
  Canvas.Font.Style := FontCaption.Style;

  s :=IntToStr(Progress)+'%';
  TextOut((CaptionPanelWidth-TextWidth(s)-3),3,s);

    {............}
    { Highlights }
    {............}
  Pen.Color :=Color3DShadow;
  MoveTo(2,2);
  LineTo(CaptionPanelWidth,2);
  MoveTo(2,2);
  LineTo(2,Height-3);
  Pen.Color :=Color3DHighlight;
  MoveTo(CaptionPanelWidth,Height-3);
  LineTo(CaptionPanelWidth,2);
  MoveTo(CaptionPanelWidth,Height-3);
  LineTo(2,Height-3);

  end;
  Canvas.Pen :=InPen;
  Canvas.Brush :=InBrush;

end;


{===================================================================}
procedure TpsiProgressBar.PaintProgressBar;
{===================================================================}
var
  InPen: TPen;
  InBrush: TBrush;
  TotBarWidth: Integer;
  NumElements: LongInt;
  PercentPerElement: Double;
  NumToPaint: LongInt;
  Painted: Integer;
  ElementPoint: LongInt;
  TotElementWidth: Integer;
  ElementWidth: Integer;
begin
  With Self.Canvas as TCanvas do
  begin
  InPen :=Pen;
  InBrush :=Brush;
  {====================}
  {....................}
  { Progress Bar Panel }
  {....................}
  {====================}
    {........................................}
    { To reduce flicker,                     }
    { only paint the background if necessary }
    {........................................}
  if (Progress =0) or (Progress <LastProgress)
                   or (csDesigning in ComponentState) then
    begin
    Brush.Color :=ColorBack;
    Brush.Style :=bsSolid;
    Rectangle(CaptionPanelWidth+3,2,Width-2,Height-2);
    end;

    {..............................}
    { Paint the highlights/shadows }
    {..............................}
  Pen.Color :=Color3DShadow;
  MoveTo(CaptionPanelWidth+3,2);
  LineTo(Width-3,2);
  MoveTo(CaptionPanelWidth+3,2);
  LineTo(CaptionPanelWidth+3,Height-3);
  Pen.Color :=Color3DHighlight;
  MoveTo(Width-3,Height-3);
  LineTo(Width-3,2);
  MoveTo(Width-3,Height-3);
  LineTo(CaptionPanelWidth+3,Height-3);

  {====================}
  {....................}
  { Progress Bar       }
  {....................}
  {====================}
    { How wide is each element in the bar }
  ElementWidth :=Round(((Height-9)/3)*2);
    { How wide is area available for the bar }
  TotBarWidth :=Width-CaptionPanelWidth-9;
    { How many elements will fit in the available area }
  NumElements :=Trunc((TotBarWidth-4)/(ElementWidth+2));
    { What percent does each element of the bar represent }
  PercentPerElement :=100/NumElements;
    { How many elements do we need to paint }
  NumToPaint :=Round(Progress/PercentPerElement);
    { Just to make sure we don't over run our available area }
  if NumToPaint >NumElements then NumToPaint :=NumElements;
    { How wide would the elements be if 100% (so we divide freespace evenly }
  TotElementWidth :=(NumElements*(ElementWidth+2))-2;
    { What is the starting point of the first element }
  ElementPoint :=(CaptionPanelWidth+5) +Trunc((TotBarWidth -TotElementWidth) /2);
    { Paint the elements }
  if NumToPaint >0 then
    begin
    Brush.Color :=ColorBar;
    Brush.Style :=bsSolid;
    Pen.Color :=ColorBar;
    Pen.Width :=1;
    For Painted :=1 to NumToPaint do
      begin
      Rectangle(ElementPoint,4,ElementPoint+ElementWidth,Height-4);
      ElementPoint :=ElementPoint+ElementWidth+2;
      end;
    end;
  Pen :=InPen;
  Brush :=InBrush;
  end;
end;


{===================================================================}
procedure TpsiProgressBar.SetNumDone(Value: Integer);
{===================================================================}
begin
  if Value <0 then Value :=0;
  if Value >100 then Value :=100;
  FNumDone :=Value;
  if NumTodo >0 then Progress :=Round((FNumDone/NumTodo)*100);
end;


{===================================================================}
procedure TpsiProgressBar.SetNumTodo(Value: Integer);
{===================================================================}
begin
  if Value <0 then Value :=0;
  if Value >100 then Value :=100;
  FNumTodo :=Value;
  if FNumTodo >0 then Progress :=Round((NumDone/FNumTodo)*100);
end;


{===================================================================}
procedure TpsiProgressBar.SetProgress(Value: Integer);
{===================================================================}
begin
  LastProgress :=Progress;
  if Value <0 then Value :=0;
  if Value >100 then Value :=100;
  FProgress :=Value;
  if (GetParentForm(Self) <>NIL) and (Visible =True) then
    begin
    PaintCaption;
    PaintProgressBar;
    end;
end;


{===================================================================}
procedure TpsiProgressBar.SetColor3DHighlight(Value: TColor);
{===================================================================}
begin
  FColor3DHighlight :=Value;
  if Visible = True then Self.Repaint;
end;


{===================================================================}
procedure TpsiProgressBar.SetColor3DShadow(Value: TColor);
{===================================================================}
begin
  FColor3DShadow :=Value;
  if Visible =True then Self.Repaint;
end;


{===================================================================}
procedure TpsiProgressBar.SetColorBack(Value: TColor);
{===================================================================}
begin
  FColorBack :=Value;
  if Visible =True then Self.Repaint;
end;


{===================================================================}
procedure TpsiProgressBar.SetColorBar(Value: TColor);
{===================================================================}
begin
  FColorBar :=Value;
  if Visible = True then Self.Repaint;
end;


{===================================================================}
procedure TpsiProgressBar.SetFontCaption(Value: TFont);
{===================================================================}
begin
  with FFontCaption do
    begin
    Color :=Value.Color;
    Height :=Value.Height;
    Name :=Value.Name;
    Pitch :=Value.Pitch;
    Size :=Value.Size;
    Style :=Value.Style;
    end;
  if Visible = True then self.refresh;
end;





end.
