// MyOle ver. 0.91
// Author: Matej Trampus, Slovenija
// E-MAIL:
//   - Matej.Trampus@kiss.uni-lj.si
// See readme.txt for details.
//

unit Richole;

interface
// About controlling reObject.dwFlags flags when data is dropped:
//   When IOleObject is dropped on RichEdit control, RichEdit sets reObject.dwFlags
//   to default values. The only way to set this flags to non-default value is to
//   provide RIHCHEDIT (or RichEditWithObjects) clipboard format.
//   Because I could not find specification for this format, I create an invisible
//   RichEdit control, insert the object and then query this RichEdit control for
//   RICHEDIT format.
//   This behaviour is currenly only implemented for VCL drag'n'drop operation
//   (which is started with BehinDrag). Data dropped from another application
//   has default reObject.dwFlags flags. Anyone know the solution?

// About destroyin RichEdit window"
//   Delphi destroys controls within a main form without calling DestroyWindowHandle
//   procedure. So we do not know, when out RichEdit control gets destroyed. I browsed
//   through controls.pas and discovered, that the only point, where the window handle is
//   actually set to 0 is in TWinControl.WMNCDestroy.
//   We intercept this message (TRichEditWithOle.WMNCDestroy) and release the riche edit's
//   interface. Alternate solution would be to check if handle is still allocated in.
//   Destroy method and call Release only when we would have a valid handle. But... Who
//   knows what happens if we have a refrence to IRichEditOle and RichEdit window gets
//   destroyed. My guess is, that interface is released too, and we get a pointer,
//   which points to already freed interface If someone knows more about this problem, please,
//   let me know...

// BUGS:
// - Calling Debug() during drag and drop of OLE object [with text] cauese form to repaint
//   after text has been dropped. I have no idea why (must be something with changes in _debugMemo)
// - I do not know, what is the value of IID_IRichEditOleCallBack, so I was not able
//   to correctly implement TMyRichEditOleCallBack.QueryInterface
//
//
// History:
// --- ver 0.91:
// 26.10.1996 Added support for Delphi drag and drop. When TRichEditWithOle need data,
//            OnGetOleDragFromSource gets called. If there is no OleData normal Delphi
//            drag'n'drop events are called
// 25.10.1996 Added TRichEditWithOle: OnQueryInsertDelphiObject OnQueryInsertObject,
//            BUGFIX: TRichEditWithOle.GetReObject TReObject.cbStruct was not set
//            BUGFIX: EnumFormats now handle also nil enumerator object
//            BUGFIX: TMyRichEditOleCallBack.GetNewStorage:  LockBytes.release was missing
// --- ver 0.90:
// 23.10.1996 BUGFIX: EnumFormats  enum.release was missing

uses windows,ole2,messages,richEdit,comctrls,controls,myOle;

const EM_GETOLEINTERFACE=(WM_USER + 60);
      EM_SETOLECALLBACK=(WM_USER + 70); // LUCKY GUESS???
      EM_SETOLEINTERFACE=EM_SETOLECALLBACK;



      REO_GETOBJ_NO_INTERFACES =0;
      REO_GETOBJ_POLEOBJ       =1;
      REO_GETOBJ_PSTG          =2;
      REO_GETOBJ_POLESITE      =4;
      REO_GETOBJ_ALL_INTERFACES=7;

 const
      REO_NULL             =0;   // No flags
      REO_READWRITEMASK    =$1F; // Mask out RO bits
      REO_BLANK            =$10; // Object is blank
      REO_DYNAMICSIZE      =$8;  // Object defines size always
      REO_INVERTEDSELECT   =$4;  // Object drawn all inverted if sel
      REO_BELOWBASELINE    =$2;  // Object sits below the baseline
      REO_RESIZABLE        =$1;  // Object may be resized
      REO_LINK             =$80000000; // Object is a link (RO)
      REO_STATIC           =$40000000; // Object is static (RO)
      REO_SELECTED         =$08000000; // Object selected (RO)
      REO_OPEN             =$04000000; // Object open in its server (RO)
      REO_INPLACEACTIVE    =$02000000; // Object in place active (RO)
      REO_HILITED          =$01000000; // Object is to be hilited (RO)
      REO_LINKAVAILABLE    =$00800000; // Link believed available (RO)
      REO_GETMETAFILE      =$00400000; // Object requires metafile (RO)

      RECO_PASTE=0;        // paste from clipboard
      RECO_DROP=1;         // drop
      RECO_COPY=2;         // copy to the clipboard
      RECO_CUT=3;          // cut to the clipboard
      RECO_DRAG=4;         // drag

      DVASPECT_CONTENT         =1;

type
   PReObject=^TreObject;
   TReObject=record
    cbStruct:DWORD;              // size of structure in bytes
    xp:longint;                  // character position of object
    clsid:TCLSID  ;              // class ID of object
    poleobj:IOleObject;          // OLE object interface
    pstg:Istorage;               // associated storage interface
    polesite:IOleClientSite;     // associated client site interface
    sizel:Tsize;                 // size of object (may be 0,0)
    dvaspect:dword;              // display aspect to use
    dwFlags:dword;               // object status flags
    dwUser:dword;                // user-defined value
end;



// The following to interfaces (to interfaces) was created by me. I am not shure, that all parameters are OK.
// I am not using functions startig with underscore, so I did not provide any parameters for
// them. If you would like to use them, look up definitions in OLE.HLP.

type IRichEditOle=class(IUnknown)
       public
         function GetClientSite(var p:IOleClientSite):hresult; virtual; stdcall; abstract;
         function GetObjectCount:longint; virtual; stdcall; abstract;
         function _GetLinkCount:hresult; virtual; stdcall; abstract;
         function GetObject(iob:longint;var obj:TReObject;dwFlags:DWORD):hresult; virtual; stdcall; abstract;
         function InsertObject(var r:TReObject):hresult; virtual; stdcall; abstract;
         function _ConvertObject:hresult; virtual; stdcall; abstract;
         function _ActivateAS:hresult; virtual; stdcall; abstract;
         function _SetHostNames:hresult; virtual; stdcall; abstract;
         function _SetLinkAvailable:hresult; virtual; stdcall; abstract;
         function _Setdvaspect:hresult; virtual; stdcall; abstract;
         function _HandsOffStorage:hresult; virtual; stdcall; abstract;
         function _SaveCompleted:hresult; virtual; stdcall; abstract;
         function _InPlaceDeactivate:hresult; virtual; stdcall; abstract;
         function _ContextSensitiveHelp:hresult; virtual; stdcall; abstract;
         function GetClipboardData(var range:TCharRange;reco:DWORD;var data:IDataObject):hresult; virtual; stdcall; abstract;
         function ImportDataObject(dataObject:IDataObject;cf:tclipformat;hMetaPict:hglobal):hresult; virtual; stdcall; abstract;
     end;

     IRichEditOleCallback=class(IUnknown)
       private
         fRefCount:Integer;
       public
         function GetNewStorage(var newStg:IStorage):hresult; virtual; stdcall; abstract;
         function GetInPlaceContext(var frame:IOleInPlaceFrame;var uiWindow:IOleInPlaceUIWindow;inplInfo:POleInPlaceFrameInfo):hresult; virtual; stdcall; abstract;
         function ShowContainerUI(fShow:BOOL):hresult; virtual; stdcall; abstract;
         function QueryInsertObject(const clsid:TClsid;storage:IStorage;cp:longint):hresult; virtual; stdcall; abstract;
         function DeleteObject(obj:IOleObject):hresult; virtual; stdcall; abstract;
         function QueryAcceptData(data:IDataObject;var clipFormat:TClipFormat;reco:DWORD;fReally:BOOL;hMetaPict:HGLOBAL):hresult; virtual; stdcall; abstract;
         function ContextSensitiveHelp(fEnterMode:BOOL):hresult; virtual; stdcall; abstract;
         function GetClipboardData(const range:TCharRange;reco:DWORD;var data:IDataObject):hresult; virtual; stdcall; abstract;
         function GetDragDropEffect(fDrag:BOOL;KeyState:DWORD;var Effect:DWORD):hresult; virtual; stdcall; abstract;
         function GetContextMenu(selType:word;oleObject:IOleObject;const range:TCharRange;var menu:HMENU):hresult; virtual; stdcall; abstract;
     end;

     TRichEditWithOle=class;
     TRichEditWithOleClass=class of TRichEditWithOle;
     TMyRichEditOleCallBack=class(IRichEditOleCallBack)
       protected
         fOwner:TRichEditWithOle;
       public
         constructor Create(AOwner:TRichEditWithOle);
         destructor Destroy; override;
         function QueryInterface(const iid: TIID; var obj): HResult; override;
         function AddRef: Longint; override;
         function Release: Longint; override;
         function GetNewStorage(var newStg:IStorage):hresult; override;
         function GetInPlaceContext(var frame:IOleInPlaceFrame;var uiWindow:IOleInPlaceUIWindow;inplInfo:POleInPlaceFrameInfo):hresult; override;
         function ShowContainerUI(fShow:BOOL):hresult; override;
         function QueryInsertObject(const clsid:TClsid;storage:IStorage;cp:longint):hresult; override;
         function DeleteObject(obj:IOleObject):hresult; override;
         function QueryAcceptData(data:IDataObject;var clipFormat:TClipFormat;reco:DWORD;fReally:BOOL;hMetaPict:HGLOBAL):hresult; override;
         function ContextSensitiveHelp(fEnterMode:BOOL):hresult; override;
         function GetClipboardData(const range:TCharRange;reco:DWORD;var data:IDataObject):hresult; override;
         function GetDragDropEffect(fDrag:BOOL;KeyState:DWORD;var Effect:DWORD):hresult; override;
         function GetContextMenu(selType:word;oleObject:IOleObject;const range:TCharRange;var menu:HMENU):hresult; override;
     end;

     TQueryInsertObjectEvent=procedure(Sender:TObject;const clsid:TCLSID;cp:Longint;var canInsert:boolean) of object;
     TQueryInsertDelphiObjectEvent=procedure(Sender:TObject;classRef:TDelphiOleClass;cp:Longint;var canInsert:boolean) of Object;
     TGetOleFromDragSourceEvent=procedure(sender:TObject;source:TObject;var oleObject:TDelphiOle) of object;

     TRichEditWithOle=class(TRichEdit)
       private
         fQueryInsertObject:TQueryInsertObjectEvent;
         fQueryInsertDelphiObject:TQueryInsertDelphiObjectEvent;
         fGetOleFromDragSource:TGetOleFromDragSourceEvent;
       protected
         fRichEditOle:IRichEditOle;
         draggingDelphiData:boolean;    // true, while user is dragging Delphi data over us
         fDragSource:TObject; // valid only during drag
         //function  QueryAcceptDelphiData(source:TObject;cp:Dword):boolean; virtual;
         procedure CreateWindowHandle(const Params: TCreateParams); override;
         procedure WMNCDestroy(var Message: TWMNCDestroy); message WM_NCDESTROY;
         procedure DestroyWindowHandle; override;

         function GetReObject(i:integer):TReObject;
         function GetObjectCount:longint;
         function GetObject(i:integer):IOleObject;
         function GetDelphiObject(i:integer):TDelphiOle;
         function QueryInsertObject(const clsid:TClsid;storage:IStorage;cp:longint):hresult; virtual;
         procedure GetOleFromDragSource(Source:TObject;var OleObject:TDelphiOle); virtual;
       public
         procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;var Accept: Boolean); override;
         destructor destroy; override;
         procedure InsertObject(theObject:TMyOle;pos:integer);
         procedure InsertObject2(theObject:IOleObject;pos:integer;storage:IStorage);
         property  richEditOle:IRichEditOle read fRichEditOle;
         property  objectCount:longint read GetObjectCount;
         // warning: Do not forget to call Release, when you don't need object anymore!!!
         property  objects[i:integer]:IOleObject read getObject;
         property  delphiObjects[i:integer]:TDelphiOle read GetDelphiObject; // nil if ..[i] is not Delphi object
       published
         property OnQueryInsertObject:TQueryInsertObjectEvent read fQueryInsertObject write fQueryInsertObject;
         property OnQueryInsertDelphiObject:TQueryInsertDelphiObjectEvent read fQueryInsertDelphiObject write fQueryInsertDelphiObject;
         property OnGetOleDragFromSource:TGetOleFromDragSourceEvent
                                          read fGetOleFromDragSource write fGetOleFromDragSource;
     end;

    TReDropSource=class(IDropSource) // RichEdit drop source, used to emulate OLE drag'n'drop
     private
       fRefCount:longint;
       fParent:TRichEditWithOle;
       captureWindow:hwnd;
     public
      dropped:boolean; // TRUE, if mouse buutton has been release
      lastPos:TPoint;  // last mouse position (QueryContinueDrag)
      constructor create(Parent:TRichEditWithOle);
      function QueryInterface(const iid: TIID; var obj): HResult; override;
      function AddRef: Longint; override;
      function Release: Longint; override;
      function QueryContinueDrag(fEscapePressed: BOOL;grfKeyState: Longint): HResult; override;
      function GiveFeedback(dwEffect: Longint): HResult; override;
    end;


procedure Register;

function iid2String(const iid:TGUid):string;

implementation
uses debugMemo1,sysUtils,oleAuto,classFactory,forms;


const errObjectIndexOutOfBounds='Object index out of bounds';
      errCantGetObject         ='Can''t get object';

function iid2String(const iid:TGUid):string;
var s:array[0..100] of WideChar;
begin
  stringFromGuid2(iid,@s,90);
  result:=wideCharToString(s);
end;

function EnumFormats(data:IDataObject):string;
var enum:IEnumFormatEtc; i:integer; dummy:Longint;
    formatName:array[0..200] of char; fmt:TFormatEtc;
begin
  data.EnumFormatEtc(1,enum);
  if enum=nil then
    result:='no enum object!'
  else begin
     result:='';
     i:=enum.Next(1,fmt,@dummy);
     while i=S_OK do begin
       GetClipBoardFormatName(fmt.cfFormat,@formatName,200);
       result:=result+format('%d(%s)',[fmt.cfFormat,formatName]);
       i:=enum.Next(1,fmt,@dummy);
     end;
     enum.release;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
/////   TMyRichEditOleCallBack
////////////////////////////////////////////////////////////////////////////////

constructor TMyRichEditOleCallBack.Create(AOwner:TRichEditWithOle);
begin
   debug2(self,'Create ');
   fOwner:=AOwner;
   FRefCount := 1; // Call release after EM_SETOLEINTERFACE!
end;

destructor TMyRichEditOleCallBack.destroy;
begin
  debug2(self,'destroy');
end;

function TMyRichEditOleCallBack.QueryInterface(const iid: TIID; var obj): HResult;
var  p:IUnknown;
begin
  p:=nil;
  // ***** WHAT IS THE VALUE OF IID_IRichEditOleCallBack ?????
  if {IsEqualIID(iid, IID_IRichEditOleCallBack) or} IsEqualIID(iid, IID_IUnknown) then p:=self;

 Pointer(obj) := P;
  if P = nil then begin debug2(self,'not suported: '+iid2String(iid)); Result := E_NOINTERFACE end else
  begin
    //warning: p must be Delphi's object for p.ClassName to work properly
    debug2(self,'QueryInterface + '+iid2String(iid)+' '+p.ClassName);
    P.AddRef;
    Result := S_OK;
  end;
end;


function TMyRichEditOleCallBack.AddRef: Longint;
begin
  debug2(self,'Addref');
  Inc(FRefCount);
  Result := FRefCount;
end;

function TMyRichEditOleCallBack.Release: Longint;
begin
  Dec(FRefCount);
  debug2(self,'Release - count='+intToStr(FRefCount));
  Result := FRefCount;
  if FRefCount = 0 then Free;
end;


function TMyRichEditOleCallBack.GetNewStorage(var newStg:IStorage):hresult;
var lockBytes:ILockBytes;
begin
 debug2(self,'GetNewStorage implementation');
  //try
    OleCheck(CreateILockBytesOnHGlobal(0, True, LockBytes));
    OleCheck(StgCreateDocfileOnILockBytes(LockBytes, STGM_READWRITE or
                                                     STGM_SHARE_EXCLUSIVE or STGM_CREATE, 0, newStg));
    lockBytes.Release;
  //except result:=E_FAIL???
 result:=S_OK;
end;

function TMyRichEditOleCallBack.GetInPlaceContext(var frame:IOleInPlaceFrame;var uiWindow:IOleInPlaceUIWindow;inplInfo:POleInPlaceFrameInfo):hresult;
begin
 debug2(self,'GetInPlaceContext - E_NOTIMPL');
 result:=E_NOTIMPL;
end;

function TMyRichEditOleCallBack.ShowContainerUI(fShow:BOOL):hresult;
begin
 debug2(self,'ShowContainerUI - E_NOTIMPL');
 result:=E_NOTIMPL;
end;

function TMyRichEditOleCallBack.QueryInsertObject(const clsid:TClsid;storage:IStorage;cp:longint):hresult;
begin
 debug2(self,Format('QueryInsertObject %s at %d S_OK',[iid2String(clsid),cp]));
 if fOwner=nil then  result:=S_OK else
 result:=fOwner.QueryInsertObject(clsid,storage,cp);
end;

function TMyRichEditOleCallBack.DeleteObject(obj:IOleObject):hresult;
begin
 debug2(self,Format('DeleteObject %p',[pointer(obj)]));
 result:=S_OK; // No return value
end;

function RenderRICHEDITformat(obj:IOleObject):IDataObject;
var   range:TCharRange; r:TRichEditWithOle;
      newStg:IStorage;   lockBytes:ILockBytes;
      f:TForm;
// Renders RichEdit text format, containing only obj object. This is the only
// way to preserve reobject.dwFlags.
// Becaue I do not have specification for richedit format, I create a temporary
// richedit controls and query it for data object
begin
  result:=nil;
  OleCheck(CreateILockBytesOnHGlobal(0, True, LockBytes));
  OleCheck(StgCreateDocfileOnILockBytes(LockBytes, STGM_READWRITE or
                                                   STGM_SHARE_EXCLUSIVE or STGM_CREATE, 0, newStg));
  r:=TRichEditWithOle.Create(nil);
  f:=tForm.Create(nil); // RichEdit needs parent, so that it can get valid handle
  r.Parent:=f;
  r.handle; // force creation of window handle
  range.cpMin:=0; range.cpMax:=-1; // select all;
  r.InsertObject2(obj,0,newStg); // try to insert object in richEdit control
  r.fRichEditOle.getClipboardData(range,RECO_COPY,result); // this added a reference to IData

  newStg.release; // release storage
  r.Destroy;  f.Destroy; // release richEdit and form (This does ??not?? (shouldn't) destroy IDataObject)
end;

function TMyRichEditOleCallBack.QueryAcceptData(data:IDataObject;var clipFormat:TClipFormat;reco:DWORD;fReally:BOOL;hMetaPict:HGLOBAL):hresult;
begin
{ if fowner.DraggingDelphiData then begin
   debug2(self,'QueryAcceptData - handling Delphi data '+fOwner.fDragSource.ClassName);
   if fowner.QueryAcceptDelphiData(fOwner.fDragSource,12) then result:=S_FALSE // we will import the object
   else result:=S_OK; // we do not accept data, RichEdit will try to handle it

   result:=S_OK;

 end else begin}

   debug2(self,format('QueryAcceptData = cplFormat=%d really=%d',[clipFormat,integer(fReally)]));
   debug2(self,'enum formats: '+enumFormats(data));


   result:=S_OK;
{ end;}
end;

function TMyRichEditOleCallBack.ContextSensitiveHelp(fEnterMode:BOOL):hresult;
begin
 debug2(self,'Context sensitive help '+intToStr(integer(fEnterMode))+' E_NOTIMP');
 result:=E_NOTIMPL;
end;

function TMyRichEditOleCallBack.GetClipboardData(const range:TCharRange;reco:DWORD;var data:IDataObject):hresult;
begin
 debug2(self,'GetClipBoardData ');
 result:=E_NOTIMPL; // the rich edit control will create its own clipboard object
end;

function TMyRichEditOleCallBack.GetDragDropEffect(fDrag:BOOL;KeyState:DWORD;var Effect:DWORD):hresult;
begin
 debug2(self,'GetDragDropEffect - default');
 //effect:=1;
 result:=S_OK; // No return value
end;

function TMyRichEditOleCallBack.GetContextMenu(selType:word;oleObject:IOleObject;const range:TCharRange;var menu:HMENU):hresult;
begin
 debug2(self,'GetContextMenu E_NOTIMPL');
 result:=E_NOTIMPL;
end;

////////////////////////////////////////////////////////////////////////////////
/////   TRichEditWithOle
////////////////////////////////////////////////////////////////////////////////

procedure TRichEditWithOle.CreateWindowHandle(const Params: TCreateParams);
var   callBack:TMyRichEditOleCallBack;
begin
  debug2(self,'CreateWindowHandle');
  inherited CreateWindowHandle(Params);
  debug2(self,'CreateWindowHandle - inherited OK');

  if handle<>0 then begin
     debug2(self,'CreateWindowHandle - getting OLE interface');
     sendMessage(Handle,EM_GETOLEINTERFACE,0,longint(@fRichEditOle)); // get IRichEditOle

     callBack:=TMyRichEditOleCallBack.Create(self);
     debug('Set callBack result='+intToStr(sendMessage(Handle,EM_SETOLEINTERFACE,0,longint(callBack))));
     callBack.Release; // we do not need this anymore. RichEdit already holds on it
  end;
end;

procedure TRichEditWithOle.DestroyWindowHandle;
begin
  debug2(self,'DestroyWindowHandle');
  if fRichEditOle<>nil then begin fRichEditOle.Release; fRichEditOle:=nil; end;
  inherited destroyWindowHandle;
end;

procedure TRichEditWithOle.WMNCDestroy(var Message: TWMNCDestroy);
begin
  debug2(self,'WMNCDestroy');
  if fRichEditOle<>nil then begin fRichEditOle.Release; fRichEditOle:=nil; end;
  inherited; // FHandle is set to 0 in inherited procedure, so we release IRichEditOle
end;

destructor TRichEditWithOle.destroy;
begin
  if handleAllocated then debug2(self,'destroy - handle is allocated '+inttoStr(handle))
                     else debug2(self,'destroy - handle is *NOT* allocated ');
  if fRichEditOle<>nil then   begin
    debug2(self,'RichEditOleRelease returned'+intToStr(fRichEditOle.Release));
    fRichEditOle:=nil
  end;
  inherited destroy;
end;

procedure TRichEditWithOle.InsertObject(theObject:TMyOle;pos:integer);
var storage:IStorage; lockBytes:ILockBytes; perStorage:IPersistStorage;
begin
  if not theObject.HasStorage then begin
    OleCheck(CreateILockBytesOnHGlobal(0, True, LockBytes));
    OleCheck(StgCreateDocfileOnILockBytes(LockBytes, STGM_READWRITE
    or STGM_SHARE_EXCLUSIVE or STGM_CREATE, 0, Storage));
    lockBytes.release;
    theObject.GetPersistStorage(perStorage);
    perStorage.initNew(storage);
    perStorage.Release;
    storage.Release; // Release for StgCreateDocfileOnILockBytes
  end;
  theObject.GetStorage(storage);
  insertObject2(theObject,pos,storage);
  storage.Release;
end;

procedure TRichEditWithOle.InsertObject2(theObject:IOleObject;pos:integer;storage:IStorage);
var id:TClsid;
    r:TREObject;
    cs:IOleClientSite; // rich edit's client site
begin

 with r do begin // fill data structure
    cbStruct:=sizeof(TreObject);
    xp:=pos;                                           // character position of object
    theObject.getUserClassId(id); clsid:=id;           // class ID of object
    poleobj:=theObject;                                // OLE object interface
    pstg:=storage;                                     // associated storage interface
    fRichEditOle.GetClientSite(cs); polesite:=cs;      // associated client site interface
    sizel.cx:=0; sizel.cy:=0;                          // size of object (may be 0,0)
    dvaspect:=dvaspect_content;                        // display aspect to use
    dwFlags:=REO_DYNAMICSIZE	or REO_BELOWBASELINE or REO_INVERTEDSELECT;  // object status flags
    // when REO_DYNAMICSIZE is used object properly resizes itself if dragged to another RichEdit
    dwUser:=0;                                         // user-defined value
 end;

 debug2(self,'Insert object start');
 fRichEditOle.InsertObject(r); // insertObject
 debug2(self,'Insert object end');
 theObject.SetClientSite(cs); // IS THIS NECCESSARY? (my guess)
end;

procedure TRichEditWithOle.GetOleFromDragSource(Source:TObject;var OleObject:TDelphiOle);
begin
  oleObject:=nil;
  if assigned(fGetOleFromDragSource) then fGetOleFromDragSource(self,source,oleObject);
end;

function TRichEditWithOle.QueryInsertObject(const clsid:TClsid;storage:IStorage;cp:longint):hresult;
var canInsert:boolean; objClass:TDelphiOleClass;
begin
  objClass:=clsid2TMyOle(clsid); canInsert:=TRUE;
  // call QueryInsertDelphiObject whenever possible, otherwise call QueryInsertObject
  if (objClass=nil) or ((objClass<>nil) and not assigned(fQueryInsertDelphiObject)) then begin
    if assigned(fQueryInsertObject) then fQueryInsertObject(self,clsid,cp,canInsert);
  end else
    if assigned(fQueryInsertDelphiObject) then fQueryInsertDelphiObject(self,objClass,cp,canInsert);
  if canInsert then result:=S_OK else result:=S_FALSE;
end;

// property access methods
function TRichEditWithOle.GetObjectCount:longint;
begin
  result:=fRichEditOle.GetObjectCount;
end;

function TRichEditWithOle.GetObject(i:integer):IOleObject;
begin
  result:=GetReObject(i).pOleObj;
end;

function TRichEditWithOle.GetDelphiObject(i:integer):TDelphiOle;
begin
  result:=IOleObject2TDelphiOle(getObject(i)); // nil, if object is not Delphi Object
end;

function TRichEditWithOle.GetReObject(i:integer):TReObject;
begin
  if (i>objectCount) or (i<0) then raise EListError.Create(errObjectIndexOutOfBounds);
  result.cbStruct:=sizeOf(result);
  if fRichEditOle.GetObject(i,result,REO_GETOBJ_POLEOBJ)<>S_OK then raise EListError.Create(errCantGetObject);
end;


procedure TRichEditWithOle.DragOver(Source: TObject; X, Y: Integer; State: TDragState;var Accept: Boolean);
var dwResult:longint; o:TDelphiOle; ds:TReDropSource; data:IDataObject;
begin
  inherited DragOver(source,x,y,State,Accept);
  if state=dsDragEnter then begin
     debug2(self,'Drag Enter***');
     GetOleFromDragSource(source,o);
     if o<>nil then begin
        fDragSource:=source; draggingDelphiData:=true;
        ds:=TReDropSource.Create(Self); // create temporary deag source
        debug2(self,'Start drag');
        //DoDragDrop(o.owner.fDataObject,ds,DROPEFFECT_COPY,dwResult); // only copy supported
        data:=renderRichEditFormat(o.owner);
          DoDragDrop(data,ds,DROPEFFECT_COPY,dwResult); // only copy supported
        data.release; // release Data object
        debug2(self,'end drag***');
        //if ds.Dropped and assigned then
        ds.Release;
        o.Release; // object has already been inserted in richEdit,
        // finally:
        fdragSource:=nil; draggingDelphiData:=false;
     end;
  end
  else if state=dsDragMove then   debug('Drag move***')
  else if state=dsDragLeave  then   debug('Drag leave***');
end;

////////////////////////////////////////////////////////////////////////////////
// TReDropSource
////////////////////////////////////////////////////////////////////////////////
// Strange, but RichEdit control, which supports IRichEditOle does not support
// IDropTarget interface (I guess, this is violation of COM rules)
// So we take an approach from the other site -w e emulate IDropSource

constructor TReDropSource.Create(Parent:TRichEditWithOle);
begin
  // At the beginnign of drag and drop operations
  // delphi creates TDragObject, which captures the focus. When OLE's DoDragDrop
  // is called, TDragObject looses capture. We must restore capture after
  // DoDragDrop is finished, or VCL would crash.
  fRefCount := 1; fParent:=Parent;
//  SendMessage(GetCapture,WM_LBUTTONUP,0, Longint(PointToSmallPoint(lastPos)));
  // CaptureWindow=0;
  captureWindow:=GetCapture; // find the window, that has capture
end;

function TReDropSource.QueryInterface(const iid: TIID; var obj): HResult;
var  p:IUnknown;
begin
  p:=nil;
  debug2(self,'QueryInterface + '+iid2String(iid));
    if IsEqualIID(iid, IID_IUnknown) or IsEqualIID(iid, IID_IDROPSOURCE) then p:=self;
  Pointer(obj) := P;
  if P = nil then begin debug2(self,'not suported!'); Result := E_NOINTERFACE end  else
  begin   P.AddRef;  Result := S_OK;  end;
end;

function TReDropSource.AddRef: Longint;
begin  Inc(FRefCount);   Result := FRefCount; end;

function TReDropSource.Release: Longint;
begin
  Dec(FRefCount);  Result := FRefCount;
  if FRefCount = 0 then begin
    if captureWindow<>0 then begin
      windows.setCapture(captureWindow);
      if dropped then PostMessage(captureWindow,WM_LBUTTONUP,0, Longint(PointToSmallPoint(lastPos)));
    end;
    Free;
 end;
end;

function TReDropSource.QueryContinueDrag(fEscapePressed: BOOL;grfKeyState: Longint): HResult;
var h:HWND; c:TWinControl;
begin
  // we do *not* cancel drag when ESC is pressed, because Delphi does not, too

  GetCursorpos(lastPos);  h:=WindowFromPoint(lastPos); c:=findControl(h);
  dropped:=dropped or ((grfKeyState and MK_LBUTTON)=0);
  // when mouse leaves RichEdit's control area, we have to continue with Delphi's
  // drag and drop mechanism

  if dropped then begin debug2(self,'QueryContinueDrag: DROP!'); result:=DRAGDROP_S_DROP; end
  else if (c=nil) or (c<>fParent) then begin
    beep; result:=DRAGDROP_S_CANCEL;
  end else result:=S_OK;
end;

function TReDropSource.GiveFeedback(dwEffect: Longint): HResult;
begin  result:=DRAGDROP_S_USEDEFAULTCURSORS; end;


procedure Register;
begin
  RegisterComponents('Win95',[TRichEditWithOle]);
end;


end.
