unit Rprev;
{**********************
  Ry Printer (another print preview component)
by Ryan Peterson
rpetersn@usit.net

Last version had "uses Main" in the file so it didn't
work, sorry.  This one works, I even tried it.

This is a quickie and small Print Preview object.
  Create it
  call NewJob
  Draw on it (TextOut, Rectangle, etc.)
  call Preview, PrintIt, or PrintPage

This was designed around a HP Laserjet 4L, but
I don't see why it wouldn't work with different
resolutions.

It has a few bare bones Canvas routines, but more can
easily be added, just look at how I implemented
the TextOut and TextOutCenter routines.

There are already so many PPrev components out there,
but only the expensive ones fit my needs.  Those
components had too many functions and most also didn't
fit my needs.

This object is based on metafiles. All the pages are open
at one time so you can draw on page 50 at the same time you're
drawing on page 2 (real handy for putting those
"Page 1 of 50" labels).  Since it uses metafiles, it's fast,
real fast at the drawing.  It works great for those small
projects where you need print preview, but don't need a big
report printer.

All I ask is that if you improve on this object, please send it
to me so I can use it too.
*********************}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, Printers, ExtCtrls, Buttons, Dialogs;

type
  TRyPrinter = class;
  TRyPreviewFrm = class(TForm)
    Panel1: TPanel;
    PrintBtn: TButton;
    NextBtn: TButton;
    LastBtn: TButton;
    FirstBtn: TButton;
    PriorBtn: TButton;
    Button6: TButton;
    Panel2: TPanel;
    Button2: TButton;
    Button7: TButton;
    sb: TScrollBox;
    PaintBox1: TPaintBox;
    ShwGrdBtn: TSpeedButton;
    Button1: TButton;
    Button3: TButton;
    procedure PrintBtnClick(Sender: TObject);
    procedure FirstBtnClick(Sender: TObject);
    procedure PriorBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure LastBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure ShwGrdBtnClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
    Ry : TRyPrinter;
    PageDisplaying : Integer;
  public
    { Public declarations }
  end;

  TRyPrinter = class(TObject)
  private
    { Private declarations }
      {Current page}
    FCurrentPage : Integer;
      {Raw canvases - with Handles connected to OPEN metafiles}
    FCanvases : TList;
    FDonePrinting : Boolean;
      {Metafiles - with Handles connected to CLOSED metafiles}
    FMetafiles : TList;
      {Pointer to the preview form, which isn't created until Preview is called}
    FRyPreviewForm : TRyPreviewFrm;
    FOrientation : TPrinterOrientation;

    function GetPageCount : Integer;
    procedure SetCurrentPage(Index : Integer);
    function GetCanvas(Index : Integer): TCanvas;
    function GetMetafile(Index : Integer): TMetafile;
    function GetFont : TFont;
    procedure SetFont(Value : TFont);
    function GetPen : TPen;
    procedure SetPen(Value : TPen);
    procedure SetOrientation(Value : TPrinterOrientation);
  protected
    { Protected declarations }
  public
    { Public declarations }
    pXppi : Integer;
    pYppi : Integer;

      {Reports where the Printable area starts}
    XOff : Integer;
    YOff : Integer;
      {Assign Title before call to PrintIt or PrintPage (not necessary)}
    Title : String;

      {reports the physical page}
    function PageSize : TPoint;
      {reports the Printable area}
    function PrintSize : TPoint;

     {Here's the gold - when you're done drawing,
     call one or all of these to display or print your work}
    procedure PrintPage(const PageNum : Integer);
    procedure PrintIt;
    procedure Preview;
       {for some reason this doesn't work - can't figure out why -
        it write the files, but the files are unreadable}
    procedure SaveToFile(const TheFile : String);

     {Routine that is called by the Preview form to display page
      Actually called in PaintBox1.OnPaint - Don't call this (it might be
      best to put this in the preview form)}
    procedure DisplayPage(Page : Integer);

     {Takes Inches and outputs Pixels}
    function XInch(const aSingle : Single): Integer;
    function YInch(const aSingle : Single): Integer;

    procedure TextOut(const X, Y : Integer; const Text : String);
    procedure TextOutRight(const X, Y : Integer; const Text : String);
    procedure TextOutCenter(const X, Y : Integer; const Text : String);
    procedure TextRect(const aRect : TRect; const X, Y : Integer;
      const Text : String);
    function TextWidth(const Text : String): Integer;
    function TextHeight: Integer;
    procedure Line(const a, b : TPoint);
    procedure Rectangle(const aRect : TRect);
      {Spits out a memo between two bounds (bottom is irrelevant)
       includes TabStop functionality - outputs Height of finished text}
    function MemoOut(const aRect : TRect; p : PChar; DisposePChar : Boolean;
      DontPrint : Boolean): Integer;
      {steps through the pages and puts the page numbers at the position you
       specify, for ex. "Page 1 of 2" - call after all your drawing for
       best results}
    procedure PutPageNums(const X, Y : Integer; Alignment : TAlignment);

      {Must call before drawing - automatically creates the first page}
    procedure NewJob;
      {Called once before Preview, PrintIt, or PringPage -
       if you don't call it, it will be called automatically}
    procedure DonePrinting;
    function NewPage : Integer;
    constructor Create;
    destructor Destroy; override;

    property PageCount : Integer  read GetPageCount;
    property CurrentPage : Integer read FCurrentPage write
      SetCurrentPage;
      {don't try using canvases after DonePrinting - the handles will be
        invalid}
    property Canvases[Index : Integer] : TCanvas read GetCanvas;
    property Metafiles[Index : Integer] : TMetafile read GetMetafile;
    property Font : TFont read GetFont write SetFont;
    property Pen : TPen read GetPen write SetPen;
      {best to set Orientation before calling NewJob}
    property Orientation : TPrinterOrientation read FOrientation write
      SetOrientation;
  end;

implementation

{$R *.DFM}

function TRyPrinter.GetPageCount : Integer;
begin
Result := FCanvases.Count;
end;

procedure TRyPrinter.SetCurrentPage(Index : Integer);
begin
if (Index <= PageCount) AND (Index > 0) then
  FCurrentPage := Index;
end;

function TRyPrinter.GetCanvas(Index : Integer): TCanvas;
begin
if (Index > 0) AND (Index <= PageCount) then
  Result := TCanvas(FCanvases[Index - 1]);
end;

function TRyPrinter.GetMetafile(Index : Integer): TMetafile;
begin
if (Index > 0) AND (Index <= PageCount) AND FDonePrinting then
  Result := TMetafile(FMetafiles[Index - 1]);
end;

function TRyPrinter.GetFont : TFont;
begin
Result := Canvases[CurrentPage].Font;
end;

procedure TRyPrinter.SetFont(Value : TFont);
begin
Canvases[CurrentPage].Font := Value;
end;

function TRyPrinter.GetPen : TPen;
begin
Result := Canvases[CurrentPage].Pen;
end;

procedure TRyPrinter.SetPen(Value : TPen);
begin
Canvases[CurrentPage].Pen := Value;
end;

procedure TRyPrinter.SetOrientation(Value : TPrinterOrientation);
begin
FOrientation := Value;
Printer.Orientation := Value;
end;

function TRyPrinter.PageSize : TPoint;
begin
Escape(Printer.Handle, GETPHYSPAGESIZE, 0, nil, @Result);
end;

function TRyPrinter.PrintSize : TPoint;
begin
Result.X := Printer.PageWidth;
Result.Y := Printer.PageHeight;
end;

procedure TRyPrinter.PrintPage(const PageNum : Integer);
var
 s : String;
begin
if not FDonePrinting then DonePrinting;
if (PageNum > 0) AND (PageNum <= PageCount) then begin
  Printer.Title := Title;
  Printer.BeginDoc;
  if Assigned(FRyPreviewForm) then s := FRyPreviewForm.Panel2.Caption;
  try
  if Assigned(FRyPreviewForm) then
    FRyPreviewForm.Panel2.Caption := Format('Printing page %d of %d',
      [PageNum, PageCount]);
  Printer.Canvas.StretchDraw(
    Rect(0,0,Printer.PageWidth, Printer.PageHeight), Metafiles[PageNum]);
  finally
  Printer.EndDoc;
  if Assigned(FRyPreviewForm) then FRyPreviewForm.Panel2.Caption := s;
  end;
  end;
end;

procedure TRyPrinter.PrintIt;
var
 i : Integer;
 s : String;
begin
if not FDonePrinting then DonePrinting;
if PageCount > 0 then begin
  Printer.Title := Title;
  Printer.BeginDoc;
  i := 1;
  if Assigned(FRyPreviewForm) then
    s := FRyPreviewForm.Panel2.Caption;
  try
  if Assigned(FRyPreviewForm) then
    FRyPreviewForm.Panel2.Caption := Format('Printing page %d of %d',
      [1, PageCount]);
  Printer.Canvas.StretchDraw(
    Rect(0,0,Printer.PageWidth, Printer.PageHeight), Metafiles[i]);
  for i := 2 to PageCount do begin
    if Assigned(FRyPreviewForm) then
      FRyPreviewForm.Panel2.Caption := Format('Printing page %d of %d',
        [i, PageCount]);
    Printer.NewPage;
    Printer.Canvas.StretchDraw(
      Rect(0,0,Printer.PageWidth, Printer.PageHeight), Metafiles[i]);
    end;
  finally
  Printer.EndDoc;
  if Assigned(FRyPreviewForm) then
    FRyPreviewForm.Panel2.Caption := s;
  end;
  end;
end;

procedure TRyPrinter.Preview;
begin
if not FDonePrinting then DonePrinting;
FRyPreviewForm := TRyPreviewFrm.Create(Application);
FRyPreviewForm.Ry := Self;
FRyPreviewForm.ShowModal;
FRyPreviewForm.Release;
FRyPreviewForm := nil;
end;

procedure TRyPrinter.SaveToFile(const TheFile : String);
var
 i : integer;
begin
if FDonePrinting then
  for i := 1 to PageCount do begin
    Metafiles[i].SaveToFile(Format('%6s%.2d', [TheFile, i]));
    end;
end;

procedure TRyPrinter.DisplayPage(Page : Integer);
var
  p : TPoint;
  Sc : Single;
  r : TRect;
  i : integer;
begin
if (Page > 0) AND (Page <= PageCount) then begin
  Sc := FRyPreviewForm.PaintBox1.Width / PageSize.X;

  FRyPreviewForm.PaintBox1.Canvas.Rectangle(
    0, 0, FRyPreviewForm.PaintBox1.Width,
    FRyPreviewForm.PaintBox1.Height);
  FRyPreviewForm.PaintBox1.Canvas.FillRect(
    Rect(1, 1, FRyPreviewForm.PaintBox1.Width - 2,
    FRyPreviewForm.PaintBox1.Height - 2));

  Escape(Printer.Handle, GETPRINTINGOFFSET, 0, nil, @p);

  r.Left := Trunc(p.X * Sc);
  r.Top := Trunc(p.Y * Sc);
  r.Right := r.Left + Trunc(Printer.PageWidth * Sc);
  r.Bottom := r.Top + Trunc(Printer.PageHeight * Sc);

  if FRyPreviewForm.ShwGrdBtn.Down then begin
    with FRyPreviewForm.PaintBox1.Canvas do begin
      Pen.Style := psDash;
      Rectangle(r.Left, r.Top, r.Right, r.Bottom);
      for i := 1 to Self.PageSize.X div Self.pXppi do begin
        MoveTo(Trunc(i * Self.pXppi * Sc), 0);
        LineTo(Trunc(i * Self.pXppi * Sc), Trunc(PageSize.Y * Sc));
        end;
      for i := 1 to Self.PageSize.Y div Self.pYppi do begin
        MoveTo(0, Trunc(i * Self.pYppi * Sc));
        LineTo(Trunc(PageSize.X * Sc), Trunc(i * Self.pYppi * Sc));
        end;
      Pen.Style := psSolid;
      end;
    end;

  FRyPreviewForm.PaintBox1.Canvas.StretchDraw(r, Metafiles[Page]);

  FRyPreviewForm.Panel2.Caption :=
    Format('Page %d of %d', [Page, PageCount]);

  FRyPreviewForm.PageDisplaying := Page;

  if Page = 1 then begin
    FRyPreviewForm.FirstBtn.Enabled := False;
    FRyPreviewForm.PriorBtn.Enabled := False;
    end else begin
    FRyPreviewForm.FirstBtn.Enabled := True;
    FRyPreviewForm.PriorBtn.Enabled := True;
    end;

  if PageCount > Page then begin
    FRyPreviewForm.NextBtn.Enabled := True;
    FRyPreviewForm.LastBtn.Enabled := True;
    end else begin
    FRyPreviewForm.NextBtn.Enabled := False;
    FRyPreviewForm.LastBtn.Enabled := False;
    end;
  end;
end;

function TRyPrinter.XInch(const aSingle : Single): Integer;
begin
Result := Trunc(pXppi * aSingle);
end;

function TRyPrinter.YInch(const aSingle : Single): Integer;
begin
Result := Trunc(pYppi * aSingle);
end;

procedure TRyPrinter.TextOut(const X, Y : Integer; const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextOut(X - XOff, Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.TextOutRight(const X, Y : Integer; const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextOut(X - Self.TextWidth(Text) - XOff, Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.TextOutCenter(const X, Y : Integer; const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextOut(X - (Self.TextWidth(Text) div 2) - XOff, Y - YOff, Text);
    end;
  end;
end;

procedure TRyPrinter.TextRect(const aRect : TRect; const X, Y : Integer;
  const Text : String);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    TextRect(Rect(aRect.Left - XOff, aRect.Top - YOff, aRect.Right - XOff,
      aRect.Bottom - YOff), X - XOff, Y - YOff, Text);
    end;
  end;
end;

function TRyPrinter.TextWidth(const Text : String): Integer;
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  Printer.Canvas.Font := Canvases[FCurrentPage].Font;
  Result := Printer.Canvas.TextWidth(Text);
  end else Result := -1;
end;

function TRyPrinter.TextHeight: Integer;
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  Printer.Canvas.Font := Canvases[FCurrentPage].Font;
  Result := Printer.Canvas.TextHeight('X');
  end else Result := -1;
end;

procedure TRyPrinter.Line(const a, b : TPoint);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    MoveTo(a.X - XOff, a.Y - YOff);
    LineTo(b.X - XOff, b.Y - YOff);
    end;
  end;
end;

procedure TRyPrinter.Rectangle(const aRect : TRect);
begin
if (FCurrentPage > 0) AND (not FDonePrinting) then begin
  with Canvases[FCurrentPage] do begin
    Rectangle(aRect.Left - XOff, aRect.Top - YOff, aRect.Right - XOff,
       aRect.Bottom - YOff);
    end;
  end;
end;

function TRyPrinter.MemoOut(const aRect : TRect; p : PChar; DisposePChar : Boolean;
  DontPrint : Boolean): Integer;
var
  x, y : Integer;
  l : PChar;
  t : Integer;
  i : Longint;
  InMiddleOfReturn : Boolean;
  InMiddleOfWord : Boolean;
  WordBegin : PChar;

    procedure DoTab;
    var d : Integer;
    begin
    d := aRect.Left;
    repeat
    Inc(d, t);
    until d > x;
    x := d;
    end;

    procedure NextLine;
    begin
    Inc(y, TextHeight);
    x := aRect.Left;
    end;

    function WordLength : Integer;
    var  temp : PChar;
    begin
      temp := StrAlloc(l - WordBegin + 1);
      StrLCopy(temp, l, l - WordBegin);
      Result := TextWidth(StrPas(temp));
      StrDispose(temp);
    end;

    procedure DoWord;
    var  temp : PChar;
    begin
    temp := StrAlloc(l - WordBegin + 1);
    StrLCopy(temp, WordBegin, l - WordBegin);

    if (TextWidth(StrPas(temp)) + x) > (aRect.Right) then NextLine;
    if not DontPrint then TextOut(x, y, StrPas(temp));

    Inc(x, TextWidth(StrPas(temp)));

    StrDispose(temp);

    InMiddleOfWord := False;
    end;

begin
if (FCurrentPage > 0) AND (not FDonePrinting) AND Assigned(p) then begin
  t := TextWidth('        ');
  l := p;
  x := aRect.Left;
  y := aRect.Top;
  InMiddleOfWord := False;
  InMiddleOfReturn := False;

  for i := 0 to StrLen(p) do begin
    case l^ of
      #9 : begin {it's a tab}
        if InMiddleOfWord then DoWord;
        DoTab;
        if x > (aRect.Right) then NextLine;
        InMiddleOfWord := False;
        InMiddleOfReturn := False;
        end;
      #10 : begin
        if InMiddleOfWord then DoWord;
        if not InMiddleOfReturn then NextLine;
        InMiddleOfWord := False;
        InMiddleOfReturn := False;
        end;
      #13 : begin
        if InMiddleOfWord then DoWord;
        NextLine;
        InMiddleOfWord := False;
        InMiddleOfReturn := True;
        end;
      #32 : begin
        if InMiddleOfWord then DoWord;
        Inc(x, TextWidth(' '));
        if (x > (aRect.Right)) then NextLine;
        InMiddleOfWord := False;
        InMiddleOfReturn := False;
        end;
      else
        if not InMiddleOfWord then WordBegin := l;
        InMiddleOfWord := True;
        InMiddleOfReturn := False
      end;
    Inc(l);
    end;
  if InMiddleOfWord then DoWord;
  if DisposePChar then StrDispose(p);
  Result := y + TextHeight - aRect.Top;
  end else Result := 0;
end;

procedure TRyPrinter.PutPageNums(const X, Y : Integer;
  Alignment : TAlignment);
var
  i : Integer;
  s : String;
  o : Integer;
begin
o := CurrentPage;
for i := 1 to PageCount do begin
  s := Format('Page %d of %d', [i, PageCount]);
  CurrentPage := i;
  case Alignment of
    taLeftJustify : TextOut(X, Y, s);
    taRightJustify : TextOutRight(X, Y, s);
    taCenter : TextOutCenter(X, Y, s);
    end;
  end;
CurrentPage := o;
end;

procedure TRyPrinter.NewJob;
var
 i : Integer;
 p : TPoint;
begin
for i := FCanvases.Count - 1 downto 0 do begin
  TCanvas(FCanvases[i]).Free;
  end;
for i := FMetafiles.Count - 1 downto 0 do begin
  TMetafile(FMetafiles[i]).Free;
  end;
FCanvases.Clear;
FMetafiles.Clear;
FCurrentPage := 0;
FDonePrinting := False;
Escape(Printer.Handle, GETPRINTINGOFFSET, 0, nil, @p);
XOff := p.X;
YOff := p.Y;
NewPage;
end;

procedure TRyPrinter.DonePrinting;
var
  i : integer;
begin
if not FDonePrinting then begin
  for i := 1 to PageCount do begin
    FMetafiles.Add(TMetafile.Create);
    TMetafile(FMetaFiles[i - 1]).Handle := CloseMetaFile(Canvases[i].Handle);
    end;
  FDonePrinting := True;
  end;
end;

function TRyPrinter.NewPage : Integer;
begin
if not FDonePrinting then begin
  Result := FCanvases.Add(TCanvas.Create) + 1;
  FCurrentPage := Result;
  TCanvas(FCanvases[Result - 1]).Handle := CreateMetafile(nil);

  SetMapMode(TCanvas(FCanvases[Result - 1]).Handle, MM_ANISOTROPIC);
  SetWindowExt(TCanvas(FCanvases[Result - 1]).Handle,
    Printer.PageWidth, Printer.PageHeight);
  with Canvases[Result] do begin
    Brush.style := bsClear;
    Font.Name := 'Arial';
    Font.PixelsPerInch := pYppi;
    Font.Size := 10;
    end;
  end;
end;

constructor TRyPrinter.Create;
begin
inherited Create;
FCanvases := TList.Create;
FMetafiles := TList.Create;
FCurrentPage := 0;
FDonePrinting := False;
pXppi := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
pYppi := GetDeviceCaps(Printer.Handle, LOGPIXELSY);
FOrientation := poPortrait;
end;

destructor TRyPrinter.Destroy;
var
 i : Integer;
begin
for i := FCanvases.Count - 1 downto 0 do begin
  TCanvas(FCanvases[i]).Free;
  end;
for i := FMetafiles.Count - 1 downto 0 do begin
  TMetafile(FMetafiles[i]).Free;
  end;
FCanvases.Free;
FMetafiles.Free;
inherited Destroy;
end;

{***************** this is for the preview form ****************}

procedure TRyPreviewFrm.PrintBtnClick(Sender: TObject);
begin
Ry.PrintIt;
end;

procedure TRyPreviewFrm.FirstBtnClick(Sender: TObject);
begin
Ry.DisplayPage(1);
end;

procedure TRyPreviewFrm.PriorBtnClick(Sender: TObject);
begin
Ry.DisplayPage(PageDisplaying - 1);
end;

procedure TRyPreviewFrm.NextBtnClick(Sender: TObject);
begin
Ry.DisplayPage(PageDisplaying + 1);
end;

procedure TRyPreviewFrm.LastBtnClick(Sender: TObject);
begin
Ry.DisplayPage(Ry.PageCount);
end;

procedure TRyPreviewFrm.FormCreate(Sender: TObject);
begin
PageDisplaying := 1;
Button7Click(Self);
end;

procedure TRyPreviewFrm.Button7Click(Sender: TObject);
begin
PaintBox1.Visible := False;
PaintBox1.Top := 15;
PaintBox1.Height := Sb.height - 30;
PaintBox1.Width := (Longint(PaintBox1.Height) * Longint(Ry.PageSize.X)) div
   Longint(Ry.PageSize.Y);
PaintBox1.Left := (Width div 2) - (PaintBox1.Width div 2);
PaintBox1.Visible := True;
end;

procedure TRyPreviewFrm.PaintBox1Paint(Sender: TObject);
begin
Ry.DisplayPage(PageDisplaying);
end;

procedure TRyPreviewFrm.Button2Click(Sender: TObject);
begin
PaintBox1.Visible := False;
PaintBox1.Top := Panel1.Height + 15;
PaintBox1.Left := 15;
PaintBox1.Width := ClientWidth - 45;
PaintBox1.Height := (Longint(PaintBox1.Width) * Longint(Ry.PageSize.Y)) div
   Longint(Ry.PageSize.X);
PaintBox1.Visible := True;
end;

procedure TRyPreviewFrm.ShwGrdBtnClick(Sender: TObject);
begin
PaintBox1.Refresh;
end;

procedure TRyPreviewFrm.Button1Click(Sender: TObject);
var
 s : String;
begin
s := '1';
if InputQuery('Print Page', 'Enter page number to print', s) then begin
  StrToInt(s);
  Ry.PrintPage(StrToInt(s));
  end;
end;

procedure TRyPreviewFrm.PaintBox1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 i : Integer;
begin
case Button of
  mbLeft : begin
    PaintBox1.Visible := False;
    PaintBox1.Top := (ClientHeight div 2) - Y;
    PaintBox1.Left := (ClientWidth div 2) - X;
    i := Trunc(PaintBox1.Width * 0.35) div 2;
    PaintBox1.Width := PaintBox1.Width + i;
    PaintBox1.Left := PaintBox1.Left - i;
    i := Trunc(PaintBox1.Height * 0.35) div 2;
    PaintBox1.Height := PaintBox1.Height + i;
    PaintBox1.Top := PaintBox1.Top - i;
    PaintBox1.Visible := True;
    end;
  mbRight : begin
    PaintBox1.Visible := False;
    i := Trunc(PaintBox1.Width * 0.35) div 2;
    PaintBox1.Width := PaintBox1.Width - i;
    PaintBox1.Left := PaintBox1.Left + i;
    i := Trunc(PaintBox1.Height * 0.35) div 2;
    PaintBox1.Height := PaintBox1.Height - i;
    PaintBox1.Top := PaintBox1.Top + i;
    PaintBox1.Top := (ClientHeight div 2) - Y;
    PaintBox1.Left := (ClientWidth div 2) - X;
    PaintBox1.Visible := True;
    end;
  end;
end;

procedure TRyPreviewFrm.Button3Click(Sender: TObject);
var
  s : String;
begin
s := 'REPORT';
if InputQuery('Save Printout as WMF', 'Enter filename', s) then
  Ry.SaveToFile(s);
end;

end.
