unit udemo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, async32, ExtCtrls;

type
  TForm1 = class(TForm)
    Comm32: TComm32;
    Memo1: TMemo;
    Memo2: TMemo;
    Label1: TLabel;
    Edit1: TEdit;
    OpenButton: TButton;
    CloseButton: TButton;
    BaudRateCombo: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    DatabitsCombo: TComboBox;
    Label4: TLabel;
    ParityCombo: TComboBox;
    Label5: TLabel;
    StopBitsCombo: TComboBox;
    Edit2: TEdit;
    LfCheckBox: TCheckBox;
    TransmitButton: TButton;
    Button1: TButton;
    BreakCheckBox: TCheckBox;
    DTRCheckBox: TCheckBox;
    Timer1: TTimer;
    Label6: TLabel;
    RtsCheckBox: TCheckBox;
    XOffButton: TButton;
    XOnButton: TButton;
    Button4: TButton;
    Bevel1: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure Comm32Open(Sender: TObject; ProviderSubType: Integer;
      Error: Integer);
    procedure CloseButtonClick(Sender: TObject);
    procedure BaudRateComboChange(Sender: TObject);
    procedure DatabitsComboChange(Sender: TObject);
    procedure ParityComboChange(Sender: TObject);
    procedure StopBitsComboChange(Sender: TObject);
    procedure TransmitButtonClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Comm32Error(Sender: TObject; Errors: Integer);
    procedure Comm32Ring(Sender: TObject);
    procedure Comm32RLSD(Sender: TObject);
    procedure Comm32TxEmpty(Sender: TObject);
    procedure Comm32RxChar(Sender: TObject; Count: Integer);
    procedure Comm32RxFlag(Sender: TObject);
    procedure Comm32Break(Sender: TObject);
    procedure Comm32CTS(Sender: TObject);
    procedure Comm32DSR(Sender: TObject);
    procedure DTRCheckBoxClick(Sender: TObject);
    procedure RTSCheckBoxClick(Sender: TObject);
    procedure BreakCheckBoxClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure XOffButtonClick(Sender: TObject);
    procedure XOnButtonClick(Sender: TObject);
  private
    FCurrentLine: Integer;
    procedure UpdateControls;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  ONOFF: array[0..1] of string = ('Off', 'On');


procedure TForm1.FormCreate(Sender: TObject);
begin
  with BaudRateCombo do
    ItemIndex := Items.IndexOf('cbr57600');
  with DataBitsCombo do
    ItemIndex := Items.IndexOf('da8');
  with ParityCombo do
    ItemIndex := Items.IndexOf('paNone');
  with StopBitsCombo do
    ItemIndex := Items.IndexOf('sb10');

  Comm32.BaudRate := TBaudrate(BaudrateCombo.ItemIndex);
  Comm32.Databits := TDataBits(DataBitsCombo.ItemIndex);
  Comm32.Parity := TParity(ParityCombo.ItemIndex);
  Comm32.StopBits := TStopBits(StopBitsCombo.ItemIndex);

  FCurrentLine := 0;   //prepare receive buffer
  Memo2.Lines.add('');

  UpdateControls;
end;

procedure TForm1.UpdateControls;
begin
  CloseButton.Enabled := Comm32.Enabled;
  TransmitButton.Enabled := (Comm32.Enabled) and (Comm32.DSR);
  DTRCheckBox.Enabled := Comm32.Enabled;
  RTSCheckBox.Enabled := Comm32.Enabled;
  BREAKCheckBox.Enabled := Comm32.Enabled;
  XOnButton.Enabled := Comm32.Enabled;
  XOffButton.Enabled := Comm32.Enabled;
end;

procedure TForm1.OpenButtonClick(Sender: TObject);
begin
   if Comm32.Enabled then
    CloseButton.Click;
  Comm32.DeviceName := Edit1.Text;
  Comm32.Open;
end;

procedure TForm1.Comm32Open(Sender: TObject; ProviderSubType: Integer; Error: Integer);
var
  S: string;
begin
  if Error <> 0 then
  begin
    Memo1.Lines.add('Error accessing device: ' + Comm32.DeviceName +
      ' (code: ' + IntToStr(Error) + ')');
  end
  else
    begin
      Memo1.Lines.add('Device ready: ' + Comm32.DeviceName +
        ' (code: ' + IntToStr(Error) + ')');
      case ProviderSubType of
        PST_FAX: S := 'FAX device';
        PST_LAT: S := 'LAT protocol';
        PST_MODEM: S := 'Modem device';
        PST_NETWORK_BRIDGE: S := 'Unspecified network bridge';
        PST_PARALLELPORT: S := 'Parallel port';
        PST_RS232: S := 'RS-232 serial port';
        PST_RS422: S := 'RS-422 port';
        PST_RS423: S := 'RS-423 port';
        PST_RS449: S := 'RS-449 port';
        PST_SCANNER: S := 'Scanner device';
        PST_TCPIP_TELNET: S := 'TCP/IP Telnet protocol';
        PST_UNSPECIFIED: S := 'Unspecified';
        PST_X25: S := 'X.25 standards';
      end;
      Memo1.Lines.add(S);
    end;

  UpdateControls;
end;

procedure TForm1.CloseButtonClick(Sender: TObject);
begin
  Memo1.Lines.add('Closing: ' + Comm32.DeviceName);
  Comm32.Close;
  UpdateControls;
end;

procedure TForm1.BaudRateComboChange(Sender: TObject);
begin
  Comm32.BaudRate := TBaudrate(BaudrateCombo.ItemIndex);
  Memo1.Lines.add('Baudrate: ' + BaudRateCombo.Text);
end;

procedure TForm1.DatabitsComboChange(Sender: TObject);
begin
  Comm32.Databits := TDataBits(DataBitsCombo.ItemIndex);
  Memo1.Lines.add('Databits: ' + DatabitsCombo.Text);
end;

procedure TForm1.ParityComboChange(Sender: TObject);
begin
  Comm32.Parity := TParity(ParityCombo.ItemIndex);
  Memo1.Lines.add('Parity: ' + ParityCombo.Text);
end;

procedure TForm1.StopBitsComboChange(Sender: TObject);
begin
  Comm32.StopBits := TStopBits(StopBitsCombo.ItemIndex);
  Memo1.Lines.add('StopBits: ' + StopBitsCombo.Text);
end;

procedure TForm1.Comm32Error(Sender: TObject; Errors: Integer);
begin
  if (Errors and CE_BREAK > 0) then
    Memo1.Lines.add('The hardware detected a break condition.');
  if (Errors and CE_DNS > 0) then
    Memo1.Lines.add('Windows 95 only: A parallel device is not selected.');
  if (Errors and CE_FRAME > 0) then
    Memo1.Lines.add('The hardware detected a framing error.');
  if (Errors and CE_IOE > 0) then
    Memo1.Lines.add('An I/O error occurred during communications with the device.');
  if (Errors and CE_MODE > 0) then
  begin
    Memo1.Lines.add('The requested mode is not supported, or the hFile parameter');
    Memo1.Lines.add('is invalid. If this value is specified, it is the only valid error.');
  end;
  if (Errors and CE_OOP > 0) then
    Memo1.Lines.add('Windows 95 only: A parallel device signaled that it is out of paper.');
  if (Errors and CE_OVERRUN > 0) then
    Memo1.Lines.add('A character-buffer overrun has occurred. The next character is lost.');
  if (Errors and CE_PTO > 0) then
    Memo1.Lines.add('Windows 95 only: A time-out occurred on a parallel device.');
  if (Errors and CE_RXOVER > 0) then
  begin
    Memo1.Lines.add('An input buffer overflow has occurred. There is either no');
    Memo1.Lines.add('room in the input buffer, or a character was received after');
    Memo1.Lines.add('the end-of-file (EOF) character.');
  end;
  if (Errors and CE_RXPARITY > 0) then
    Memo1.Lines.add('The hardware detected a parity error.');
  if (Errors and CE_TXFULL > 0) then
  begin
    Memo1.Lines.add('The application tried to transmit a character, but the output');
    Memo1.Lines.add('buffer was full.');
  end;
end;

procedure TForm1.TransmitButtonClick(Sender: TObject);
var
  S: string;
  Count: Integer;
begin
  S := Edit2.Text;
  if lfCheckBox.Checked then
    S := S + #13#10;
  Count := Comm32.Write(S[1], Length(S));
  if Count = -1 then
    Memo1.Lines.add('Error!')
  else
    Memo1.Lines.add('Transmitting ' + IntToStr(Count) + ' characters');
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Memo1.Lines.Clear;
  Memo2.Lines.Clear;
  FCurrentLine := 0;   //prepare receive buffer
  Memo2.Lines.add('');
end;

procedure TForm1.Comm32CTS(Sender: TObject);
begin
  Memo1.Lines.add('CTS SIGNAL: ' + ONOFF[ord(Comm32.CTS)]);
end;

procedure TForm1.Comm32DSR(Sender: TObject);
begin
  Memo1.Lines.add('DSR SIGNAL: ' + ONOFF[ord(Comm32.DSR)]);
  UpdateControls;
end;

procedure TForm1.Comm32Ring(Sender: TObject);
begin
  Memo1.Lines.add('RING SIGNAL: ' + ONOFF[ord(Comm32.Ring)]);
end;

procedure TForm1.Comm32RLSD(Sender: TObject);
begin
  Memo1.Lines.add('RLSD SIGNAL: ' + ONOFF[ord(Comm32.RLSD)]);
end;

procedure TForm1.Comm32TxEmpty(Sender: TObject);
begin
  Memo1.Lines.add('TXEMPTY SIGNAL');
end;

procedure TForm1.Comm32RxChar(Sender: TObject; Count: Integer);
var
  Buffer: array[0..1024] of Char;
  Bytes, P: Integer;
begin
  Memo1.Lines.add('RXCHAR SIGNAL');
  Fillchar(Buffer, Sizeof(Buffer), 0);
  Bytes := Comm32.Read(Buffer, Count);
  if Bytes = -1 then
    Memo1.Lines.add('Error reading incoming data!')
  else
  begin
    Memo1.Lines.add('Reading ' + IntToStr(Bytes) + ' characters');
    for P := 0 to Bytes do
      case Buffer[P] of
        #10: begin
               Memo2.Lines.add('');
               Inc(FCurrentLine);
             end;
        #13:;
         else
           begin
             Memo2.Lines[FCurrentLine] := Memo2.Lines[FCurrentLine] + Buffer[P];
             Memo2.Refresh;
           end;
      end;
  end;
end;

procedure TForm1.Comm32RxFlag(Sender: TObject);
begin
  Memo1.Lines.add('RXFLAG SIGNAL');
end;

procedure TForm1.Comm32Break(Sender: TObject);
begin
  Memo1.Lines.add('BREAK SIGNAL');
end;

procedure TForm1.DTRCheckBoxClick(Sender: TObject);
begin
  Comm32.DTR(DTRCheckBox.Checked);
end;

procedure TForm1.RTSCheckBoxClick(Sender: TObject);
begin
  Comm32.RTS(RTSCheckBox.Checked);
end;

procedure TForm1.BreakCheckBoxClick(Sender: TObject);
begin
  Comm32.BREAK(BreakCheckBox.Checked);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  Label6.Visible := not Label6.Visible;
end;

procedure TForm1.XOffButtonClick(Sender: TObject);
begin
  Comm32.XON(false);
end;

procedure TForm1.XOnButtonClick(Sender: TObject);
begin
  Comm32.XON(true);
end;

end.
